@@ -1430,7 +1430,7 @@ TOOLTIPS_HitTestT (const TOOLTIPS_INFO *infoPtr, LPTTHITTESTINFOW lptthit,
 
 
 static LRESULT
-TOOLTIPS_NewToolRectT (TOOLTIPS_INFO *infoPtr, const TTTOOLINFOW *ti, BOOL isW)
+TOOLTIPS_NewToolRectT (TOOLTIPS_INFO *infoPtr, const TTTOOLINFOW *ti)
 {
     INT nTool;
 
@@ -1839,14 +1839,6 @@ TOOLTIPS_UpdateTipTextT (TOOLTIPS_INFO *infoPtr, const TTTOOLINFOW *ti, BOOL isW
 }
 
 
-static LRESULT
-TOOLTIPS_WindowFromPoint (HWND hwnd, WPARAM wParam, LPARAM lParam)
-{
-    return (LRESULT)WindowFromPoint (*((LPPOINT)lParam));
-}
-
-
-
 static LRESULT
 TOOLTIPS_Create (HWND hwnd)
 {
@@ -2220,8 +2212,8 @@ TOOLTIPS_WindowProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
                                       uMsg == TTM_HITTESTW);
 	case TTM_NEWTOOLRECTA:
 	case TTM_NEWTOOLRECTW:
-	    return TOOLTIPS_NewToolRectT (infoPtr, (LPTTTOOLINFOW)lParam,
-                                          uMsg == TTM_NEWTOOLRECTW);
+	    return TOOLTIPS_NewToolRectT (infoPtr, (LPTTTOOLINFOW)lParam);
+
 	case TTM_POP:
 	    return TOOLTIPS_Pop (infoPtr);
 
@@ -2268,8 +2260,7 @@ TOOLTIPS_WindowProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
                                             uMsg == TTM_UPDATETIPTEXTW);
 
 	case TTM_WINDOWFROMPOINT:
-	    return TOOLTIPS_WindowFromPoint (hwnd, wParam, lParam);
-
+	    return (LRESULT)WindowFromPoint (*((LPPOINT)lParam));
 
 	case WM_CREATE:
 	    return TOOLTIPS_Create (hwnd);
