@@ -40,8 +40,6 @@ extern BOOL FD32_GetTemplate(PFD31_DATA lfs);
 extern BOOL FD31_Init(void);
 extern PFD31_DATA FD31_AllocPrivate(LPARAM lParam, UINT dlgType, BOOL IsUnicode);
 extern void FD31_DestroyPrivate(PFD31_DATA lfs);
-extern void FD31_MapOfnStructA(const OPENFILENAMEA *ofnA, LPOPENFILENAMEW ofnW, BOOL open);
-extern void FD31_FreeOfnW(OPENFILENAMEW *ofnW);
 extern BOOL FD31_CallWindowProc(const FD31_DATA *lfs, UINT wMsg, WPARAM wParam,
                                 LPARAM lParam);
 extern LONG FD31_WMInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam);
