@@ -304,7 +304,7 @@ static void test_create_view_template(void)
 }
 
 /* test cases for resizing of the file dialog */
-struct {
+static const struct {
     DWORD flags;
     int resize_folderchange;/* change in CDN_FOLDERCHANGE handler */
     int resize_timer1;      /* change in first WM_TIMER handler */
@@ -562,14 +562,16 @@ static void test_resize(void)
 
 /* test cases for control message IDOK */
 /* Show case for bug #19079 */
-static struct {
+typedef struct {
     int  retval;        /* return code of the message handler */
     BOOL setmsgresult;  /* set the result in the DWLP_MSGRESULT */
     BOOL usemsgokstr;   /* use the FILEOKSTRING message instead of WM_NOTIFY:CDN_FILEOK */
     BOOL do_subclass;   /* subclass the dialog hook procedure */
     BOOL expclose;      /* is the dialog expected to close ? */
     BOOL actclose;      /* has the dialog actually closed ? */
-} ok_testcases[] = {
+} ok_wndproc_testcase;
+
+static ok_wndproc_testcase ok_testcases[] = {
     { 0,        FALSE,  FALSE,  FALSE,  TRUE},
     { 0,         TRUE,  FALSE,  FALSE,  TRUE},
     { 0,        FALSE,  FALSE,   TRUE,  TRUE},
@@ -590,12 +592,12 @@ static struct {
 static LONG_PTR WINAPI test_ok_wndproc(HWND dlg, UINT msg, WPARAM wParam, LPARAM lParam)
 {
     HWND parent = GetParent( dlg);
-    static int index;
+    static ok_wndproc_testcase *testcase = NULL;
     static UINT msgFILEOKSTRING;
     if (msg == WM_INITDIALOG)
     {
-        index = ((OPENFILENAME*)lParam)->lCustData;
-        ok_testcases[index].actclose = TRUE;
+        testcase = (ok_wndproc_testcase*)((OPENFILENAME*)lParam)->lCustData;
+        testcase->actclose = TRUE;
         msgFILEOKSTRING = RegisterWindowMessageA( FILEOKSTRING);
     }
     if( msg == WM_NOTIFY) {
@@ -604,28 +606,28 @@ static LONG_PTR WINAPI test_ok_wndproc(HWND dlg, UINT msg, WPARAM wParam, LPARAM
             PostMessage( parent, WM_COMMAND, IDOK, 0);
             return FALSE;
         } else if(((LPNMHDR)lParam)->code == CDN_FILEOK) {
-            if( ok_testcases[index].usemsgokstr)
+            if( testcase->usemsgokstr)
                 return FALSE;
-            if( ok_testcases[index].setmsgresult)
-                SetWindowLongPtrA( dlg, DWLP_MSGRESULT, ok_testcases[index].retval);
-            return ok_testcases[index].retval;
+            if( testcase->setmsgresult)
+                SetWindowLongPtrA( dlg, DWLP_MSGRESULT, testcase->retval);
+            return testcase->retval;
         }
     }
     if( msg == msgFILEOKSTRING) {
-        if( !ok_testcases[index].usemsgokstr)
+        if( !testcase->usemsgokstr)
             return FALSE;
-        if( ok_testcases[index].setmsgresult)
-            SetWindowLongPtrA( dlg, DWLP_MSGRESULT, ok_testcases[index].retval);
-        return ok_testcases[index].retval;
+        if( testcase->setmsgresult)
+            SetWindowLongPtrA( dlg, DWLP_MSGRESULT, testcase->retval);
+        return testcase->retval;
     }
     if( msg == WM_TIMER) {
         /* the dialog did not close automatically */
-        ok_testcases[index].actclose = FALSE;
+        testcase->actclose = FALSE;
         KillTimer( dlg, 0);
         PostMessage( parent, WM_COMMAND, IDCANCEL, 0);
         return FALSE;
     }
-    if( ok_testcases[index].do_subclass)
+    if( testcase && testcase->do_subclass)
         return DefWindowProc( dlg, msg, wParam, lParam);
     return FALSE;
 }
@@ -658,7 +660,7 @@ static void test_ok(void)
     ofn.Flags =  OFN_ENABLEHOOK | OFN_EXPLORER| OFN_ENABLETEMPLATE ;
     for( i = 0; ok_testcases[i].retval != -1; i++) {
         strcpy( filename, tmpfilename);
-        ofn.lCustData = i;
+        ofn.lCustData = (LPARAM)(ok_testcases + i);
         ofn.lpfnHook = ok_testcases[i].do_subclass
             ? (LPOFNHOOKPROC) ok_template_hook
             : (LPOFNHOOKPROC) test_ok_wndproc;
@@ -876,7 +878,7 @@ static void test_arrange(void)
     }
 }
 
-static CHAR WINDIR[MAX_PATH];
+static CHAR SYSDIR[MAX_PATH];
 
 static UINT_PTR CALLBACK path_hook_proc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
 {
@@ -896,8 +898,8 @@ static UINT_PTR CALLBACK path_hook_proc( HWND hDlg, UINT msg, WPARAM wParam, LPA
 
             memset(buf, 0x66, sizeof(buf));
             ret = SendMessageA( GetParent(hDlg), CDM_GETFOLDERPATH, sizeof(buf), (LPARAM)buf);
-            ok(!lstrcmpA(WINDIR, buf), "Expected '%s', got '%s'\n", WINDIR, buf);
-            ok(lstrlenA(WINDIR) + 1 == ret, "Expected %d, got %d\n", lstrlenA(WINDIR) + 1, ret);
+            ok(!lstrcmpiA(SYSDIR, buf), "Expected '%s', got '%s'\n", SYSDIR, buf);
+            ok(lstrlenA(SYSDIR) + 1 == ret, "Expected %d, got %d\n", lstrlenA(SYSDIR) + 1, ret);
         }
     }
 
@@ -911,8 +913,11 @@ static void test_getfolderpath(void)
     char szFileName[MAX_PATH] = "";
     char szInitialDir[MAX_PATH];
 
-    GetWindowsDirectory(szInitialDir, MAX_PATH);
-    lstrcpyA(WINDIR, szInitialDir);
+    /* We need to pick a different directory as the other tests because of new
+     * Windows 7 behavior.
+     */
+    GetSystemDirectory(szInitialDir, MAX_PATH);
+    lstrcpyA(SYSDIR, szInitialDir);
 
     ZeroMemory(&ofn, sizeof(ofn));
 
@@ -980,6 +985,102 @@ static void test_resizable2(void)
 #undef ISSIZABLE
 }
 
+static void test_mru(void)
+{
+    ok_wndproc_testcase testcase = {0};
+    OPENFILENAME ofn = {sizeof(OPENFILENAME)};
+    const char *test_dir_name = "C:\\mru_test";
+    const char *test_file_name = "test.txt";
+    const char *test_full_path = "C:\\mru_test\\test.txt";
+    char filename_buf[MAX_PATH];
+    DWORD ret;
+
+    ofn.lpstrFile = filename_buf;
+    ofn.nMaxFile = sizeof(filename_buf);
+    ofn.lpTemplateName = "template1";
+    ofn.hInstance = GetModuleHandle(NULL);
+    ofn.Flags =  OFN_ENABLEHOOK | OFN_EXPLORER | OFN_ENABLETEMPLATE | OFN_NOCHANGEDIR;
+    ofn.lCustData = (LPARAM)&testcase;
+    ofn.lpfnHook = (LPOFNHOOKPROC)test_ok_wndproc;
+
+    SetLastError(0xdeadbeef);
+    ret = CreateDirectoryA(test_dir_name, NULL);
+    ok(ret == TRUE, "CreateDirectoryA should have succeeded: %d\n", GetLastError());
+
+    /* "teach" comdlg32 about this directory */
+    strcpy(filename_buf, test_full_path);
+    SetLastError(0xdeadbeef);
+    ret = GetOpenFileNameA(&ofn);
+    ok(ret, "GetOpenFileNameA should have succeeded: %d\n", GetLastError());
+    ret = CommDlgExtendedError();
+    ok(!ret, "CommDlgExtendedError returned %x\n", ret);
+    ok(testcase.actclose, "Open File dialog should have closed.\n");
+    ok(!strcmp(ofn.lpstrFile, test_full_path), "Expected to get %s, got %s\n", test_full_path, ofn.lpstrFile);
+
+    /* get a filename without a full path. it should return the file in
+     * test_dir_name, not in the CWD */
+    strcpy(filename_buf, test_file_name);
+    SetLastError(0xdeadbeef);
+    ret = GetOpenFileNameA(&ofn);
+    ok(ret, "GetOpenFileNameA should have succeeded: %d\n", GetLastError());
+    ret = CommDlgExtendedError();
+    ok(!ret, "CommDlgExtendedError returned %x\n", ret);
+    ok(testcase.actclose, "Open File dialog should have closed.\n");
+    if(strcmp(ofn.lpstrFile, test_full_path) != 0)
+        win_skip("Platform doesn't save MRU data\n");
+
+    SetLastError(0xdeadbeef);
+    ret = RemoveDirectoryA(test_dir_name);
+    ok(ret == TRUE, "RemoveDirectoryA should have succeeded: %d\n", GetLastError());
+}
+
+static UINT_PTR WINAPI test_extension_wndproc(HWND dlg, UINT msg, WPARAM wParam, LPARAM lParam)
+{
+    HWND parent = GetParent( dlg);
+    if( msg == WM_NOTIFY) {
+        SetTimer( dlg, 0, 100, 0);
+        PostMessage( parent, WM_COMMAND, IDOK, 0);
+    }
+    if( msg == WM_TIMER) {
+        /* the dialog did not close automatically */
+        KillTimer( dlg, 0);
+        PostMessage( parent, WM_COMMAND, IDCANCEL, 0);
+    }
+    return FALSE;
+}
+
+static void test_extension(void)
+{
+    OPENFILENAME ofn = { sizeof(OPENFILENAME)};
+    char filename[1024] = {0};
+    char curdir[MAX_PATH];
+    char *filename_ptr;
+    const char *test_file_name = "deadbeef";
+    DWORD ret;
+
+    ok(GetCurrentDirectoryA(sizeof(curdir), curdir) != 0, "Failed to get current dir err %d\n", GetLastError());
+
+    /* Ignore .* extension */
+    ofn.lStructSize = sizeof(ofn);
+    ofn.hwndOwner = NULL;
+    ofn.lpstrFile = filename;
+    ofn.nMaxFile = MAX_PATH;
+    ofn.Flags = OFN_EXPLORER | OFN_ENABLEHOOK;
+    ofn.lpstrDefExt = NULL;
+    ofn.lpstrInitialDir = curdir;
+    ofn.lpfnHook = test_extension_wndproc;
+    ofn.nFileExtension = 0;
+    ofn.lpstrFilter = "All Files (*.*)\0*.*\0";
+    strcpy(filename, test_file_name);
+
+    ret = GetSaveFileNameA(&ofn);
+    filename_ptr = ofn.lpstrFile + strlen( ofn.lpstrFile ) - strlen( test_file_name );
+    ok(1 == ret, "expected 1, got %d\n", ret);
+    ok(strlen(ofn.lpstrFile) >= strlen(test_file_name), "Filename %s is too short\n", ofn.lpstrFile );
+    ok( strcmp(filename_ptr, test_file_name) == 0,
+        "Filename is %s, expected %s\n", filename_ptr, test_file_name );
+}
+
 START_TEST(filedlg)
 {
     test_DialogCancel();
@@ -989,5 +1090,7 @@ START_TEST(filedlg)
     test_resize();
     test_ok();
     test_getfolderpath();
+    test_mru();
     if( resizesupported) test_resizable2();
+    test_extension();
 }
