@@ -29,10 +29,30 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(crypt);
 
+static inline BOOL is_quotable_char(char c)
+{
+    switch(c)
+    {
+    case '+':
+    case ',':
+    case '"':
+    case '=':
+    case '<':
+    case '>':
+    case ';':
+    case '#':
+    case '\n':
+        return TRUE;
+    default:
+        return FALSE;
+    }
+}
+
 DWORD WINAPI CertRDNValueToStrA(DWORD dwValueType, PCERT_RDN_VALUE_BLOB pValue,
  LPSTR psz, DWORD csz)
 {
-    DWORD ret = 0;
+    DWORD ret = 0, len, i;
+    BOOL needsQuotes = FALSE;
 
     TRACE("(%d, %p, %p, %d)\n", dwValueType, pValue, psz, csz);
 
@@ -48,29 +68,78 @@ DWORD WINAPI CertRDNValueToStrA(DWORD dwValueType, PCERT_RDN_VALUE_BLOB pValue,
     case CERT_RDN_GRAPHIC_STRING:
     case CERT_RDN_VISIBLE_STRING:
     case CERT_RDN_GENERAL_STRING:
+        len = pValue->cbData;
+        if (pValue->cbData && isspace(pValue->pbData[0]))
+            needsQuotes = TRUE;
+        if (pValue->cbData && isspace(pValue->pbData[pValue->cbData - 1]))
+            needsQuotes = TRUE;
+        for (i = 0; i < pValue->cbData; i++)
+        {
+            if (is_quotable_char(pValue->pbData[i]))
+                needsQuotes = TRUE;
+            if (pValue->pbData[i] == '"')
+                len += 1;
+        }
+        if (needsQuotes)
+            len += 2;
         if (!psz || !csz)
-            ret = pValue->cbData;
+            ret = len;
         else
         {
-            DWORD chars = min(pValue->cbData, csz - 1);
+            char *ptr = psz;
 
-            if (chars)
+            if (needsQuotes)
+                *ptr++ = '"';
+            for (i = 0; i < pValue->cbData && ptr - psz < csz; ptr++, i++)
             {
-                memcpy(psz, pValue->pbData, chars);
-                ret += chars;
-                csz -= chars;
+                *ptr = pValue->pbData[i];
+                if (pValue->pbData[i] == '"' && ptr - psz < csz - 1)
+                    *(++ptr) = '"';
             }
+            if (needsQuotes && ptr - psz < csz)
+                *ptr++ = '"';
+            ret = ptr - psz;
         }
         break;
+    case CERT_RDN_BMP_STRING:
     case CERT_RDN_UTF8_STRING:
+        len = WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)pValue->pbData,
+         pValue->cbData / sizeof(WCHAR), NULL, 0, NULL, NULL);
+        if (pValue->cbData && isspaceW(((LPCWSTR)pValue->pbData)[0]))
+            needsQuotes = TRUE;
+        if (pValue->cbData &&
+         isspaceW(((LPCWSTR)pValue->pbData)[pValue->cbData / sizeof(WCHAR)-1]))
+            needsQuotes = TRUE;
+        for (i = 0; i < pValue->cbData / sizeof(WCHAR); i++)
+        {
+            if (is_quotable_char(((LPCWSTR)pValue->pbData)[i]))
+                needsQuotes = TRUE;
+            if (((LPCWSTR)pValue->pbData)[i] == '"')
+                len += 1;
+        }
+        if (needsQuotes)
+            len += 2;
         if (!psz || !csz)
-            ret = WideCharToMultiByte(CP_UTF8, 0, (LPWSTR)pValue->pbData,
-             pValue->cbData / sizeof(WCHAR) + 1, NULL, 0, NULL, NULL);
+            ret = len;
         else
         {
-            ret = WideCharToMultiByte(CP_UTF8, 0, (LPWSTR)pValue->pbData,
-             pValue->cbData / sizeof(WCHAR) + 1, psz, csz - 1, NULL, NULL);
-            csz -= ret;
+            char *dst = psz;
+
+            if (needsQuotes)
+                *dst++ = '"';
+            for (i = 0; i < pValue->cbData / sizeof(WCHAR) &&
+             dst - psz < csz; dst++, i++)
+            {
+                LPCWSTR src = (LPCWSTR)pValue->pbData + i;
+
+                WideCharToMultiByte(CP_ACP, 0, src, 1, dst,
+                 csz - (dst - psz) - 1, NULL, NULL);
+                if (*src == '"' && dst - psz < csz - 1)
+                    *(++dst) = '"';
+            }
+            if (needsQuotes && dst - psz < csz)
+                *dst++ = '"';
+            ret = dst - psz;
         }
         break;
     default:
@@ -91,7 +160,8 @@ DWORD WINAPI CertRDNValueToStrA(DWORD dwValueType, PCERT_RDN_VALUE_BLOB pValue,
 DWORD WINAPI CertRDNValueToStrW(DWORD dwValueType, PCERT_RDN_VALUE_BLOB pValue,
  LPWSTR psz, DWORD csz)
 {
-    DWORD ret = 0;
+    DWORD ret = 0, len, i, strLen;
+    BOOL needsQuotes = FALSE;
 
     TRACE("(%d, %p, %p, %d)\n", dwValueType, pValue, psz, csz);
 
@@ -107,39 +177,72 @@ DWORD WINAPI CertRDNValueToStrW(DWORD dwValueType, PCERT_RDN_VALUE_BLOB pValue,
     case CERT_RDN_GRAPHIC_STRING:
     case CERT_RDN_VISIBLE_STRING:
     case CERT_RDN_GENERAL_STRING:
+        len = pValue->cbData;
+        if (pValue->cbData && isspace(pValue->pbData[0]))
+            needsQuotes = TRUE;
+        if (pValue->cbData && isspace(pValue->pbData[pValue->cbData - 1]))
+            needsQuotes = TRUE;
+        for (i = 0; i < pValue->cbData; i++)
+        {
+            if (is_quotable_char(pValue->pbData[i]))
+                needsQuotes = TRUE;
+            if (pValue->pbData[i] == '"')
+                len += 1;
+        }
+        if (needsQuotes)
+            len += 2;
         if (!psz || !csz)
-            ret = pValue->cbData;
+            ret = len;
         else
         {
-            DWORD chars = min(pValue->cbData, csz - 1);
+            WCHAR *ptr = psz;
 
-            if (chars)
+            if (needsQuotes)
+                *ptr++ = '"';
+            for (i = 0; i < pValue->cbData && ptr - psz < csz; ptr++, i++)
             {
-                DWORD i;
-
-                for (i = 0; i < chars; i++)
-                    psz[i] = pValue->pbData[i];
-                ret += chars;
-                csz -= chars;
+                *ptr = pValue->pbData[i];
+                if (pValue->pbData[i] == '"' && ptr - psz < csz - 1)
+                    *(++ptr) = '"';
             }
+            if (needsQuotes && ptr - psz < csz)
+                *ptr++ = '"';
+            ret = ptr - psz;
         }
         break;
+    case CERT_RDN_BMP_STRING:
     case CERT_RDN_UTF8_STRING:
+        strLen = len = pValue->cbData / sizeof(WCHAR);
+        if (pValue->cbData && isspace(pValue->pbData[0]))
+            needsQuotes = TRUE;
+        if (pValue->cbData && isspace(pValue->pbData[strLen - 1]))
+            needsQuotes = TRUE;
+        for (i = 0; i < strLen; i++)
+        {
+            if (is_quotable_char(((LPCWSTR)pValue->pbData)[i]))
+                needsQuotes = TRUE;
+            if (((LPCWSTR)pValue->pbData)[i] == '"')
+                len += 1;
+        }
+        if (needsQuotes)
+            len += 2;
         if (!psz || !csz)
-            ret = pValue->cbData / sizeof(WCHAR);
+            ret = len;
         else
         {
-            DWORD chars = min(pValue->cbData / sizeof(WCHAR), csz - 1);
+            WCHAR *ptr = psz;
 
-            if (chars)
+            if (needsQuotes)
+                *ptr++ = '"';
+            for (i = 0; i < strLen && ptr - psz < csz; ptr++, i++)
             {
-                DWORD i;
-
-                for (i = 0; i < chars; i++)
-                    psz[i] = *((LPWSTR)pValue->pbData + i);
-                ret += chars;
-                csz -= chars;
+                *ptr = ((LPCWSTR)pValue->pbData)[i];
+                if (((LPCWSTR)pValue->pbData)[i] == '"' && ptr - psz < csz - 1)
+                    *(++ptr) = '"';
             }
+            if (needsQuotes && ptr - psz < csz)
+                *ptr++ = '"';
+            ret = ptr - psz;
         }
         break;
     default:
@@ -255,7 +358,6 @@ DWORD WINAPI CertNameToStrA(DWORD dwCertEncodingType, PCERT_NAME_BLOB pName,
                      psz ? psz + ret : NULL, psz ? csz - ret - 1 : 0);
                     ret += chars;
                 }
-                /* FIXME: handle quoting */
                 chars = CertRDNValueToStrA(
                  rdn->rgRDNAttr[j].dwValueType,
                  &rdn->rgRDNAttr[j].Value, psz ? psz + ret : NULL,
@@ -435,7 +537,6 @@ DWORD cert_name_to_str_with_indent(DWORD dwCertEncodingType, DWORD indentLevel,
                      psz ? psz + ret : NULL, psz ? csz - ret - 1 : 0);
                     ret += chars;
                 }
-                /* FIXME: handle quoting */
                 chars = CertRDNValueToStrW(
                  rdn->rgRDNAttr[j].dwValueType,
                  &rdn->rgRDNAttr[j].Value, psz ? psz + ret : NULL,
@@ -970,7 +1071,7 @@ static PCERT_ALT_NAME_ENTRY cert_find_alt_name_entry(PCCERT_CONTEXT cert,
 }
 
 static DWORD cert_get_name_from_rdn_attr(DWORD encodingType,
- PCERT_NAME_BLOB name, LPCSTR oid, LPWSTR pszNameString, DWORD cchNameString)
+ const CERT_NAME_BLOB *name, LPCSTR oid, LPWSTR pszNameString, DWORD cchNameString)
 {
     CERT_NAME_INFO *nameInfo;
     DWORD bytes = 0, ret = 0;
