@@ -79,6 +79,14 @@ const char* pe_map_section(struct image_section_map* ism)
         fmap->sect[ism->sidx].mapped == IMAGE_NO_MAP)
     {
         IMAGE_NT_HEADERS*       nth;
+
+        if (fmap->sect[ism->sidx].shdr.Misc.VirtualSize > fmap->sect[ism->sidx].shdr.SizeOfRawData)
+        {
+            FIXME("Section %ld: virtual (0x%x) > raw (0x%x) size - not supported\n",
+                  ism->sidx, fmap->sect[ism->sidx].shdr.Misc.VirtualSize,
+                  fmap->sect[ism->sidx].shdr.SizeOfRawData);
+            return IMAGE_NO_MAP;
+        }
         /* FIXME: that's rather drastic, but that will do for now
          * that's ok if the full file map exists, but we could be less agressive otherwise and
          * only map the relevant section
@@ -162,13 +170,13 @@ DWORD_PTR pe_get_map_rva(const struct image_section_map* ism)
 /******************************************************************
  *		pe_get_map_size
  *
- * Get the size of an PE section
+ * Get the size of a PE section
  */
 unsigned pe_get_map_size(const struct image_section_map* ism)
 {
     if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.pe.ntheader.FileHeader.NumberOfSections)
         return 0;
-    return ism->fmap->u.pe.sect[ism->sidx].shdr.SizeOfRawData;
+    return ism->fmap->u.pe.sect[ism->sidx].shdr.Misc.VirtualSize;
 }
 
 /******************************************************************
@@ -455,17 +463,6 @@ static BOOL pe_load_coff_symbol_table(struct module* module)
     return TRUE;
 }
 
-static inline void* pe_get_sect(IMAGE_NT_HEADERS* nth, void* mapping,
-                                IMAGE_SECTION_HEADER* sect)
-{
-    return (sect) ? RtlImageRvaToVa(nth, mapping, sect->VirtualAddress, NULL) : NULL;
-}
-
-static inline DWORD pe_get_sect_size(IMAGE_SECTION_HEADER* sect)
-{
-    return (sect) ? sect->SizeOfRawData : 0;
-}
-
 /******************************************************************
  *		pe_load_stabs
  *
@@ -736,7 +733,7 @@ BOOL pe_load_debug_info(const struct process* pcs, struct module* module)
  *
  */
 struct module* pe_load_native_module(struct process* pcs, const WCHAR* name,
-                                     HANDLE hFile, DWORD base, DWORD size)
+                                     HANDLE hFile, DWORD64 base, DWORD size)
 {
     struct module*              module = NULL;
     BOOL                        opened = FALSE;
