@@ -33,7 +33,7 @@ static LPDIRECTDRAW lpDD = NULL;
 static LPDIRECTDRAWSURFACE lpDDSPrimary = NULL;
 static LPDIRECTDRAWSURFACE lpDDSBack = NULL;
 static WNDCLASS wc;
-static HWND hwnd;
+static HWND hwnd, hwnd2;
 static int modes_cnt;
 static int modes_size;
 static LPDDSURFACEDESC modes;
@@ -54,31 +54,21 @@ static void init_function_pointers(void)
     pDirectDrawEnumerateExW = (void*)GetProcAddress(hmod, "DirectDrawEnumerateExW");
 }
 
-static void createwindow(void)
+static HWND createwindow(void)
 {
-    wc.style = CS_HREDRAW | CS_VREDRAW;
-    wc.lpfnWndProc = DefWindowProcA;
-    wc.cbClsExtra = 0;
-    wc.cbWndExtra = 0;
-    wc.hInstance = GetModuleHandleA(0);
-    wc.hIcon = LoadIconA(wc.hInstance, IDI_APPLICATION);
-    wc.hCursor = LoadCursorA(NULL, IDC_ARROW);
-    wc.hbrBackground = GetStockObject(BLACK_BRUSH);
-    wc.lpszMenuName = NULL;
-    wc.lpszClassName = "TestWindowClass";
-    if(!RegisterClassA(&wc))
-        assert(0);
+    HWND hwnd;
 
     hwnd = CreateWindowExA(0, "TestWindowClass", "TestWindowClass",
         WS_POPUP, 0, 0,
         GetSystemMetrics(SM_CXSCREEN),
         GetSystemMetrics(SM_CYSCREEN),
         NULL, NULL, GetModuleHandleA(0), NULL);
-    assert(hwnd != NULL);
 
     ShowWindow(hwnd, SW_HIDE);
     UpdateWindow(hwnd);
     SetFocus(hwnd);
+
+    return hwnd;
 }
 
 static BOOL createdirectdraw(void)
@@ -546,6 +536,7 @@ static void testdisplaymodes(void)
 
 static void testcooperativelevels_normal(void)
 {
+    BOOL sfw;
     HRESULT rc;
     DDSURFACEDESC surfacedesc;
     IDirectDrawSurface *surface = (IDirectDrawSurface *) 0xdeadbeef;
@@ -557,26 +548,58 @@ static void testcooperativelevels_normal(void)
     surfacedesc.dwBackBufferCount = 1;
     surfacedesc.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_COMPLEX | DDSCAPS_FLIP;
 
+    rc = IDirectDraw_SetCooperativeLevel(lpDD, hwnd, DDSCL_SETFOCUSWINDOW | DDSCL_CREATEDEVICEWINDOW);
+    ok(rc==DDERR_INVALIDPARAMS,"SetCooperativeLevel(DDSCL_SETFOCUSWINDOW | DDSCL_CREATEDEVICEWINDOW) returned: %x\n",rc);
+
     /* Do some tests with DDSCL_NORMAL mode */
+
     /* Fullscreen mode + normal mode + exclusive mode */
+
+    rc = IDirectDraw_SetCooperativeLevel(lpDD, NULL, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE | DDSCL_NORMAL);
+    ok(rc==DDERR_INVALIDPARAMS, "Expected DDERR_INVALIDPARAMS, received: %x\n", rc);
+
+    sfw=FALSE;
+    if(hwnd2)
+        sfw=SetForegroundWindow(hwnd2);
+    else
+        skip("Failed to create the second window\n");
+
     rc = IDirectDraw_SetCooperativeLevel(lpDD, hwnd, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE | DDSCL_NORMAL);
-    todo_wine ok(rc==DD_OK,"SetCooperativeLevel(DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE | DDSCL_NORMAL) returned: %x\n",rc);
+    ok(rc==DD_OK,"SetCooperativeLevel(DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE | DDSCL_NORMAL) returned: %x\n",rc);
+
+    if(sfw)
+        ok(GetForegroundWindow()==hwnd,"Expected the main windows (%p) for foreground, received the second one (%p)\n",hwnd, hwnd2);
+
     /* Try creating a double buffered primary in fullscreen + exclusive + normal mode */
     rc = IDirectDraw_CreateSurface(lpDD, &surfacedesc, &surface, NULL);
+
     if (rc == DDERR_UNSUPPORTEDMODE)
         skip("Unsupported mode\n");
     else
     {
-        todo_wine ok(rc == DD_OK, "IDirectDraw_CreateSurface returned %08x\n", rc);
-        todo_wine ok(surface!=NULL, "Returned NULL surface pointer \n");
+        ok(rc == DD_OK, "IDirectDraw_CreateSurface returned %08x\n", rc);
+        ok(surface!=NULL, "Returned NULL surface pointer\n");
     }
     if(surface && surface != (IDirectDrawSurface *)0xdeadbeef) IDirectDrawSurface_Release(surface);
+
     /* Exclusive mode + normal mode */
     rc = IDirectDraw_SetCooperativeLevel(lpDD, hwnd, DDSCL_EXCLUSIVE | DDSCL_NORMAL);
     ok(rc==DDERR_INVALIDPARAMS,"SetCooperativeLevel(DDSCL_EXCLUSIVE | DDSCL_NORMAL) returned: %x\n",rc);
+
     /* Fullscreen mode + normal mode */
+
+    sfw=FALSE;
+    if(hwnd2) sfw=SetForegroundWindow(hwnd2);
+
     rc = IDirectDraw_SetCooperativeLevel(lpDD, hwnd, DDSCL_FULLSCREEN | DDSCL_NORMAL);
-    todo_wine ok(rc==DD_OK,"SetCooperativeLevel(DDSCL_FULLSCREEN | DDSCL_NORMAL) returned: %x\n",rc);
+    ok(rc==DD_OK,"SetCooperativeLevel(DDSCL_FULLSCREEN | DDSCL_NORMAL) returned: %x\n",rc);
+
+    if(sfw)
+        ok(GetForegroundWindow()==hwnd2,"Expected the second windows (%p) for foreground, received the main one (%p)\n",hwnd2, hwnd);
+
+    rc = IDirectDraw_SetCooperativeLevel(lpDD, NULL, DDSCL_FULLSCREEN | DDSCL_NORMAL);
+    ok(rc==DD_OK, "Expected DD_OK, received %x\n", rc);
+
     /* Try creating a double buffered primary in fullscreen + normal mode */
     rc = IDirectDraw_CreateSurface(lpDD, &surfacedesc, &surface, NULL);
     if (rc == DDERR_UNSUPPORTEDMODE)
@@ -586,12 +609,24 @@ static void testcooperativelevels_normal(void)
         ok(rc == DDERR_NOEXCLUSIVEMODE, "IDirectDraw_CreateSurface returned %08x\n", rc);
         ok(surface == NULL, "Returned surface pointer is %p\n", surface);
     }
+
     if(surface && surface != (IDirectDrawSurface *)0xdeadbeef) IDirectDrawSurface_Release(surface);
-    /* Normal mode */
+
+    /* switching from Fullscreen mode to Normal mode */
+
+    sfw=FALSE;
+    if(hwnd2) sfw=SetForegroundWindow(hwnd2);
+
     rc = IDirectDraw_SetCooperativeLevel(lpDD,
         hwnd, DDSCL_NORMAL);
     ok(rc==DD_OK,"SetCooperativeLevel(DDSCL_NORMAL) returned: %x\n",rc);
 
+    if(sfw)
+        ok(GetForegroundWindow()==hwnd2,"Expected the second windows (%p) for foreground, received the main one (%p)\n",hwnd2, hwnd);
+
+    rc = IDirectDraw_SetCooperativeLevel(lpDD, NULL, DDSCL_NORMAL);
+    ok(rc==DD_OK, "Expected DD_OK, received %x\n", rc);
+
     /* Try creating a double buffered primary in normal mode */
     rc = IDirectDraw_CreateSurface(lpDD, &surfacedesc, &surface, NULL);
     if (rc == DDERR_UNSUPPORTEDMODE)
@@ -603,7 +638,36 @@ static void testcooperativelevels_normal(void)
     }
     if(surface && surface != (IDirectDrawSurface *)0xdeadbeef) IDirectDrawSurface_Release(surface);
 
+    /* switching from Normal mode to Fullscreen + Normal mode */
+
+    sfw=FALSE;
+    if(hwnd2) sfw=SetForegroundWindow(hwnd2);
+
+    rc = IDirectDraw_SetCooperativeLevel(lpDD,
+        hwnd, DDSCL_NORMAL | DDSCL_FULLSCREEN);
+    ok(rc==DD_OK,"SetCooperativeLevel(DDSCL_NORMAL | FULLSCREEN) returned: %x\n",rc);
+
+    if(sfw)
+        ok(GetForegroundWindow()==hwnd2,"Expected the second windows (%p) for foreground, received the main one (%p)\n",hwnd2, hwnd);
+
+    rc = IDirectDraw_SetCooperativeLevel(lpDD, NULL, DDSCL_NORMAL | DDSCL_FULLSCREEN);
+    ok(rc==DD_OK, "Expected DD_OK, received %x\n", rc);
+
     /* Set the focus window */
+
+    rc = IDirectDraw_SetCooperativeLevel(lpDD, hwnd, DDSCL_SETFOCUSWINDOW | DDSCL_CREATEDEVICEWINDOW);
+    ok(rc==DDERR_INVALIDPARAMS,"SetCooperativeLevel(DDSCL_SETFOCUSWINDOW | DDSCL_CREATEDEVICEWINDOW) returned: %x\n",rc);
+
+    rc = IDirectDraw_SetCooperativeLevel(lpDD, NULL, DDSCL_SETFOCUSWINDOW);
+
+    if (rc == DDERR_INVALIDPARAMS)
+    {
+        win_skip("NT4/Win95 do not support cooperative levels DDSCL_SETDEVICEWINDOW and DDSCL_SETFOCUSWINDOW\n");
+        return;
+    }
+
+    ok(rc==DD_OK, "Expected DD_OK, received %x\n", rc);
+
     rc = IDirectDraw_SetCooperativeLevel(lpDD,
         hwnd, DDSCL_SETFOCUSWINDOW);
 
@@ -674,7 +738,7 @@ static void testcooperativelevels_normal(void)
 
 static void testcooperativelevels_exclusive(void)
 {
-    BOOL success;
+    BOOL sfw, success;
     HRESULT rc;
     RECT window_rect;
 
@@ -695,13 +759,27 @@ static void testcooperativelevels_exclusive(void)
     ok(rc==DDERR_INVALIDPARAMS,"SetCooperativeLevel(DDSCL_FULLSCREEN) returned: %x\n",rc);
 
     /* Full screen mode + exclusive mode */
+
+    rc = IDirectDraw_SetCooperativeLevel(lpDD, NULL, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE);
+    ok(rc==DDERR_INVALIDPARAMS, "Expected DDERR_INVALIDPARAMS, received %x\n", rc);
+
+    sfw=FALSE;
+    if(hwnd2)
+        sfw=SetForegroundWindow(hwnd2);
+    else
+        skip("Failed to create the second window\n");
+
     rc = IDirectDraw_SetCooperativeLevel(lpDD,
         hwnd, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE);
     ok(rc==DD_OK,"SetCooperativeLevel(DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN) returned: %x\n",rc);
-    GetClientRect(hwnd, &window_rect);
+
+    if(sfw)
+        ok(GetForegroundWindow()==hwnd,"Expected the main windows (%p) for foreground, received the second one (%p)\n",hwnd, hwnd2);
+
     /* rect_before_create is assumed to hold the screen rect */
+    GetClientRect(hwnd, &window_rect);
     rc = EqualRect(&rect_before_create, &window_rect);
-    todo_wine ok(rc!=0, "Fullscreen window has wrong size\n");
+    ok(rc, "Fullscreen window has wrong size.\n");
 
     /* Set the focus window. Should fail */
     rc = IDirectDraw_SetCooperativeLevel(lpDD,
@@ -776,9 +854,36 @@ START_TEST(ddrawmodes)
 {
     init_function_pointers();
 
-    createwindow();
-    if (!createdirectdraw())
+    wc.style = CS_HREDRAW | CS_VREDRAW;
+    wc.lpfnWndProc = DefWindowProcA;
+    wc.cbClsExtra = 0;
+    wc.cbWndExtra = 0;
+    wc.hInstance = GetModuleHandleA(0);
+    wc.hIcon = LoadIconA(wc.hInstance, IDI_APPLICATION);
+    wc.hCursor = LoadCursorA(NULL, IDC_ARROW);
+    wc.hbrBackground = GetStockObject(BLACK_BRUSH);
+    wc.lpszMenuName = NULL;
+    wc.lpszClassName = "TestWindowClass";
+    if (!RegisterClassA(&wc))
+    {
+        skip("RegisterClassA failed\n");
         return;
+    }
+
+    hwnd2=createwindow();
+    hwnd=createwindow();
+
+    if (!hwnd)
+    {
+        skip("Failed to create the main window\n");
+        return;
+    }
+
+    if (!createdirectdraw())
+    {
+        skip("Failed to create the direct draw object\n");
+        return;
+    }
 
     test_DirectDrawEnumerateA();
     test_DirectDrawEnumerateW();
