@@ -190,16 +190,9 @@ static HRESULT STDMETHODCALLTYPE dxgi_device_CreateSurface(IWineDXGIDevice *ifac
             goto fail;
         }
 
-        hr = IWineD3DSurface_GetParent(wined3d_surface, &parent);
-        IWineD3DSurface_Release(wined3d_surface);
-        if (FAILED(hr))
-        {
-            ERR("GetParent failed, returning %#x\n", hr);
-            goto fail;
-        }
-
+        parent = IWineD3DSurface_GetParent(wined3d_surface);
         hr = IUnknown_QueryInterface(parent, &IID_IDXGISurface, (void **)&surface[i]);
-        IUnknown_Release(parent);
+        IWineD3DSurface_Release(wined3d_surface);
         if (FAILED(hr))
         {
             ERR("Surface should implement IDXGISurface\n");
@@ -394,7 +387,7 @@ HRESULT dxgi_device_init(struct dxgi_device *device, struct dxgi_device_layer *l
     FIXME("Ignoring adapter type.\n");
     EnterCriticalSection(&dxgi_cs);
     hr = IWineD3D_CreateDevice(wined3d, adapter_ordinal, WINED3DDEVTYPE_HAL, NULL, 0,
-            (IUnknown *)device, wined3d_device_parent, &device->wined3d_device);
+            wined3d_device_parent, &device->wined3d_device);
     IWineD3DDeviceParent_Release(wined3d_device_parent);
     IWineD3D_Release(wined3d);
     LeaveCriticalSection(&dxgi_cs);
