@@ -868,12 +868,9 @@ static void test_CreateAssemblyNameObject(void)
     to_widechar(namestr, "wine PublicKeyToken=1234567890abcdef");
     name = (IAssemblyName *)0xdeadbeef;
     hr = pCreateAssemblyNameObject(&name, namestr, CANOF_PARSE_DISPLAY_NAME, NULL);
-    todo_wine
-    {
-        ok(hr == FUSION_E_INVALID_NAME,
-           "Expected FUSION_E_INVALID_NAME, got %08x\n", hr);
-        ok(name == (IAssemblyName *)0xdeadbeef, "Expected 0xdeadbeef, got %p\n", name);
-    }
+    ok(hr == FUSION_E_INVALID_NAME,
+       "Expected FUSION_E_INVALID_NAME, got %08x\n", hr);
+    ok(name == (IAssemblyName *)0xdeadbeef, "Expected 0xdeadbeef, got %p\n", name);
     if(SUCCEEDED(hr)) IAssemblyName_Release(name);
 
     /* no '=' */
