@@ -234,11 +234,11 @@ static void test_gamestatisticsmgr( void )
 
     /* this should fail, cause statistics doesn't yet exists */
     hr = IGameStatisticsMgr_GetGameStatistics(gsm, sExeName, GAMESTATS_OPEN_OPENONLY, &dwOpenResult, &gs);
-    todo_wine ok(hr == HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND), "GetGameStatistics returned unexpected value: 0x%08x\n", hr);
+    ok(hr == HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND), "GetGameStatistics returned unexpected value: 0x%08x\n", hr);
 
     /* now, allow to create */
     hr = IGameStatisticsMgr_GetGameStatistics(gsm, sExeName, GAMESTATS_OPEN_OPENORCREATE, &dwOpenResult, &gs);
-    todo_wine ok(SUCCEEDED(hr), "GetGameStatistics returned error: 0x%x\n", hr);
+    ok(SUCCEEDED(hr), "GetGameStatistics returned error: 0x%x\n", hr);
     ok(gs!=NULL, "GetGameStatistics did not return valid interface pointer\n");
     if(gs)
     {
@@ -340,71 +340,71 @@ static void test_gamestatisticsmgr( void )
 
         /* try to read written statisticd */
         hr = IGameStatisticsMgr_GetGameStatistics(gsm, sExeName, GAMESTATS_OPEN_OPENORCREATE, &dwOpenResult, &gs);
-        todo_wine ok(SUCCEEDED(hr), "GetGameStatistics returned error: 0x%08x\n", hr);
-        todo_wine ok(dwOpenResult == GAMESTATS_OPEN_OPENED, "GetGameStatistics returned invalid open result: 0x%x\n", dwOpenResult);
+        ok(SUCCEEDED(hr), "GetGameStatistics returned error: 0x%08x\n", hr);
+        ok(dwOpenResult == GAMESTATS_OPEN_OPENED, "GetGameStatistics returned invalid open result: 0x%x\n", dwOpenResult);
         ok(gs!=NULL, "GetGameStatistics did not return valid interface pointer\n");
 
         /* verify values with these which we stored before*/
         hr = IGameStatistics_GetCategoryTitle(gs, 0, &lpName);
-        todo_wine ok(hr == S_OK, "getting category title failed\n");
-        todo_wine ok(lstrcmpW(lpName, sCategory0)==0, "getting category title returned invalid string (%s)\n", wine_dbgstr_w(lpName));
+        ok(hr == S_OK, "getting category title failed\n");
+        ok(lstrcmpW(lpName, sCategory0)==0, "getting category title returned invalid string (%s)\n", wine_dbgstr_w(lpName));
         CoTaskMemFree(lpName);
 
         hr = IGameStatistics_GetCategoryTitle(gs, 1, &lpName);
-        todo_wine ok(hr == S_OK, "getting category title failed\n");
-        todo_wine ok(lstrcmpW(lpName, sCategory1)==0, "getting category title returned invalid string (%s)\n", wine_dbgstr_w(lpName));
+        ok(hr == S_OK, "getting category title failed\n");
+        ok(lstrcmpW(lpName, sCategory1)==0, "getting category title returned invalid string (%s)\n", wine_dbgstr_w(lpName));
         CoTaskMemFree(lpName);
 
         hr = IGameStatistics_GetCategoryTitle(gs, 2, &lpName);
-        todo_wine ok(hr == S_OK, "getting category title failed\n");
-        todo_wine ok(lstrcmpW(lpName, sCategory2)==0, "getting category title returned invalid string (%s)\n", wine_dbgstr_w(lpName));
+        ok(hr == S_OK, "getting category title failed\n");
+        ok(lstrcmpW(lpName, sCategory2)==0, "getting category title returned invalid string (%s)\n", wine_dbgstr_w(lpName));
         CoTaskMemFree(lpName);
 
         /* check result if category doesn't exists */
         hr = IGameStatistics_GetCategoryTitle(gs, 3, &lpName);
-        todo_wine ok(hr == S_OK, "getting category title failed\n");
+        ok(hr == S_OK, "getting category title failed\n");
         ok(lpName == NULL, "getting category title failed\n");
         CoTaskMemFree(lpName);
 
         hr = IGameStatistics_GetStatistic(gs, 0, 0, &lpName, &lpValue);
-        todo_wine ok(hr == S_OK, "getting statistic failed\n");
-        todo_wine ok(lstrcmpW(lpName, sStatistic00)==0, "getting statistic returned invalid name\n");
-        todo_wine ok(lstrcmpW(lpValue, sValue00)==0, "getting statistic returned invalid value\n");
+        ok(hr == S_OK, "getting statistic failed\n");
+        ok(lstrcmpW(lpName, sStatistic00)==0, "getting statistic returned invalid name\n");
+        ok(lstrcmpW(lpValue, sValue00)==0, "getting statistic returned invalid value\n");
         CoTaskMemFree(lpName);
         CoTaskMemFree(lpValue);
 
         hr = IGameStatistics_GetStatistic(gs, 0, 1, &lpName, &lpValue);
-        todo_wine ok(hr == S_OK, "getting statistic failed\n");
-        todo_wine ok(lstrcmpW(lpName, sStatistic01)==0, "getting statistic returned invalid name\n");
-        todo_wine ok(lstrcmpW(lpValue, sValue01)==0, "getting statistic returned invalid value\n");
+        ok(hr == S_OK, "getting statistic failed\n");
+        ok(lstrcmpW(lpName, sStatistic01)==0, "getting statistic returned invalid name\n");
+        ok(lstrcmpW(lpValue, sValue01)==0, "getting statistic returned invalid value\n");
         CoTaskMemFree(lpName);
         CoTaskMemFree(lpValue);
 
         hr = IGameStatistics_GetStatistic(gs, 1, 0, &lpName, &lpValue);
-        todo_wine ok(hr == S_OK, "getting statistic failed\n");
-        todo_wine ok(lstrcmpW(lpName, sStatistic10)==0, "getting statistic returned invalid name\n");
-        todo_wine ok(lstrcmpW(lpValue, sValue10)==0, "getting statistic returned invalid value\n");
+        ok(hr == S_OK, "getting statistic failed\n");
+        ok(lstrcmpW(lpName, sStatistic10)==0, "getting statistic returned invalid name\n");
+        ok(lstrcmpW(lpValue, sValue10)==0, "getting statistic returned invalid value\n");
         CoTaskMemFree(lpName);
         CoTaskMemFree(lpValue);
 
         hr = IGameStatistics_GetStatistic(gs, 1, 1, &lpName, &lpValue);
-        todo_wine ok(hr == S_OK, "getting statistic failed\n");
-        todo_wine ok(lstrcmpW(lpName, sStatistic11)==0, "getting statistic returned invalid name\n");
-        todo_wine ok(lstrcmpW(lpValue, sValue11)==0, "getting statistic returned invalid value\n");
+        ok(hr == S_OK, "getting statistic failed\n");
+        ok(lstrcmpW(lpName, sStatistic11)==0, "getting statistic returned invalid name\n");
+        ok(lstrcmpW(lpValue, sValue11)==0, "getting statistic returned invalid value\n");
         CoTaskMemFree(lpName);
         CoTaskMemFree(lpValue);
 
         hr = IGameStatistics_GetStatistic(gs, 2, 0, &lpName, &lpValue);
-        todo_wine ok(hr == S_OK, "getting statistic failed\n");
-        todo_wine ok(lstrcmpW(lpName, sStatistic20)==0, "getting statistic returned invalid name\n");
-        todo_wine ok(lstrcmpW(lpValue, sValue20)==0, "getting statistic returned invalid value\n");
+        ok(hr == S_OK, "getting statistic failed\n");
+        ok(lstrcmpW(lpName, sStatistic20)==0, "getting statistic returned invalid name\n");
+        ok(lstrcmpW(lpValue, sValue20)==0, "getting statistic returned invalid value\n");
         CoTaskMemFree(lpName);
         CoTaskMemFree(lpValue);
 
         hr = IGameStatistics_GetStatistic(gs, 2, 1, &lpName, &lpValue);
-        todo_wine ok(hr == S_OK, "getting statistic failed\n");
-        todo_wine ok(lstrcmpW(lpName, sStatistic21)==0, "getting statistic returned invalid name\n");
-        todo_wine ok(lstrcmpW(lpValue, sValue21)==0, "getting statistic returned invalid value\n");
+        ok(hr == S_OK, "getting statistic failed\n");
+        ok(lstrcmpW(lpName, sStatistic21)==0, "getting statistic returned invalid name\n");
+        ok(lstrcmpW(lpValue, sValue21)==0, "getting statistic returned invalid value\n");
         CoTaskMemFree(lpName);
         CoTaskMemFree(lpValue);
 
