@@ -37,13 +37,12 @@ static void test_widenpath(void)
 {
     HDC hdc = GetDC(0);
     HPEN greenPen, narrowPen;
-    HPEN oldPen;
     POINT pnt[6];
     INT nSize, ret;
 
     /* Create a pen to be used in WidenPath */
     greenPen = CreatePen(PS_SOLID, 10, RGB(0,0,0));
-    oldPen = SelectObject(hdc, greenPen);
+    SelectObject(hdc, greenPen);
 
     /* Prepare a path */
     pnt[0].x = 100;
@@ -85,7 +84,7 @@ static void test_widenpath(void)
 
     /* Test when the pen width is equal to 1. The path should change too */
     narrowPen = CreatePen(PS_SOLID, 1, RGB(0,0,0));
-    oldPen = SelectObject(hdc, narrowPen);
+    SelectObject(hdc, narrowPen);
     BeginPath(hdc);
     Polyline(hdc, pnt, 6);
     EndPath(hdc);
@@ -260,9 +259,9 @@ static const path_test_t anglearc_path[] = {
     {245, 200, PT_BEZIERTO, 0, 0}, /* 5 */
     {200, 245, PT_BEZIERTO, 0, 0}, /* 6 */
     {200, 300, PT_BEZIERTO, 0, 0}, /* 7 */
-    {200, 300, PT_BEZIERTO, 0, 2}, /* 8 */
-    {200, 300, PT_BEZIERTO, 0, 2}, /* 9 */
-    {200, 300, PT_BEZIERTO, 0, 2}, /* 10 */
+    {200, 300, PT_BEZIERTO, 0, 0}, /* 8 */
+    {200, 300, PT_BEZIERTO, 0, 0}, /* 9 */
+    {200, 300, PT_BEZIERTO, 0, 0}, /* 10 */
     {231, 260, PT_LINETO, 0, 0}, /* 11 */
     {245, 235, PT_BEZIERTO, 0, 0}, /* 12 */
     {271, 220, PT_BEZIERTO, 0, 0}, /* 13 */
@@ -289,7 +288,7 @@ static void test_anglearc(void)
     CloseFigure(hdc);
     EndPath(hdc);
 
-    ok_path(hdc, "anglearc_path", anglearc_path, sizeof(anglearc_path)/sizeof(path_test_t), 1);
+    ok_path(hdc, "anglearc_path", anglearc_path, sizeof(anglearc_path)/sizeof(path_test_t), 0);
 done:
     ReleaseDC(0, hdc);
 }
@@ -393,7 +392,6 @@ done:
 }
 
 static void test_closefigure(void) {
-    BOOL retb;
     int nSize, nSizeWitness;
     HDC hdc = GetDC(0);
 
@@ -402,7 +400,7 @@ static void test_closefigure(void) {
     LineTo(hdc, 95,  0);
     LineTo(hdc,  0, 95);
 
-    retb = CloseFigure(hdc);
+    CloseFigure(hdc);
     EndPath(hdc);
     nSize = GetPath(hdc, NULL, NULL, 0);
 
