@@ -27,6 +27,9 @@
 #include "winnt.h"
 #include "winuser.h"
 #include "ole2.h"
+#include "ocidl.h"
+#include "rpcproxy.h"
+#include "initguid.h"
 #include "mimeole.h"
 
 #include "inetcomm_private.h"
@@ -35,6 +38,8 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(inetcomm);
 
+static HINSTANCE instance;
+
 BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 {
     static IMimeInternational *international;
@@ -47,6 +52,7 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
         return FALSE;
     case DLL_PROCESS_ATTACH:
         DisableThreadLibraryCalls(hinstDLL);
+        instance = hinstDLL;
         if (!InternetTransport_RegisterClass(hinstDLL))
             return FALSE;
         MimeInternational_Construct(&international);
@@ -199,3 +205,19 @@ HRESULT WINAPI DllCanUnloadNow(void)
 {
     return S_FALSE;
 }
+
+/***********************************************************************
+ *		DllRegisterServer (INETCOMM.@)
+ */
+HRESULT WINAPI DllRegisterServer(void)
+{
+    return __wine_register_resources( instance, NULL );
+}
+
+/***********************************************************************
+ *		DllUnregisterServer (INETCOMM.@)
+ */
+HRESULT WINAPI DllUnregisterServer(void)
+{
+    return __wine_unregister_resources( instance, NULL );
+}
