@@ -33,7 +33,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(jscript);
 #define TIME_EPOCH  ((ULONGLONG)(369 * 365 + 89) * 86400 * 1000)
 
 typedef struct {
-    DispatchEx dispex;
+    jsdisp_t dispex;
 
     /* ECMA-262 3rd Edition    15.9.1.1 */
     DOUBLE time;
@@ -91,6 +91,7 @@ static const WCHAR setUTCMonthW[] = {'s','e','t','U','T','C','M','o','n','t','h'
 static const WCHAR setFullYearW[] = {'s','e','t','F','u','l','l','Y','e','a','r',0};
 static const WCHAR setUTCFullYearW[] = {'s','e','t','U','T','C','F','u','l','l','Y','e','a','r',0};
 static const WCHAR getYearW[] = {'g','e','t','Y','e','a','r',0};
+static const WCHAR setYearW[] = {'s','e','t','Y','e','a','r',0};
 
 static const WCHAR UTCW[] = {'U','T','C',0};
 static const WCHAR parseW[] = {'p','a','r','s','e',0};
@@ -2023,6 +2024,14 @@ static HRESULT Date_getYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISP
     return S_OK;
 }
 
+/* ECMA-262 3rd Edition    B2.5 */
+static HRESULT Date_setYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+        VARIANT *retv, jsexcept_t *ei, IServiceProvider *caller)
+{
+    FIXME("\n");
+    return E_NOTIMPL;
+}
+
 static HRESULT Date_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
         VARIANT *retv, jsexcept_t *ei, IServiceProvider *caller)
 {
@@ -2074,6 +2083,7 @@ static const builtin_prop_t Date_props[] = {
     {setUTCMinutesW,         Date_setUTCMinutes,         PROPF_METHOD|3},
     {setUTCMonthW,           Date_setUTCMonth,           PROPF_METHOD|2},
     {setUTCSecondsW,         Date_setUTCSeconds,         PROPF_METHOD|2},
+    {setYearW,               Date_setYear,               PROPF_METHOD|1},
     {toDateStringW,          Date_toDateString,          PROPF_METHOD},
     {toGMTStringW,           Date_toGMTString,           PROPF_METHOD},
     {toLocaleDateStringW,    Date_toLocaleDateString,    PROPF_METHOD},
@@ -2094,7 +2104,7 @@ static const builtin_info_t Date_info = {
     NULL
 };
 
-static HRESULT create_date(script_ctx_t *ctx, DispatchEx *object_prototype, DOUBLE time, DispatchEx **ret)
+static HRESULT create_date(script_ctx_t *ctx, jsdisp_t *object_prototype, DOUBLE time, jsdisp_t **ret)
 {
     DateInstance *date;
     HRESULT hres;
@@ -2511,7 +2521,7 @@ static HRESULT DateConstr_UTC(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DI
 static HRESULT DateConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
         VARIANT *retv, jsexcept_t *ei, IServiceProvider *sp)
 {
-    DispatchEx *date;
+    jsdisp_t *date;
     HRESULT hres;
 
     TRACE("\n");
@@ -2575,8 +2585,7 @@ static HRESULT DateConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
         }
         }
 
-        V_VT(retv) = VT_DISPATCH;
-        V_DISPATCH(retv) = (IDispatch*)_IDispatchEx_(date);
+        var_set_jsdisp(retv, date);
         return S_OK;
 
     case INVOKE_FUNC: {
@@ -2613,9 +2622,9 @@ static const builtin_info_t DateConstr_info = {
     NULL
 };
 
-HRESULT create_date_constr(script_ctx_t *ctx, DispatchEx *object_prototype, DispatchEx **ret)
+HRESULT create_date_constr(script_ctx_t *ctx, jsdisp_t *object_prototype, jsdisp_t **ret)
 {
-    DispatchEx *date;
+    jsdisp_t *date;
     HRESULT hres;
 
     static const WCHAR DateW[] = {'D','a','t','e',0};
