@@ -24,7 +24,7 @@
 WINE_DEFAULT_DEBUG_CHANNEL(jscript);
 
 /*
- * This IID is used to get DispatchEx objecto from interface.
+ * This IID is used to get jsdisp_t objecto from interface.
  * We might consider using private insteface instead.
  */
 static const IID IID_IDispatchJS =
@@ -51,12 +51,12 @@ struct _dispex_prop_t {
     } u;
 };
 
-static inline DISPID prop_to_id(DispatchEx *This, dispex_prop_t *prop)
+static inline DISPID prop_to_id(jsdisp_t *This, dispex_prop_t *prop)
 {
     return prop - This->props;
 }
 
-static inline dispex_prop_t *get_prop(DispatchEx *This, DISPID id)
+static inline dispex_prop_t *get_prop(jsdisp_t *This, DISPID id)
 {
     if(id < 0 || id >= This->prop_cnt || This->props[id].type == PROP_DELETED)
         return NULL;
@@ -64,7 +64,7 @@ static inline dispex_prop_t *get_prop(DispatchEx *This, DISPID id)
     return This->props+id;
 }
 
-static DWORD get_flags(DispatchEx *This, dispex_prop_t *prop)
+static DWORD get_flags(jsdisp_t *This, dispex_prop_t *prop)
 {
     if(prop->type == PROP_PROTREF) {
         dispex_prop_t *parent = get_prop(This->prototype, prop->u.ref);
@@ -79,7 +79,7 @@ static DWORD get_flags(DispatchEx *This, dispex_prop_t *prop)
     return prop->flags;
 }
 
-static const builtin_prop_t *find_builtin_prop(DispatchEx *This, const WCHAR *name)
+static const builtin_prop_t *find_builtin_prop(jsdisp_t *This, const WCHAR *name)
 {
     int min = 0, max, i, r;
 
@@ -100,7 +100,7 @@ static const builtin_prop_t *find_builtin_prop(DispatchEx *This, const WCHAR *na
     return NULL;
 }
 
-static dispex_prop_t *alloc_prop(DispatchEx *This, const WCHAR *name, prop_type_t type, DWORD flags)
+static dispex_prop_t *alloc_prop(jsdisp_t *This, const WCHAR *name, prop_type_t type, DWORD flags)
 {
     dispex_prop_t *ret;
 
@@ -121,7 +121,7 @@ static dispex_prop_t *alloc_prop(DispatchEx *This, const WCHAR *name, prop_type_
     return ret;
 }
 
-static dispex_prop_t *alloc_protref(DispatchEx *This, const WCHAR *name, DWORD ref)
+static dispex_prop_t *alloc_protref(jsdisp_t *This, const WCHAR *name, DWORD ref)
 {
     dispex_prop_t *ret;
 
@@ -133,7 +133,7 @@ static dispex_prop_t *alloc_protref(DispatchEx *This, const WCHAR *name, DWORD r
     return ret;
 }
 
-static HRESULT find_prop_name(DispatchEx *This, const WCHAR *name, dispex_prop_t **ret)
+static HRESULT find_prop_name(jsdisp_t *This, const WCHAR *name, dispex_prop_t **ret)
 {
     const builtin_prop_t *builtin;
     dispex_prop_t *prop;
@@ -160,7 +160,7 @@ static HRESULT find_prop_name(DispatchEx *This, const WCHAR *name, dispex_prop_t
     return S_OK;
 }
 
-static HRESULT find_prop_name_prot(DispatchEx *This, const WCHAR *name, dispex_prop_t **ret)
+static HRESULT find_prop_name_prot(jsdisp_t *This, const WCHAR *name, dispex_prop_t **ret)
 {
     dispex_prop_t *prop;
     HRESULT hres;
@@ -190,7 +190,7 @@ static HRESULT find_prop_name_prot(DispatchEx *This, const WCHAR *name, dispex_p
     return S_OK;
 }
 
-static HRESULT ensure_prop_name(DispatchEx *This, const WCHAR *name, BOOL search_prot, DWORD create_flags, dispex_prop_t **ret)
+static HRESULT ensure_prop_name(jsdisp_t *This, const WCHAR *name, BOOL search_prot, DWORD create_flags, dispex_prop_t **ret)
 {
     dispex_prop_t *prop;
     HRESULT hres;
@@ -254,7 +254,7 @@ static HRESULT set_this(DISPPARAMS *dp, DISPPARAMS *olddp, IDispatch *jsthis)
     return S_OK;
 }
 
-static HRESULT invoke_prop_func(DispatchEx *This, DispatchEx *jsthis, dispex_prop_t *prop, WORD flags,
+static HRESULT invoke_prop_func(jsdisp_t *This, jsdisp_t *jsthis, dispex_prop_t *prop, WORD flags,
         DISPPARAMS *dp, VARIANT *retv, jsexcept_t *ei, IServiceProvider *caller)
 {
     HRESULT hres;
@@ -263,7 +263,7 @@ static HRESULT invoke_prop_func(DispatchEx *This, DispatchEx *jsthis, dispex_pro
     case PROP_BUILTIN: {
         vdisp_t vthis;
 
-        if(flags == DISPATCH_CONSTRUCT && (prop->flags & DISPATCH_METHOD)) {
+        if(flags == DISPATCH_CONSTRUCT && (prop->flags & PROPF_METHOD)) {
             WARN("%s is not a constructor\n", debugstr_w(prop->name));
             return E_INVALIDARG;
         }
@@ -285,7 +285,7 @@ static HRESULT invoke_prop_func(DispatchEx *This, DispatchEx *jsthis, dispex_pro
 
         TRACE("call %s %p\n", debugstr_w(prop->name), V_DISPATCH(&prop->u.var));
 
-        hres = set_this(&new_dp, dp, (IDispatch*)_IDispatchEx_(jsthis));
+        hres = set_this(&new_dp, dp, to_disp(jsthis));
         if(FAILED(hres))
             return hres;
 
@@ -306,7 +306,7 @@ static HRESULT invoke_prop_func(DispatchEx *This, DispatchEx *jsthis, dispex_pro
     return E_FAIL;
 }
 
-static HRESULT prop_get(DispatchEx *This, dispex_prop_t *prop, DISPPARAMS *dp,
+static HRESULT prop_get(jsdisp_t *This, dispex_prop_t *prop, DISPPARAMS *dp,
         VARIANT *retv, jsexcept_t *ei, IServiceProvider *caller)
 {
     HRESULT hres;
@@ -314,16 +314,14 @@ static HRESULT prop_get(DispatchEx *This, dispex_prop_t *prop, DISPPARAMS *dp,
     switch(prop->type) {
     case PROP_BUILTIN:
         if(prop->u.p->flags & PROPF_METHOD) {
-            DispatchEx *obj;
+            jsdisp_t *obj;
             hres = create_builtin_function(This->ctx, prop->u.p->invoke, prop->u.p->name, NULL,
                     prop->u.p->flags, NULL, &obj);
             if(FAILED(hres))
                 break;
 
             prop->type = PROP_VARIANT;
-            V_VT(&prop->u.var) = VT_DISPATCH;
-            V_DISPATCH(&prop->u.var) = (IDispatch*)_IDispatchEx_(obj);
-
+            var_set_jsdisp(&prop->u.var, obj);
             hres = VariantCopy(retv, &prop->u.var);
         }else {
             vdisp_t vthis;
@@ -353,11 +351,14 @@ static HRESULT prop_get(DispatchEx *This, dispex_prop_t *prop, DISPPARAMS *dp,
     return hres;
 }
 
-static HRESULT prop_put(DispatchEx *This, dispex_prop_t *prop, VARIANT *val,
+static HRESULT prop_put(jsdisp_t *This, dispex_prop_t *prop, VARIANT *val,
         jsexcept_t *ei, IServiceProvider *caller)
 {
     HRESULT hres;
 
+    if(prop->flags & PROPF_CONST)
+        return S_OK;
+
     switch(prop->type) {
     case PROP_BUILTIN:
         if(!(prop->flags & PROPF_METHOD)) {
@@ -393,7 +394,7 @@ static HRESULT prop_put(DispatchEx *This, dispex_prop_t *prop, VARIANT *val,
     return S_OK;
 }
 
-static HRESULT fill_protrefs(DispatchEx *This)
+static HRESULT fill_protrefs(jsdisp_t *This)
 {
     dispex_prop_t *iter, *prop;
     HRESULT hres;
@@ -419,11 +420,11 @@ static HRESULT fill_protrefs(DispatchEx *This)
     return S_OK;
 }
 
-#define DISPATCHEX_THIS(iface) DEFINE_THIS(DispatchEx, IDispatchEx, iface)
+#define DISPATCHEX_THIS(iface) DEFINE_THIS(jsdisp_t, IDispatchEx, iface)
 
 static HRESULT WINAPI DispatchEx_QueryInterface(IDispatchEx *iface, REFIID riid, void **ppv)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
 
     if(IsEqualGUID(&IID_IUnknown, riid)) {
         TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
@@ -436,7 +437,7 @@ static HRESULT WINAPI DispatchEx_QueryInterface(IDispatchEx *iface, REFIID riid,
         *ppv = _IDispatchEx_(This);
     }else if(IsEqualGUID(&IID_IDispatchJS, riid)) {
         TRACE("(%p)->(IID_IDispatchJS %p)\n", This, ppv);
-        IUnknown_AddRef(_IDispatchEx_(This));
+        jsdisp_addref(This);
         *ppv = This;
         return S_OK;
     }else {
@@ -451,7 +452,7 @@ static HRESULT WINAPI DispatchEx_QueryInterface(IDispatchEx *iface, REFIID riid,
 
 static ULONG WINAPI DispatchEx_AddRef(IDispatchEx *iface)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
@@ -461,7 +462,7 @@ static ULONG WINAPI DispatchEx_AddRef(IDispatchEx *iface)
 
 static ULONG WINAPI DispatchEx_Release(IDispatchEx *iface)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
@@ -490,7 +491,7 @@ static ULONG WINAPI DispatchEx_Release(IDispatchEx *iface)
 
 static HRESULT WINAPI DispatchEx_GetTypeInfoCount(IDispatchEx *iface, UINT *pctinfo)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
 
     TRACE("(%p)->(%p)\n", This, pctinfo);
 
@@ -501,7 +502,7 @@ static HRESULT WINAPI DispatchEx_GetTypeInfoCount(IDispatchEx *iface, UINT *pcti
 static HRESULT WINAPI DispatchEx_GetTypeInfo(IDispatchEx *iface, UINT iTInfo, LCID lcid,
                                               ITypeInfo **ppTInfo)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     FIXME("(%p)->(%u %u %p)\n", This, iTInfo, lcid, ppTInfo);
     return E_NOTIMPL;
 }
@@ -510,7 +511,7 @@ static HRESULT WINAPI DispatchEx_GetIDsOfNames(IDispatchEx *iface, REFIID riid,
                                                 LPOLESTR *rgszNames, UINT cNames, LCID lcid,
                                                 DISPID *rgDispId)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     UINT i;
     HRESULT hres;
 
@@ -530,7 +531,7 @@ static HRESULT WINAPI DispatchEx_Invoke(IDispatchEx *iface, DISPID dispIdMember,
                                         REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
                             VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
 
     TRACE("(%p)->(%d %s %d %d %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
           lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
@@ -541,7 +542,7 @@ static HRESULT WINAPI DispatchEx_Invoke(IDispatchEx *iface, DISPID dispIdMember,
 
 static HRESULT WINAPI DispatchEx_GetDispID(IDispatchEx *iface, BSTR bstrName, DWORD grfdex, DISPID *pid)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
 
     TRACE("(%p)->(%s %x %p)\n", This, debugstr_w(bstrName), grfdex, pid);
 
@@ -556,7 +557,7 @@ static HRESULT WINAPI DispatchEx_GetDispID(IDispatchEx *iface, BSTR bstrName, DW
 static HRESULT WINAPI DispatchEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lcid, WORD wFlags, DISPPARAMS *pdp,
         VARIANT *pvarRes, EXCEPINFO *pei, IServiceProvider *pspCaller)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     dispex_prop_t *prop;
     jsexcept_t jsexcept;
     HRESULT hres;
@@ -575,6 +576,8 @@ static HRESULT WINAPI DispatchEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lc
     memset(&jsexcept, 0, sizeof(jsexcept));
 
     switch(wFlags) {
+    case DISPATCH_METHOD|DISPATCH_PROPERTYGET:
+        wFlags = DISPATCH_METHOD;
     case DISPATCH_METHOD:
     case DISPATCH_CONSTRUCT:
         hres = invoke_prop_func(This, This, prop, wFlags, pdp, pvarRes, &jsexcept, pspCaller);
@@ -620,7 +623,7 @@ static HRESULT delete_prop(dispex_prop_t *prop)
 
 static HRESULT WINAPI DispatchEx_DeleteMemberByName(IDispatchEx *iface, BSTR bstrName, DWORD grfdex)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     dispex_prop_t *prop;
     HRESULT hres;
 
@@ -642,7 +645,7 @@ static HRESULT WINAPI DispatchEx_DeleteMemberByName(IDispatchEx *iface, BSTR bst
 
 static HRESULT WINAPI DispatchEx_DeleteMemberByDispID(IDispatchEx *iface, DISPID id)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     dispex_prop_t *prop;
 
     TRACE("(%p)->(%x)\n", This, id);
@@ -658,14 +661,14 @@ static HRESULT WINAPI DispatchEx_DeleteMemberByDispID(IDispatchEx *iface, DISPID
 
 static HRESULT WINAPI DispatchEx_GetMemberProperties(IDispatchEx *iface, DISPID id, DWORD grfdexFetch, DWORD *pgrfdex)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     FIXME("(%p)->(%x %x %p)\n", This, id, grfdexFetch, pgrfdex);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI DispatchEx_GetMemberName(IDispatchEx *iface, DISPID id, BSTR *pbstrName)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     dispex_prop_t *prop;
 
     TRACE("(%p)->(%x %p)\n", This, id, pbstrName);
@@ -683,7 +686,7 @@ static HRESULT WINAPI DispatchEx_GetMemberName(IDispatchEx *iface, DISPID id, BS
 
 static HRESULT WINAPI DispatchEx_GetNextDispID(IDispatchEx *iface, DWORD grfdex, DISPID id, DISPID *pid)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     dispex_prop_t *iter;
     HRESULT hres;
 
@@ -715,7 +718,7 @@ static HRESULT WINAPI DispatchEx_GetNextDispID(IDispatchEx *iface, DWORD grfdex,
 
 static HRESULT WINAPI DispatchEx_GetNameSpaceParent(IDispatchEx *iface, IUnknown **ppunk)
 {
-    DispatchEx *This = DISPATCHEX_THIS(iface);
+    jsdisp_t *This = DISPATCHEX_THIS(iface);
     FIXME("(%p)->(%p)\n", This, ppunk);
     return E_NOTIMPL;
 }
@@ -740,7 +743,7 @@ static IDispatchExVtbl DispatchExVtbl = {
     DispatchEx_GetNameSpaceParent
 };
 
-HRESULT init_dispex(DispatchEx *dispex, script_ctx_t *ctx, const builtin_info_t *builtin_info, DispatchEx *prototype)
+HRESULT init_dispex(jsdisp_t *dispex, script_ctx_t *ctx, const builtin_info_t *builtin_info, jsdisp_t *prototype)
 {
     TRACE("%p (%p)\n", dispex, prototype);
 
@@ -754,7 +757,7 @@ HRESULT init_dispex(DispatchEx *dispex, script_ctx_t *ctx, const builtin_info_t
 
     dispex->prototype = prototype;
     if(prototype)
-        IDispatchEx_AddRef(_IDispatchEx_(prototype));
+        jsdisp_addref(prototype);
 
     dispex->prop_cnt = 1;
     dispex->props[0].name = NULL;
@@ -780,12 +783,12 @@ static const builtin_info_t dispex_info = {
     NULL
 };
 
-HRESULT create_dispex(script_ctx_t *ctx, const builtin_info_t *builtin_info, DispatchEx *prototype, DispatchEx **dispex)
+HRESULT create_dispex(script_ctx_t *ctx, const builtin_info_t *builtin_info, jsdisp_t *prototype, jsdisp_t **dispex)
 {
-    DispatchEx *ret;
+    jsdisp_t *ret;
     HRESULT hres;
 
-    ret = heap_alloc_zero(sizeof(DispatchEx));
+    ret = heap_alloc_zero(sizeof(jsdisp_t));
     if(!ret)
         return E_OUTOFMEMORY;
 
@@ -797,13 +800,13 @@ HRESULT create_dispex(script_ctx_t *ctx, const builtin_info_t *builtin_info, Dis
     return S_OK;
 }
 
-HRESULT init_dispex_from_constr(DispatchEx *dispex, script_ctx_t *ctx, const builtin_info_t *builtin_info, DispatchEx *constr)
+HRESULT init_dispex_from_constr(jsdisp_t *dispex, script_ctx_t *ctx, const builtin_info_t *builtin_info, jsdisp_t *constr)
 {
-    DispatchEx *prot = NULL;
+    jsdisp_t *prot = NULL;
     dispex_prop_t *prop;
     HRESULT hres;
 
-    static const WCHAR constructorW[] = {'c','o','n','s','t','r','u','c','t','o','r'};
+    static const WCHAR constructorW[] = {'c','o','n','s','t','r','u','c','t','o','r',0};
     static const WCHAR prototypeW[] = {'p','r','o','t','o','t','y','p','e',0};
 
     hres = find_prop_name_prot(constr, prototypeW, &prop);
@@ -836,8 +839,7 @@ HRESULT init_dispex_from_constr(DispatchEx *dispex, script_ctx_t *ctx, const bui
         jsexcept_t jsexcept;
         VARIANT var;
 
-        V_VT(&var) = VT_DISPATCH;
-        V_DISPATCH(&var) = (IDispatch*)_IDispatchEx_(constr);
+        var_set_jsdisp(&var, constr);
         memset(&jsexcept, 0, sizeof(jsexcept));
         hres = prop_put(dispex, prop, &var, &jsexcept, NULL/*FIXME*/);
     }
@@ -847,9 +849,9 @@ HRESULT init_dispex_from_constr(DispatchEx *dispex, script_ctx_t *ctx, const bui
     return hres;
 }
 
-DispatchEx *iface_to_jsdisp(IUnknown *iface)
+jsdisp_t *iface_to_jsdisp(IUnknown *iface)
 {
-    DispatchEx *ret;
+    jsdisp_t *ret;
     HRESULT hres;
 
     hres = IUnknown_QueryInterface(iface, &IID_IDispatchJS, (void**)&ret);
@@ -859,7 +861,7 @@ DispatchEx *iface_to_jsdisp(IUnknown *iface)
     return ret;
 }
 
-HRESULT jsdisp_get_id(DispatchEx *jsdisp, const WCHAR *name, DWORD flags, DISPID *id)
+HRESULT jsdisp_get_id(jsdisp_t *jsdisp, const WCHAR *name, DWORD flags, DISPID *id)
 {
     dispex_prop_t *prop;
     HRESULT hres;
@@ -880,7 +882,7 @@ HRESULT jsdisp_get_id(DispatchEx *jsdisp, const WCHAR *name, DWORD flags, DISPID
     return DISP_E_UNKNOWNNAME;
 }
 
-HRESULT jsdisp_call_value(DispatchEx *jsthis, WORD flags, DISPPARAMS *dp, VARIANT *retv,
+HRESULT jsdisp_call_value(jsdisp_t *jsthis, WORD flags, DISPPARAMS *dp, VARIANT *retv,
         jsexcept_t *ei, IServiceProvider *caller)
 {
     vdisp_t vdisp;
@@ -892,7 +894,7 @@ HRESULT jsdisp_call_value(DispatchEx *jsthis, WORD flags, DISPPARAMS *dp, VARIAN
     return hres;
 }
 
-HRESULT jsdisp_call(DispatchEx *disp, DISPID id, WORD flags, DISPPARAMS *dp, VARIANT *retv,
+HRESULT jsdisp_call(jsdisp_t *disp, DISPID id, WORD flags, DISPPARAMS *dp, VARIANT *retv,
         jsexcept_t *ei, IServiceProvider *caller)
 {
     dispex_prop_t *prop;
@@ -908,7 +910,7 @@ HRESULT jsdisp_call(DispatchEx *disp, DISPID id, WORD flags, DISPPARAMS *dp, VAR
     return invoke_prop_func(disp, disp, prop, flags, dp, retv, ei, caller);
 }
 
-HRESULT jsdisp_call_name(DispatchEx *disp, const WCHAR *name, WORD flags, DISPPARAMS *dp, VARIANT *retv,
+HRESULT jsdisp_call_name(jsdisp_t *disp, const WCHAR *name, WORD flags, DISPPARAMS *dp, VARIANT *retv,
         jsexcept_t *ei, IServiceProvider *caller)
 {
     dispex_prop_t *prop;
@@ -928,7 +930,7 @@ HRESULT jsdisp_call_name(DispatchEx *disp, const WCHAR *name, WORD flags, DISPPA
 HRESULT disp_call(script_ctx_t *ctx, IDispatch *disp, DISPID id, WORD flags, DISPPARAMS *dp, VARIANT *retv,
         jsexcept_t *ei, IServiceProvider *caller)
 {
-    DispatchEx *jsdisp;
+    jsdisp_t *jsdisp;
     IDispatchEx *dispex;
     HRESULT hres;
 
@@ -940,6 +942,8 @@ HRESULT disp_call(script_ctx_t *ctx, IDispatch *disp, DISPID id, WORD flags, DIS
     }
 
     memset(ei, 0, sizeof(*ei));
+    if(retv && arg_cnt(dp))
+        flags |= DISPATCH_PROPERTYGET;
 
     if(retv)
         V_VT(retv) = VT_EMPTY;
@@ -962,7 +966,7 @@ HRESULT disp_call(script_ctx_t *ctx, IDispatch *disp, DISPID id, WORD flags, DIS
     return hres;
 }
 
-HRESULT jsdisp_propput_name(DispatchEx *obj, const WCHAR *name, VARIANT *val, jsexcept_t *ei, IServiceProvider *caller)
+HRESULT jsdisp_propput_name(jsdisp_t *obj, const WCHAR *name, VARIANT *val, jsexcept_t *ei, IServiceProvider *caller)
 {
     dispex_prop_t *prop;
     HRESULT hres;
@@ -974,7 +978,19 @@ HRESULT jsdisp_propput_name(DispatchEx *obj, const WCHAR *name, VARIANT *val, js
     return prop_put(obj, prop, val, ei, caller);
 }
 
-HRESULT jsdisp_propput_idx(DispatchEx *obj, DWORD idx, VARIANT *val, jsexcept_t *ei, IServiceProvider *caller)
+HRESULT jsdisp_propput_const(jsdisp_t *obj, const WCHAR *name, VARIANT *val)
+{
+    dispex_prop_t *prop;
+    HRESULT hres;
+
+    hres = ensure_prop_name(obj, name, FALSE, PROPF_ENUM|PROPF_CONST, &prop);
+    if(FAILED(hres))
+        return hres;
+
+    return VariantCopy(&prop->u.var, val);
+}
+
+HRESULT jsdisp_propput_idx(jsdisp_t *obj, DWORD idx, VARIANT *val, jsexcept_t *ei, IServiceProvider *caller)
 {
     WCHAR buf[12];
 
@@ -986,7 +1002,7 @@ HRESULT jsdisp_propput_idx(DispatchEx *obj, DWORD idx, VARIANT *val, jsexcept_t
 
 HRESULT disp_propput(script_ctx_t *ctx, IDispatch *disp, DISPID id, VARIANT *val, jsexcept_t *ei, IServiceProvider *caller)
 {
-    DispatchEx *jsdisp;
+    jsdisp_t *jsdisp;
     HRESULT hres;
 
     jsdisp = iface_to_jsdisp((IUnknown*)disp);
@@ -1020,7 +1036,7 @@ HRESULT disp_propput(script_ctx_t *ctx, IDispatch *disp, DISPID id, VARIANT *val
     return hres;
 }
 
-HRESULT jsdisp_propget_name(DispatchEx *obj, const WCHAR *name, VARIANT *var, jsexcept_t *ei, IServiceProvider *caller)
+HRESULT jsdisp_propget_name(jsdisp_t *obj, const WCHAR *name, VARIANT *var, jsexcept_t *ei, IServiceProvider *caller)
 {
     DISPPARAMS dp = {NULL, NULL, 0, 0};
     dispex_prop_t *prop;
@@ -1037,7 +1053,7 @@ HRESULT jsdisp_propget_name(DispatchEx *obj, const WCHAR *name, VARIANT *var, js
     return prop_get(obj, prop, &dp, var, ei, caller);
 }
 
-HRESULT jsdisp_get_idx(DispatchEx *obj, DWORD idx, VARIANT *var, jsexcept_t *ei, IServiceProvider *caller)
+HRESULT jsdisp_get_idx(jsdisp_t *obj, DWORD idx, VARIANT *var, jsexcept_t *ei, IServiceProvider *caller)
 {
     WCHAR name[12];
     DISPPARAMS dp = {NULL, NULL, 0, 0};
@@ -1059,7 +1075,7 @@ HRESULT jsdisp_get_idx(DispatchEx *obj, DWORD idx, VARIANT *var, jsexcept_t *ei,
     return prop_get(obj, prop, &dp, var, ei, caller);
 }
 
-HRESULT jsdisp_propget(DispatchEx *jsdisp, DISPID id, VARIANT *val, jsexcept_t *ei, IServiceProvider *caller)
+HRESULT jsdisp_propget(jsdisp_t *jsdisp, DISPID id, VARIANT *val, jsexcept_t *ei, IServiceProvider *caller)
 {
     DISPPARAMS dp  = {NULL,NULL,0,0};
     dispex_prop_t *prop;
@@ -1076,7 +1092,7 @@ HRESULT disp_propget(script_ctx_t *ctx, IDispatch *disp, DISPID id, VARIANT *val
 {
     DISPPARAMS dp  = {NULL,NULL,0,0};
     IDispatchEx *dispex;
-    DispatchEx *jsdisp;
+    jsdisp_t *jsdisp;
     HRESULT hres;
 
     jsdisp = iface_to_jsdisp((IUnknown*)disp);
@@ -1100,7 +1116,7 @@ HRESULT disp_propget(script_ctx_t *ctx, IDispatch *disp, DISPID id, VARIANT *val
     return hres;
 }
 
-HRESULT jsdisp_delete_idx(DispatchEx *obj, DWORD idx)
+HRESULT jsdisp_delete_idx(jsdisp_t *obj, DWORD idx)
 {
     static const WCHAR formatW[] = {'%','d',0};
     WCHAR buf[12];
