@@ -149,6 +149,8 @@ tmp = Object.prototype.toString.call(this);
 ok(tmp === "[object Object]", "toString.call(this) = " + tmp);
 (function () { tmp = Object.prototype.toString.call(arguments); })();
 ok(tmp === "[object Object]", "toString.call(arguments) = " + tmp);
+tmp = Object.prototype.toString.call(new VBArray(createArray()));
+ok(tmp === "[object Object]", "toString.call(new VBArray()) = " + tmp);
 
 ok(Object(1) instanceof Number, "Object(1) is not instance of Number");
 ok(Object("") instanceof String, "Object('') is not instance of String");
@@ -1897,6 +1899,12 @@ exception_test(function() {"test" in nullDisp;}, "TypeError", -2146823281);
 exception_test(function() {new 3;}, "TypeError", -2146827843);
 exception_test(function() {new null;}, "TypeError", -2146823281);
 exception_test(function() {new nullDisp;}, "TypeError", -2146827850);
+exception_test(function() {new VBArray();}, "TypeError", -2146823275);
+exception_test(function() {new VBArray(new VBArray(createArray()));}, "TypeError", -2146823275);
+exception_test(function() {createArray().lbound("aaa");}, "RangeError", -2146828279);
+exception_test(function() {createArray().lbound(3);}, "RangeError", -2146828279);
+exception_test(function() {VBArray.prototype.lbound.call(new Object());}, "TypeError", -2146823275);
+exception_test(function() {createArray().getItem(3);}, "RangeError", -2146828279);
 
 function testThisExcept(func, number) {
     exception_test(function() {func.call(new Object())}, "TypeError", number);
@@ -2128,6 +2136,7 @@ testFunctions(Date.prototype, [
         ["setUTCMinutes", 3],
         ["setUTCMonth", 2],
         ["setUTCSeconds", 2],
+        ["setYear", 1],
         ["toDateString", 0],
         ["toLocaleDateString", 0],
         ["toLocaleString", 0],
@@ -2194,6 +2203,14 @@ testFunctions(Function.prototype, [
         ["toString", 0]
     ]);
 
+testFunctions(VBArray.prototype, [
+        ["dimensions", 0],
+        ["getItem", 1],
+        ["lbound", 0],
+        ["toArray", 0],
+        ["ubound", 0]
+    ]);
+
 ok(ActiveXObject.length == 1, "ActiveXObject.length = " + ActiveXObject.length);
 ok(Array.length == 1, "Array.length = " + Array.length);
 ok(Boolean.length == 1, "Boolean.length = " + Boolean.length);
@@ -2236,4 +2253,22 @@ ok(unescape.length == 1, "unescape.length = " + unescape.length);
 String.length = 3;
 ok(String.length == 1, "String.length = " + String.length);
 
+var tmp = createArray();
+ok(getVT(tmp) == "VT_ARRAY|VT_VARIANT", "getVT(createArray()) = " + getVT(tmp));
+ok(getVT(VBArray(tmp)) == "VT_ARRAY|VT_VARIANT", "getVT(VBArray(tmp)) = " + getVT(VBArray(tmp)));
+tmp = new VBArray(tmp);
+tmp = new VBArray(VBArray(createArray()));
+ok(tmp.dimensions() == 2, "tmp.dimensions() = " + tmp.dimensions());
+ok(tmp.lbound() == 0, "tmp.lbound() = " + tmp.lbound());
+ok(tmp.lbound(1) == 0, "tmp.lbound(1) = " + tmp.lbound(1));
+ok(tmp.lbound(2, 1) == 2, "tmp.lbound(2, 1) = " + tmp.lbound(2, 1));
+ok(tmp.ubound() == 4, "tmp.ubound() = " + tmp.ubound());
+ok(tmp.ubound("2") == 3, "tmp.ubound(\"2\") = " + tmp.ubound("2"));
+ok(tmp.getItem(1, 2) == 3, "tmp.getItem(1, 2) = " + tmp.getItem(1, 2));
+ok(tmp.getItem(2, 3) == 33, "tmp.getItem(2, 3) = " + tmp.getItem(2, 3));
+ok(tmp.getItem(3, 2) == 13, "tmp.getItem(3, 2) = " + tmp.getItem(3, 2));
+ok(tmp.toArray() == "2,3,12,13,22,23,32,33,42,43", "tmp.toArray() = " + tmp.toArray());
+ok(createArray().toArray() == "2,3,12,13,22,23,32,33,42,43",
+        "createArray.toArray()=" + createArray().toArray());
+
 reportSuccess();
