@@ -310,11 +310,14 @@ static void test_InitPathA(CHAR *newdir, CHAR *curDrive, CHAR *otherDrive)
 {
   CHAR tmppath[MAX_PATH], /*path to TEMP */
        tmpstr[MAX_PATH],
-       tmpstr1[MAX_PATH];
+       tmpstr1[MAX_PATH],
+       invalid_dir[MAX_PATH];
+
   DWORD len,len1,drives;
   INT id;
   HANDLE hndl;
   BOOL bRes;
+  UINT unique;
 
   *curDrive = *otherDrive = NOT_A_VALID_DRIVE;
 
@@ -334,11 +337,7 @@ static void test_InitPathA(CHAR *newdir, CHAR *curDrive, CHAR *otherDrive)
   ok(len1==len+1 || broken(len1 == len), /* WinME */
      "GetTempPathA should return string length %d instead of %d\n",len+1,len1);
 
-/* Test GetTmpFileNameA
-   The only test we do here is whether GetTempFileNameA passes or not.
-   We do not thoroughly test this function yet (specifically, whether
-   it behaves correctly when 'unique' is non zero)
-*/
+/* Test GetTmpFileNameA */
   ok((id=GetTempFileNameA(tmppath,"path",0,newdir)),"GetTempFileNameA failed\n");
   sprintf(tmpstr,"pat%.4x.tmp",id & 0xffff);
   sprintf(tmpstr1,"pat%x.tmp",id & 0xffff);
@@ -361,6 +360,22 @@ static void test_InitPathA(CHAR *newdir, CHAR *curDrive, CHAR *otherDrive)
     ok(DeleteFileA(newdir),"Couldn't delete the temporary file we just created\n");
   }
 
+  for(unique=0;unique<3;unique++) {
+    /* Nonexistent path */
+    sprintf(invalid_dir, "%s\\%s",tmppath,"non_existent_dir_1jwj3y32nb3");
+    SetLastError(0xdeadbeef);
+    ok(!GetTempFileNameA(invalid_dir,"tfn",unique,newdir),"GetTempFileNameA should have failed\n");
+    ok(GetLastError()==ERROR_DIRECTORY || broken(GetLastError()==ERROR_PATH_NOT_FOUND)/*win98*/,
+    "got %d, expected ERROR_DIRECTORY\n", GetLastError());
+
+    /* Check return value for unique !=0 */
+    if(unique) {
+      ok((GetTempFileNameA(tmppath,"tfn",unique,newdir) == unique),"GetTempFileNameA unexpectedly failed\n");
+      /* if unique != 0, the actual temp files are not created: */
+      ok(!DeleteFileA(newdir) && GetLastError() == ERROR_FILE_NOT_FOUND,"Deleted a file that shouldn't exist!\n");
+    }
+  }
+
 /* Find first valid drive letter that is neither newdir[0] nor curDrive */
   drives = GetLogicalDrives() & ~(1<<(newdir[0]-'A'));
   if( *curDrive != NOT_A_VALID_DRIVE)
@@ -540,6 +555,28 @@ static void test_CleanupPathA(CHAR *origdir, CHAR *curdir)
   ok(RemoveDirectoryA(curdir),"RemoveDirectoryA failed\n");
 }
 
+/* test that short path name functions work regardless of case */
+static void test_ShortPathCase(const char *tmpdir, const char *dirname,
+                               const char *filename)
+{
+    char buf[MAX_PATH], shortbuf[MAX_PATH];
+    HANDLE hndl;
+    int i;
+
+    snprintf(buf,sizeof(buf),"%s\\%s\\%s",tmpdir,dirname,filename);
+    GetShortPathNameA(buf,shortbuf,sizeof(shortbuf));
+    hndl = CreateFileA(shortbuf,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
+    ok(hndl!=INVALID_HANDLE_VALUE,"CreateFileA failed (%d)\n",GetLastError());
+    CloseHandle(hndl);
+    /* Now for the real test */
+    for(i=0;i<strlen(shortbuf);i++)
+        if (i % 2)
+            shortbuf[i] = tolower(shortbuf[i]);
+    hndl = CreateFileA(shortbuf,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
+    ok(hndl!=INVALID_HANDLE_VALUE,"CreateFileA failed (%d)\n",GetLastError());
+    CloseHandle(hndl);
+}
+
 /* This routine will test Get(Full|Short|Long)PathNameA */
 static void test_PathNameA(CHAR *curdir, CHAR curDrive, CHAR otherDrive)
 {
@@ -863,6 +900,10 @@ static void test_PathNameA(CHAR *curdir, CHAR curDrive, CHAR otherDrive)
     sprintf(tmpstr,"Long File %c",funny_chars[i]);
     test_FunnyChars(curdir,curdir_short,tmpstr,valid,tmpstr1);
   }
+  /* Now try it on mixed case short names */
+  test_ShortPathCase(curdir,SHORTDIR,LONGFILE);
+  test_ShortPathCase(curdir,LONGDIR,SHORTFILE);
+  test_ShortPathCase(curdir,LONGDIR,LONGFILE);
 }
 
 static void test_GetTempPathA(char* tmp_dir)
@@ -1569,6 +1610,120 @@ if (0)
       "Expected ERROR_INVALID_PARAMETER, got %x\n", GetLastError());
 }
 
+static void test_GetFullPathNameA(void)
+{
+    char output[MAX_PATH], *filepart;
+    DWORD ret;
+    int is_win9x, i;
+
+    const struct
+    {
+        LPCSTR name;
+        DWORD len;
+        LPSTR buffer;
+        LPSTR *lastpart;
+        int win9x_crash;
+    } invalid_parameters[] =
+    {
+        {NULL, 0,        NULL,   NULL,      1},
+        {NULL, MAX_PATH, NULL,   NULL,      1},
+        {NULL, MAX_PATH, output, NULL,      1},
+        {NULL, MAX_PATH, output, &filepart, 1},
+        {"",   0,        NULL,   NULL},
+        {"",   MAX_PATH, NULL,   NULL},
+        {"",   MAX_PATH, output, NULL},
+        {"",   MAX_PATH, output, &filepart},
+    };
+
+    SetLastError(0xdeadbeef);
+    ret = GetFullPathNameW(NULL, 0, NULL, NULL);
+    is_win9x = !ret && GetLastError() == ERROR_CALL_NOT_IMPLEMENTED;
+
+    if (is_win9x)
+        win_skip("Skipping some tests that cause GetFullPathNameA to crash on Win9x\n");
+
+    for (i = 0; i < sizeof(invalid_parameters)/sizeof(invalid_parameters[0]); i++)
+    {
+        if (is_win9x && invalid_parameters[i].win9x_crash)
+            continue;
+
+        SetLastError(0xdeadbeef);
+        strcpy(output, "deadbeef");
+        filepart = (char *)0xdeadbeef;
+        ret = GetFullPathNameA(invalid_parameters[i].name,
+                               invalid_parameters[i].len,
+                               invalid_parameters[i].buffer,
+                               invalid_parameters[i].lastpart);
+        ok(!ret, "[%d] Expected GetFullPathNameA to return 0, got %u\n", i, ret);
+        ok(!strcmp(output, "deadbeef"), "[%d] Expected the output buffer to be unchanged, got \"%s\"\n", i, output);
+        ok(filepart == (char *)0xdeadbeef, "[%d] Expected output file part pointer to be untouched, got %p\n", i, filepart);
+        ok(GetLastError() == 0xdeadbeef ||
+           GetLastError() == ERROR_BAD_PATHNAME || /* Win9x */
+           GetLastError() == ERROR_INVALID_NAME, /* Win7 */
+           "[%d] Expected GetLastError() to return 0xdeadbeef, got %u\n",
+           i, GetLastError());
+    }
+}
+
+static void test_GetFullPathNameW(void)
+{
+    static const WCHAR emptyW[] = {0};
+    static const WCHAR deadbeefW[] = {'d','e','a','d','b','e','e','f',0};
+
+    WCHAR output[MAX_PATH], *filepart;
+    DWORD ret;
+    int i;
+
+    const struct
+    {
+        LPCWSTR name;
+        DWORD len;
+        LPWSTR buffer;
+        LPWSTR *lastpart;
+        int win7_expect;
+    } invalid_parameters[] =
+    {
+        {NULL,   0,        NULL,   NULL},
+        {NULL,   0,        NULL,   &filepart, 1},
+        {NULL,   MAX_PATH, NULL,   NULL},
+        {NULL,   MAX_PATH, output, NULL},
+        {NULL,   MAX_PATH, output, &filepart, 1},
+        {emptyW, 0,        NULL,   NULL},
+        {emptyW, 0,        NULL,   &filepart, 1},
+        {emptyW, MAX_PATH, NULL,   NULL},
+        {emptyW, MAX_PATH, output, NULL},
+        {emptyW, MAX_PATH, output, &filepart, 1},
+    };
+
+    SetLastError(0xdeadbeef);
+    ret = GetFullPathNameW(NULL, 0, NULL, NULL);
+    if (!ret && GetLastError() == ERROR_CALL_NOT_IMPLEMENTED)
+    {
+        win_skip("GetFullPathNameW is not available\n");
+        return;
+    }
+
+    for (i = 0; i < sizeof(invalid_parameters)/sizeof(invalid_parameters[0]); i++)
+    {
+        SetLastError(0xdeadbeef);
+        lstrcpyW(output, deadbeefW);
+        filepart = (WCHAR *)0xdeadbeef;
+        ret = GetFullPathNameW(invalid_parameters[i].name,
+                               invalid_parameters[i].len,
+                               invalid_parameters[i].buffer,
+                               invalid_parameters[i].lastpart);
+        ok(!ret, "[%d] Expected GetFullPathNameW to return 0, got %u\n", i, ret);
+        ok(!lstrcmpW(output, deadbeefW), "[%d] Expected the output buffer to be unchanged, got %s\n", i, wine_dbgstr_w(output));
+        ok(filepart == (WCHAR *)0xdeadbeef ||
+           (invalid_parameters[i].win7_expect && filepart == NULL),
+           "[%d] Expected output file part pointer to be untouched, got %p\n", i, filepart);
+        ok(GetLastError() == 0xdeadbeef ||
+           GetLastError() == ERROR_INVALID_NAME, /* Win7 */
+           "[%d] Expected GetLastError() to return 0xdeadbeef, got %u\n",
+           i, GetLastError());
+    }
+}
+
 static void init_pointers(void)
 {
     HMODULE hKernel32 = GetModuleHandleA("kernel32.dll");
@@ -1610,4 +1765,6 @@ START_TEST(path)
     test_drive_letter_case();
     test_SearchPathA();
     test_SearchPathW();
+    test_GetFullPathNameA();
+    test_GetFullPathNameW();
 }
