@@ -83,85 +83,11 @@ static ITypeInfo *typeinfos[LAST_tid];
 static struct list dispex_data_list = LIST_INIT(dispex_data_list);
 
 static REFIID tid_ids[] = {
-    &IID_NULL,
-    &DIID_DispCEventObj,
-    &DIID_DispDOMChildrenCollection,
-    &DIID_DispHTMLAnchorElement,
-    &DIID_DispHTMLBody,
-    &DIID_DispHTMLCommentElement,
-    &DIID_DispHTMLCurrentStyle,
-    &DIID_DispHTMLDocument,
-    &DIID_DispHTMLDOMTextNode,
-    &DIID_DispHTMLElementCollection,
-    &DIID_DispHTMLFormElement,
-    &DIID_DispHTMLGenericElement,
-    &DIID_DispHTMLFrameElement,
-    &DIID_DispHTMLIFrame,
-    &DIID_DispHTMLImg,
-    &DIID_DispHTMLInputElement,
-    &DIID_DispHTMLLocation,
-    &DIID_DispHTMLNavigator,
-    &DIID_DispHTMLOptionElement,
-    &DIID_DispHTMLScreen,
-    &DIID_DispHTMLScriptElement,
-    &DIID_DispHTMLSelectElement,
-    &DIID_DispHTMLStyle,
-    &DIID_DispHTMLTable,
-    &DIID_DispHTMLTableRow,
-    &DIID_DispHTMLTextAreaElement,
-    &DIID_DispHTMLUnknownElement,
-    &DIID_DispHTMLWindow2,
-    &DIID_HTMLDocumentEvents,
-    &IID_IHTMLAnchorElement,
-    &IID_IHTMLBodyElement,
-    &IID_IHTMLBodyElement2,
-    &IID_IHTMLCommentElement,
-    &IID_IHTMLCurrentStyle,
-    &IID_IHTMLCurrentStyle2,
-    &IID_IHTMLCurrentStyle3,
-    &IID_IHTMLCurrentStyle4,
-    &IID_IHTMLDocument2,
-    &IID_IHTMLDocument3,
-    &IID_IHTMLDocument4,
-    &IID_IHTMLDocument5,
-    &IID_IHTMLDOMChildrenCollection,
-    &IID_IHTMLDOMNode,
-    &IID_IHTMLDOMNode2,
-    &IID_IHTMLDOMTextNode,
-    &IID_IHTMLElement,
-    &IID_IHTMLElement2,
-    &IID_IHTMLElement3,
-    &IID_IHTMLElement4,
-    &IID_IHTMLElementCollection,
-    &IID_IHTMLEventObj,
-    &IID_IHTMLFiltersCollection,
-    &IID_IHTMLFormElement,
-    &IID_IHTMLFrameBase,
-    &IID_IHTMLFrameBase2,
-    &IID_IHTMLGenericElement,
-    &IID_IHTMLFrameElement3,
-    &IID_IHTMLIFrameElement,
-    &IID_IHTMLImageElementFactory,
-    &IID_IHTMLImgElement,
-    &IID_IHTMLInputElement,
-    &IID_IHTMLLocation,
-    &IID_IHTMLOptionElement,
-    &IID_IHTMLScreen,
-    &IID_IHTMLScriptElement,
-    &IID_IHTMLSelectElement,
-    &IID_IHTMLStyle,
-    &IID_IHTMLStyle2,
-    &IID_IHTMLStyle3,
-    &IID_IHTMLStyle4,
-    &IID_IHTMLTable,
-    &IID_IHTMLTableRow,
-    &IID_IHTMLTextAreaElement,
-    &IID_IHTMLTextContainer,
-    &IID_IHTMLUniqueName,
-    &IID_IHTMLWindow2,
-    &IID_IHTMLWindow3,
-    &IID_IHTMLWindow4,
-    &IID_IOmNavigator
+#define XIID(iface) &IID_ ## iface,
+#define XDIID(iface) &DIID_ ## iface,
+TID_LIST
+#undef XIID
+#undef XDIID
 };
 
 static HRESULT get_typeinfo(tid_t tid, ITypeInfo **typeinfo)
