@@ -35,6 +35,7 @@
 
 #include "mshtml_private.h"
 #include "htmlevent.h"
+#include "pluginhost.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
@@ -110,7 +111,9 @@ static HRESULT WINAPI HTMLDocument_get_all(IHTMLDocument2 *iface, IHTMLElementCo
 {
     HTMLDocument *This = HTMLDOC_THIS(iface);
     nsIDOMElement *nselem = NULL;
+    HTMLDOMNode *node;
     nsresult nsres;
+    HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
@@ -125,14 +128,16 @@ static HRESULT WINAPI HTMLDocument_get_all(IHTMLDocument2 *iface, IHTMLElementCo
         return E_FAIL;
     }
 
-    if(nselem) {
-        *p = create_all_collection(get_node(This->doc_node, (nsIDOMNode*)nselem, TRUE), TRUE);
-        nsIDOMElement_Release(nselem);
-    }else {
+    if(!nselem) {
         *p = NULL;
+        return S_OK;
     }
 
-    return S_OK;
+    hres = get_node(This->doc_node, (nsIDOMNode*)nselem, TRUE, &node);
+    if(SUCCEEDED(hres))
+        *p = create_all_collection(node, TRUE);
+    nsIDOMElement_Release(nselem);
+    return hres;
 }
 
 static HRESULT WINAPI HTMLDocument_get_body(IHTMLDocument2 *iface, IHTMLElement **p)
@@ -140,32 +145,31 @@ static HRESULT WINAPI HTMLDocument_get_body(IHTMLDocument2 *iface, IHTMLElement
     HTMLDocument *This = HTMLDOC_THIS(iface);
     nsIDOMHTMLElement *nsbody = NULL;
     HTMLDOMNode *node;
-    nsresult nsres;
+    HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
-    if(!This->doc_node->nsdoc) {
-        WARN("NULL nsdoc\n");
-        return E_UNEXPECTED;
+    if(This->doc_node->nsdoc) {
+        nsresult nsres;
+
+        nsres = nsIDOMHTMLDocument_GetBody(This->doc_node->nsdoc, &nsbody);
+        if(NS_FAILED(nsres)) {
+            TRACE("Could not get body: %08x\n", nsres);
+            return E_UNEXPECTED;
+        }
     }
 
-    nsres = nsIDOMHTMLDocument_GetBody(This->doc_node->nsdoc, &nsbody);
-    if(NS_FAILED(nsres)) {
-        TRACE("Could not get body: %08x\n", nsres);
-        return E_UNEXPECTED;
-    }
-
-    if(nsbody) {
-        node = get_node(This->doc_node, (nsIDOMNode*)nsbody, TRUE);
-        nsIDOMHTMLElement_Release(nsbody);
-
-        IHTMLDOMNode_QueryInterface(HTMLDOMNODE(node), &IID_IHTMLElement, (void**)p);
-    }else {
+    if(!nsbody) {
         *p = NULL;
+        return S_OK;
     }
 
-    TRACE("*p = %p\n", *p);
-    return S_OK;
+    hres = get_node(This->doc_node, (nsIDOMNode*)nsbody, TRUE, &node);
+    nsIDOMHTMLElement_Release(nsbody);
+    if(FAILED(hres))
+        return hres;
+
+    return IHTMLDOMNode_QueryInterface(HTMLDOMNODE(node), &IID_IHTMLElement, (void**)p);
 }
 
 static HRESULT WINAPI HTMLDocument_get_activeElement(IHTMLDocument2 *iface, IHTMLElement **p)
@@ -567,6 +571,11 @@ static HRESULT WINAPI HTMLDocument_get_location(IHTMLDocument2 *iface, IHTMLLoca
 
     TRACE("(%p)->(%p)\n", This, p);
 
+    if(!This->doc_node->nsdoc) {
+        WARN("NULL nsdoc\n");
+        return E_UNEXPECTED;
+    }
+
     return IHTMLWindow2_get_location(HTMLWINDOW2(This->window), p);
 }
 
@@ -889,8 +898,25 @@ static HRESULT WINAPI HTMLDocument_close(IHTMLDocument2 *iface)
 static HRESULT WINAPI HTMLDocument_clear(IHTMLDocument2 *iface)
 {
     HTMLDocument *This = HTMLDOC_THIS(iface);
-    FIXME("(%p)\n", This);
-    return E_NOTIMPL;
+    nsIDOMNSHTMLDocument *nsdoc;
+    nsresult nsres;
+
+    TRACE("(%p)\n", This);
+
+    nsres = nsIDOMHTMLDocument_QueryInterface(This->doc_node->nsdoc, &IID_nsIDOMNSHTMLDocument, (void**)&nsdoc);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDOMNSHTMLDocument iface: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    nsres = nsIDOMNSHTMLDocument_Clear(nsdoc);
+    nsIDOMNSHTMLDocument_Release(nsdoc);
+    if(NS_FAILED(nsres)) {
+        ERR("Clear failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLDocument_queryCommandSupported(IHTMLDocument2 *iface, BSTR cmdID,
@@ -971,8 +997,10 @@ static HRESULT WINAPI HTMLDocument_createElement(IHTMLDocument2 *iface, BSTR eTa
     if(FAILED(hres))
         return hres;
 
-    elem = HTMLElement_Create(This->doc_node, (nsIDOMNode*)nselem, TRUE);
+    hres = HTMLElement_Create(This->doc_node, (nsIDOMNode*)nselem, TRUE, &elem);
     nsIDOMHTMLElement_Release(nselem);
+    if(FAILED(hres))
+        return hres;
 
     *newElem = HTMLELEM(elem);
     IHTMLElement_AddRef(HTMLELEM(elem));
@@ -1870,6 +1898,7 @@ static void HTMLDocumentNode_destructor(HTMLDOMNode *iface)
 
     detach_selection(This);
     detach_ranges(This);
+    detach_plugin_hosts(This);
     release_nodes(This);
 
     if(This->nsdoc) {
@@ -1881,11 +1910,39 @@ static void HTMLDocumentNode_destructor(HTMLDOMNode *iface)
     destroy_htmldoc(&This->basedoc);
 }
 
-#undef HTMLDOCNODE_NODE_THIS
+static HRESULT HTMLDocumentNode_clone(HTMLDOMNode *iface, nsIDOMNode *nsnode, HTMLDOMNode **ret)
+{
+    HTMLDocumentNode *This = HTMLDOCNODE_NODE_THIS(iface);
+    FIXME("%p\n", This);
+    return E_NOTIMPL;
+}
 
 static const NodeImplVtbl HTMLDocumentNodeImplVtbl = {
     HTMLDocumentNode_QI,
-    HTMLDocumentNode_destructor
+    HTMLDocumentNode_destructor,
+    HTMLDocumentNode_clone
+};
+
+static HRESULT HTMLDocumentFragment_clone(HTMLDOMNode *iface, nsIDOMNode *nsnode, HTMLDOMNode **ret)
+{
+    HTMLDocumentNode *This = HTMLDOCNODE_NODE_THIS(iface);
+    HTMLDocumentNode *new_node;
+    HRESULT hres;
+
+    hres = create_document_fragment(nsnode, This->node.doc, &new_node);
+    if(FAILED(hres))
+        return hres;
+
+    *ret = &new_node->node;
+    return S_OK;
+}
+
+#undef HTMLDOCNODE_NODE_THIS
+
+static const NodeImplVtbl HTMLDocumentFragmentImplVtbl = {
+    HTMLDocumentNode_QI,
+    HTMLDocumentNode_destructor,
+    HTMLDocumentFragment_clone
 };
 
 static const tid_t HTMLDocumentNode_iface_tids[] = {
@@ -1905,35 +1962,48 @@ static dispex_static_data_t HTMLDocumentNode_dispex = {
     HTMLDocumentNode_iface_tids
 };
 
+static HTMLDocumentNode *alloc_doc_node(HTMLDocumentObj *doc_obj, HTMLWindow *window)
+{
+    HTMLDocumentNode *doc;
+
+    doc = heap_alloc_zero(sizeof(HTMLDocumentNode));
+    if(!doc)
+        return NULL;
+
+    doc->ref = 1;
+    doc->basedoc.doc_node = doc;
+    doc->basedoc.doc_obj = doc_obj;
+    doc->basedoc.window = window;
+
+    init_dispex(&doc->node.dispex, (IUnknown*)HTMLDOMNODE(&doc->node), &HTMLDocumentNode_dispex);
+    init_doc(&doc->basedoc, (IUnknown*)HTMLDOMNODE(&doc->node), DISPATCHEX(&doc->node.dispex));
+    HTMLDocumentNode_SecMgr_Init(doc);
+
+    init_nsevents(doc);
+
+    list_init(&doc->bindings);
+    list_init(&doc->selection_list);
+    list_init(&doc->range_list);
+    list_init(&doc->plugin_hosts);
+
+    return doc;
+}
+
 HRESULT create_doc_from_nsdoc(nsIDOMHTMLDocument *nsdoc, HTMLDocumentObj *doc_obj, HTMLWindow *window, HTMLDocumentNode **ret)
 {
     HTMLDocumentNode *doc;
     HRESULT hres;
 
-    doc = heap_alloc_zero(sizeof(HTMLDocumentNode));
+    doc = alloc_doc_node(doc_obj, window);
     if(!doc)
         return E_OUTOFMEMORY;
 
-    doc->basedoc.doc_node = doc;
-    doc->basedoc.doc_obj = doc_obj;
-
-    init_dispex(&doc->node.dispex, (IUnknown*)HTMLDOMNODE(&doc->node), &HTMLDocumentNode_dispex);
-    init_doc(&doc->basedoc, (IUnknown*)HTMLDOMNODE(&doc->node), DISPATCHEX(&doc->node.dispex));
-    HTMLDocumentNode_SecMgr_Init(doc);
-    doc->ref = 1;
-
-    doc->basedoc.window = window;
     if(window == doc_obj->basedoc.window)
         doc->basedoc.cp_container.forward_container = &doc_obj->basedoc.cp_container;
 
     nsIDOMHTMLDocument_AddRef(nsdoc);
     doc->nsdoc = nsdoc;
     init_mutation(doc);
-    init_nsevents(doc);
-
-    list_init(&doc->bindings);
-    list_init(&doc->selection_list);
-    list_init(&doc->range_list);
 
     HTMLDOMNode_Init(doc, &doc->node, (nsIDOMNode*)nsdoc);
     doc->node.vtbl = &HTMLDocumentNodeImplVtbl;
@@ -1949,6 +2019,23 @@ HRESULT create_doc_from_nsdoc(nsIDOMHTMLDocument *nsdoc, HTMLDocumentObj *doc_ob
     return S_OK;
 }
 
+HRESULT create_document_fragment(nsIDOMNode *nsnode, HTMLDocumentNode *doc_node, HTMLDocumentNode **ret)
+{
+    HTMLDocumentNode *doc_frag;
+
+    doc_frag = alloc_doc_node(doc_node->basedoc.doc_obj, doc_node->basedoc.window);
+    if(!doc_frag)
+        return E_OUTOFMEMORY;
+
+    HTMLDOMNode_Init(doc_node, &doc_frag->node, nsnode);
+    doc_frag->node.vtbl = &HTMLDocumentFragmentImplVtbl;
+    doc_frag->node.cp_container = &doc_frag->basedoc.cp_container;
+
+    htmldoc_addref(&doc_frag->basedoc);
+    *ret = doc_frag;
+    return S_OK;
+}
+
 /**********************************************************
  * ICustomDoc implementation
  */
@@ -2010,6 +2097,8 @@ static ULONG WINAPI CustomDoc_Release(ICustomDoc *iface)
             IAdviseSink_Release(This->view_sink);
         if(This->client)
             IOleObject_SetClientSite(OLEOBJ(&This->basedoc), NULL);
+        if(This->hostui)
+            ICustomDoc_SetUIHandler(CUSTOMDOC(This), NULL);
         if(This->in_place_active)
             IOleInPlaceObjectWindowless_InPlaceDeactivate(INPLACEWIN(&This->basedoc));
         if(This->ipsite)
@@ -2037,8 +2126,31 @@ static ULONG WINAPI CustomDoc_Release(ICustomDoc *iface)
 static HRESULT WINAPI CustomDoc_SetUIHandler(ICustomDoc *iface, IDocHostUIHandler *pUIHandler)
 {
     HTMLDocumentObj *This = CUSTOMDOC_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, pUIHandler);
-    return E_NOTIMPL;
+    IOleCommandTarget *cmdtrg;
+    HRESULT hres;
+
+    TRACE("(%p)->(%p)\n", This, pUIHandler);
+
+    if(This->custom_hostui && This->hostui == pUIHandler)
+        return S_OK;
+
+    This->custom_hostui = TRUE;
+
+    if(This->hostui)
+        IDocHostUIHandler_Release(This->hostui);
+    if(pUIHandler)
+        IDocHostUIHandler_AddRef(pUIHandler);
+    This->hostui = pUIHandler;
+    if(!pUIHandler)
+        return S_OK;
+
+    hres = IDocHostUIHandler_QueryInterface(pUIHandler, &IID_IOleCommandTarget, (void**)&cmdtrg);
+    if(SUCCEEDED(hres)) {
+        FIXME("custom UI handler supports IOleCommandTarget\n");
+        IOleCommandTarget_Release(cmdtrg);
+    }
+
+    return S_OK;
 }
 
 #undef CUSTOMDOC_THIS
