@@ -108,6 +108,7 @@ static HRESULT WINAPI HTMLDocument3_createTextNode(IHTMLDocument3 *iface, BSTR t
     HTMLDOMNode *node;
     nsAString text_str;
     nsresult nsres;
+    HRESULT hres;
 
     TRACE("(%p)->(%s %p)\n", This, debugstr_w(text), newTextNode);
 
@@ -124,8 +125,10 @@ static HRESULT WINAPI HTMLDocument3_createTextNode(IHTMLDocument3 *iface, BSTR t
         return E_FAIL;
     }
 
-    node = HTMLDOMTextNode_Create(This->doc_node, (nsIDOMNode*)nstext);
+    hres = HTMLDOMTextNode_Create(This->doc_node, (nsIDOMNode*)nstext, &node);
     nsIDOMElement_Release(nstext);
+    if(FAILED(hres))
+        return hres;
 
     *newTextNode = HTMLDOMNODE(node);
     IHTMLDOMNode_AddRef(HTMLDOMNODE(node));
@@ -138,6 +141,7 @@ static HRESULT WINAPI HTMLDocument3_get_documentElement(IHTMLDocument3 *iface, I
     nsIDOMElement *nselem = NULL;
     HTMLDOMNode *node;
     nsresult nsres;
+    HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
@@ -157,15 +161,17 @@ static HRESULT WINAPI HTMLDocument3_get_documentElement(IHTMLDocument3 *iface, I
         return E_FAIL;
     }
 
-    if(nselem) {
-        node = get_node(This->doc_node, (nsIDOMNode *)nselem, TRUE);
-        nsIDOMElement_Release(nselem);
-        IHTMLDOMNode_QueryInterface(HTMLDOMNODE(node), &IID_IHTMLElement, (void**)p);
-    }else {
+    if(!nselem) {
         *p = NULL;
+        return S_OK;
     }
 
-    return S_OK;
+    hres = get_node(This->doc_node, (nsIDOMNode *)nselem, TRUE, &node);
+    nsIDOMElement_Release(nselem);
+    if(FAILED(hres))
+        return hres;
+
+    return IHTMLDOMNode_QueryInterface(HTMLDOMNODE(node), &IID_IHTMLElement, (void**)p);
 }
 
 static HRESULT WINAPI HTMLDocument3_uniqueID(IHTMLDocument3 *iface, BSTR *p)
@@ -343,8 +349,31 @@ static HRESULT WINAPI HTMLDocument3_createDocumentFragment(IHTMLDocument3 *iface
                                                            IHTMLDocument2 **ppNewDoc)
 {
     HTMLDocument *This = HTMLDOC3_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, ppNewDoc);
-    return E_NOTIMPL;
+    nsIDOMDocumentFragment *doc_frag;
+    HTMLDocumentNode *docnode;
+    nsresult nsres;
+    HRESULT hres;
+
+    TRACE("(%p)->(%p)\n", This, ppNewDoc);
+
+    if(!This->doc_node->nsdoc) {
+        FIXME("NULL nsdoc\n");
+        return E_NOTIMPL;
+    }
+
+    nsres = nsIDOMHTMLDocument_CreateDocumentFragment(This->doc_node->nsdoc, &doc_frag);
+    if(NS_FAILED(nsres)) {
+        ERR("CreateDocumentFragment failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    hres = create_document_fragment((nsIDOMNode*)doc_frag, This->doc_node, &docnode);
+    nsIDOMDocumentFragment_Release(doc_frag);
+    if(FAILED(hres))
+        return hres;
+
+    *ppNewDoc = HTMLDOC(&docnode->basedoc);
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLDocument3_get_parentDocument(IHTMLDocument3 *iface,
@@ -441,6 +470,7 @@ static HRESULT WINAPI HTMLDocument3_getElementById(IHTMLDocument3 *iface, BSTR v
     nsIDOMNodeList *nsnode_list;
     nsAString id_str;
     nsresult nsres;
+    HRESULT hres;
 
     TRACE("(%p)->(%s %p)\n", This, debugstr_w(v), pel);
 
@@ -505,15 +535,17 @@ static HRESULT WINAPI HTMLDocument3_getElementById(IHTMLDocument3 *iface, BSTR v
         nsnode = nsnode_by_name ? nsnode_by_name : nsnode_by_id;
 
     if(nsnode) {
-        node = get_node(This->doc_node, nsnode, TRUE);
+        hres = get_node(This->doc_node, nsnode, TRUE, &node);
         nsIDOMNode_Release(nsnode);
 
-        IHTMLDOMNode_QueryInterface(HTMLDOMNODE(node), &IID_IHTMLElement, (void**)pel);
+        if(SUCCEEDED(hres))
+            hres = IHTMLDOMNode_QueryInterface(HTMLDOMNODE(node), &IID_IHTMLElement, (void**)pel);
     }else {
         *pel = NULL;
+        hres = S_OK;
     }
 
-    return S_OK;
+    return hres;
 }
 
 
