@@ -56,6 +56,9 @@ static const WCHAR onchangeW[] = {'o','n','c','h','a','n','g','e',0};
 static const WCHAR clickW[] = {'c','l','i','c','k',0};
 static const WCHAR onclickW[] = {'o','n','c','l','i','c','k',0};
 
+static const WCHAR contextmenuW[] = {'c','o','n','t','e','x','t','m','e','n','u',0};
+static const WCHAR oncontextmenuW[] = {'o','n','c','o','n','t','e','x','t','m','e','n','u',0};
+
 static const WCHAR dblclickW[] = {'d','b','l','c','l','i','c','k',0};
 static const WCHAR ondblclickW[] = {'o','n','d','b','l','c','l','i','c','k',0};
 
@@ -141,6 +144,8 @@ static const event_info_t event_info[] = {
         EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
     {clickW,             onclickW,             EVENTT_MOUSE,  DISPID_EVMETH_ONCLICK,
         EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {contextmenuW,       oncontextmenuW,       EVENTT_MOUSE,  DISPID_EVMETH_ONCONTEXTMENU,
+        EVENT_BUBBLE},
     {dblclickW,          ondblclickW,          EVENTT_MOUSE,  DISPID_EVMETH_ONDBLCLICK,
         EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
     {dragW,              ondragW,              EVENTT_MOUSE,  DISPID_EVMETH_ONDRAG,
@@ -215,7 +220,7 @@ static DWORD get_node_handler_mask(eventid_t eid)
 
 typedef struct {
     DispatchEx dispex;
-    const IHTMLEventObjVtbl  *lpIHTMLEventObjVtbl;
+    IHTMLEventObj IHTMLEventObj_iface;
 
     LONG ref;
 
@@ -224,22 +229,23 @@ typedef struct {
     nsIDOMEvent *nsevent;
 } HTMLEventObj;
 
-#define HTMLEVENTOBJ(x) ((IHTMLEventObj*) &(x)->lpIHTMLEventObjVtbl)
-
-#define HTMLEVENTOBJ_THIS(iface) DEFINE_THIS(HTMLEventObj, IHTMLEventObj, iface)
+static inline HTMLEventObj *impl_from_IHTMLEventObj(IHTMLEventObj *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLEventObj, IHTMLEventObj_iface);
+}
 
 static HRESULT WINAPI HTMLEventObj_QueryInterface(IHTMLEventObj *iface, REFIID riid, void **ppv)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     *ppv = NULL;
 
     if(IsEqualGUID(&IID_IUnknown, riid)) {
         TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
-        *ppv = HTMLEVENTOBJ(This);
+        *ppv = &This->IHTMLEventObj_iface;
     }else if(IsEqualGUID(&IID_IHTMLEventObj, riid)) {
         TRACE("(%p)->(IID_IHTMLEventObj %p)\n", This, ppv);
-        *ppv = HTMLEVENTOBJ(This);
+        *ppv = &This->IHTMLEventObj_iface;
     }else if(dispex_query_interface(&This->dispex, riid, ppv)) {
         return *ppv ? S_OK : E_NOINTERFACE;
     }
@@ -255,7 +261,7 @@ static HRESULT WINAPI HTMLEventObj_QueryInterface(IHTMLEventObj *iface, REFIID r
 
 static ULONG WINAPI HTMLEventObj_AddRef(IHTMLEventObj *iface)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
@@ -265,7 +271,7 @@ static ULONG WINAPI HTMLEventObj_AddRef(IHTMLEventObj *iface)
 
 static ULONG WINAPI HTMLEventObj_Release(IHTMLEventObj *iface)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
@@ -282,42 +288,37 @@ static ULONG WINAPI HTMLEventObj_Release(IHTMLEventObj *iface)
 
 static HRESULT WINAPI HTMLEventObj_GetTypeInfoCount(IHTMLEventObj *iface, UINT *pctinfo)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, pctinfo);
-    return E_NOTIMPL;
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
+    return IDispatchEx_GetTypeInfoCount(DISPATCHEX(&This->dispex), pctinfo);
 }
 
 static HRESULT WINAPI HTMLEventObj_GetTypeInfo(IHTMLEventObj *iface, UINT iTInfo,
                                               LCID lcid, ITypeInfo **ppTInfo)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
-    FIXME("(%p)->(%u %u %p)\n", This, iTInfo, lcid, ppTInfo);
-    return E_NOTIMPL;
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
+    return IDispatchEx_GetTypeInfo(DISPATCHEX(&This->dispex), iTInfo, lcid, ppTInfo);
 }
 
 static HRESULT WINAPI HTMLEventObj_GetIDsOfNames(IHTMLEventObj *iface, REFIID riid,
                                                 LPOLESTR *rgszNames, UINT cNames,
                                                 LCID lcid, DISPID *rgDispId)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
-    FIXME("(%p)->(%s %p %u %u %p)\n", This, debugstr_guid(riid), rgszNames, cNames,
-          lcid, rgDispId);
-    return E_NOTIMPL;
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
+    return IDispatchEx_GetIDsOfNames(DISPATCHEX(&This->dispex), riid, rgszNames, cNames, lcid, rgDispId);
 }
 
 static HRESULT WINAPI HTMLEventObj_Invoke(IHTMLEventObj *iface, DISPID dispIdMember,
                             REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
                             VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
-    FIXME("(%p)->(%d %s %d %d %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
-          lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
-    return E_NOTIMPL;
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
+    return IDispatchEx_Invoke(DISPATCHEX(&This->dispex), dispIdMember, riid, lcid,
+            wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 }
 
 static HRESULT WINAPI HTMLEventObj_get_srcElement(IHTMLEventObj *iface, IHTMLElement **p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     TRACE("(%p)->(%p)\n", This, p);
 
@@ -326,7 +327,7 @@ static HRESULT WINAPI HTMLEventObj_get_srcElement(IHTMLEventObj *iface, IHTMLEle
 
 static HRESULT WINAPI HTMLEventObj_get_altKey(IHTMLEventObj *iface, VARIANT_BOOL *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     PRBool ret = FALSE;
 
     TRACE("(%p)->(%p)\n", This, p);
@@ -356,7 +357,7 @@ static HRESULT WINAPI HTMLEventObj_get_altKey(IHTMLEventObj *iface, VARIANT_BOOL
 
 static HRESULT WINAPI HTMLEventObj_get_ctrlKey(IHTMLEventObj *iface, VARIANT_BOOL *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     PRBool ret = FALSE;
 
     TRACE("(%p)->(%p)\n", This, p);
@@ -386,7 +387,7 @@ static HRESULT WINAPI HTMLEventObj_get_ctrlKey(IHTMLEventObj *iface, VARIANT_BOO
 
 static HRESULT WINAPI HTMLEventObj_get_shiftKey(IHTMLEventObj *iface, VARIANT_BOOL *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     PRBool ret = FALSE;
 
     TRACE("(%p)->(%p)\n", This, p);
@@ -416,14 +417,14 @@ static HRESULT WINAPI HTMLEventObj_get_shiftKey(IHTMLEventObj *iface, VARIANT_BO
 
 static HRESULT WINAPI HTMLEventObj_put_returnValue(IHTMLEventObj *iface, VARIANT v)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     FIXME("(%p)->()\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLEventObj_get_returnValue(IHTMLEventObj *iface, VARIANT *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -433,14 +434,14 @@ static HRESULT WINAPI HTMLEventObj_get_returnValue(IHTMLEventObj *iface, VARIANT
 
 static HRESULT WINAPI HTMLEventObj_put_cancelBubble(IHTMLEventObj *iface, VARIANT_BOOL v)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     FIXME("(%p)->(%x)\n", This, v);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLEventObj_get_cancelBubble(IHTMLEventObj *iface, VARIANT_BOOL *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -450,7 +451,7 @@ static HRESULT WINAPI HTMLEventObj_get_cancelBubble(IHTMLEventObj *iface, VARIAN
 
 static HRESULT WINAPI HTMLEventObj_get_fromElement(IHTMLEventObj *iface, IHTMLElement **p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -460,7 +461,7 @@ static HRESULT WINAPI HTMLEventObj_get_fromElement(IHTMLEventObj *iface, IHTMLEl
 
 static HRESULT WINAPI HTMLEventObj_get_toElement(IHTMLEventObj *iface, IHTMLElement **p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -470,14 +471,14 @@ static HRESULT WINAPI HTMLEventObj_get_toElement(IHTMLEventObj *iface, IHTMLElem
 
 static HRESULT WINAPI HTMLEventObj_put_keyCode(IHTMLEventObj *iface, LONG v)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     FIXME("(%p)->(%d)\n", This, v);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI HTMLEventObj_get_keyCode(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     PRUint32 key_code = 0;
 
     TRACE("(%p)->(%p)\n", This, p);
@@ -499,7 +500,7 @@ static HRESULT WINAPI HTMLEventObj_get_keyCode(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_button(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     PRUint16 button = 0;
 
     TRACE("(%p)->(%p)\n", This, p);
@@ -521,7 +522,7 @@ static HRESULT WINAPI HTMLEventObj_get_button(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_type(IHTMLEventObj *iface, BSTR *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     TRACE("(%p)->(%p)\n", This, p);
 
@@ -531,7 +532,7 @@ static HRESULT WINAPI HTMLEventObj_get_type(IHTMLEventObj *iface, BSTR *p)
 
 static HRESULT WINAPI HTMLEventObj_get_qualifier(IHTMLEventObj *iface, BSTR *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -541,7 +542,7 @@ static HRESULT WINAPI HTMLEventObj_get_qualifier(IHTMLEventObj *iface, BSTR *p)
 
 static HRESULT WINAPI HTMLEventObj_get_reason(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -551,7 +552,7 @@ static HRESULT WINAPI HTMLEventObj_get_reason(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_x(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -561,7 +562,7 @@ static HRESULT WINAPI HTMLEventObj_get_x(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_y(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -571,7 +572,7 @@ static HRESULT WINAPI HTMLEventObj_get_y(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_clientX(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     PRInt32 x = 0;
 
     TRACE("(%p)->(%p)\n", This, p);
@@ -593,7 +594,7 @@ static HRESULT WINAPI HTMLEventObj_get_clientX(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_clientY(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     PRInt32 y = 0;
 
     TRACE("(%p)->(%p)\n", This, p);
@@ -615,7 +616,7 @@ static HRESULT WINAPI HTMLEventObj_get_clientY(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_offsetX(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -625,7 +626,7 @@ static HRESULT WINAPI HTMLEventObj_get_offsetX(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_offsetY(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -635,7 +636,7 @@ static HRESULT WINAPI HTMLEventObj_get_offsetY(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_screenX(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     PRInt32 x = 0;
 
     TRACE("(%p)->(%p)\n", This, p);
@@ -657,7 +658,7 @@ static HRESULT WINAPI HTMLEventObj_get_screenX(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_screenY(IHTMLEventObj *iface, LONG *p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
     PRInt32 y = 0;
 
     TRACE("(%p)->(%p)\n", This, p);
@@ -679,7 +680,7 @@ static HRESULT WINAPI HTMLEventObj_get_screenY(IHTMLEventObj *iface, LONG *p)
 
 static HRESULT WINAPI HTMLEventObj_get_srcFilter(IHTMLEventObj *iface, IDispatch **p)
 {
-    HTMLEventObj *This = HTMLEVENTOBJ_THIS(iface);
+    HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
     FIXME("(%p)->(%p)\n", This, p);
 
@@ -687,8 +688,6 @@ static HRESULT WINAPI HTMLEventObj_get_srcFilter(IHTMLEventObj *iface, IDispatch
     return S_OK;
 }
 
-#undef HTMLEVENTOBJ_THIS
-
 static const IHTMLEventObjVtbl HTMLEventObjVtbl = {
     HTMLEventObj_QueryInterface,
     HTMLEventObj_AddRef,
@@ -744,7 +743,7 @@ static IHTMLEventObj *create_event(HTMLDOMNode *target, eventid_t eid, nsIDOMEve
     if(!ret)
         return NULL;
 
-    ret->lpIHTMLEventObjVtbl = &HTMLEventObjVtbl;
+    ret->IHTMLEventObj_iface.lpVtbl = &HTMLEventObjVtbl;
     ret->ref = 1;
     ret->type = event_info+eid;
 
@@ -767,7 +766,7 @@ static IHTMLEventObj *create_event(HTMLDOMNode *target, eventid_t eid, nsIDOMEve
         }
         if(NS_FAILED(nsres)) {
             ERR("Could not create event: %08x\n", nsres);
-            IHTMLEventObj_Release(HTMLEVENTOBJ(ret));
+            IHTMLEventObj_Release(&ret->IHTMLEventObj_iface);
             return NULL;
         }
     }
@@ -775,9 +774,9 @@ static IHTMLEventObj *create_event(HTMLDOMNode *target, eventid_t eid, nsIDOMEve
     ret->target = target;
     IHTMLDOMNode_AddRef(HTMLDOMNODE(target));
 
-    init_dispex(&ret->dispex, (IUnknown*)HTMLEVENTOBJ(ret), &HTMLEventObj_dispex);
+    init_dispex(&ret->dispex, (IUnknown*)&ret->IHTMLEventObj_iface, &HTMLEventObj_dispex);
 
-    return HTMLEVENTOBJ(ret);
+    return &ret->IHTMLEventObj_iface;
 }
 
 static HRESULT call_cp_func(IDispatch *disp, DISPID dispid)
@@ -905,12 +904,18 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
     nsIDOMNode *parent, *nsnode;
     HTMLDOMNode *node;
     PRUint16 node_type;
+    HRESULT hres;
 
     TRACE("(%p) %s\n", doc, debugstr_w(event_info[eid].name));
 
     prev_event = doc->basedoc.window->event;
-    if(set_event)
-        event_obj = create_event(get_node(doc, target, TRUE), eid, nsevent);
+    if(set_event) {
+        hres = get_node(doc, target, TRUE, &node);
+        if(FAILED(hres))
+            return;
+
+        event_obj = create_event(node, eid, nsevent);
+    }
     doc->basedoc.window->event = event_obj;
 
     nsIDOMNode_GetNodeType(target, &node_type);
@@ -920,8 +925,8 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
     switch(node_type) {
     case ELEMENT_NODE:
         do {
-            node = get_node(doc, nsnode, FALSE);
-            if(node)
+            hres = get_node(doc, nsnode, FALSE, &node);
+            if(SUCCEEDED(hres) && node)
                 call_event_handlers(doc, event_obj, *get_node_event_target(node), node->cp_container, eid,
                         (IDispatch*)HTMLDOMNODE(node));
 
@@ -947,8 +952,8 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
 
             nsres = nsIDOMHTMLDocument_GetBody(doc->nsdoc, &nsbody);
             if(NS_SUCCEEDED(nsres) && nsbody) {
-                node = get_node(doc, (nsIDOMNode*)nsbody, FALSE);
-                if(node)
+                hres = get_node(doc, (nsIDOMNode*)nsbody, FALSE, &node);
+                if(SUCCEEDED(hres) && node)
                     call_event_handlers(doc, event_obj, *get_node_event_target(node), node->cp_container,
                             eid, (IDispatch*)HTMLDOMNODE(node));
                 nsIDOMHTMLElement_Release(nsbody);
@@ -1077,7 +1082,7 @@ static HRESULT ensure_nsevent_handler(HTMLDocumentNode *doc, event_target_t *eve
 
 static HRESULT remove_event_handler(event_target_t **event_target, eventid_t eid)
 {
-    if(*event_target && (*event_target)->event_table[eid]->handler_prop) {
+    if(*event_target && (*event_target)->event_table[eid] && (*event_target)->event_table[eid]->handler_prop) {
         IDispatch_Release((*event_target)->event_table[eid]->handler_prop);
         (*event_target)->event_table[eid]->handler_prop = NULL;
     }
@@ -1225,6 +1230,7 @@ void check_event_attr(HTMLDocumentNode *doc, nsIDOMElement *nselem)
     HTMLDOMNode *node;
     int i;
     nsresult nsres;
+    HRESULT hres;
 
     nsAString_Init(&attr_value_str, NULL);
     nsAString_Init(&attr_name_str, NULL);
@@ -1241,8 +1247,9 @@ void check_event_attr(HTMLDocumentNode *doc, nsIDOMElement *nselem)
 
             disp = script_parse_event(doc->basedoc.window, attr_value);
             if(disp) {
-                node = get_node(doc, (nsIDOMNode*)nselem, TRUE);
-                set_event_handler_disp(get_node_event_target(node), node->nsnode, node->doc, i, disp);
+                hres = get_node(doc, (nsIDOMNode*)nselem, TRUE, &node);
+                if(SUCCEEDED(hres))
+                    set_event_handler_disp(get_node_event_target(node), node->nsnode, node->doc, i, disp);
                 IDispatch_Release(disp);
             }
         }
