@@ -41,6 +41,42 @@ struct HTMLFormElement {
 
 #define HTMLFORM(x)  (&(x)->lpHTMLFormElementVtbl)
 
+static HRESULT htmlform_item(HTMLFormElement *This, int i, IDispatch **ret)
+{
+    nsIDOMHTMLCollection *elements;
+    nsIDOMNode *item;
+    HTMLDOMNode *node;
+    nsresult nsres;
+    HRESULT hres;
+
+    nsres = nsIDOMHTMLFormElement_GetElements(This->nsform, &elements);
+    if(NS_FAILED(nsres)) {
+        FIXME("GetElements failed: 0x%08x\n", nsres);
+        return E_FAIL;
+    }
+
+    nsres = nsIDOMHTMLCollection_Item(elements, i, &item);
+    nsIDOMHTMLCollection_Release(elements);
+    if(NS_FAILED(nsres)) {
+        FIXME("Item failed: 0x%08x\n", nsres);
+        return E_FAIL;
+    }
+
+    if(item) {
+        hres = get_node(This->element.node.doc, item, TRUE, &node);
+        if(FAILED(hres))
+            return hres;
+
+        IHTMLDOMNode_AddRef(HTMLDOMNODE(node));
+        nsIDOMNode_Release(item);
+        *ret = (IDispatch*)HTMLDOMNODE(node);
+    }else {
+        *ret = NULL;
+    }
+
+    return S_OK;
+}
+
 #define HTMLFORM_THIS(iface) DEFINE_THIS(HTMLFormElement, HTMLFormElement, iface)
 
 static HRESULT WINAPI HTMLFormElement_QueryInterface(IHTMLFormElement *iface,
@@ -98,15 +134,43 @@ static HRESULT WINAPI HTMLFormElement_Invoke(IHTMLFormElement *iface, DISPID dis
 static HRESULT WINAPI HTMLFormElement_put_action(IHTMLFormElement *iface, BSTR v)
 {
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(%s)\n", This, wine_dbgstr_w(v));
-    return E_NOTIMPL;
+    nsAString action_str;
+    nsresult nsres;
+
+    TRACE("(%p)->(%s)\n", This, wine_dbgstr_w(v));
+
+    nsAString_InitDepend(&action_str, v);
+    nsres = nsIDOMHTMLFormElement_SetAction(This->nsform, &action_str);
+    nsAString_Finish(&action_str);
+    if(NS_FAILED(nsres)) {
+        ERR("SetAction failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLFormElement_get_action(IHTMLFormElement *iface, BSTR *p)
 {
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsAString action_str;
+    nsresult nsres;
+    HRESULT hres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsAString_Init(&action_str, NULL);
+    nsres = nsIDOMHTMLFormElement_GetAction(This->nsform, &action_str);
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *action;
+        nsAString_GetData(&action_str, &action);
+        hres = nsuri_to_url(action, FALSE, p);
+    }else {
+        ERR("GetAction failed: %08x\n", nsres);
+        hres = E_FAIL;
+    }
+
+    return hres;
 }
 
 static HRESULT WINAPI HTMLFormElement_put_dir(IHTMLFormElement *iface, BSTR v)
@@ -125,30 +189,101 @@ static HRESULT WINAPI HTMLFormElement_get_dir(IHTMLFormElement *iface, BSTR *p)
 
 static HRESULT WINAPI HTMLFormElement_put_encoding(IHTMLFormElement *iface, BSTR v)
 {
+    static const WCHAR urlencodedW[] = {'a','p','p','l','i','c','a','t','i','o','n','/',
+        'x','-','w','w','w','-','f','o','r','m','-','u','r','l','e','n','c','o','d','e','d',0};
+    static const WCHAR dataW[] = {'m','u','l','t','i','p','a','r','t','/',
+        'f','o','r','m','-','d','a','t','a',0};
+    static const WCHAR plainW[] = {'t','e','x','t','/','p','l','a','i','n',0};
+
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(%s)\n", This, wine_dbgstr_w(v));
-    return E_NOTIMPL;
+    nsAString encoding_str;
+    nsresult nsres;
+
+    TRACE("(%p)->(%s)\n", This, wine_dbgstr_w(v));
+
+    if(lstrcmpiW(v, urlencodedW) && lstrcmpiW(v, dataW) && lstrcmpiW(v, plainW)) {
+        WARN("incorrect enctype\n");
+        return E_INVALIDARG;
+    }
+
+    nsAString_InitDepend(&encoding_str, v);
+    nsres = nsIDOMHTMLFormElement_SetEnctype(This->nsform, &encoding_str);
+    nsAString_Finish(&encoding_str);
+    if(NS_FAILED(nsres))
+        return E_FAIL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLFormElement_get_encoding(IHTMLFormElement *iface, BSTR *p)
 {
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsAString encoding_str;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsAString_Init(&encoding_str, NULL);
+    nsres = nsIDOMHTMLFormElement_GetEnctype(This->nsform, &encoding_str);
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *encoding;
+        nsAString_GetData(&encoding_str, &encoding);
+
+        *p = SysAllocString(encoding);
+        if(!*p)
+            return E_OUTOFMEMORY;
+    }else
+        return E_FAIL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLFormElement_put_method(IHTMLFormElement *iface, BSTR v)
 {
+    static const WCHAR postW[] = {'P','O','S','T',0};
+    static const WCHAR getW[] = {'G','E','T',0};
+
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(%s)\n", This, wine_dbgstr_w(v));
-    return E_NOTIMPL;
+    nsAString method_str;
+    nsresult nsres;
+
+    TRACE("(%p)->(%s)\n", This, wine_dbgstr_w(v));
+
+    if(lstrcmpiW(v, postW) && lstrcmpiW(v, getW)) {
+        WARN("unrecognized method\n");
+        return E_INVALIDARG;
+    }
+
+    nsAString_InitDepend(&method_str, v);
+    nsres = nsIDOMHTMLFormElement_SetMethod(This->nsform, &method_str);
+    nsAString_Finish(&method_str);
+    if(NS_FAILED(nsres))
+        return E_FAIL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLFormElement_get_method(IHTMLFormElement *iface, BSTR *p)
 {
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsAString method_str;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsAString_Init(&method_str, NULL);
+    nsres = nsIDOMHTMLFormElement_GetMethod(This->nsform, &method_str);
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *method;
+        nsAString_GetData(&method_str, &method);
+
+        *p = SysAllocString(method);
+        if(!*p)
+            return E_OUTOFMEMORY;
+    }else
+        return E_FAIL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLFormElement_get_elements(IHTMLFormElement *iface, IDispatch **p)
@@ -175,15 +310,44 @@ static HRESULT WINAPI HTMLFormElement_get_target(IHTMLFormElement *iface, BSTR *
 static HRESULT WINAPI HTMLFormElement_put_name(IHTMLFormElement *iface, BSTR v)
 {
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(%s)\n", This, wine_dbgstr_w(v));
-    return E_NOTIMPL;
+    nsAString name_str;
+    nsresult nsres;
+
+    TRACE("(%p)->(%s)\n", This, wine_dbgstr_w(v));
+
+    nsAString_InitDepend(&name_str, v);
+    nsres = nsIDOMHTMLFormElement_SetName(This->nsform, &name_str);
+    nsAString_Finish(&name_str);
+    if(NS_FAILED(nsres))
+        return E_FAIL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLFormElement_get_name(IHTMLFormElement *iface, BSTR *p)
 {
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsAString name_str;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsAString_Init(&name_str, NULL);
+    nsres = nsIDOMHTMLFormElement_GetName(This->nsform, &name_str);
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *name;
+        nsAString_GetData(&name_str, &name);
+
+        if(*name) {
+            *p = SysAllocString(name);
+            if(!*p)
+                return E_OUTOFMEMORY;
+        }else
+            *p = NULL;
+    }else
+        return E_FAIL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLFormElement_put_onsubmit(IHTMLFormElement *iface, VARIANT v)
@@ -238,8 +402,19 @@ static HRESULT WINAPI HTMLFormElement_put_length(IHTMLFormElement *iface, LONG v
 static HRESULT WINAPI HTMLFormElement_get_length(IHTMLFormElement *iface, LONG *p)
 {
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    PRInt32 length;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsres = nsIDOMHTMLFormElement_GetLength(This->nsform, &length);
+    if(NS_FAILED(nsres)) {
+        ERR("GetLength failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    *p = length;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLFormElement__newEnum(IHTMLFormElement *iface, IUnknown **p)
@@ -253,7 +428,20 @@ static HRESULT WINAPI HTMLFormElement_item(IHTMLFormElement *iface, VARIANT name
         VARIANT index, IDispatch **pdisp)
 {
     HTMLFormElement *This = HTMLFORM_THIS(iface);
-    FIXME("(%p)->(v v %p)\n", This, pdisp);
+
+    TRACE("(%p)->(%s %s %p)\n", This, debugstr_variant(&name), debugstr_variant(&index), pdisp);
+
+    if(!pdisp)
+        return E_INVALIDARG;
+    *pdisp = NULL;
+
+    if(V_VT(&name) == VT_I4) {
+        if(V_I4(&name) < 0)
+            return E_INVALIDARG;
+        return htmlform_item(This, V_I4(&name), pdisp);
+    }
+
+    FIXME("Unsupported args\n");
     return E_NOTIMPL;
 }
 
@@ -428,34 +616,21 @@ static HRESULT HTMLFormElement_invoke(HTMLDOMNode *iface,
         EXCEPINFO *ei, IServiceProvider *caller)
 {
     HTMLFormElement *This = HTMLFORM_NODE_THIS(iface);
-    nsIDOMHTMLCollection *elements;
-    nsIDOMNode *item;
-    HTMLDOMNode *node;
-    nsresult nsres;
+    IDispatch *ret;
+    HRESULT hres;
 
     TRACE("(%p)->(%x %x %x %p %p %p %p)\n", This, id, lcid, flags, params, res, ei, caller);
 
-    nsres = nsIDOMHTMLFormElement_GetElements(This->nsform, &elements);
-    if(NS_FAILED(nsres)) {
-        FIXME("GetElements failed: 0x%08x\n", nsres);
-        return E_FAIL;
+    hres = htmlform_item(This, id - MSHTML_DISPID_CUSTOM_MIN, &ret);
+    if(FAILED(hres))
+        return hres;
+
+    if(ret) {
+        V_VT(res) = VT_DISPATCH;
+        V_DISPATCH(res) = ret;
+    }else {
+        V_VT(res) = VT_NULL;
     }
-
-    nsres = nsIDOMHTMLCollection_Item(elements, id - MSHTML_DISPID_CUSTOM_MIN, &item);
-    nsIDOMHTMLCollection_Release(elements);
-    if(NS_FAILED(nsres)) {
-        FIXME("Item failed: 0x%08x\n", nsres);
-        return E_FAIL;
-    }
-
-    node = get_node(This->element.node.doc, item, TRUE);
-
-    V_VT(res) = VT_DISPATCH;
-    V_DISPATCH(res) = (IDispatch*)node;
-
-    IHTMLDOMNode_AddRef(HTMLDOMNODE(node));
-    nsIDOMNode_Release(item);
-
     return S_OK;
 }
 
@@ -464,6 +639,7 @@ static HRESULT HTMLFormElement_invoke(HTMLDOMNode *iface,
 static const NodeImplVtbl HTMLFormElementImplVtbl = {
     HTMLFormElement_QI,
     HTMLFormElement_destructor,
+    HTMLElement_clone,
     NULL,
     NULL,
     NULL,
@@ -487,19 +663,27 @@ static dispex_static_data_t HTMLFormElement_dispex = {
     HTMLFormElement_iface_tids
 };
 
-HTMLElement *HTMLFormElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem)
+HRESULT HTMLFormElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem, HTMLElement **elem)
 {
-    HTMLFormElement *ret = heap_alloc_zero(sizeof(HTMLFormElement));
+    HTMLFormElement *ret;
     nsresult nsres;
 
+    ret = heap_alloc_zero(sizeof(HTMLFormElement));
+    if(!ret)
+        return E_OUTOFMEMORY;
+
     ret->lpHTMLFormElementVtbl = &HTMLFormElementVtbl;
     ret->element.node.vtbl = &HTMLFormElementImplVtbl;
 
+    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLFormElement, (void**)&ret->nsform);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDOMHTMLFormElement interface: %08x\n", nsres);
+        heap_free(ret);
+        return E_FAIL;
+    }
+
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLFormElement_dispex);
 
-    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLFormElement, (void**)&ret->nsform);
-    if(NS_FAILED(nsres))
-        ERR("Could not get nsIDOMHTMLFormElement interface: %08x\n", nsres);
-
-    return &ret->element;
+    *elem = &ret->element;
+    return S_OK;
 }
