@@ -34,13 +34,22 @@
 
 #include "nsiface.h"
 
+#define NS_ERROR_GENERATE_FAILURE(module,code) \
+    ((nsresult) (((PRUint32)(1<<31)) | ((PRUint32)(module+0x45)<<16) | ((PRUint32)(code))))
+
 #define NS_OK                     ((nsresult)0x00000000L)
 #define NS_ERROR_FAILURE          ((nsresult)0x80004005L)
+#define NS_ERROR_OUT_OF_MEMORY    ((nsresult)0x8007000EL)
 #define NS_NOINTERFACE            ((nsresult)0x80004002L)
 #define NS_ERROR_NOT_IMPLEMENTED  ((nsresult)0x80004001L)
+#define NS_ERROR_NOT_AVAILABLE    ((nsresult)0x80040111L)
 #define NS_ERROR_INVALID_ARG      ((nsresult)0x80070057L) 
 #define NS_ERROR_UNEXPECTED       ((nsresult)0x8000ffffL)
-#define NS_ERROR_UNKNOWN_PROTOCOL ((nsresult)0x804b0012L)
+
+#define NS_ERROR_MODULE_NETWORK    6
+
+#define NS_BINDING_ABORTED         NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_NETWORK, 2)
+#define NS_ERROR_UNKNOWN_PROTOCOL  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_NETWORK, 18)
 
 #define NS_FAILED(res) ((res) & 0x80000000)
 #define NS_SUCCEEDED(res) (!NS_FAILED(res))
@@ -54,87 +63,104 @@ typedef struct ConnectionPoint ConnectionPoint;
 typedef struct BSCallback BSCallback;
 typedef struct event_target_t event_target_t;
 
-/* NOTE: make sure to keep in sync with dispex.c */
+#define TID_LIST \
+    XIID(NULL) \
+    XDIID(DispCEventObj) \
+    XDIID(DispCPlugins) \
+    XDIID(DispDOMChildrenCollection) \
+    XDIID(DispHTMLAnchorElement) \
+    XDIID(DispHTMLBody) \
+    XDIID(DispHTMLCommentElement) \
+    XDIID(DispHTMLCurrentStyle) \
+    XDIID(DispHTMLDocument) \
+    XDIID(DispHTMLDOMTextNode) \
+    XDIID(DispHTMLElementCollection) \
+    XDIID(DispHTMLEmbed) \
+    XDIID(DispHTMLFormElement) \
+    XDIID(DispHTMLGenericElement) \
+    XDIID(DispHTMLFrameElement) \
+    XDIID(DispHTMLIFrame) \
+    XDIID(DispHTMLImg) \
+    XDIID(DispHTMLInputElement) \
+    XDIID(DispHTMLLocation) \
+    XDIID(DispHTMLNavigator) \
+    XDIID(DispHTMLObjectElement) \
+    XDIID(DispHTMLOptionElement) \
+    XDIID(DispHTMLScreen) \
+    XDIID(DispHTMLScriptElement) \
+    XDIID(DispHTMLSelectElement) \
+    XDIID(DispHTMLStyle) \
+    XDIID(DispHTMLStyleElement) \
+    XDIID(DispHTMLStyleSheetsCollection) \
+    XDIID(DispHTMLTable) \
+    XDIID(DispHTMLTableRow) \
+    XDIID(DispHTMLTextAreaElement) \
+    XDIID(DispHTMLUnknownElement) \
+    XDIID(DispHTMLWindow2) \
+    XDIID(HTMLDocumentEvents) \
+    XIID(IHTMLAnchorElement) \
+    XIID(IHTMLBodyElement) \
+    XIID(IHTMLBodyElement2) \
+    XIID(IHTMLCommentElement) \
+    XIID(IHTMLCurrentStyle) \
+    XIID(IHTMLCurrentStyle2) \
+    XIID(IHTMLCurrentStyle3) \
+    XIID(IHTMLCurrentStyle4) \
+    XIID(IHTMLDocument2) \
+    XIID(IHTMLDocument3) \
+    XIID(IHTMLDocument4) \
+    XIID(IHTMLDocument5) \
+    XIID(IHTMLDOMChildrenCollection) \
+    XIID(IHTMLDOMNode) \
+    XIID(IHTMLDOMNode2) \
+    XIID(IHTMLDOMTextNode) \
+    XIID(IHTMLElement) \
+    XIID(IHTMLElement2) \
+    XIID(IHTMLElement3) \
+    XIID(IHTMLElement4) \
+    XIID(IHTMLElementCollection) \
+    XIID(IHTMLEmbedElement) \
+    XIID(IHTMLEventObj) \
+    XIID(IHTMLFiltersCollection) \
+    XIID(IHTMLFormElement) \
+    XIID(IHTMLFrameBase) \
+    XIID(IHTMLFrameBase2) \
+    XIID(IHTMLFrameElement3) \
+    XIID(IHTMLGenericElement) \
+    XIID(IHTMLIFrameElement) \
+    XIID(IHTMLImageElementFactory) \
+    XIID(IHTMLImgElement) \
+    XIID(IHTMLInputElement) \
+    XIID(IHTMLLocation) \
+    XIID(IHTMLObjectElement) \
+    XIID(IHTMLOptionElement) \
+    XIID(IHTMLPluginsCollection) \
+    XIID(IHTMLRect) \
+    XIID(IHTMLScreen) \
+    XIID(IHTMLScriptElement) \
+    XIID(IHTMLSelectElement) \
+    XIID(IHTMLStyle) \
+    XIID(IHTMLStyle2) \
+    XIID(IHTMLStyle3) \
+    XIID(IHTMLStyle4) \
+    XIID(IHTMLStyleElement) \
+    XIID(IHTMLStyleSheetsCollection) \
+    XIID(IHTMLTable) \
+    XIID(IHTMLTableRow) \
+    XIID(IHTMLTextAreaElement) \
+    XIID(IHTMLTextContainer) \
+    XIID(IHTMLUniqueName) \
+    XIID(IHTMLWindow2) \
+    XIID(IHTMLWindow3) \
+    XIID(IHTMLWindow4) \
+    XIID(IOmNavigator)
+
 typedef enum {
-    NULL_tid,
-    DispCEventObj_tid,
-    DispDOMChildrenCollection_tid,
-    DispHTMLAnchorElement_tid,
-    DispHTMLBody_tid,
-    DispHTMLCommentElement_tid,
-    DispHTMLCurrentStyle_tid,
-    DispHTMLDocument_tid,
-    DispHTMLDOMTextNode_tid,
-    DispHTMLElementCollection_tid,
-    DispHTMLFormElement_tid,
-    DispHTMLGenericElement_tid,
-    DispHTMLFrameElement_tid,
-    DispHTMLIFrame_tid,
-    DispHTMLImg_tid,
-    DispHTMLInputElement_tid,
-    DispHTMLLocation_tid,
-    DispHTMLNavigator_tid,
-    DispHTMLOptionElement_tid,
-    DispHTMLScreen_tid,
-    DispHTMLScriptElement_tid,
-    DispHTMLSelectElement_tid,
-    DispHTMLStyle_tid,
-    DispHTMLTable_tid,
-    DispHTMLTableRow_tid,
-    DispHTMLTextAreaElement_tid,
-    DispHTMLUnknownElement_tid,
-    DispHTMLWindow2_tid,
-    HTMLDocumentEvents_tid,
-    IHTMLAnchorElement_tid,
-    IHTMLBodyElement_tid,
-    IHTMLBodyElement2_tid,
-    IHTMLCommentElement_tid,
-    IHTMLCurrentStyle_tid,
-    IHTMLCurrentStyle2_tid,
-    IHTMLCurrentStyle3_tid,
-    IHTMLCurrentStyle4_tid,
-    IHTMLDocument2_tid,
-    IHTMLDocument3_tid,
-    IHTMLDocument4_tid,
-    IHTMLDocument5_tid,
-    IHTMLDOMChildrenCollection_tid,
-    IHTMLDOMNode_tid,
-    IHTMLDOMNode2_tid,
-    IHTMLDOMTextNode_tid,
-    IHTMLElement_tid,
-    IHTMLElement2_tid,
-    IHTMLElement3_tid,
-    IHTMLElement4_tid,
-    IHTMLElementCollection_tid,
-    IHTMLEventObj_tid,
-    IHTMLFiltersCollection_tid,
-    IHTMLFormElement_tid,
-    IHTMLFrameBase_tid,
-    IHTMLFrameBase2_tid,
-    IHTMLFrameElement3_tid,
-    IHTMLGenericElement_tid,
-    IHTMLIFrameElement_tid,
-    IHTMLImageElementFactory_tid,
-    IHTMLImgElement_tid,
-    IHTMLInputElement_tid,
-    IHTMLLocation_tid,
-    IHTMLOptionElement_tid,
-    IHTMLScreen_tid,
-    IHTMLScriptElement_tid,
-    IHTMLSelectElement_tid,
-    IHTMLStyle_tid,
-    IHTMLStyle2_tid,
-    IHTMLStyle3_tid,
-    IHTMLStyle4_tid,
-    IHTMLTable_tid,
-    IHTMLTableRow_tid,
-    IHTMLTextAreaElement_tid,
-    IHTMLTextContainer_tid,
-    IHTMLUniqueName_tid,
-    IHTMLWindow2_tid,
-    IHTMLWindow3_tid,
-    IHTMLWindow4_tid,
-    IOmNavigator_tid,
+#define XIID(iface) iface ## _tid,
+#define XDIID(iface) iface ## _tid,
+TID_LIST
+#undef XIID
+#undef XDIID
     LAST_tid
 } tid_t;
 
@@ -143,6 +169,7 @@ typedef struct dispex_dynamic_data_t dispex_dynamic_data_t;
 
 #define MSHTML_DISPID_CUSTOM_MIN 0x60000000
 #define MSHTML_DISPID_CUSTOM_MAX 0x6fffffff
+#define MSHTML_CUSTOM_DISPID_CNT (MSHTML_DISPID_CUSTOM_MAX-MSHTML_DISPID_CUSTOM_MIN)
 
 typedef struct {
     HRESULT (*value)(IUnknown*,LCID,WORD,DISPPARAMS*,VARIANT*,EXCEPINFO*,IServiceProvider*);
@@ -237,7 +264,8 @@ struct HTMLWindow {
     const IHTMLWindow3Vtbl *lpHTMLWindow3Vtbl;
     const IHTMLWindow4Vtbl *lpHTMLWindow4Vtbl;
     const IHTMLPrivateWindowVtbl *lpIHTMLPrivateWindowVtbl;
-    const IDispatchExVtbl  *lpIDispatchExVtbl;
+    const IDispatchExVtbl         *lpIDispatchExVtbl;
+    const IServiceProviderVtbl    *lpServiceProviderVtbl;
 
     LONG ref;
 
@@ -383,6 +411,7 @@ struct HTMLDocumentObj {
 
     IOleClientSite *client;
     IDocHostUIHandler *hostui;
+    BOOL custom_hostui;
     IOleInPlaceSite *ipsite;
     IOleInPlaceFrame *frame;
     IOleInPlaceUIWindow *ip_window;
@@ -436,8 +465,6 @@ struct NSContainer {
     nsIURIContentListener *content_listener;
 
     HWND hwnd;
-
-    HWND reset_focus; /* hack */
 };
 
 typedef struct nsWineURI nsWineURI;
@@ -445,6 +472,13 @@ typedef struct nsWineURI nsWineURI;
 HRESULT set_wine_url(nsWineURI*,LPCWSTR);
 nsresult on_start_uri_open(NSContainer*,nsIURI*,PRBool*);
 
+/* Keep sync with request_method_strings in nsio.c */
+typedef enum {
+    METHOD_GET,
+    METHOD_PUT,
+    METHOD_POST
+} REQUEST_METHOD;
+
 typedef struct {
     const nsIHttpChannelVtbl *lpHttpChannelVtbl;
     const nsIUploadChannelVtbl *lpUploadChannelVtbl;
@@ -459,15 +493,28 @@ typedef struct {
     nsISupports *owner;
     nsLoadFlags load_flags;
     nsIURI *original_uri;
+    nsIURI *referrer;
     char *content_type;
     char *charset;
     PRUint32 response_status;
+    REQUEST_METHOD request_method;
+    struct list response_headers;
+    struct list request_headers;
     UINT url_scheme;
 } nsChannel;
 
+typedef struct {
+    struct list entry;
+    WCHAR *header;
+    WCHAR *data;
+} http_header_t;
+
+HRESULT set_http_header(struct list*,const WCHAR*,int,const WCHAR*,int);
+
 typedef struct {
     HRESULT (*qi)(HTMLDOMNode*,REFIID,void**);
     void (*destructor)(HTMLDOMNode*);
+    HRESULT (*clone)(HTMLDOMNode*,nsIDOMNode*,HTMLDOMNode**);
     event_target_t **(*get_event_target)(HTMLDOMNode*);
     HRESULT (*call_event)(HTMLDOMNode*,DWORD,BOOL*);
     HRESULT (*put_disabled)(HTMLDOMNode*,VARIANT_BOOL);
@@ -564,12 +611,14 @@ struct HTMLDocumentNode {
     nsDocumentEventListener *nsevent_listener;
     BOOL *event_vector;
 
+    BOOL skip_mutation_notif;
     mutation_queue_t *mutation_queue;
     mutation_queue_t *mutation_queue_tail;
 
     struct list bindings;
     struct list selection_list;
     struct list range_list;
+    struct list plugin_hosts;
 };
 
 #define HTMLWINDOW2(x)   ((IHTMLWindow2*)                 &(x)->lpHTMLWindow2Vtbl)
@@ -654,6 +703,7 @@ struct HTMLDocumentNode {
 HRESULT HTMLDocument_Create(IUnknown*,REFIID,void**);
 HRESULT HTMLLoadOptions_Create(IUnknown*,REFIID,void**);
 HRESULT create_doc_from_nsdoc(nsIDOMHTMLDocument*,HTMLDocumentObj*,HTMLWindow*,HTMLDocumentNode**);
+HRESULT create_document_fragment(nsIDOMNode*,HTMLDocumentNode*,HTMLDocumentNode**);
 
 HRESULT HTMLWindow_Create(HTMLDocumentObj*,nsIDOMWindow*,HTMLWindow*,HTMLWindow**);
 void update_window_doc(HTMLWindow*);
@@ -704,7 +754,7 @@ void close_gecko(void);
 void register_nsservice(nsIComponentRegistrar*,nsIServiceManager*);
 void init_nsio(nsIComponentManager*,nsIComponentRegistrar*);
 void release_nsio(void);
-BOOL install_wine_gecko(BOOL);
+BOOL is_gecko_path(const char*);
 
 HRESULT nsuri_to_url(LPCWSTR,BOOL,BSTR*);
 HRESULT create_doc_uri(HTMLWindow*,WCHAR*,nsWineURI**);
@@ -721,8 +771,10 @@ HRESULT call_set_active_object(IOleInPlaceUIWindow*,IOleInPlaceActiveObject*);
 void *nsalloc(size_t) __WINE_ALLOC_SIZE(1);
 void nsfree(void*);
 
+void nsACString_InitDepend(nsACString*,const char*);
 void nsACString_SetData(nsACString*,const char*);
 PRUint32 nsACString_GetData(const nsACString*,const char**);
+void nsACString_Finish(nsACString*);
 
 BOOL nsAString_Init(nsAString*,const PRUnichar*);
 void nsAString_InitDepend(nsAString*,const PRUnichar*);
@@ -766,24 +818,27 @@ HRESULT get_node_text(HTMLDOMNode*,BSTR*);
 
 HRESULT create_nselem(HTMLDocumentNode*,const WCHAR*,nsIDOMHTMLElement**);
 
-HTMLDOMNode *HTMLDOMTextNode_Create(HTMLDocumentNode*,nsIDOMNode*);
+HRESULT HTMLDOMTextNode_Create(HTMLDocumentNode*,nsIDOMNode*,HTMLDOMNode**);
 
-HTMLElement *HTMLElement_Create(HTMLDocumentNode*,nsIDOMNode*,BOOL);
-HTMLElement *HTMLCommentElement_Create(HTMLDocumentNode*,nsIDOMNode*);
-HTMLElement *HTMLAnchorElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLBodyElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLFormElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLFrameElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLIFrame_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLImgElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLInputElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLOptionElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLScriptElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLSelectElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLTable_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLTableRow_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLTextAreaElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
-HTMLElement *HTMLGenericElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*);
+HRESULT HTMLElement_Create(HTMLDocumentNode*,nsIDOMNode*,BOOL,HTMLElement**);
+HRESULT HTMLCommentElement_Create(HTMLDocumentNode*,nsIDOMNode*,HTMLElement**);
+HRESULT HTMLAnchorElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLBodyElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLEmbedElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLFormElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLFrameElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLIFrame_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLStyleElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLImgElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLInputElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLObjectElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLOptionElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLScriptElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLSelectElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLTable_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLTableRow_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLTextAreaElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
+HRESULT HTMLGenericElement_Create(HTMLDocumentNode*,nsIDOMHTMLElement*,HTMLElement**);
 
 void HTMLDOMNode_Init(HTMLDocumentNode*,HTMLDOMNode*,nsIDOMNode*);
 void HTMLElement_Init(HTMLElement*,HTMLDocumentNode*,nsIDOMHTMLElement*,dispex_static_data_t*);
@@ -797,17 +852,19 @@ void HTMLDOMNode_destructor(HTMLDOMNode*);
 
 HRESULT HTMLElement_QI(HTMLDOMNode*,REFIID,void**);
 void HTMLElement_destructor(HTMLDOMNode*);
+HRESULT HTMLElement_clone(HTMLDOMNode*,nsIDOMNode*,HTMLDOMNode**);
 
 HRESULT HTMLFrameBase_QI(HTMLFrameBase*,REFIID,void**);
 void HTMLFrameBase_destructor(HTMLFrameBase*);
 
-HTMLDOMNode *get_node(HTMLDocumentNode*,nsIDOMNode*,BOOL);
+HRESULT get_node(HTMLDocumentNode*,nsIDOMNode*,BOOL,HTMLDOMNode**);
 void release_nodes(HTMLDocumentNode*);
 
 void release_script_hosts(HTMLWindow*);
 void connect_scripts(HTMLWindow*);
 void doc_insert_script(HTMLWindow*,nsIDOMHTMLScriptElement*);
 IDispatch *script_parse_event(HTMLWindow*,LPCWSTR);
+HRESULT exec_script(HTMLWindow*,const WCHAR*,const WCHAR*,VARIANT*);
 void set_script_mode(HTMLWindow*,SCRIPTMODE);
 BOOL find_global_prop(HTMLWindow*,BSTR,DWORD,ScriptHost**,DISPID*);
 IDispatch *get_script_disp(ScriptHost*);
@@ -837,7 +894,6 @@ void update_title(HTMLDocumentObj*);
 
 /* editor */
 void init_editor(HTMLDocument*);
-void set_ns_editmode(NSContainer*);
 void handle_edit_event(HTMLDocument*,nsIDOMEvent*);
 HRESULT editor_exec_copy(HTMLDocument*,DWORD,VARIANT*,VARIANT*);
 HRESULT editor_exec_cut(HTMLDocument*,DWORD,VARIANT*,VARIANT*);
@@ -934,6 +990,19 @@ static inline LPWSTR heap_strdupW(LPCWSTR str)
     return ret;
 }
 
+static inline LPWSTR heap_strndupW(LPCWSTR str, unsigned len)
+{
+    LPWSTR ret = NULL;
+
+    if(str) {
+        ret = heap_alloc((len+1)*sizeof(WCHAR));
+        memcpy(ret, str, len*sizeof(WCHAR));
+        ret[len] = 0;
+    }
+
+    return ret;
+}
+
 static inline char *heap_strdupA(const char *str)
 {
     char *ret = NULL;
@@ -990,5 +1059,6 @@ static inline void windowref_release(windowref_t *ref)
 
 HDC get_display_dc(void);
 HINSTANCE get_shdoclc(void);
+void set_statustext(HTMLDocumentObj*,INT,LPCWSTR);
 
 extern HINSTANCE hInst;
