@@ -113,7 +113,8 @@ DEFINE_EXPECT(SetScriptState_STARTED);
 DEFINE_EXPECT(SetScriptState_CONNECTED);
 DEFINE_EXPECT(SetScriptState_DISCONNECTED);
 DEFINE_EXPECT(AddNamedItem);
-DEFINE_EXPECT(ParseScriptText);
+DEFINE_EXPECT(ParseScriptText_script);
+DEFINE_EXPECT(ParseScriptText_execScript);
 DEFINE_EXPECT(GetScriptDispatch);
 DEFINE_EXPECT(funcDisp);
 DEFINE_EXPECT(script_divid_d);
@@ -123,7 +124,8 @@ DEFINE_EXPECT(script_testprop2_d);
 DEFINE_EXPECT(AXQueryInterface_IActiveScript);
 DEFINE_EXPECT(AXQueryInterface_IObjectSafety);
 DEFINE_EXPECT(AXGetInterfaceSafetyOptions);
-DEFINE_EXPECT(AXSetInterfaceSafetyOptions);
+DEFINE_EXPECT(AXSetInterfaceSafetyOptions_IDispatch);
+DEFINE_EXPECT(AXSetInterfaceSafetyOptions_IDispatchEx);
 DEFINE_EXPECT(external_success);
 
 #define TESTSCRIPT_CLSID "{178fc163-f585-4e24-9c13-4bb7faf80746}"
@@ -148,6 +150,8 @@ static BOOL doc_complete;
 static IDispatch *script_disp;
 static BOOL ax_objsafe;
 static HWND container_hwnd;
+static HRESULT ax_getopt_hres = S_OK, ax_setopt_dispex_hres = S_OK, ax_setopt_disp_hres = S_OK;
+static DWORD ax_setopt;
 
 static const char *debugstr_guid(REFIID riid)
 {
@@ -420,7 +424,7 @@ static HRESULT WINAPI scriptDisp_GetDispID(IDispatchEx *iface, BSTR bstrName, DW
         return E_FAIL;
     }
 
-    ok(0, "unexpected call\n");
+    ok(0, "unexpected call %s\n", wine_dbgstr_w(bstrName));
     return E_NOTIMPL;
 }
 
@@ -1265,25 +1269,35 @@ static HRESULT WINAPI AXObjectSafety_GetInterfaceSafetyOptions(IObjectSafety *if
     ok(pdwSupportedOptions != NULL, "pdwSupportedOptions == NULL\n");
     ok(pdwEnabledOptions != NULL, "pdwEnabledOptions == NULL\n");
 
-    *pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_DATA|INTERFACE_USES_DISPEX|INTERFACE_USES_SECURITY_MANAGER;
-    *pdwEnabledOptions = INTERFACE_USES_DISPEX;
+    if(SUCCEEDED(ax_getopt_hres)) {
+        *pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_DATA|INTERFACE_USES_DISPEX|INTERFACE_USES_SECURITY_MANAGER;
+        *pdwEnabledOptions = INTERFACE_USES_DISPEX;
+    }
 
-    return S_OK;
+    return ax_getopt_hres;
 }
 
 static HRESULT WINAPI AXObjectSafety_SetInterfaceSafetyOptions(IObjectSafety *iface, REFIID riid,
         DWORD dwOptionSetMask, DWORD dwEnabledOptions)
 {
-    CHECK_EXPECT(AXSetInterfaceSafetyOptions);
+    if(IsEqualGUID(&IID_IDispatchEx, riid)) {
+        CHECK_EXPECT(AXSetInterfaceSafetyOptions_IDispatchEx);
+        ok(dwOptionSetMask == ax_setopt, "dwOptionSetMask=%x, expected %x\n", dwOptionSetMask, ax_setopt);
+        ok(dwEnabledOptions == ax_setopt, "dwEnabledOptions=%x, expected %x\n", dwOptionSetMask, ax_setopt);
+        return ax_setopt_dispex_hres;
+    }
 
-    ok(IsEqualGUID(&IID_IDispatchEx, riid), "unexpected riid %s\n", debugstr_guid(riid));
+    if(IsEqualGUID(&IID_IDispatch, riid)) {
+        DWORD exopt = ax_setopt & ~INTERFACE_USES_SECURITY_MANAGER;
 
-    ok(dwOptionSetMask == (INTERFACESAFE_FOR_UNTRUSTED_CALLER|INTERFACE_USES_SECURITY_MANAGER),
-       "dwOptionSetMask=%x\n", dwOptionSetMask);
-    ok(dwEnabledOptions == (INTERFACESAFE_FOR_UNTRUSTED_CALLER|INTERFACE_USES_SECURITY_MANAGER),
-       "dwEnabledOptions=%x\n", dwOptionSetMask);
+        CHECK_EXPECT(AXSetInterfaceSafetyOptions_IDispatch);
+        ok(dwOptionSetMask == exopt, "dwOptionSetMask=%x, expected %x\n", dwOptionSetMask, exopt);
+        ok(dwEnabledOptions == exopt, "dwEnabledOptions=%x, expected %x\n", dwOptionSetMask, exopt);
+        return ax_setopt_disp_hres;
+    }
 
-    return S_OK;
+    ok(0, "unexpected riid %s\n", debugstr_guid(riid));
+    return E_NOINTERFACE;
 }
 
 static const IObjectSafetyVtbl AXObjectSafetyVtbl = {
@@ -1311,6 +1325,8 @@ static void test_security(void)
     BYTE *ppolicy;
     HRESULT hres;
 
+    ax_setopt = INTERFACESAFE_FOR_UNTRUSTED_CALLER|INTERFACE_USES_SECURITY_MANAGER;
+
     hres = IActiveScriptSite_QueryInterface(site, &IID_IServiceProvider, (void**)&sp);
     ok(hres == S_OK, "Could not get IServiceProvider iface: %08x\n", hres);
 
@@ -1332,13 +1348,13 @@ static void test_security(void)
     SET_EXPECT(AXQueryInterface_IActiveScript);
     SET_EXPECT(AXQueryInterface_IObjectSafety);
     SET_EXPECT(AXGetInterfaceSafetyOptions);
-    SET_EXPECT(AXSetInterfaceSafetyOptions);
+    SET_EXPECT(AXSetInterfaceSafetyOptions_IDispatchEx);
     hres = IInternetHostSecurityManager_QueryCustomPolicy(sec_mgr, &GUID_CUSTOM_CONFIRMOBJECTSAFETY,
             &ppolicy, &policy_size, (BYTE*)&cs, sizeof(cs), 0);
     CHECK_CALLED(AXQueryInterface_IActiveScript);
     CHECK_CALLED(AXQueryInterface_IObjectSafety);
     CHECK_CALLED(AXGetInterfaceSafetyOptions);
-    CHECK_CALLED(AXSetInterfaceSafetyOptions);
+    CHECK_CALLED(AXSetInterfaceSafetyOptions_IDispatchEx);
 
     ok(hres == S_OK, "QueryCusromPolicy failed: %08x\n", hres);
     ok(policy_size == sizeof(DWORD), "policy_size = %d\n", policy_size);
@@ -1376,13 +1392,13 @@ static void test_security(void)
         SET_EXPECT(AXQueryInterface_IActiveScript);
         SET_EXPECT(AXQueryInterface_IObjectSafety);
         SET_EXPECT(AXGetInterfaceSafetyOptions);
-        SET_EXPECT(AXSetInterfaceSafetyOptions);
+        SET_EXPECT(AXSetInterfaceSafetyOptions_IDispatchEx);
         hres = IInternetHostSecurityManager_QueryCustomPolicy(sec_mgr, &GUID_CUSTOM_CONFIRMOBJECTSAFETY,
                 &ppolicy, &policy_size, (BYTE*)&cs, sizeof(cs), 0);
         CHECK_CALLED(AXQueryInterface_IActiveScript);
         CHECK_CALLED(AXQueryInterface_IObjectSafety);
         CHECK_CALLED(AXGetInterfaceSafetyOptions);
-        CHECK_CALLED(AXSetInterfaceSafetyOptions);
+        CHECK_CALLED(AXSetInterfaceSafetyOptions_IDispatchEx);
 
         ok(hres == S_OK, "QueryCusromPolicy failed: %08x\n", hres);
         ok(policy_size == sizeof(DWORD), "policy_size = %d\n", policy_size);
@@ -1394,6 +1410,69 @@ static void test_security(void)
         skip("Could not set safety registry\n");
     }
 
+    ax_objsafe = TRUE;
+
+    ax_setopt_dispex_hres = E_NOINTERFACE;
+    SET_EXPECT(AXQueryInterface_IActiveScript);
+    SET_EXPECT(AXQueryInterface_IObjectSafety);
+    SET_EXPECT(AXGetInterfaceSafetyOptions);
+    SET_EXPECT(AXSetInterfaceSafetyOptions_IDispatchEx);
+    SET_EXPECT(AXSetInterfaceSafetyOptions_IDispatch);
+    hres = IInternetHostSecurityManager_QueryCustomPolicy(sec_mgr, &GUID_CUSTOM_CONFIRMOBJECTSAFETY,
+            &ppolicy, &policy_size, (BYTE*)&cs, sizeof(cs), 0);
+    CHECK_CALLED(AXQueryInterface_IActiveScript);
+    CHECK_CALLED(AXQueryInterface_IObjectSafety);
+    CHECK_CALLED(AXGetInterfaceSafetyOptions);
+    CHECK_CALLED(AXSetInterfaceSafetyOptions_IDispatchEx);
+    CHECK_CALLED(AXSetInterfaceSafetyOptions_IDispatch);
+
+    ok(hres == S_OK, "QueryCusromPolicy failed: %08x\n", hres);
+    ok(policy_size == sizeof(DWORD), "policy_size = %d\n", policy_size);
+    ok(*(DWORD*)ppolicy == URLPOLICY_ALLOW, "policy = %x\n", *(DWORD*)ppolicy);
+    CoTaskMemFree(ppolicy);
+
+    ax_setopt_dispex_hres = E_FAIL;
+    ax_setopt_disp_hres = E_NOINTERFACE;
+    SET_EXPECT(AXQueryInterface_IActiveScript);
+    SET_EXPECT(AXQueryInterface_IObjectSafety);
+    SET_EXPECT(AXGetInterfaceSafetyOptions);
+    SET_EXPECT(AXSetInterfaceSafetyOptions_IDispatchEx);
+    SET_EXPECT(AXSetInterfaceSafetyOptions_IDispatch);
+    hres = IInternetHostSecurityManager_QueryCustomPolicy(sec_mgr, &GUID_CUSTOM_CONFIRMOBJECTSAFETY,
+            &ppolicy, &policy_size, (BYTE*)&cs, sizeof(cs), 0);
+    CHECK_CALLED(AXQueryInterface_IActiveScript);
+    CHECK_CALLED(AXQueryInterface_IObjectSafety);
+    CHECK_CALLED(AXGetInterfaceSafetyOptions);
+    CHECK_CALLED(AXSetInterfaceSafetyOptions_IDispatchEx);
+    CHECK_CALLED(AXSetInterfaceSafetyOptions_IDispatch);
+
+    ok(hres == S_OK, "QueryCusromPolicy failed: %08x\n", hres);
+    ok(policy_size == sizeof(DWORD), "policy_size = %d\n", policy_size);
+    ok(*(DWORD*)ppolicy == URLPOLICY_DISALLOW, "policy = %x\n", *(DWORD*)ppolicy);
+    CoTaskMemFree(ppolicy);
+
+    ax_setopt_dispex_hres = E_FAIL;
+    ax_setopt_disp_hres = S_OK;
+    ax_getopt_hres = E_NOINTERFACE;
+    ax_setopt = INTERFACESAFE_FOR_UNTRUSTED_CALLER;
+    SET_EXPECT(AXQueryInterface_IActiveScript);
+    SET_EXPECT(AXQueryInterface_IObjectSafety);
+    SET_EXPECT(AXGetInterfaceSafetyOptions);
+    SET_EXPECT(AXSetInterfaceSafetyOptions_IDispatchEx);
+    SET_EXPECT(AXSetInterfaceSafetyOptions_IDispatch);
+    hres = IInternetHostSecurityManager_QueryCustomPolicy(sec_mgr, &GUID_CUSTOM_CONFIRMOBJECTSAFETY,
+            &ppolicy, &policy_size, (BYTE*)&cs, sizeof(cs), 0);
+    CHECK_CALLED(AXQueryInterface_IActiveScript);
+    CHECK_CALLED(AXQueryInterface_IObjectSafety);
+    CHECK_CALLED(AXGetInterfaceSafetyOptions);
+    CHECK_CALLED(AXSetInterfaceSafetyOptions_IDispatchEx);
+    CHECK_CALLED(AXSetInterfaceSafetyOptions_IDispatch);
+
+    ok(hres == S_OK, "QueryCusromPolicy failed: %08x\n", hres);
+    ok(policy_size == sizeof(DWORD), "policy_size = %d\n", policy_size);
+    ok(*(DWORD*)ppolicy == URLPOLICY_ALLOW, "policy = %x\n", *(DWORD*)ppolicy);
+    CoTaskMemFree(ppolicy);
+
     IInternetHostSecurityManager_Release(sec_mgr);
 }
 
@@ -1654,10 +1733,7 @@ static void test_global_id(void)
     VariantClear(&var);
 }
 
-static HRESULT WINAPI ActiveScriptParse_ParseScriptText(IActiveScriptParse *iface,
-        LPCOLESTR pstrCode, LPCOLESTR pstrItemName, IUnknown *punkContext,
-        LPCOLESTR pstrDelimiter, CTXARG_T dwSourceContextCookie, ULONG ulStartingLine,
-        DWORD dwFlags, VARIANT *pvarResult, EXCEPINFO *pexcepinfo)
+static void test_script_run(void)
 {
     IDispatchEx *document, *dispex;
     IHTMLWindow2 *window;
@@ -1674,8 +1750,6 @@ static HRESULT WINAPI ActiveScriptParse_ParseScriptText(IActiveScriptParse *ifac
     static const WCHAR testW[] = {'t','e','s','t',0};
     static const WCHAR funcW[] = {'f','u','n','c',0};
 
-    CHECK_EXPECT(ParseScriptText);
-
     SET_EXPECT(GetScriptDispatch);
 
     tmp = SysAllocString(documentW);
@@ -1865,8 +1939,38 @@ static HRESULT WINAPI ActiveScriptParse_ParseScriptText(IActiveScriptParse *ifac
     test_global_id();
 
     test_security();
+}
 
-    return S_OK;
+static HRESULT WINAPI ActiveScriptParse_ParseScriptText(IActiveScriptParse *iface,
+        LPCOLESTR pstrCode, LPCOLESTR pstrItemName, IUnknown *punkContext,
+        LPCOLESTR pstrDelimiter, CTXARG_T dwSourceContextCookie, ULONG ulStartingLine,
+        DWORD dwFlags, VARIANT *pvarResult, EXCEPINFO *pexcepinfo)
+{
+    ok(!punkContext, "punkContext = %p\n", punkContext);
+    ok(pvarResult != NULL, "pvarResult == NULL\n");
+    ok(pexcepinfo != NULL, "pexcepinfo == NULL\n");
+
+    if(!strcmp_wa(pstrCode, "execScript call")) {
+        CHECK_EXPECT(ParseScriptText_execScript);
+        ok(!pstrItemName, "pstrItemName = %s\n", wine_dbgstr_w(pstrItemName));
+        ok(!strcmp_wa(pstrDelimiter, "\""), "pstrDelimiter = %s\n", wine_dbgstr_w(pstrDelimiter));
+        ok(dwFlags == SCRIPTTEXT_ISVISIBLE, "dwFlags = %x\n", dwFlags);
+
+        V_VT(pvarResult) = VT_I4;
+        V_I4(pvarResult) = 10;
+        return S_OK;
+    }else if(!strcmp_wa(pstrCode, "simple script")) {
+        CHECK_EXPECT(ParseScriptText_script);
+        ok(!strcmp_wa(pstrItemName, "window"), "pstrItemName = %s\n", wine_dbgstr_w(pstrItemName));
+        ok(!strcmp_wa(pstrDelimiter, "</SCRIPT>"), "pstrDelimiter = %s\n", wine_dbgstr_w(pstrDelimiter));
+        ok(dwFlags == (SCRIPTTEXT_ISVISIBLE|SCRIPTTEXT_HOSTMANAGESSOURCE), "dwFlags = %x\n", dwFlags);
+
+        test_script_run();
+        return S_OK;
+    }
+
+    ok(0, "unexpected script %s\n", wine_dbgstr_w(pstrCode));
+    return E_FAIL;
 }
 
 static const IActiveScriptParseVtbl ActiveScriptParseVtbl = {
@@ -1995,6 +2099,8 @@ static HRESULT WINAPI ActiveScript_SetScriptState(IActiveScript *iface, SCRIPTST
     }
 
     hres = IActiveScriptSite_OnStateChange(site, (state = ss));
+    ok(hres == S_OK, "OnStateChange failed: %08x\n", hres);
+
     return S_OK;
 }
 
@@ -2185,6 +2291,38 @@ static const char simple_script_str[] =
     "<script language=\"TestScript\">simple script</script>"
     "</body></html>";
 
+static void test_exec_script(IHTMLDocument2 *doc)
+{
+    IHTMLWindow2 *window;
+    BSTR code, lang;
+    VARIANT v;
+    HRESULT hres;
+
+    hres = IHTMLDocument2_get_parentWindow(doc, &window);
+    ok(hres == S_OK, "get_parentWindow failed: %08x\n", hres);
+
+    code = a2bstr("execScript call");
+    lang = a2bstr("TestScript");
+
+    SET_EXPECT(ParseScriptText_execScript);
+    hres = IHTMLWindow2_execScript(window, code, lang, &v);
+    ok(hres == S_OK, "execScript failed: %08x\n", hres);
+    ok(V_VT(&v) == VT_I4, "V_VT(v) = %d\n", V_VT(&v));
+    ok(V_I4(&v) == 10, "V_I4(v) = %d\n", V_I4(&v));
+    CHECK_CALLED(ParseScriptText_execScript);
+    SysFreeString(lang);
+
+    lang = a2bstr("invalid");
+    V_VT(&v) = 100;
+    hres = IHTMLWindow2_execScript(window, code, lang, &v);
+    ok(hres == CO_E_CLASSSTRING, "execScript failed: %08x, expected CO_E_CLASSSTRING\n", hres);
+    ok(V_VT(&v) == 100, "V_VT(v) = %d\n", V_VT(&v));
+    SysFreeString(lang);
+    SysFreeString(code);
+
+    IHTMLWindow2_Release(window);
+}
+
 static void test_simple_script(void)
 {
     IHTMLDocument2 *doc;
@@ -2204,7 +2342,7 @@ static void test_simple_script(void)
     SET_EXPECT(SetScriptState_STARTED);
     SET_EXPECT(AddNamedItem);
     SET_EXPECT(SetProperty_ABBREVIATE_GLOBALNAME_RESOLUTION); /* IE8 */
-    SET_EXPECT(ParseScriptText);
+    SET_EXPECT(ParseScriptText_script);
     SET_EXPECT(SetScriptState_CONNECTED);
 
     load_doc(doc, simple_script_str);
@@ -2220,9 +2358,11 @@ static void test_simple_script(void)
     CHECK_CALLED(SetScriptState_STARTED);
     CHECK_CALLED(AddNamedItem);
     CHECK_CALLED_BROKEN(SetProperty_ABBREVIATE_GLOBALNAME_RESOLUTION); /* IE8 */
-    CHECK_CALLED(ParseScriptText);
+    CHECK_CALLED(ParseScriptText_script);
     CHECK_CALLED(SetScriptState_CONNECTED);
 
+    test_exec_script(doc);
+
     if(site)
         IActiveScriptSite_Release(site);
     if(window_dispex)
