@@ -1439,6 +1439,55 @@ unsigned char* CDECL _mbsnbcat(unsigned char* dst, const unsigned char* src, MSV
     return u_strncat(dst, src, len); /* ASCII CP */
 }
 
+int CDECL _mbsnbcat_s(unsigned char *dst, MSVCRT_size_t size, const unsigned char *src, MSVCRT_size_t len)
+{
+    unsigned char *ptr = dst;
+    MSVCRT_size_t i;
+
+    if (!dst && !size && !src && !len)
+        return 0;
+
+    if (!dst || !size || !src)
+    {
+        if (dst && size)
+            *dst = '\0';
+
+        *MSVCRT__errno() = MSVCRT_EINVAL;
+        return MSVCRT_EINVAL;
+    }
+
+    /* Find the null terminator of the destination buffer. */
+    while (size && *ptr)
+        size--, ptr++;
+
+    if (!size)
+    {
+        *dst = '\0';
+        *MSVCRT__errno() = MSVCRT_EINVAL;
+        return MSVCRT_EINVAL;
+    }
+
+    /* If necessary, check that the character preceding the null terminator is
+     * a lead byte and move the pointer back by one for later overwrite. */
+    if (ptr != dst && get_locale()->locinfo->mb_cur_max > 1 && MSVCRT_isleadbyte(*(ptr - 1)))
+        size++, ptr--;
+
+    for (i = 0; *src && i < len; i++)
+    {
+        *ptr++ = *src++;
+        size--;
+
+        if (!size)
+        {
+            *dst = '\0';
+            *MSVCRT__errno() = MSVCRT_ERANGE;
+            return MSVCRT_ERANGE;
+        }
+    }
+
+    *ptr = '\0';
+    return 0;
+}
 
 /*********************************************************************
  *		_mbsncat(MSVCRT.@)
@@ -1775,8 +1824,7 @@ int CDECL MSVCRT__mbstowcs_s_l(MSVCRT_size_t *ret, MSVCRT_wchar_t *wcstr,
         return 0;
     }
 
-    if(!mbstr || !wcstr) {
-        MSVCRT__invalid_parameter(NULL, NULL, NULL, 0, 0);
+    if(!MSVCRT_CHECK_PMT(mbstr != NULL) || !MSVCRT_CHECK_PMT(wcstr != NULL)) {
         if(wcstr && size)
             wcstr[0] = '\0';
         *MSVCRT__errno() = MSVCRT_EINVAL;
@@ -1794,7 +1842,7 @@ int CDECL MSVCRT__mbstowcs_s_l(MSVCRT_size_t *ret, MSVCRT_wchar_t *wcstr,
     else if(conv==size && (count==MSVCRT__TRUNCATE || wcstr[conv-1]=='\0'))
         wcstr[conv-1] = '\0';
     else {
-        MSVCRT__invalid_parameter(NULL, NULL, NULL, 0, 0);
+        MSVCRT_INVALID_PMT("wcstr[size] is too small");
         if(size)
             wcstr[0] = '\0';
         *MSVCRT__errno() = MSVCRT_ERANGE;
