@@ -425,7 +425,11 @@ static void test_defines(void)
     CHECK_DEF(_FPE_STACKOVERFLOW);
     CHECK_DEF(_FPE_STACKUNDERFLOW);
     CHECK_DEF(_FPE_EXPLICITGEN);
-#ifdef __i386__
+    CHECK_DEF(_MCW_EM);
+    CHECK_DEF(_MCW_IC);
+    CHECK_DEF(_MCW_RC);
+    CHECK_DEF(_MCW_PC);
+    CHECK_DEF(_MCW_DN);
     CHECK_DEF(_EM_INVALID);
     CHECK_DEF(_EM_DENORMAL);
     CHECK_DEF(_EM_ZERODIVIDE);
@@ -441,7 +445,11 @@ static void test_defines(void)
     CHECK_DEF(_PC_24);
     CHECK_DEF(_PC_53);
     CHECK_DEF(_PC_64);
-#endif
+    CHECK_DEF(_DN_SAVE);
+    CHECK_DEF(_DN_FLUSH);
+    CHECK_DEF(_DN_FLUSH_OPERANDS_SAVE_RESULTS);
+    CHECK_DEF(_DN_SAVE_OPERANDS_FLUSH_RESULTS);
+    CHECK_DEF(_EM_AMBIGUOUS);
 }
 
 #endif /* __WINE_USE_MSVCRT */
