@@ -36,7 +36,7 @@
 #include "msxml.h"
 #include "xmldom.h"
 #include "xmldso.h"
-#include "msxml2.h"
+#include "msxml6.h"
 
 /* undef the #define in msxml2 so that we can access the v.2 version
    independent CLSID as well as the v.3 one. */
@@ -515,8 +515,23 @@ static struct regsvr_coclass const coclass_list[] = {
         "Both",
         "Microsoft.FreeThreadedXMLDOM.1.0",
         "1.0"
+    },
+    {   &CLSID_FreeThreadedDOMDocument26,
+        "Free Threaded XML DOM Document 2.6",
+        NULL,
+        "msxml3.dll",
+        "Both",
+        "Microsoft.FreeThreadedXMLDOM.1.0",
+        "1.0"
+     },
+    {   &CLSID_FreeThreadedDOMDocument30,
+        "Free Threaded XML DOM Document 3.0",
+        NULL,
+        "msxml3.dll",
+        "Both",
+        "Microsoft.FreeThreadedDOMDocument.1.0",
+        "1.0"
      },
-
     {   &CLSID_XMLHTTPRequest,
 	"XML HTTP Request",
 	NULL,
@@ -525,6 +540,22 @@ static struct regsvr_coclass const coclass_list[] = {
 	"Microsoft.XMLHTTP",
 	"1.0"
     },
+    {   &CLSID_XMLHTTP26,
+	"XML HTTP 2.6",
+	NULL,
+	"msxml3.dll",
+	"Apartment",
+	"Msxml2.XMLHTTP.2.6",
+	"2.6"
+    },
+    {   &CLSID_XMLHTTP30,
+	"XML HTTP 3.0",
+	NULL,
+	"msxml3.dll",
+	"Apartment",
+	"Msxml2.XMLHTTP.3.0",
+	"3.0"
+    },
     {   &CLSID_XMLDSOControl,
 	"XML Data Source Object",
 	NULL,
@@ -548,6 +579,14 @@ static struct regsvr_coclass const coclass_list[] = {
 	"Msxml2.XMLSchemaCache",
         "3.0"
     },
+    {   &CLSID_XMLSchemaCache26,
+    "XML Schema Cache 2.6",
+    NULL,
+    "msxml3.dll",
+    "Both",
+    "Msxml2.XMLSchemaCache",
+    "2.6"
+    },
     {   &CLSID_XMLSchemaCache30,
 	"XML Schema Cache 3.0",
 	NULL,
@@ -588,6 +627,22 @@ static struct regsvr_coclass const coclass_list[] = {
         "Msxml2.MXXMLWriter",
         "3.0"
     },
+    {   &CLSID_SAXAttributes,
+        "SAX Attribute",
+        NULL,
+        "msxml3.dll",
+        "Both",
+        "Msxml2.SAXAttributes",
+        NULL
+    },
+    {   &CLSID_SAXAttributes30,
+        "SAX Attribute 3.0",
+        NULL,
+        "msxml3.dll",
+        "Both",
+        "Msxml2.SAXAttributes",
+        "3.0"
+    },
     { NULL }			/* list terminator */
 };
 
@@ -642,6 +697,16 @@ static struct progid const progid_list[] = {
 	&CLSID_DOMFreeThreadedDocument,
 	"Microsoft.FreeThreadedXMLDOM.1.0"
     },
+    {   "MSXML.FreeThreadedDOMDocument26",
+    "Free threaded XML DOM Document 2.6",
+    &CLSID_FreeThreadedDOMDocument26,
+    NULL
+    },
+    {   "MSXML.FreeThreadedDOMDocument30",
+    "Free threaded XML DOM Document 3.0",
+    &CLSID_FreeThreadedDOMDocument30,
+    NULL
+    },
     {   "Microsoft.XMLHTTP",
 	"XML HTTP Request",
 	&CLSID_XMLHTTPRequest,
@@ -652,6 +717,16 @@ static struct progid const progid_list[] = {
 	&CLSID_XMLHTTPRequest,
 	NULL
     },
+    {   "Msxml2.XMLHTTP.2.6",
+	"XML HTTP 2.6",
+	&CLSID_XMLHTTP26,
+	NULL
+    },
+    {   "Msxml2.XMLHTTP.3.0",
+	"XML HTTP 3.0",
+	&CLSID_XMLHTTP30,
+	NULL
+    },
     {   "Microsoft.XMLDSO",
 	"XML Data Source Object",
 	&CLSID_XMLDSOControl,
@@ -672,6 +747,11 @@ static struct progid const progid_list[] = {
         &CLSID_XMLSchemaCache,
         "Msxml2.XMLSchemaCache.3.0"
     },
+    {   "Msxml2.XMLSchemaCache.2.6",
+        "XML Schema Cache 2.6",
+        &CLSID_XMLSchemaCache26,
+        "Msxml2.XMLSchemaCache.2.6"
+    },
     {   "Msxml2.XMLSchemaCache.3.0",
         "XML Schema Cache 3.0",
         &CLSID_XMLSchemaCache30,
@@ -697,7 +777,16 @@ static struct progid const progid_list[] = {
         &CLSID_MXXMLWriter30,
         NULL
     },
-
+    {   "Msxml2.SAXAttributes",
+        "SAX Attribute",
+        &CLSID_SAXAttributes,
+        NULL
+    },
+    {   "Msxml2.SAXAttributes.3.0",
+        "SAX Attribute 3.0",
+        &CLSID_SAXAttributes30,
+        NULL
+    },
     { NULL }			/* list terminator */
 };
 
