@@ -93,6 +93,8 @@ static const enum cpu_type client_cpu = CPU_ALPHA;
 static const enum cpu_type client_cpu = CPU_POWERPC;
 #elif defined(__sparc__)
 static const enum cpu_type client_cpu = CPU_SPARC;
+#elif defined(__arm__)
+static const enum cpu_type client_cpu = CPU_ARM;
 #else
 #error Unsupported CPU
 #endif
@@ -559,7 +561,7 @@ int server_get_unix_fd( HANDLE handle, unsigned int wanted_access, int *unix_fd,
             if ((fd = receive_fd( &fd_handle )) != -1)
             {
                 assert( wine_server_ptr_handle(fd_handle) == handle );
-                *needs_close = (reply->removable ||
+                *needs_close = (!reply->cacheable ||
                                 !add_fd_to_cache( handle, fd, reply->type,
                                                   reply->access, reply->options ));
             }
@@ -942,9 +944,9 @@ static int get_unix_tid(void)
 {
     int ret = -1;
 #if defined(linux) && defined(__i386__)
-    __asm__("int $0x80" : "=a" (ret) : "0" (224) /* SYS_gettid */);
+    ret = syscall(224 /*SYS_gettid*/);
 #elif defined(linux) && defined(__x86_64__)
-    __asm__("syscall" : "=a" (ret) : "0" (186) /* SYS_gettid */);
+    ret = syscall(186 /*SYS_gettid*/);
 #elif defined(__sun)
     ret = pthread_self();
 #elif defined(__APPLE__)
@@ -1119,7 +1121,7 @@ size_t server_init_thread( void *entry_point )
             if (!strcmp( arch, "win32" ) && (is_win64 || is_wow64))
                 fatal_error( "WINEARCH set to win32 but '%s' is a 64-bit installation.\n",
                              wine_get_config_dir() );
-            if (!strcmp( arch, "win64" ) && !is_win64 && !is_wow64)
+            if (!strcmp( arch, "win64" ) && !is_wow64)
                 fatal_error( "WINEARCH set to win64 but '%s' is a 32-bit installation.\n",
                              wine_get_config_dir() );
         }
