@@ -24,6 +24,7 @@
 #include <stdlib.h>
 
 #include "ntdll_test.h"
+#include "inaddr.h"
 
 #ifndef __WINE_WINTERNL_H
 
@@ -45,10 +46,25 @@ typedef struct _RTL_HANDLE_TABLE
 
 #endif
 
+/* avoid #include <winsock2.h> */
+#undef htons
+#ifdef WORDS_BIGENDIAN
+#define htons(s) ((USHORT)(s))
+#else  /* WORDS_BIGENDIAN */
+static inline USHORT __my_ushort_swap(USHORT s)
+{
+    return (s >> 8) | (s << 8);
+}
+#define htons(s) __my_ushort_swap(s)
+#endif  /* WORDS_BIGENDIAN */
+
+
+
 /* Function ptrs for ntdll calls */
 static HMODULE hntdll = 0;
 static SIZE_T    (WINAPI  *pRtlCompareMemory)(LPCVOID,LPCVOID,SIZE_T);
 static SIZE_T    (WINAPI  *pRtlCompareMemoryUlong)(PULONG, SIZE_T, ULONG);
+static NTSTATUS  (WINAPI  *pRtlDeleteTimer)(HANDLE, HANDLE, HANDLE);
 static VOID      (WINAPI  *pRtlMoveMemory)(LPVOID,LPCVOID,SIZE_T);
 static VOID      (WINAPI  *pRtlFillMemory)(LPVOID,SIZE_T,BYTE);
 static VOID      (WINAPI  *pRtlFillMemoryUlong)(LPVOID,SIZE_T,ULONG);
@@ -66,6 +82,17 @@ static RTL_HANDLE * (WINAPI * pRtlAllocateHandle)(RTL_HANDLE_TABLE *, ULONG *);
 static BOOLEAN   (WINAPI * pRtlFreeHandle)(RTL_HANDLE_TABLE *, RTL_HANDLE *);
 static NTSTATUS  (WINAPI *pRtlAllocateAndInitializeSid)(PSID_IDENTIFIER_AUTHORITY,BYTE,DWORD,DWORD,DWORD,DWORD,DWORD,DWORD,DWORD,DWORD,PSID*);
 static NTSTATUS  (WINAPI *pRtlFreeSid)(PSID);
+static struct _TEB * (WINAPI *pNtCurrentTeb)(void);
+static DWORD     (WINAPI *pRtlGetThreadErrorMode)(void);
+static NTSTATUS  (WINAPI *pRtlSetThreadErrorMode)(DWORD, LPDWORD);
+static IMAGE_BASE_RELOCATION *(WINAPI *pLdrProcessRelocationBlock)(void*,UINT,USHORT*,INT_PTR);
+static CHAR *    (WINAPI *pRtlIpv4AddressToStringA)(const IN_ADDR *, LPSTR);
+static NTSTATUS  (WINAPI *pRtlIpv4AddressToStringExA)(const IN_ADDR *, USHORT, LPSTR, PULONG);
+
+static HMODULE hkernel32 = 0;
+static BOOL      (WINAPI *pIsWow64Process)(HANDLE, PBOOL);
+
+
 #define LEN 16
 static const char* src_src = "This is a test!"; /* 16 bytes long, incl NUL */
 static ULONG src_aligned_block[4];
@@ -80,6 +107,7 @@ static void InitFunctionPtrs(void)
     if (hntdll) {
 	pRtlCompareMemory = (void *)GetProcAddress(hntdll, "RtlCompareMemory");
 	pRtlCompareMemoryUlong = (void *)GetProcAddress(hntdll, "RtlCompareMemoryUlong");
+        pRtlDeleteTimer = (void *)GetProcAddress(hntdll, "RtlDeleteTimer");
 	pRtlMoveMemory = (void *)GetProcAddress(hntdll, "RtlMoveMemory");
 	pRtlFillMemory = (void *)GetProcAddress(hntdll, "RtlFillMemory");
 	pRtlFillMemoryUlong = (void *)GetProcAddress(hntdll, "RtlFillMemoryUlong");
@@ -97,6 +125,17 @@ static void InitFunctionPtrs(void)
 	pRtlFreeHandle = (void *)GetProcAddress(hntdll, "RtlFreeHandle");
         pRtlAllocateAndInitializeSid = (void *)GetProcAddress(hntdll, "RtlAllocateAndInitializeSid");
         pRtlFreeSid = (void *)GetProcAddress(hntdll, "RtlFreeSid");
+        pNtCurrentTeb = (void *)GetProcAddress(hntdll, "NtCurrentTeb");
+        pRtlGetThreadErrorMode = (void *)GetProcAddress(hntdll, "RtlGetThreadErrorMode");
+        pRtlSetThreadErrorMode = (void *)GetProcAddress(hntdll, "RtlSetThreadErrorMode");
+        pLdrProcessRelocationBlock  = (void *)GetProcAddress(hntdll, "LdrProcessRelocationBlock");
+        pRtlIpv4AddressToStringA = (void *)GetProcAddress(hntdll, "RtlIpv4AddressToStringA");
+        pRtlIpv4AddressToStringExA = (void *)GetProcAddress(hntdll, "RtlIpv4AddressToStringExA");
+    }
+    hkernel32 = LoadLibraryA("kernel32.dll");
+    ok(hkernel32 != 0, "LoadLibrary failed\n");
+    if (hkernel32) {
+        pIsWow64Process = (void *)GetProcAddress(hkernel32, "IsWow64Process");
     }
     strcpy((char*)src_aligned_block, src_src);
     ok(strlen(src) == 15, "Source must be 16 bytes long!\n");
@@ -110,7 +149,10 @@ static void test_RtlCompareMemory(void)
   SIZE_T size;
 
   if (!pRtlCompareMemory)
+  {
+    win_skip("RtlCompareMemory is not available\n");
     return;
+  }
 
   strcpy(dest, src);
 
@@ -125,6 +167,12 @@ static void test_RtlCompareMemoryUlong(void)
     ULONG a[10];
     ULONG result;
 
+    if (!pRtlCompareMemoryUlong)
+    {
+        win_skip("RtlCompareMemoryUlong is not available\n");
+        return;
+    }
+
     a[0]= 0x0123;
     a[1]= 0x4567;
     a[2]= 0x89ab;
@@ -171,7 +219,10 @@ static void test_RtlCompareMemoryUlong(void)
 static void test_RtlMoveMemory(void)
 {
   if (!pRtlMoveMemory)
+  {
+    win_skip("RtlMoveMemory is not available\n");
     return;
+  }
 
   /* Length should be in bytes and not rounded. Use strcmp to ensure we
    * didn't write past the end (it checks for the final NUL left by memset)
@@ -199,7 +250,10 @@ static void test_RtlMoveMemory(void)
 static void test_RtlFillMemory(void)
 {
   if (!pRtlFillMemory)
+  {
+    win_skip("RtlFillMemory is not available\n");
     return;
+  }
 
   /* Length should be in bytes and not rounded. Use strcmp to ensure we
    * didn't write past the end (the remainder of the string should match)
@@ -222,7 +276,10 @@ static void test_RtlFillMemoryUlong(void)
 {
   ULONG val = ('x' << 24) | ('x' << 16) | ('x' << 8) | 'x';
   if (!pRtlFillMemoryUlong)
+  {
+    win_skip("RtlFillMemoryUlong is not available\n");
     return;
+  }
 
   /* Length should be in bytes and not rounded. Use strcmp to ensure we
    * didn't write past the end (the remainder of the string should match)
@@ -245,7 +302,10 @@ static void test_RtlFillMemoryUlong(void)
 static void test_RtlZeroMemory(void)
 {
   if (!pRtlZeroMemory)
+  {
+    win_skip("RtlZeroMemory is not available\n");
     return;
+  }
 
   /* Length should be in bytes and not rounded. */
   ZERO(0); MCMP("This is a test!");
@@ -264,6 +324,18 @@ static void test_RtlUlonglongByteSwap(void)
 {
     ULONGLONG result;
 
+    if ( !pRtlUlonglongByteSwap )
+    {
+        win_skip("RtlUlonglongByteSwap is not available\n");
+        return;
+    }
+
+    if ( pRtlUlonglongByteSwap( 0 ) != 0 )
+    {
+        win_skip("Broken RtlUlonglongByteSwap in win2k\n");
+        return;
+    }
+
     result = pRtlUlonglongByteSwap( ((ULONGLONG)0x76543210 << 32) | 0x87654321 );
     ok( (((ULONGLONG)0x21436587 << 32) | 0x10325476) == result,
        "RtlUlonglongByteSwap(0x7654321087654321) returns 0x%x%08x, expected 0x2143658710325476\n",
@@ -279,6 +351,12 @@ static void test_RtlUniform(void)
     ULONG expected;
     ULONG result;
 
+    if (!pRtlUniform)
+    {
+        win_skip("RtlUniform is not available\n");
+        return;
+    }
+
 /*
  * According to the documentation RtlUniform is using D.H. Lehmer's 1948
  * algorithm. This algorithm is:
@@ -563,7 +641,7 @@ static void test_RtlUniform(void)
 }
 
 
-static ULONG WINAPI my_RtlRandom(PULONG seed)
+static ULONG my_RtlRandom(PULONG seed)
 {
     static ULONG saved_value[128] =
     { /*   0 */ 0x4c8bc0aa, 0x4c022957, 0x2232827a, 0x2f1e7626, 0x7f8bdafb, 0x5c37d02a, 0x0ab48f72, 0x2f0c4ffa,
@@ -604,6 +682,12 @@ static void test_RtlRandom(void)
     ULONG result;
     ULONG result_expected;
 
+    if (!pRtlRandom)
+    {
+        win_skip("RtlRandom is not available\n");
+        return;
+    }
+
 /*
  * Unlike RtlUniform, RtlRandom is not documented. We guess that for
  * RtlRandom D.H. Lehmer's 1948 algorithm is used like stated in
@@ -812,6 +896,12 @@ static void test_RtlAreAllAccessesGranted(void)
     unsigned int test_num;
     BOOLEAN result;
 
+    if (!pRtlAreAllAccessesGranted)
+    {
+        win_skip("RtlAreAllAccessesGranted is not available\n");
+        return;
+    }
+
     for (test_num = 0; test_num < NB_ALL_ACCESSES; test_num++) {
 	result = pRtlAreAllAccessesGranted(all_accesses[test_num].GrantedAccess,
 					   all_accesses[test_num].DesiredAccess);
@@ -849,6 +939,12 @@ static void test_RtlAreAnyAccessesGranted(void)
     unsigned int test_num;
     BOOLEAN result;
 
+    if (!pRtlAreAnyAccessesGranted)
+    {
+        win_skip("RtlAreAnyAccessesGranted is not available\n");
+        return;
+    }
+
     for (test_num = 0; test_num < NB_ANY_ACCESSES; test_num++) {
 	result = pRtlAreAnyAccessesGranted(any_accesses[test_num].GrantedAccess,
 					   any_accesses[test_num].DesiredAccess);
@@ -865,7 +961,10 @@ static void test_RtlComputeCrc32(void)
   DWORD crc = 0;
 
   if (!pRtlComputeCrc32)
+  {
+    win_skip("RtlComputeCrc32 is not available\n");
     return;
+  }
 
   crc = pRtlComputeCrc32(crc, (const BYTE *)src, LEN);
   ok(crc == 0x40861dc2,"Expected 0x40861dc2, got %8x\n", crc);
@@ -892,6 +991,12 @@ static void test_HandleTables(void)
     MY_HANDLE * MyHandle;
     RTL_HANDLE_TABLE HandleTable;
 
+    if (!pRtlInitializeHandleTable)
+    {
+        win_skip("RtlInitializeHandleTable is not available\n");
+        return;
+    }
+
     pRtlInitializeHandleTable(0x3FFF, sizeof(MY_HANDLE), &HandleTable);
     MyHandle = (MY_HANDLE *)pRtlAllocateHandle(&HandleTable, &Index);
     ok(MyHandle != NULL, "RtlAllocateHandle failed\n");
@@ -911,49 +1016,324 @@ static void test_RtlAllocateAndInitializeSid(void)
     SID_IDENTIFIER_AUTHORITY sia = {{ 1, 2, 3, 4, 5, 6 }};
     PSID psid;
 
+    if (!pRtlAllocateAndInitializeSid)
+    {
+        win_skip("RtlAllocateAndInitializeSid is not available\n");
+        return;
+    }
+
     ret = pRtlAllocateAndInitializeSid(&sia, 0, 1, 2, 3, 4, 5, 6, 7, 8, &psid);
     ok(!ret, "RtlAllocateAndInitializeSid error %08x\n", ret);
     ret = pRtlFreeSid(psid);
     ok(!ret, "RtlFreeSid error %08x\n", ret);
 
-    /* these tests crash on XP
-    ret = pRtlAllocateAndInitializeSid(NULL, 0, 1, 2, 3, 4, 5, 6, 7, 8, &psid);
-    ret = pRtlAllocateAndInitializeSid(&sia, 0, 1, 2, 3, 4, 5, 6, 7, 8, NULL);*/
+    /* these tests crash on XP */
+    if (0)
+    {
+        ret = pRtlAllocateAndInitializeSid(NULL, 0, 1, 2, 3, 4, 5, 6, 7, 8, &psid);
+        ret = pRtlAllocateAndInitializeSid(&sia, 0, 1, 2, 3, 4, 5, 6, 7, 8, NULL);
+    }
 
     ret = pRtlAllocateAndInitializeSid(&sia, 9, 1, 2, 3, 4, 5, 6, 7, 8, &psid);
     ok(ret == STATUS_INVALID_SID, "wrong error %08x\n", ret);
 }
 
+static void test_RtlDeleteTimer(void)
+{
+    NTSTATUS ret;
+
+    if (!pRtlDeleteTimer)
+    {
+        win_skip("RtlDeleteTimer is not available\n");
+        return;
+    }
+
+    ret = pRtlDeleteTimer(NULL, NULL, NULL);
+    ok(ret == STATUS_INVALID_PARAMETER_1 ||
+       ret == STATUS_INVALID_PARAMETER, /* W2K */
+       "expected STATUS_INVALID_PARAMETER_1 or STATUS_INVALID_PARAMETER, got %x\n", ret);
+}
+
+static void test_RtlThreadErrorMode(void)
+{
+    DWORD oldmode;
+    BOOL is_wow64;
+    DWORD mode;
+    NTSTATUS status;
+
+    if (!pRtlGetThreadErrorMode || !pRtlSetThreadErrorMode)
+    {
+        win_skip("RtlGetThreadErrorMode and/or RtlSetThreadErrorMode not available\n");
+        return;
+    }
+
+    if (!pIsWow64Process || !pIsWow64Process(GetCurrentProcess(), &is_wow64))
+        is_wow64 = FALSE;
+
+    oldmode = pRtlGetThreadErrorMode();
+
+    status = pRtlSetThreadErrorMode(0x70, &mode);
+    ok(status == STATUS_SUCCESS ||
+       status == STATUS_WAIT_1, /* Vista */
+       "RtlSetThreadErrorMode failed with error 0x%08x\n", status);
+    ok(mode == oldmode,
+       "RtlSetThreadErrorMode returned mode 0x%x, expected 0x%x\n",
+       mode, oldmode);
+    ok(pRtlGetThreadErrorMode() == 0x70,
+       "RtlGetThreadErrorMode returned 0x%x, expected 0x%x\n", mode, 0x70);
+    if (!is_wow64 && pNtCurrentTeb)
+        ok(pNtCurrentTeb()->HardErrorDisabled == 0x70,
+           "The TEB contains 0x%x, expected 0x%x\n",
+           pNtCurrentTeb()->HardErrorDisabled, 0x70);
+
+    status = pRtlSetThreadErrorMode(0, &mode);
+    ok(status == STATUS_SUCCESS ||
+       status == STATUS_WAIT_1, /* Vista */
+       "RtlSetThreadErrorMode failed with error 0x%08x\n", status);
+    ok(mode == 0x70,
+       "RtlSetThreadErrorMode returned mode 0x%x, expected 0x%x\n",
+       mode, 0x70);
+    ok(pRtlGetThreadErrorMode() == 0,
+       "RtlGetThreadErrorMode returned 0x%x, expected 0x%x\n", mode, 0);
+    if (!is_wow64 && pNtCurrentTeb)
+        ok(pNtCurrentTeb()->HardErrorDisabled == 0,
+           "The TEB contains 0x%x, expected 0x%x\n",
+           pNtCurrentTeb()->HardErrorDisabled, 0);
+
+    for (mode = 1; mode; mode <<= 1)
+    {
+        status = pRtlSetThreadErrorMode(mode, NULL);
+        if (mode & 0x70)
+            ok(status == STATUS_SUCCESS ||
+               status == STATUS_WAIT_1, /* Vista */
+               "RtlSetThreadErrorMode(%x,NULL) failed with error 0x%08x\n",
+               mode, status);
+        else
+            ok(status == STATUS_INVALID_PARAMETER_1,
+               "RtlSetThreadErrorMode(%x,NULL) returns 0x%08x, "
+               "expected STATUS_INVALID_PARAMETER_1\n",
+               mode, status);
+    }
+
+    pRtlSetThreadErrorMode(oldmode, NULL);
+}
+
+static void test_LdrProcessRelocationBlock(void)
+{
+    IMAGE_BASE_RELOCATION *ret;
+    USHORT reloc;
+    DWORD addr32;
+    SHORT addr16;
+
+    if(!pLdrProcessRelocationBlock) {
+        win_skip("LdrProcessRelocationBlock not available\n");
+        return;
+    }
+
+    addr32 = 0x50005;
+    reloc = IMAGE_REL_BASED_HIGHLOW<<12;
+    ret = pLdrProcessRelocationBlock(&addr32, 1, &reloc, 0x500050);
+    ok((USHORT*)ret == &reloc+1, "ret = %p, expected %p\n", ret, &reloc+1);
+    ok(addr32 == 0x550055, "addr32 = %x, expected 0x550055\n", addr32);
+
+    addr16 = 0x505;
+    reloc = IMAGE_REL_BASED_HIGH<<12;
+    ret = pLdrProcessRelocationBlock(&addr16, 1, &reloc, 0x500060);
+    ok((USHORT*)ret == &reloc+1, "ret = %p, expected %p\n", ret, &reloc+1);
+    ok(addr16 == 0x555, "addr16 = %x, expected 0x555\n", addr16);
+
+    addr16 = 0x505;
+    reloc = IMAGE_REL_BASED_LOW<<12;
+    ret = pLdrProcessRelocationBlock(&addr16, 1, &reloc, 0x500060);
+    ok((USHORT*)ret == &reloc+1, "ret = %p, expected %p\n", ret, &reloc+1);
+    ok(addr16 == 0x565, "addr16 = %x, expected 0x565\n", addr16);
+}
+
+static void test_RtlIpv4AddressToString(void)
+{
+    CHAR buffer[20];
+    CHAR *res;
+    IN_ADDR ip;
+    DWORD len;
+
+    if (!pRtlIpv4AddressToStringA)
+    {
+        win_skip("RtlIpv4AddressToStringA not available\n");
+        return;
+    }
+
+    ip.S_un.S_un_b.s_b1 = 1;
+    ip.S_un.S_un_b.s_b2 = 2;
+    ip.S_un.S_un_b.s_b3 = 3;
+    ip.S_un.S_un_b.s_b4 = 4;
+
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringA(&ip, buffer);
+    len = strlen(buffer);
+    ok(res == (buffer + len), "got %p with '%s' (expected %p)\n", res, buffer, buffer + len);
+
+    res = pRtlIpv4AddressToStringA(&ip, NULL);
+    ok( (res == (char *)~0) ||
+        broken(res == (char *)0 + len),        /* XP and w2003 */
+        "got %p (expected ~0)\n", res);
+
+    if (0) {
+        /* this crashes in windows */
+        memset(buffer, '#', sizeof(buffer) - 1);
+        buffer[sizeof(buffer) -1] = 0;
+        res = pRtlIpv4AddressToStringA(NULL, buffer);
+        trace("got %p with '%s'\n", res, buffer);
+    }
+
+    if (0) {
+        /* this crashes in windows */
+        res = pRtlIpv4AddressToStringA(NULL, NULL);
+        trace("got %p\n", res);
+    }
+}
+
+static void test_RtlIpv4AddressToStringEx(void)
+{
+    CHAR ip_1234[] = "1.2.3.4";
+    CHAR ip_1234_80[] = "1.2.3.4:80";
+    LPSTR expect;
+    CHAR buffer[30];
+    NTSTATUS res;
+    IN_ADDR ip;
+    ULONG size;
+    DWORD used;
+    USHORT port;
+
+    if (!pRtlIpv4AddressToStringExA)
+    {
+        win_skip("RtlIpv4AddressToStringExA not available\n");
+        return;
+    }
+
+    ip.S_un.S_un_b.s_b1 = 1;
+    ip.S_un.S_un_b.s_b2 = 2;
+    ip.S_un.S_un_b.s_b3 = 3;
+    ip.S_un.S_un_b.s_b4 = 4;
+
+    port = htons(80);
+    expect = ip_1234_80;
+
+    size = sizeof(buffer);
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(&ip, port, buffer, &size);
+    used = strlen(buffer);
+    ok( (res == STATUS_SUCCESS) &&
+        (size == strlen(expect) + 1) && !strcmp(buffer, expect),
+        "got 0x%x and size %d with '%s'\n", res, size, buffer);
+
+    size = used + 1;
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(&ip, port, buffer, &size);
+    ok( (res == STATUS_SUCCESS) &&
+        (size == strlen(expect) + 1) && !strcmp(buffer, expect),
+        "got 0x%x and size %d with '%s'\n", res, size, buffer);
+
+    size = used;
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(&ip, port, buffer, &size);
+    ok( (res == STATUS_INVALID_PARAMETER) && (size == used + 1),
+        "got 0x%x and %d with '%s' (expected STATUS_INVALID_PARAMETER and %d)\n",
+        res, size, buffer, used + 1);
+
+    size = used - 1;
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(&ip, port, buffer, &size);
+    ok( (res == STATUS_INVALID_PARAMETER) && (size == used + 1),
+        "got 0x%x and %d with '%s' (expected STATUS_INVALID_PARAMETER and %d)\n",
+        res, size, buffer, used + 1);
+
+
+    /* to get only the ip, use 0 as port */
+    port = 0;
+    expect = ip_1234;
+
+    size = sizeof(buffer);
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(&ip, port, buffer, &size);
+    used = strlen(buffer);
+    ok( (res == STATUS_SUCCESS) &&
+        (size == strlen(expect) + 1) && !strcmp(buffer, expect),
+        "got 0x%x and size %d with '%s'\n", res, size, buffer);
+
+    size = used + 1;
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(&ip, port, buffer, &size);
+    ok( (res == STATUS_SUCCESS) &&
+        (size == strlen(expect) + 1) && !strcmp(buffer, expect),
+        "got 0x%x and size %d with '%s'\n", res, size, buffer);
+
+    size = used;
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(&ip, port, buffer, &size);
+    ok( (res == STATUS_INVALID_PARAMETER) && (size == used + 1),
+        "got 0x%x and %d with '%s' (expected STATUS_INVALID_PARAMETER and %d)\n",
+        res, size, buffer, used + 1);
+
+    size = used - 1;
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(&ip, port, buffer, &size);
+    ok( (res == STATUS_INVALID_PARAMETER) && (size == used + 1),
+        "got 0x%x and %d with '%s' (expected STATUS_INVALID_PARAMETER and %d)\n",
+        res, size, buffer, used + 1);
+
+
+    /* parameters are checked */
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(&ip, 0, buffer, NULL);
+    ok(res == STATUS_INVALID_PARAMETER,
+        "got 0x%x with '%s' (expected STATUS_INVALID_PARAMETER)\n", res, buffer);
+
+    size = sizeof(buffer);
+    res = pRtlIpv4AddressToStringExA(&ip, 0, NULL, &size);
+    ok( res == STATUS_INVALID_PARAMETER,
+        "got 0x%x and size %d (expected STATUS_INVALID_PARAMETER)\n", res, size);
+
+    size = sizeof(buffer);
+    memset(buffer, '#', sizeof(buffer) - 1);
+    buffer[sizeof(buffer) -1] = 0;
+    res = pRtlIpv4AddressToStringExA(NULL, 0, buffer, &size);
+    ok( res == STATUS_INVALID_PARAMETER,
+        "got 0x%x and size %d with '%s' (expected STATUS_INVALID_PARAMETER)\n",
+        res, size, buffer);
+}
+
+
 START_TEST(rtl)
 {
     InitFunctionPtrs();
 
-    if (pRtlCompareMemory)
-        test_RtlCompareMemory();
-    if (pRtlCompareMemoryUlong)
-        test_RtlCompareMemoryUlong();
-    if (pRtlMoveMemory)
-        test_RtlMoveMemory();
-    if (pRtlFillMemory)
-        test_RtlFillMemory();
-    if (pRtlFillMemoryUlong)
-        test_RtlFillMemoryUlong();
-    if (pRtlZeroMemory)
-        test_RtlZeroMemory();
-    if (pRtlUlonglongByteSwap)
-        test_RtlUlonglongByteSwap();
-    if (pRtlUniform)
-        test_RtlUniform();
-    if (pRtlRandom)
-        test_RtlRandom();
-    if (pRtlAreAllAccessesGranted)
-        test_RtlAreAllAccessesGranted();
-    if (pRtlAreAnyAccessesGranted)
-        test_RtlAreAnyAccessesGranted();
-    if (pRtlComputeCrc32)
-        test_RtlComputeCrc32();
-    if (pRtlInitializeHandleTable)
-        test_HandleTables();
-    if (pRtlAllocateAndInitializeSid)
-        test_RtlAllocateAndInitializeSid();
+    test_RtlCompareMemory();
+    test_RtlCompareMemoryUlong();
+    test_RtlMoveMemory();
+    test_RtlFillMemory();
+    test_RtlFillMemoryUlong();
+    test_RtlZeroMemory();
+    test_RtlUlonglongByteSwap();
+    test_RtlUniform();
+    test_RtlRandom();
+    test_RtlAreAllAccessesGranted();
+    test_RtlAreAnyAccessesGranted();
+    test_RtlComputeCrc32();
+    test_HandleTables();
+    test_RtlAllocateAndInitializeSid();
+    test_RtlDeleteTimer();
+    test_RtlThreadErrorMode();
+    test_LdrProcessRelocationBlock();
+    test_RtlIpv4AddressToString();
+    test_RtlIpv4AddressToStringEx();
 }
