@@ -156,10 +156,15 @@ static IClassFactory Test_ClassFactory = { &TestClassFactory_Vtbl };
 
 typedef struct
 {
-    const IUnknownVtbl *lpVtbl;
+    IUnknown IUnknown_iface;
     ULONG refs;
 } HeapUnknown;
 
+static inline HeapUnknown *impl_from_IUnknown(IUnknown *iface)
+{
+    return CONTAINING_RECORD(iface, HeapUnknown, IUnknown_iface);
+}
+
 static HRESULT WINAPI HeapUnknown_QueryInterface(IUnknown *iface, REFIID riid, void **ppv)
 {
     if (IsEqualIID(riid, &IID_IUnknown))
@@ -174,13 +179,13 @@ static HRESULT WINAPI HeapUnknown_QueryInterface(IUnknown *iface, REFIID riid, v
 
 static ULONG WINAPI HeapUnknown_AddRef(IUnknown *iface)
 {
-    HeapUnknown *This = (HeapUnknown *)iface;
+    HeapUnknown *This = impl_from_IUnknown(iface);
     return InterlockedIncrement((LONG*)&This->refs);
 }
 
 static ULONG WINAPI HeapUnknown_Release(IUnknown *iface)
 {
-    HeapUnknown *This = (HeapUnknown *)iface;
+    HeapUnknown *This = impl_from_IUnknown(iface);
     ULONG refs = InterlockedDecrement((LONG*)&This->refs);
     if (!refs) HeapFree(GetProcessHeap(), 0, This);
     return refs;
@@ -1880,9 +1885,9 @@ static void test_bind_context(void)
     ok(hr == E_INVALIDARG, "IBindCtx_RegisterObjectParam should have returned E_INVALIDARG instead of 0x%08x\n", hr);
 
     unknown = HeapAlloc(GetProcessHeap(), 0, sizeof(*unknown));
-    unknown->lpVtbl = &HeapUnknown_Vtbl;
+    unknown->IUnknown_iface.lpVtbl = &HeapUnknown_Vtbl;
     unknown->refs = 1;
-    hr = IBindCtx_RegisterObjectParam(pBindCtx, (WCHAR *)wszParamName, (IUnknown *)&unknown->lpVtbl);
+    hr = IBindCtx_RegisterObjectParam(pBindCtx, (WCHAR *)wszParamName, &unknown->IUnknown_iface);
     ok_ole_success(hr, "IBindCtx_RegisterObjectParam");
 
     hr = IBindCtx_GetObjectParam(pBindCtx, (WCHAR *)wszParamName, &param_obj);
@@ -1907,23 +1912,23 @@ static void test_bind_context(void)
     ok(hr == E_INVALIDARG, "IBindCtx_RevokeObjectBound(NULL) should have return E_INVALIDARG instead of 0x%08x\n", hr);
 
     unknown2 = HeapAlloc(GetProcessHeap(), 0, sizeof(*unknown));
-    unknown2->lpVtbl = &HeapUnknown_Vtbl;
+    unknown2->IUnknown_iface.lpVtbl = &HeapUnknown_Vtbl;
     unknown2->refs = 1;
-    hr = IBindCtx_RegisterObjectBound(pBindCtx, (IUnknown *)&unknown2->lpVtbl);
+    hr = IBindCtx_RegisterObjectBound(pBindCtx, &unknown2->IUnknown_iface);
     ok_ole_success(hr, "IBindCtx_RegisterObjectBound");
 
-    hr = IBindCtx_RevokeObjectBound(pBindCtx, (IUnknown *)&unknown2->lpVtbl);
+    hr = IBindCtx_RevokeObjectBound(pBindCtx, &unknown2->IUnknown_iface);
     ok_ole_success(hr, "IBindCtx_RevokeObjectBound");
 
-    hr = IBindCtx_RevokeObjectBound(pBindCtx, (IUnknown *)&unknown2->lpVtbl);
+    hr = IBindCtx_RevokeObjectBound(pBindCtx, &unknown2->IUnknown_iface);
     ok(hr == MK_E_NOTBOUND, "IBindCtx_RevokeObjectBound with not bound object should have returned MK_E_NOTBOUND instead of 0x%08x\n", hr);
 
     IBindCtx_Release(pBindCtx);
 
-    refs = IUnknown_Release((IUnknown *)&unknown->lpVtbl);
+    refs = IUnknown_Release(&unknown->IUnknown_iface);
     ok(!refs, "object param should have been destroyed, instead of having %d refs\n", refs);
 
-    refs = IUnknown_Release((IUnknown *)&unknown2->lpVtbl);
+    refs = IUnknown_Release(&unknown2->IUnknown_iface);
     ok(!refs, "bound object should have been destroyed, instead of having %d refs\n", refs);
 }
 
@@ -1998,6 +2003,11 @@ static void test_save_load_filemoniker(void)
 
 START_TEST(moniker)
 {
+    if (!GetProcAddress(GetModuleHandleA("ole32.dll"), "CoRegisterSurrogateEx")) {
+        win_skip("skipping test on win9x\n");
+        return;
+    }
+
     CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
 
     test_ROT();
