@@ -480,15 +480,6 @@ static HRESULT WINAPI OLEFontImpl_QueryInterface(
   OLEFontImpl *this = (OLEFontImpl *)iface;
   TRACE("(%p)->(%s, %p)\n", this, debugstr_guid(riid), ppvObject);
 
-  /*
-   * Perform a sanity check on the parameters.
-   */
-  if ( (this==0) || (ppvObject==0) )
-    return E_INVALIDARG;
-
-  /*
-   * Initialize the return parameter.
-   */
   *ppvObject = 0;
 
   /*
@@ -1143,7 +1134,7 @@ static HRESULT WINAPI OLEFontImpl_IsEqual(
   INT ret;
   INT left_len,right_len;
 
-  if((iface == NULL) || (pFontOther == NULL))
+  if(pFontOther == NULL)
     return E_POINTER;
   else if (left->description.cySize.s.Lo != right->description.cySize.s.Lo)
     return S_FALSE;
@@ -2140,111 +2131,106 @@ static HRESULT WINAPI OLEFontImpl_IPersistPropertyBag_Load(
     static const WCHAR sAttrUnderline[] = {'U','n','d','e','r','l','i','n','e',0};
     static const WCHAR sAttrItalic[] = {'I','t','a','l','i','c',0};
     static const WCHAR sAttrStrikethrough[] = {'S','t','r','i','k','e','t','h','r','o','u','g','h',0};
-    VARIANT rawAttr;
-    VARIANT valueAttr;
-    HRESULT iRes = S_OK;
     OLEFontImpl *this = impl_from_IPersistPropertyBag(iface);
+    VARIANT value;
+    HRESULT iRes;
 
-    VariantInit(&rawAttr);
-    VariantInit(&valueAttr);
+    VariantInit(&value);
+
+    iRes = IPropertyBag_Read(pPropBag, sAttrName, &value, pErrorLog);
+    if (iRes == S_OK)
+    {
+        iRes = VariantChangeType(&value, &value, 0, VT_BSTR);
+        if (iRes == S_OK)
+            iRes = IFont_put_Name((IFont *)this, V_BSTR(&value));
+    }
+    else if (iRes == E_INVALIDARG)
+        iRes = S_OK;
+
+    VariantClear(&value);
 
     if (iRes == S_OK) {
-        iRes = IPropertyBag_Read(pPropBag, sAttrName, &rawAttr, pErrorLog);
+        iRes = IPropertyBag_Read(pPropBag, sAttrSize, &value, pErrorLog);
         if (iRes == S_OK)
         {
-            iRes = VariantChangeType(&rawAttr, &valueAttr, 0, VT_BSTR);
+            iRes = VariantChangeType(&value, &value, 0, VT_CY);
             if (iRes == S_OK)
-                iRes = IFont_put_Name((IFont *)this, V_BSTR(&valueAttr));
+                iRes = IFont_put_Size((IFont *)this, V_CY(&value));
         }
         else if (iRes == E_INVALIDARG)
             iRes = S_OK;
-        VariantClear(&rawAttr);
-        VariantClear(&valueAttr);
+
+        VariantClear(&value);
     }
 
     if (iRes == S_OK) {
-        iRes = IPropertyBag_Read(pPropBag, sAttrSize, &rawAttr, pErrorLog);
+        iRes = IPropertyBag_Read(pPropBag, sAttrCharset, &value, pErrorLog);
         if (iRes == S_OK)
         {
-            iRes = VariantChangeType(&rawAttr, &valueAttr, 0, VT_CY);
+            iRes = VariantChangeType(&value, &value, 0, VT_I2);
             if (iRes == S_OK)
-                iRes = IFont_put_Size((IFont *)this, V_CY(&valueAttr));
+                iRes = IFont_put_Charset((IFont *)this, V_I2(&value));
         }
         else if (iRes == E_INVALIDARG)
             iRes = S_OK;
-        VariantClear(&rawAttr);
-        VariantClear(&valueAttr);
+
+        VariantClear(&value);
     }
 
     if (iRes == S_OK) {
-        iRes = IPropertyBag_Read(pPropBag, sAttrCharset, &rawAttr, pErrorLog);
+        iRes = IPropertyBag_Read(pPropBag, sAttrWeight, &value, pErrorLog);
         if (iRes == S_OK)
         {
-            iRes = VariantChangeType(&rawAttr, &valueAttr, 0, VT_I2);
+            iRes = VariantChangeType(&value, &value, 0, VT_I2);
             if (iRes == S_OK)
-                iRes = IFont_put_Charset((IFont *)this, V_I2(&valueAttr));
+                iRes = IFont_put_Weight((IFont *)this, V_I2(&value));
         }
         else if (iRes == E_INVALIDARG)
             iRes = S_OK;
-        VariantClear(&rawAttr);
-        VariantClear(&valueAttr);
+
+        VariantClear(&value);
     }
 
     if (iRes == S_OK) {
-        iRes = IPropertyBag_Read(pPropBag, sAttrWeight, &rawAttr, pErrorLog);
+        iRes = IPropertyBag_Read(pPropBag, sAttrUnderline, &value, pErrorLog);
         if (iRes == S_OK)
         {
-            iRes = VariantChangeType(&rawAttr, &valueAttr, 0, VT_I2);
+            iRes = VariantChangeType(&value, &value, 0, VT_BOOL);
             if (iRes == S_OK)
-                iRes = IFont_put_Weight((IFont *)this, V_I2(&valueAttr));
+                iRes = IFont_put_Underline((IFont *)this, V_BOOL(&value));
         }
         else if (iRes == E_INVALIDARG)
             iRes = S_OK;
-        VariantClear(&rawAttr);
-        VariantClear(&valueAttr);
 
+        VariantClear(&value);
     }
 
     if (iRes == S_OK) {
-        iRes = IPropertyBag_Read(pPropBag, sAttrUnderline, &rawAttr, pErrorLog);
+        iRes = IPropertyBag_Read(pPropBag, sAttrItalic, &value, pErrorLog);
         if (iRes == S_OK)
         {
-            iRes = VariantChangeType(&rawAttr, &valueAttr, 0, VT_BOOL);
+            iRes = VariantChangeType(&value, &value, 0, VT_BOOL);
             if (iRes == S_OK)
-                iRes = IFont_put_Underline((IFont *)this, V_BOOL(&valueAttr));
+                iRes = IFont_put_Italic((IFont *)this, V_BOOL(&value));
         }
         else if (iRes == E_INVALIDARG)
             iRes = S_OK;
-        VariantClear(&rawAttr);
-        VariantClear(&valueAttr);
+
+        VariantClear(&value);
     }
 
     if (iRes == S_OK) {
-        iRes = IPropertyBag_Read(pPropBag, sAttrItalic, &rawAttr, pErrorLog);
+        iRes = IPropertyBag_Read(pPropBag, sAttrStrikethrough, &value, pErrorLog);
         if (iRes == S_OK)
         {
-            iRes = VariantChangeType(&rawAttr, &valueAttr, 0, VT_BOOL);
+            iRes = VariantChangeType(&value, &value, 0, VT_BOOL);
             if (iRes == S_OK)
-                iRes = IFont_put_Italic((IFont *)this, V_BOOL(&valueAttr));
+                IFont_put_Strikethrough((IFont *)this, V_BOOL(&value));
         }
         else if (iRes == E_INVALIDARG)
             iRes = S_OK;
-        VariantClear(&rawAttr);
-        VariantClear(&valueAttr);
-    }
 
-    if (iRes == S_OK) {
-        iRes = IPropertyBag_Read(pPropBag, sAttrStrikethrough, &rawAttr, pErrorLog);
-        if (iRes == S_OK)
-        {
-            iRes = VariantChangeType(&rawAttr, &valueAttr, 0, VT_BOOL);
-            if (iRes == S_OK)
-                IFont_put_Strikethrough((IFont *)this, V_BOOL(&valueAttr));
-        }
-        else if (iRes == E_INVALIDARG)
-            iRes = S_OK;
-        VariantClear(&rawAttr);
-        VariantClear(&valueAttr);
+        VariantClear(&value);
     }
 
     if (FAILED(iRes))
@@ -2458,13 +2444,18 @@ static void OLEFontImpl_Destroy(OLEFontImpl* fontDesc)
 typedef struct
 {
     /* IUnknown fields */
-    const IClassFactoryVtbl    *lpVtbl;
-    LONG                        ref;
+    IClassFactory IClassFactory_iface;
+    LONG          ref;
 } IClassFactoryImpl;
 
+static inline IClassFactoryImpl *impl_from_IClassFactory(IClassFactory *iface)
+{
+        return CONTAINING_RECORD(iface, IClassFactoryImpl, IClassFactory_iface);
+}
+
 static HRESULT WINAPI
 SFCF_QueryInterface(LPCLASSFACTORY iface,REFIID riid,LPVOID *ppobj) {
-	IClassFactoryImpl *This = (IClassFactoryImpl *)iface;
+	IClassFactoryImpl *This = impl_from_IClassFactory(iface);
 
 	FIXME("(%p)->(%s,%p),stub!\n",This,debugstr_guid(riid),ppobj);
 	return E_NOINTERFACE;
@@ -2472,12 +2463,12 @@ SFCF_QueryInterface(LPCLASSFACTORY iface,REFIID riid,LPVOID *ppobj) {
 
 static ULONG WINAPI
 SFCF_AddRef(LPCLASSFACTORY iface) {
-	IClassFactoryImpl *This = (IClassFactoryImpl *)iface;
+	IClassFactoryImpl *This = impl_from_IClassFactory(iface);
 	return InterlockedIncrement(&This->ref);
 }
 
 static ULONG WINAPI SFCF_Release(LPCLASSFACTORY iface) {
-	IClassFactoryImpl *This = (IClassFactoryImpl *)iface;
+	IClassFactoryImpl *This = impl_from_IClassFactory(iface);
 	/* static class, won't be  freed */
 	return InterlockedDecrement(&This->ref);
 }
@@ -2490,7 +2481,7 @@ static HRESULT WINAPI SFCF_CreateInstance(
 }
 
 static HRESULT WINAPI SFCF_LockServer(LPCLASSFACTORY iface,BOOL dolock) {
-	IClassFactoryImpl *This = (IClassFactoryImpl *)iface;
+	IClassFactoryImpl *This = impl_from_IClassFactory(iface);
 	FIXME("(%p)->(%d),stub!\n",This,dolock);
 	return S_OK;
 }
@@ -2502,6 +2493,6 @@ static const IClassFactoryVtbl SFCF_Vtbl = {
 	SFCF_CreateInstance,
 	SFCF_LockServer
 };
-static IClassFactoryImpl STDFONT_CF = {&SFCF_Vtbl, 1 };
+static IClassFactoryImpl STDFONT_CF = {{&SFCF_Vtbl}, 1 };
 
 void _get_STDFONT_CF(LPVOID *ppv) { *ppv = &STDFONT_CF; }
