@@ -33,16 +33,21 @@
 WINE_DEFAULT_DEBUG_CHANNEL(qedit);
 
 typedef struct MediaDetImpl {
-    const IMediaDetVtbl *MediaDet_Vtbl;
+    IMediaDet IMediaDet_iface;
     LONG refCount;
     IGraphBuilder *graph;
     IBaseFilter *source;
     IBaseFilter *splitter;
-    long num_streams;
-    long cur_stream;
+    LONG num_streams;
+    LONG cur_stream;
     IPin *cur_pin;
 } MediaDetImpl;
 
+static inline MediaDetImpl *impl_from_IMediaDet(IMediaDet *iface)
+{
+    return CONTAINING_RECORD(iface, MediaDetImpl, IMediaDet_iface);
+}
+
 static void MD_cleanup(MediaDetImpl *This)
 {
     if (This->cur_pin) IPin_Release(This->cur_pin);
@@ -59,7 +64,7 @@ static void MD_cleanup(MediaDetImpl *This)
 
 static ULONG WINAPI MediaDet_AddRef(IMediaDet* iface)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     ULONG refCount = InterlockedIncrement(&This->refCount);
     TRACE("(%p)->() AddRef from %d\n", This, refCount - 1);
     return refCount;
@@ -67,7 +72,7 @@ static ULONG WINAPI MediaDet_AddRef(IMediaDet* iface)
 
 static ULONG WINAPI MediaDet_Release(IMediaDet* iface)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     ULONG refCount = InterlockedDecrement(&This->refCount);
     TRACE("(%p)->() Release from %d\n", This, refCount + 1);
 
@@ -84,7 +89,7 @@ static ULONG WINAPI MediaDet_Release(IMediaDet* iface)
 static HRESULT WINAPI MediaDet_QueryInterface(IMediaDet* iface, REFIID riid,
                                               void **ppvObject)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     TRACE("(%p)->(%s %p)\n", This, debugstr_guid(riid), ppvObject);
 
     if (IsEqualIID(riid, &IID_IUnknown) ||
@@ -100,21 +105,21 @@ static HRESULT WINAPI MediaDet_QueryInterface(IMediaDet* iface, REFIID riid,
 
 static HRESULT WINAPI MediaDet_get_Filter(IMediaDet* iface, IUnknown **pVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     FIXME("(%p)->(%p): not implemented!\n", This, pVal);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI MediaDet_put_Filter(IMediaDet* iface, IUnknown *newVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     FIXME("(%p)->(%p): not implemented!\n", This, newVal);
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI MediaDet_get_OutputStreams(IMediaDet* iface, long *pVal)
+static HRESULT WINAPI MediaDet_get_OutputStreams(IMediaDet* iface, LONG *pVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     IEnumPins *pins;
     IPin *pin;
     HRESULT hr;
@@ -156,9 +161,9 @@ static HRESULT WINAPI MediaDet_get_OutputStreams(IMediaDet* iface, long *pVal)
     return S_OK;
 }
 
-static HRESULT WINAPI MediaDet_get_CurrentStream(IMediaDet* iface, long *pVal)
+static HRESULT WINAPI MediaDet_get_CurrentStream(IMediaDet* iface, LONG *pVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     TRACE("(%p)\n", This);
 
     if (!pVal)
@@ -168,7 +173,7 @@ static HRESULT WINAPI MediaDet_get_CurrentStream(IMediaDet* iface, long *pVal)
     return S_OK;
 }
 
-static HRESULT SetCurPin(MediaDetImpl *This, long strm)
+static HRESULT SetCurPin(MediaDetImpl *This, LONG strm)
 {
     IEnumPins *pins;
     IPin *pin;
@@ -209,16 +214,16 @@ static HRESULT SetCurPin(MediaDetImpl *This, long strm)
     return S_OK;
 }
 
-static HRESULT WINAPI MediaDet_put_CurrentStream(IMediaDet* iface, long newVal)
+static HRESULT WINAPI MediaDet_put_CurrentStream(IMediaDet* iface, LONG newVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     HRESULT hr;
 
-    TRACE("(%p)->(%ld)\n", This, newVal);
+    TRACE("(%p)->(%d)\n", This, newVal);
 
     if (This->num_streams == -1)
     {
-        long n;
+        LONG n;
         hr = MediaDet_get_OutputStreams(iface, &n);
         if (FAILED(hr))
             return hr;
@@ -237,28 +242,28 @@ static HRESULT WINAPI MediaDet_put_CurrentStream(IMediaDet* iface, long newVal)
 
 static HRESULT WINAPI MediaDet_get_StreamType(IMediaDet* iface, GUID *pVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     FIXME("(%p)->(%p): not implemented!\n", This, debugstr_guid(pVal));
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI MediaDet_get_StreamTypeB(IMediaDet* iface, BSTR *pVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     FIXME("(%p)->(%p): not implemented!\n", This, pVal);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI MediaDet_get_StreamLength(IMediaDet* iface, double *pVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     FIXME("(%p): stub!\n", This);
     return VFW_E_INVALIDMEDIATYPE;
 }
 
 static HRESULT WINAPI MediaDet_get_Filename(IMediaDet* iface, BSTR *pVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     IFileSourceFilter *file;
     LPOLESTR name;
     HRESULT hr;
@@ -310,7 +315,11 @@ static HRESULT GetFilterInfo(IMoniker *pMoniker, GUID *pclsid, VARIANT *pvar)
         hr = IPropertyBag_Read(pPropBagCat, wszClsidName, pvar, NULL);
 
     if (SUCCEEDED(hr))
+    {
         hr = CLSIDFromString(V_UNION(pvar, bstrVal), pclsid);
+        VariantClear(pvar);
+        V_VT(pvar) = VT_BSTR;
+    }
 
     if (SUCCEEDED(hr))
         hr = IPropertyBag_Read(pPropBagCat, wszFriendlyName, pvar, NULL);
@@ -374,48 +383,56 @@ static HRESULT GetSplitter(MediaDetImpl *This)
     if (FAILED(hr))
         return hr;
 
-    hr = IEnumMoniker_Next(filters, 1, &mon, NULL);
-    IEnumMoniker_Release(filters);
-    if (hr != S_OK)    /* No matches, what do we do?  */
-        return E_NOINTERFACE;
-
-    hr = GetFilterInfo(mon, &clsid, &var);
-    IMoniker_Release(mon);
-    if (FAILED(hr))
-        return hr;
-
-    hr = CoCreateInstance(&clsid, NULL, CLSCTX_INPROC_SERVER,
-                          &IID_IBaseFilter, (void **) &splitter);
-    if (FAILED(hr))
-        return hr;
-
-    hr = IGraphBuilder_AddFilter(This->graph, splitter,
-                                 V_UNION(&var, bstrVal));
-    if (FAILED(hr))
+    hr = E_NOINTERFACE;
+    while (IEnumMoniker_Next(filters, 1, &mon, NULL) == S_OK)
     {
-        IBaseFilter_Release(splitter);
-        return hr;
-    }
-    This->splitter = splitter;
+        hr = GetFilterInfo(mon, &clsid, &var);
+        IMoniker_Release(mon);
+        if (FAILED(hr))
+            continue;
 
-    hr = IBaseFilter_EnumPins(This->source, &pins);
-    if (FAILED(hr))
-        return hr;
-    IEnumPins_Next(pins, 1, &source_pin, NULL);
-    IEnumPins_Release(pins);
+        hr = CoCreateInstance(&clsid, NULL, CLSCTX_INPROC_SERVER,
+                              &IID_IBaseFilter, (void **) &splitter);
+        if (FAILED(hr))
+        {
+            VariantClear(&var);
+            goto retry;
+        }
 
-    hr = IBaseFilter_EnumPins(splitter, &pins);
-    if (FAILED(hr))
-    {
+        hr = IGraphBuilder_AddFilter(This->graph, splitter,
+                                     V_UNION(&var, bstrVal));
+        VariantClear(&var);
+        This->splitter = splitter;
+        if (FAILED(hr))
+            goto retry;
+
+        hr = IBaseFilter_EnumPins(This->source, &pins);
+        if (FAILED(hr))
+            goto retry;
+        IEnumPins_Next(pins, 1, &source_pin, NULL);
+        IEnumPins_Release(pins);
+
+        hr = IBaseFilter_EnumPins(splitter, &pins);
+        if (FAILED(hr))
+        {
+            IPin_Release(source_pin);
+            goto retry;
+        }
+        IEnumPins_Next(pins, 1, &splitter_pin, NULL);
+        IEnumPins_Release(pins);
+
+        hr = IPin_Connect(source_pin, splitter_pin, NULL);
         IPin_Release(source_pin);
-        return hr;
-    }
-    IEnumPins_Next(pins, 1, &splitter_pin, NULL);
-    IEnumPins_Release(pins);
+        IPin_Release(splitter_pin);
+        if (SUCCEEDED(hr))
+            break;
 
-    hr = IPin_Connect(source_pin, splitter_pin, NULL);
-    IPin_Release(source_pin);
-    IPin_Release(splitter_pin);
+retry:
+        IBaseFilter_Release(splitter);
+        This->splitter = NULL;
+    }
+
+    IEnumMoniker_Release(filters);
     if (FAILED(hr))
         return hr;
 
@@ -425,7 +442,7 @@ static HRESULT GetSplitter(MediaDetImpl *This)
 static HRESULT WINAPI MediaDet_put_Filename(IMediaDet* iface, BSTR newVal)
 {
     static const WCHAR reader[] = {'R','e','a','d','e','r',0};
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     IGraphBuilder *gb;
     IBaseFilter *bf;
     HRESULT hr;
@@ -461,28 +478,28 @@ static HRESULT WINAPI MediaDet_put_Filename(IMediaDet* iface, BSTR newVal)
 
 static HRESULT WINAPI MediaDet_GetBitmapBits(IMediaDet* iface,
                                              double StreamTime,
-                                             long *pBufferSize, char *pBuffer,
-                                             long Width, long Height)
+                                             LONG *pBufferSize, char *pBuffer,
+                                             LONG Width, LONG Height)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
-    FIXME("(%p)->(%f %p %p %ld %ld): not implemented!\n", This, StreamTime, pBufferSize, pBuffer,
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
+    FIXME("(%p)->(%f %p %p %d %d): not implemented!\n", This, StreamTime, pBufferSize, pBuffer,
           Width, Height);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI MediaDet_WriteBitmapBits(IMediaDet* iface,
-                                               double StreamTime, long Width,
-                                               long Height, BSTR Filename)
+                                               double StreamTime, LONG Width,
+                                               LONG Height, BSTR Filename)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
-    FIXME("(%p)->(%f %ld %ld %p): not implemented!\n", This, StreamTime, Width, Height, Filename);
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
+    FIXME("(%p)->(%f %d %d %p): not implemented!\n", This, StreamTime, Width, Height, Filename);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI MediaDet_get_StreamMediaType(IMediaDet* iface,
                                                    AM_MEDIA_TYPE *pVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     IEnumMediaTypes *types;
     AM_MEDIA_TYPE *pmt;
     HRESULT hr;
@@ -516,14 +533,14 @@ static HRESULT WINAPI MediaDet_get_StreamMediaType(IMediaDet* iface,
 static HRESULT WINAPI MediaDet_GetSampleGrabber(IMediaDet* iface,
                                                 ISampleGrabber **ppVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     FIXME("(%p)->(%p): not implemented!\n", This, ppVal);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI MediaDet_get_FrameRate(IMediaDet* iface, double *pVal)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     AM_MEDIA_TYPE mt;
     VIDEOINFOHEADER *vh;
     HRESULT hr;
@@ -553,7 +570,7 @@ static HRESULT WINAPI MediaDet_get_FrameRate(IMediaDet* iface, double *pVal)
 static HRESULT WINAPI MediaDet_EnterBitmapGrabMode(IMediaDet* iface,
                                                    double SeekTime)
 {
-    MediaDetImpl *This = (MediaDetImpl *)iface;
+    MediaDetImpl *This = impl_from_IMediaDet(iface);
     FIXME("(%p)->(%f): not implemented!\n", This, SeekTime);
     return E_NOTIMPL;
 }
@@ -597,7 +614,7 @@ HRESULT MediaDet_create(IUnknown * pUnkOuter, LPVOID * ppv) {
     ZeroMemory(obj, sizeof(MediaDetImpl));
 
     obj->refCount = 1;
-    obj->MediaDet_Vtbl = &IMediaDet_VTable;
+    obj->IMediaDet_iface.lpVtbl = &IMediaDet_VTable;
     obj->graph = NULL;
     obj->source = NULL;
     obj->splitter = NULL;
