@@ -52,6 +52,14 @@ static const IUnknownVtbl IInner_VTable;
 static const IBasicVideoVtbl IBasicVideo_VTable;
 static const IVideoWindowVtbl IVideoWindow_VTable;
 static const IPinVtbl VideoRenderer_InputPin_Vtbl;
+static const IAMFilterMiscFlagsVtbl IAMFilterMiscFlags_Vtbl;
+static const IQualityControlVtbl VideoRenderer_QualityControl_Vtbl = {
+    QualityControlImpl_QueryInterface,
+    QualityControlImpl_AddRef,
+    QualityControlImpl_Release,
+    QualityControlImpl_Notify,
+    QualityControlImpl_SetSink
+};
 
 typedef struct VideoRendererImpl
 {
@@ -59,7 +67,9 @@ typedef struct VideoRendererImpl
     const IBasicVideoVtbl * IBasicVideo_vtbl;
     const IVideoWindowVtbl * IVideoWindow_vtbl;
     const IUnknownVtbl * IInner_vtbl;
+    const IAMFilterMiscFlagsVtbl *IAMFilterMiscFlags_vtbl;
     IUnknown *seekthru_unk;
+    QualityControlImpl qcimpl;
 
     BaseInputPin *pInputPin;
 
@@ -72,6 +82,7 @@ typedef struct VideoRendererImpl
     BOOL ThreadResult;
     HWND hWnd;
     HWND hWndMsgDrain;
+    HWND hWndOwner;
     BOOL AutoShow;
     RECT SourceRect;
     RECT DestRect;
@@ -81,14 +92,13 @@ typedef struct VideoRendererImpl
     IUnknown * pUnkOuter;
     BOOL bUnkOuterValid;
     BOOL bAggregatable;
-    REFERENCE_TIME rtLastStop;
     LONG WindowStyle;
 
     /* During pause we can hold a single sample, for use in GetCurrentImage */
     IMediaSample *sample_held;
 } VideoRendererImpl;
 
-static LRESULT CALLBACK VideoWndProcA(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
+static LRESULT CALLBACK VideoWndProcW(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
     VideoRendererImpl* pVideoRenderer = (VideoRendererImpl*)GetWindowLongPtrW(hwnd, 0);
     LPRECT lprect = (LPRECT)lParam;
@@ -120,7 +130,7 @@ static LRESULT CALLBACK VideoWndProcA(HWND hwnd, UINT uMsg, WPARAM wParam, LPARA
             case WM_RBUTTONDBLCLK:
             case WM_RBUTTONDOWN:
             case WM_RBUTTONUP:
-                PostMessageA(pVideoRenderer->hWndMsgDrain, uMsg, wParam, lParam);
+                PostMessageW(pVideoRenderer->hWndMsgDrain, uMsg, wParam, lParam);
                 break;
             default:
                 break;
@@ -149,19 +159,21 @@ static LRESULT CALLBACK VideoWndProcA(HWND hwnd, UINT uMsg, WPARAM wParam, LPARA
                 pVideoRenderer->DestRect.bottom - pVideoRenderer->DestRect.top);
             return TRUE;
         default:
-            return DefWindowProcA(hwnd, uMsg, wParam, lParam);
+            return DefWindowProcW(hwnd, uMsg, wParam, lParam);
     }
     return 0;
 }
 
-static BOOL CreateRenderingWindow(VideoRendererImpl* This)
-{
-    WNDCLASSA winclass;
+static const WCHAR classnameW[] = { 'W','i','n','e',' ','A','c','t','i','v','e','M','o','v','i','e',' ','C','l','a','s','s',0 };
+static const WCHAR windownameW[] = { 'A','c','t','i','v','e','M','o','v','i','e',' ','W','i','n','d','o','w',0 };
+
+static BOOL video_register_windowclass(void) {
+    WNDCLASSW winclass;
+    if (wnd_class_registered)
+        return 1;
 
-    TRACE("(%p)->()\n", This);
-    
     winclass.style = 0;
-    winclass.lpfnWndProc = VideoWndProcA;
+    winclass.lpfnWndProc = VideoWndProcW;
     winclass.cbClsExtra = 0;
     winclass.cbWndExtra = sizeof(VideoRendererImpl*);
     winclass.hInstance = NULL;
@@ -169,19 +181,28 @@ static BOOL CreateRenderingWindow(VideoRendererImpl* This)
     winclass.hCursor = NULL;
     winclass.hbrBackground = GetStockObject(BLACK_BRUSH);
     winclass.lpszMenuName = NULL;
-    winclass.lpszClassName = "Wine ActiveMovie Class";
-
-    if (!wnd_class_registered)
+    winclass.lpszClassName = classnameW;
+    if (!RegisterClassW(&winclass))
     {
-        if (!RegisterClassA(&winclass))
-        {
-            ERR("Unable to register window %u\n", GetLastError());
-            return FALSE;
-        }
-        wnd_class_registered = TRUE;
+        ERR("Unable to register window class: %u\n", GetLastError());
+        return FALSE;
     }
+    wnd_class_registered = 1;
+    return 1;
+}
 
-    This->hWnd = CreateWindowExA(0, "Wine ActiveMovie Class", "Wine ActiveMovie Window", WS_SIZEBOX,
+void video_unregister_windowclass(void) {
+    if (!wnd_class_registered)
+        return;
+    UnregisterClassW(classnameW, NULL);
+}
+
+static BOOL CreateRenderingWindow(VideoRendererImpl* This)
+{
+    TRACE("(%p)->()\n", This);
+    if (!video_register_windowclass())
+        return FALSE;
+    This->hWnd = CreateWindowExW(0, classnameW, windownameW, WS_SIZEBOX,
                                  CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL,
                                  NULL, NULL, NULL);
 
@@ -214,10 +235,10 @@ static DWORD WINAPI MessageLoop(LPVOID lpParameter)
     This->ThreadResult = TRUE;
     SetEvent(This->hEvent);
 
-    while ((fGotMessage = GetMessageA(&msg, NULL, 0, 0)) != 0 && fGotMessage != -1) 
+    while ((fGotMessage = GetMessageW(&msg, NULL, 0, 0)) != 0 && fGotMessage != -1)
     {
         TranslateMessage(&msg); 
-        DispatchMessageA(&msg); 
+        DispatchMessageW(&msg);
     }
 
     TRACE("End of message loop\n");
@@ -250,6 +271,42 @@ static BOOL CreateRenderingSubsystem(VideoRendererImpl* This)
     return TRUE;
 }
 
+static void VideoRenderer_AutoShowWindow(VideoRendererImpl *This) {
+    if (!This->init && (!This->WindowPos.right || !This->WindowPos.top))
+    {
+        DWORD style = GetWindowLongW(This->hWnd, GWL_STYLE);
+        DWORD style_ex = GetWindowLongW(This->hWnd, GWL_EXSTYLE);
+
+        if (!This->WindowPos.right)
+        {
+            This->WindowPos.left = This->SourceRect.left;
+            This->WindowPos.right = This->SourceRect.right;
+        }
+        if (!This->WindowPos.bottom)
+        {
+            This->WindowPos.top = This->SourceRect.top;
+            This->WindowPos.bottom = This->SourceRect.bottom;
+        }
+
+        AdjustWindowRectEx(&This->WindowPos, style, TRUE, style_ex);
+
+        TRACE("WindowPos: %d %d %d %d\n", This->WindowPos.left, This->WindowPos.top, This->WindowPos.right, This->WindowPos.bottom);
+        SetWindowPos(This->hWnd, NULL,
+            This->WindowPos.left,
+            This->WindowPos.top,
+            This->WindowPos.right - This->WindowPos.left,
+            This->WindowPos.bottom - This->WindowPos.top,
+            SWP_NOZORDER|SWP_NOMOVE|SWP_DEFERERASE);
+
+        GetClientRect(This->hWnd, &This->DestRect);
+    }
+    else if (!This->init)
+        This->DestRect = This->WindowPos;
+    This->init = TRUE;
+    if (This->AutoShow)
+        ShowWindow(This->hWnd, SW_SHOW);
+}
+
 static DWORD VideoRenderer_SendSampleData(VideoRendererImpl* This, LPBYTE data, DWORD size)
 {
     AM_MEDIA_TYPE amt;
@@ -289,41 +346,6 @@ static DWORD VideoRenderer_SendSampleData(VideoRendererImpl* This, LPBYTE data,
     TRACE("biCompression = %s\n", debugstr_an((LPSTR)&(bmiHeader->biCompression), 4));
     TRACE("biSizeImage = %d\n", bmiHeader->biSizeImage);
 
-    if (!This->init)
-    {
-        if (!This->WindowPos.right || !This->WindowPos.top)
-        {
-            DWORD style = GetWindowLongW(This->hWnd, GWL_STYLE);
-            DWORD style_ex = GetWindowLongW(This->hWnd, GWL_EXSTYLE);
-
-            if (!This->WindowPos.right)
-            {
-                This->WindowPos.left = This->SourceRect.left;
-                This->WindowPos.right = This->SourceRect.right;
-            }
-            if (!This->WindowPos.bottom)
-            {
-                This->WindowPos.top = This->SourceRect.top;
-                This->WindowPos.bottom = This->SourceRect.bottom;
-            }
-
-            AdjustWindowRectEx(&This->WindowPos, style, TRUE, style_ex);
-
-            TRACE("WindowPos: %d %d %d %d\n", This->WindowPos.left, This->WindowPos.top, This->WindowPos.right, This->WindowPos.bottom);
-            SetWindowPos(This->hWnd, NULL,
-                This->WindowPos.left,
-                This->WindowPos.top,
-                This->WindowPos.right - This->WindowPos.left,
-                This->WindowPos.bottom - This->WindowPos.top,
-                SWP_NOZORDER|SWP_NOMOVE|SWP_DEFERERASE);
-
-            GetClientRect(This->hWnd, &This->DestRect);
-        }
-        else
-            This->DestRect = This->WindowPos;
-        This->init = TRUE;
-    }
-
     hDC = GetDC(This->hWnd);
 
     if (!hDC) {
@@ -340,8 +362,6 @@ static DWORD VideoRenderer_SendSampleData(VideoRendererImpl* This, LPBYTE data,
                   data, (BITMAPINFO *)bmiHeader, DIB_RGB_COLORS, SRCCOPY);
 
     ReleaseDC(This->hWnd, hDC);
-    if (This->AutoShow)
-        ShowWindow(This->hWnd, SW_SHOW);
 
     return S_OK;
 }
@@ -370,29 +390,11 @@ static HRESULT WINAPI VideoRenderer_Receive(BaseInputPin* pin, IMediaSample * pS
         return VFW_E_WRONG_STATE;
     }
 
-    hr = IMediaSample_GetTime(pSample, &tStart, &tStop);
-    if (FAILED(hr))
-        ERR("Cannot get sample time (%x)\n", hr);
-    else
+    if (IMediaSample_GetMediaTime(pSample, &tStart, &tStop) == S_OK)
         MediaSeekingPassThru_RegisterMediaTime(This->seekthru_unk, tStart);
 
-    if (This->rtLastStop != tStart && This->filter.state == State_Running)
-    {
-        LONG64 delta;
-        delta = tStart - This->rtLastStop;
-        if ((delta < -100000 || delta > 100000) &&
-            IMediaSample_IsDiscontinuity(pSample) == S_FALSE)
-            ERR("Unexpected discontinuity: Last: %u.%03u, tStart: %u.%03u\n",
-                (DWORD)(This->rtLastStop / 10000000),
-                (DWORD)((This->rtLastStop / 10000)%1000),
-                (DWORD)(tStart / 10000000), (DWORD)((tStart / 10000)%1000));
-        This->rtLastStop = tStart;
-    }
-
     /* Preroll means the sample isn't shown, this is used for key frames and things like that */
-    if (IMediaSample_IsPreroll(pSample) == S_OK)
-    {
-        This->rtLastStop = tStop;
+    if (IMediaSample_IsPreroll(pSample) == S_OK) {
         LeaveCriticalSection(&This->filter.csFilter);
         return S_OK;
     }
@@ -425,6 +427,7 @@ static HRESULT WINAPI VideoRenderer_Receive(BaseInputPin* pin, IMediaSample * pS
     SetEvent(This->hEvent);
     if (This->filter.state == State_Paused)
     {
+        VideoRenderer_SendSampleData(This, pbSrcStream, cbSrcStream);
         This->sample_held = pSample;
         LeaveCriticalSection(&This->filter.csFilter);
         WaitForSingleObject(This->blocked, INFINITE);
@@ -441,44 +444,15 @@ static HRESULT WINAPI VideoRenderer_Receive(BaseInputPin* pin, IMediaSample * pS
             LeaveCriticalSection(&This->filter.csFilter);
             return VFW_E_WRONG_STATE;
         }
-    }
-
-    if (This->filter.pClock && This->filter.state == State_Running)
-    {
-        REFERENCE_TIME time, trefstart, trefstop;
-        LONG delta;
-
-        /* Perhaps I <SHOULD> use the reference clock AdviseTime function here
-         * I'm not going to! When I tried, it seemed to generate lag and
-         * it caused instability.
-         */
-        IReferenceClock_GetTime(This->filter.pClock, &time);
-
-        trefstart = This->filter.rtStreamStart;
-        trefstop = (REFERENCE_TIME)((double)(tStop - tStart) / This->pInputPin->dRate) + This->filter.rtStreamStart;
-        delta = (LONG)((trefstart-time)/10000);
-        This->filter.rtStreamStart = trefstop;
-        This->rtLastStop = tStop;
-
-        if (delta > 0)
-        {
-            TRACE("Sleeping for %u ms\n", delta);
-            Sleep(delta);
-        }
-        else if (time > trefstop)
-        {
-            TRACE("Dropping sample: Time: %u.%03u ms trefstop: %u.%03u ms!\n",
-                  (DWORD)(time / 10000000), (DWORD)((time / 10000)%1000),
-                  (DWORD)(trefstop / 10000000), (DWORD)((trefstop / 10000)%1000) );
-            This->rtLastStop = tStop;
-            LeaveCriticalSection(&This->filter.csFilter);
-            return S_OK;
+    } else {
+        hr = QualityControlRender_WaitFor(&This->qcimpl, pSample, This->blocked);
+        if (hr == S_OK) {
+            QualityControlRender_BeginRender(&This->qcimpl);
+            VideoRenderer_SendSampleData(This, pbSrcStream, cbSrcStream);
+            QualityControlRender_EndRender(&This->qcimpl);
         }
+        QualityControlRender_DoQOS(&This->qcimpl);
     }
-    This->rtLastStop = tStop;
-
-    VideoRenderer_SendSampleData(This, pbSrcStream, cbSrcStream);
-
     LeaveCriticalSection(&This->filter.csFilter);
     return S_OK;
 }
@@ -581,6 +555,7 @@ HRESULT VideoRenderer_create(IUnknown * pUnkOuter, LPVOID * ppv)
     pVideoRenderer->bUnkOuterValid = FALSE;
     pVideoRenderer->bAggregatable = FALSE;
     pVideoRenderer->IInner_vtbl = &IInner_VTable;
+    pVideoRenderer->IAMFilterMiscFlags_vtbl = &IAMFilterMiscFlags_Vtbl;
 
     BaseFilter_Init(&pVideoRenderer->filter, &VideoRenderer_Vtbl, &CLSID_VideoRenderer, (DWORD_PTR)(__FILE__ ": VideoRendererImpl.csFilter"), &BaseFuncTable);
 
@@ -589,11 +564,10 @@ HRESULT VideoRenderer_create(IUnknown * pUnkOuter, LPVOID * ppv)
 
     pVideoRenderer->init = 0;
     pVideoRenderer->AutoShow = 1;
-    pVideoRenderer->rtLastStop = -1;
     ZeroMemory(&pVideoRenderer->SourceRect, sizeof(RECT));
     ZeroMemory(&pVideoRenderer->DestRect, sizeof(RECT));
     ZeroMemory(&pVideoRenderer->WindowPos, sizeof(RECT));
-    pVideoRenderer->hWndMsgDrain = NULL;
+    pVideoRenderer->hWndMsgDrain = pVideoRenderer->hWndOwner = NULL;
     pVideoRenderer->WindowStyle = WS_OVERLAPPED;
 
     /* construct input pin */
@@ -619,6 +593,9 @@ HRESULT VideoRenderer_create(IUnknown * pUnkOuter, LPVOID * ppv)
     if (FAILED(hr))
         goto fail;
 
+    QualityControlImpl_init(&pVideoRenderer->qcimpl, (IPin*)pVideoRenderer->pInputPin, (IBaseFilter*)pVideoRenderer);
+    pVideoRenderer->qcimpl.lpVtbl = &VideoRenderer_QualityControl_Vtbl;
+
     if (!CreateRenderingSubsystem(pVideoRenderer))
         return E_FAIL;
 
@@ -666,6 +643,10 @@ static HRESULT WINAPI VideoRendererInner_QueryInterface(IUnknown * iface, REFIID
         *ppv = &This->IVideoWindow_vtbl;
     else if (IsEqualIID(riid, &IID_IMediaSeeking))
         return IUnknown_QueryInterface(This->seekthru_unk, riid, ppv);
+    else if (IsEqualIID(riid, &IID_IAMFilterMiscFlags))
+        *ppv = &This->IAMFilterMiscFlags_vtbl;
+    else if (IsEqualIID(riid, &IID_IQualityControl))
+        *ppv = &This->qcimpl;
 
     if (*ppv)
     {
@@ -700,8 +681,9 @@ static ULONG WINAPI VideoRendererInner_Release(IUnknown * iface)
     {
         IPin *pConnectedTo;
 
-        DestroyWindow(This->hWnd);
-        PostThreadMessageA(This->ThreadID, WM_QUIT, 0, 0);
+        if (This->hWnd)
+            SendMessageW(This->hWnd, WM_CLOSE, 0, 0);
+        PostThreadMessageW(This->ThreadID, WM_QUIT, 0, 0);
         WaitForSingleObject(This->hThread, INFINITE);
         CloseHandle(This->hThread);
         CloseHandle(This->hEvent);
@@ -798,6 +780,9 @@ static HRESULT WINAPI VideoRenderer_Stop(IBaseFilter * iface)
         SetEvent(This->hEvent);
         SetEvent(This->blocked);
         MediaSeekingPassThru_ResetMediaTime(This->seekthru_unk);
+        if (This->AutoShow)
+            /* Black it out */
+            RedrawWindow(This->hWnd, NULL, NULL, RDW_INVALIDATE|RDW_ERASE);
     }
     LeaveCriticalSection(&This->filter.csFilter);
 
@@ -817,6 +802,7 @@ static HRESULT WINAPI VideoRenderer_Pause(IBaseFilter * iface)
         {
             This->pInputPin->end_of_stream = 0;
             ResetEvent(This->hEvent);
+            VideoRenderer_AutoShowWindow(This);
         }
 
         This->filter.state = State_Paused;
@@ -827,28 +813,54 @@ static HRESULT WINAPI VideoRenderer_Pause(IBaseFilter * iface)
     return S_OK;
 }
 
+static HRESULT WINAPI VideoRenderer_SetSyncSource(IBaseFilter *iface, IReferenceClock *clock) {
+    VideoRendererImpl *This = (VideoRendererImpl *)iface;
+    HRESULT hr;
+
+    EnterCriticalSection(&This->filter.csFilter);
+    QualityControlRender_SetClock(&This->qcimpl, clock);
+    hr = BaseFilterImpl_SetSyncSource(iface, clock);
+    LeaveCriticalSection(&This->filter.csFilter);
+    return hr;
+}
+
 static HRESULT WINAPI VideoRenderer_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
 {
+    HRESULT hr = S_OK;
     VideoRendererImpl *This = (VideoRendererImpl *)iface;
 
     TRACE("(%p/%p)->(%s)\n", This, iface, wine_dbgstr_longlong(tStart));
 
     EnterCriticalSection(&This->filter.csFilter);
-    if (This->filter.state != State_Running)
+    This->filter.rtStreamStart = tStart;
+    if (This->filter.state == State_Running)
+        goto out;
+    QualityControlRender_Start(&This->qcimpl, tStart);
+    if (This->pInputPin->pin.pConnectedTo && (This->filter.state == State_Stopped || !This->pInputPin->end_of_stream))
     {
         if (This->filter.state == State_Stopped)
         {
-            This->pInputPin->end_of_stream = 0;
             ResetEvent(This->hEvent);
+            VideoRenderer_AutoShowWindow(This);
+            This->pInputPin->end_of_stream = 0;
         }
         SetEvent(This->blocked);
-
-        This->filter.rtStreamStart = tStart;
-        This->filter.state = State_Running;
+    } else if (This->filter.filterInfo.pGraph) {
+        IMediaEventSink *pEventSink;
+        hr = IFilterGraph_QueryInterface(This->filter.filterInfo.pGraph, &IID_IMediaEventSink, (LPVOID*)&pEventSink);
+        if (SUCCEEDED(hr))
+        {
+            hr = IMediaEventSink_Notify(pEventSink, EC_COMPLETE, S_OK, (LONG_PTR)This);
+            IMediaEventSink_Release(pEventSink);
+        }
+        hr = S_OK;
     }
+    if (SUCCEEDED(hr))
+        This->filter.state = State_Running;
+out:
     LeaveCriticalSection(&This->filter.csFilter);
 
-    return S_OK;
+    return hr;
 }
 
 static HRESULT WINAPI VideoRenderer_GetState(IBaseFilter * iface, DWORD dwMilliSecsTimeout, FILTER_STATE *pState)
@@ -891,7 +903,7 @@ static const IBaseFilterVtbl VideoRenderer_Vtbl =
     VideoRenderer_Pause,
     VideoRenderer_Run,
     VideoRenderer_GetState,
-    BaseFilterImpl_SetSyncSource,
+    VideoRenderer_SetSyncSource,
     BaseFilterImpl_GetSyncSource,
     BaseFilterImpl_EnumPins,
     VideoRenderer_FindPin,
@@ -905,18 +917,24 @@ static HRESULT WINAPI VideoRenderer_InputPin_EndOfStream(IPin * iface)
     BaseInputPin* This = (BaseInputPin*)iface;
     VideoRendererImpl *pFilter;
     IMediaEventSink* pEventSink;
-    HRESULT hr;
+    HRESULT hr = S_OK;
 
     TRACE("(%p/%p)->()\n", This, iface);
 
+    EnterCriticalSection(This->pin.pCritSec);
     pFilter = (VideoRendererImpl*)This->pin.pinInfo.pFilter;
+    if (This->flushing || This->end_of_stream)
+        goto out;
     hr = IFilterGraph_QueryInterface(pFilter->filter.filterInfo.pGraph, &IID_IMediaEventSink, (LPVOID*)&pEventSink);
     if (SUCCEEDED(hr))
     {
-        hr = IMediaEventSink_Notify(pEventSink, EC_COMPLETE, S_OK, 0);
+        hr = IMediaEventSink_Notify(pEventSink, EC_COMPLETE, S_OK, (LONG_PTR)pFilter);
         IMediaEventSink_Release(pEventSink);
     }
     MediaSeekingPassThru_EOS(pFilter->seekthru_unk);
+    This->end_of_stream = 1;
+out:
+    LeaveCriticalSection(This->pin.pCritSec);
 
     return hr;
 }
@@ -925,18 +943,11 @@ static HRESULT WINAPI VideoRenderer_InputPin_BeginFlush(IPin * iface)
 {
     BaseInputPin* This = (BaseInputPin*)iface;
     VideoRendererImpl *pVideoRenderer = (VideoRendererImpl *)This->pin.pinInfo.pFilter;
-    HRESULT hr;
 
     TRACE("(%p/%p)->()\n", This, iface);
 
-    EnterCriticalSection(This->pin.pCritSec);
-    if (pVideoRenderer->filter.state == State_Paused)
-        SetEvent(pVideoRenderer->blocked);
-
-    hr = BaseInputPinImpl_BeginFlush(iface);
-    LeaveCriticalSection(This->pin.pCritSec);
-
-    return hr;
+    SetEvent(pVideoRenderer->blocked);
+    return BaseInputPinImpl_BeginFlush(iface);
 }
 
 static HRESULT WINAPI VideoRenderer_InputPin_EndFlush(IPin * iface)
@@ -948,9 +959,12 @@ static HRESULT WINAPI VideoRenderer_InputPin_EndFlush(IPin * iface)
     TRACE("(%p/%p)->()\n", This, iface);
 
     EnterCriticalSection(This->pin.pCritSec);
-    if (pVideoRenderer->filter.state == State_Paused)
+    if (pVideoRenderer->filter.state == State_Paused) {
         ResetEvent(pVideoRenderer->blocked);
+        ResetEvent(pVideoRenderer->hEvent);
+    }
 
+    QualityControlRender_Start(&pVideoRenderer->qcimpl, pVideoRenderer->filter.rtStreamStart);
     hr = BaseInputPinImpl_EndFlush(iface);
     LeaveCriticalSection(This->pin.pCritSec);
     MediaSeekingPassThru_ResetMediaTime(pVideoRenderer->seekthru_unk);
@@ -1416,9 +1430,11 @@ static HRESULT WINAPI Basicvideo_GetVideoPaletteEntries(IBasicVideo *iface,
                                                         LONG *pPalette) {
     ICOM_THIS_MULTI(VideoRendererImpl, IBasicVideo_vtbl, iface);
 
-    FIXME("(%p/%p)->(%d, %d, %p, %p): stub !!!\n", This, iface, StartIndex, Entries, pRetrieved, pPalette);
+    TRACE("(%p/%p)->(%d, %d, %p, %p)\n", This, iface, StartIndex, Entries, pRetrieved, pPalette);
 
-    return S_OK;
+    if (pRetrieved)
+        *pRetrieved = 0;
+    return VFW_E_NO_PALETTE_AVAILABLE;
 }
 
 static HRESULT WINAPI Basicvideo_GetCurrentImage(IBasicVideo *iface,
@@ -1649,14 +1665,14 @@ static HRESULT WINAPI Videowindow_put_WindowStyle(IVideoWindow *iface,
     ICOM_THIS_MULTI(VideoRendererImpl, IVideoWindow_vtbl, iface);
     LONG old;
 
-    old = GetWindowLongA(This->hWnd, GWL_STYLE);
+    old = GetWindowLongW(This->hWnd, GWL_STYLE);
 
     TRACE("(%p/%p)->(%x -> %x)\n", This, iface, old, WindowStyle);
 
     if (WindowStyle & (WS_DISABLED|WS_HSCROLL|WS_ICONIC|WS_MAXIMIZE|WS_MINIMIZE|WS_VSCROLL))
         return E_INVALIDARG;
 
-    SetWindowLongA(This->hWnd, GWL_STYLE, WindowStyle);
+    SetWindowLongW(This->hWnd, GWL_STYLE, WindowStyle);
     SetWindowPos(This->hWnd,0,0,0,0,0,SWP_FRAMECHANGED|SWP_NOSIZE|SWP_NOZORDER);
     This->WindowStyle = WindowStyle;
 
@@ -1680,7 +1696,7 @@ static HRESULT WINAPI Videowindow_put_WindowStyleEx(IVideoWindow *iface,
 
     TRACE("(%p/%p)->(%d)\n", This, iface, WindowStyleEx);
 
-    if (!SetWindowLongA(This->hWnd, GWL_EXSTYLE, WindowStyleEx))
+    if (!SetWindowLongW(This->hWnd, GWL_EXSTYLE, WindowStyleEx))
         return E_FAIL;
 
     return S_OK;
@@ -1692,7 +1708,7 @@ static HRESULT WINAPI Videowindow_get_WindowStyleEx(IVideoWindow *iface,
 
     TRACE("(%p/%p)->(%p)\n", This, iface, WindowStyleEx);
 
-    *WindowStyleEx = GetWindowLongA(This->hWnd, GWL_EXSTYLE);
+    *WindowStyleEx = GetWindowLongW(This->hWnd, GWL_EXSTYLE);
 
     return S_OK;
 }
@@ -1887,13 +1903,14 @@ static HRESULT WINAPI Videowindow_put_Owner(IVideoWindow *iface,
 
     TRACE("(%p/%p)->(%08x)\n", This, iface, (DWORD) Owner);
 
-    SetParent(This->hWnd, (HWND)Owner);
+    This->hWndOwner = (HWND)Owner;
+    SetParent(This->hWnd, This->hWndOwner);
     if (This->WindowStyle & WS_CHILD)
     {
-        LONG old = GetWindowLongA(This->hWnd, GWL_STYLE);
+        LONG old = GetWindowLongW(This->hWnd, GWL_STYLE);
         if (old != This->WindowStyle)
         {
-            SetWindowLongA(This->hWnd, GWL_STYLE, This->WindowStyle);
+            SetWindowLongW(This->hWnd, GWL_STYLE, This->WindowStyle);
             SetWindowPos(This->hWnd,0,0,0,0,0,SWP_FRAMECHANGED|SWP_NOSIZE|SWP_NOZORDER);
         }
     }
@@ -1907,7 +1924,7 @@ static HRESULT WINAPI Videowindow_get_Owner(IVideoWindow *iface,
 
     TRACE("(%p/%p)->(%p)\n", This, iface, Owner);
 
-    *(HWND*)Owner = GetParent(This->hWnd);
+    *(HWND*)Owner = This->hWndOwner;
 
     return S_OK;
 }
@@ -1967,6 +1984,22 @@ static HRESULT WINAPI Videowindow_put_FullScreenMode(IVideoWindow *iface,
 
     FIXME("(%p/%p)->(%d): stub !!!\n", This, iface, FullScreenMode);
 
+    if (FullScreenMode) {
+        ShowWindow(This->hWnd, SW_HIDE);
+        SetParent(This->hWnd, 0);
+        SetWindowLongW(This->hWnd, GWL_STYLE, WS_POPUP);
+        SetWindowPos(This->hWnd,HWND_TOP,0,0,GetSystemMetrics(SM_CXSCREEN),GetSystemMetrics(SM_CYSCREEN),SWP_SHOWWINDOW);
+        GetWindowRect(This->hWnd, &This->DestRect);
+        This->WindowPos = This->DestRect;
+    } else {
+        ShowWindow(This->hWnd, SW_HIDE);
+        SetParent(This->hWnd, This->hWndOwner);
+        SetWindowLongW(This->hWnd, GWL_STYLE, This->WindowStyle);
+        GetClientRect(This->hWnd, &This->DestRect);
+        SetWindowPos(This->hWnd,0,This->DestRect.left,This->DestRect.top,This->DestRect.right,This->DestRect.bottom,SWP_NOZORDER|SWP_SHOWWINDOW);
+        This->WindowPos = This->DestRect;
+    }
+
     return S_OK;
 }
 
@@ -2006,7 +2039,7 @@ static HRESULT WINAPI Videowindow_NotifyOwnerMessage(IVideoWindow *iface,
 
     TRACE("(%p/%p)->(%08lx, %d, %08lx, %08lx)\n", This, iface, hwnd, uMsg, wParam, lParam);
 
-    if (!PostMessageA(This->hWnd, uMsg, wParam, lParam))
+    if (!PostMessageW(This->hWnd, uMsg, wParam, lParam))
         return E_FAIL;
     
     return S_OK;
@@ -2154,3 +2187,33 @@ static const IVideoWindowVtbl IVideoWindow_VTable =
     Videowindow_HideCursor,
     Videowindow_IsCursorHidden
 };
+
+static VideoRendererImpl *from_IAMFilterMiscFlags(IAMFilterMiscFlags *iface) {
+    return (VideoRendererImpl*)((char*)iface - offsetof(VideoRendererImpl, IAMFilterMiscFlags_vtbl));
+}
+
+static HRESULT WINAPI AMFilterMiscFlags_QueryInterface(IAMFilterMiscFlags *iface, const REFIID riid, void **ppv) {
+    VideoRendererImpl *This = from_IAMFilterMiscFlags(iface);
+    return IUnknown_QueryInterface((IUnknown*)This, riid, ppv);
+}
+
+static ULONG WINAPI AMFilterMiscFlags_AddRef(IAMFilterMiscFlags *iface) {
+    VideoRendererImpl *This = from_IAMFilterMiscFlags(iface);
+    return IUnknown_AddRef((IUnknown*)This);
+}
+
+static ULONG WINAPI AMFilterMiscFlags_Release(IAMFilterMiscFlags *iface) {
+    VideoRendererImpl *This = from_IAMFilterMiscFlags(iface);
+    return IUnknown_Release((IUnknown*)This);
+}
+
+static ULONG WINAPI AMFilterMiscFlags_GetMiscFlags(IAMFilterMiscFlags *iface) {
+    return AM_FILTER_MISC_FLAGS_IS_RENDERER;
+}
+
+static const IAMFilterMiscFlagsVtbl IAMFilterMiscFlags_Vtbl = {
+    AMFilterMiscFlags_QueryInterface,
+    AMFilterMiscFlags_AddRef,
+    AMFilterMiscFlags_Release,
+    AMFilterMiscFlags_GetMiscFlags
+};
