@@ -58,14 +58,17 @@ typedef struct {
 
 static const IRpcProxyBufferVtbl StdProxy_Vtbl;
 
-#define ICOM_THIS_MULTI(impl,field,iface) impl* const This=(impl*)((char*)(iface) - offsetof(impl,field))
+static inline StdProxyImpl *impl_from_proxy_obj( void *iface )
+{
+    return (StdProxyImpl *)((char*)iface - FIELD_OFFSET(StdProxyImpl, PVtbl));
+}
 
 #if defined(__i386__)
 
 #include "pshpack1.h"
 
 struct thunk {
-  BYTE push;
+  BYTE mov_eax;
   DWORD index;
   BYTE jmp;
   LONG handler;
@@ -75,9 +78,12 @@ struct thunk {
 
 extern void call_stubless_func(void);
 __ASM_GLOBAL_FUNC(call_stubless_func,
+                  "pushl %eax\n\t"  /* method index */
+                  __ASM_CFI(".cfi_adjust_cfa_offset 4\n\t")
                   "pushl %esp\n\t"  /* pointer to index */
                   __ASM_CFI(".cfi_adjust_cfa_offset 4\n\t")
                   "call " __ASM_NAME("ObjectStubless") __ASM_STDCALL(4) "\n\t"
+                  __ASM_CFI(".cfi_adjust_cfa_offset -4\n\t")
                   "popl %edx\n\t"  /* args size */
                   __ASM_CFI(".cfi_adjust_cfa_offset -4\n\t")
                   "movl (%esp),%ecx\n\t"  /* return address */
@@ -115,7 +121,7 @@ static const struct thunk *allocate_block( unsigned int num )
 
     for (i = 0; i < BLOCK_SIZE; i++)
     {
-        block[i].push    = 0x68; /* pushl */
+        block[i].mov_eax = 0xb8; /* movl $n,%eax */
         block[i].index   = BLOCK_SIZE * num + i + 3;
         block[i].jmp     = 0xe9; /* jmp */
         block[i].handler = (char *)call_stubless_func - (char *)(&block[i].handler + 1);
@@ -226,7 +232,7 @@ HRESULT StdProxy_Construct(REFIID riid,
 
 static void StdProxy_Destruct(LPRPCPROXYBUFFER iface)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,lpVtbl,iface);
+  StdProxyImpl *This = (StdProxyImpl *)iface;
 
   if (This->pChannel)
     IRpcProxyBuffer_Disconnect(iface);
@@ -242,7 +248,7 @@ static HRESULT WINAPI StdProxy_QueryInterface(LPRPCPROXYBUFFER iface,
                                              REFIID riid,
                                              LPVOID *obj)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,lpVtbl,iface);
+  StdProxyImpl *This = (StdProxyImpl *)iface;
   TRACE("(%p)->QueryInterface(%s,%p)\n",This,debugstr_guid(riid),obj);
 
   if (IsEqualGUID(&IID_IUnknown,riid) ||
@@ -263,7 +269,7 @@ static HRESULT WINAPI StdProxy_QueryInterface(LPRPCPROXYBUFFER iface,
 
 static ULONG WINAPI StdProxy_AddRef(LPRPCPROXYBUFFER iface)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,lpVtbl,iface);
+  StdProxyImpl *This = (StdProxyImpl *)iface;
   TRACE("(%p)->AddRef()\n",This);
 
   return InterlockedIncrement(&This->RefCount);
@@ -272,7 +278,7 @@ static ULONG WINAPI StdProxy_AddRef(LPRPCPROXYBUFFER iface)
 static ULONG WINAPI StdProxy_Release(LPRPCPROXYBUFFER iface)
 {
   ULONG refs;
-  ICOM_THIS_MULTI(StdProxyImpl,lpVtbl,iface);
+  StdProxyImpl *This = (StdProxyImpl *)iface;
   TRACE("(%p)->Release()\n",This);
 
   refs = InterlockedDecrement(&This->RefCount);
@@ -284,7 +290,7 @@ static ULONG WINAPI StdProxy_Release(LPRPCPROXYBUFFER iface)
 static HRESULT WINAPI StdProxy_Connect(LPRPCPROXYBUFFER iface,
                                       LPRPCCHANNELBUFFER pChannel)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,lpVtbl,iface);
+  StdProxyImpl *This = (StdProxyImpl *)iface;
   TRACE("(%p)->Connect(%p)\n",This,pChannel);
 
   This->pChannel = pChannel;
@@ -295,7 +301,7 @@ static HRESULT WINAPI StdProxy_Connect(LPRPCPROXYBUFFER iface,
 
 static VOID WINAPI StdProxy_Disconnect(LPRPCPROXYBUFFER iface)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,lpVtbl,iface);
+  StdProxyImpl *This = (StdProxyImpl *)iface;
   TRACE("(%p)->Disconnect()\n",This);
 
   if (This->base_proxy) IRpcProxyBuffer_Disconnect( This->base_proxy );
@@ -316,7 +322,7 @@ static const IRpcProxyBufferVtbl StdProxy_Vtbl =
 static void StdProxy_GetChannel(LPVOID iface,
                                    LPRPCCHANNELBUFFER *ppChannel)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,PVtbl,iface);
+  StdProxyImpl *This = impl_from_proxy_obj( iface );
   TRACE("(%p)->GetChannel(%p) %s\n",This,ppChannel,This->name);
 
   *ppChannel = This->pChannel;
@@ -325,7 +331,7 @@ static void StdProxy_GetChannel(LPVOID iface,
 static void StdProxy_GetIID(LPVOID iface,
                                const IID **ppiid)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,PVtbl,iface);
+  StdProxyImpl *This = impl_from_proxy_obj( iface );
   TRACE("(%p)->GetIID(%p) %s\n",This,ppiid,This->name);
 
   *ppiid = This->piid;
@@ -335,21 +341,21 @@ HRESULT WINAPI IUnknown_QueryInterface_Proxy(LPUNKNOWN iface,
                                             REFIID riid,
                                             LPVOID *ppvObj)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,PVtbl,iface);
+  StdProxyImpl *This = impl_from_proxy_obj( iface );
   TRACE("(%p)->QueryInterface(%s,%p) %s\n",This,debugstr_guid(riid),ppvObj,This->name);
   return IUnknown_QueryInterface(This->pUnkOuter,riid,ppvObj);
 }
 
 ULONG WINAPI IUnknown_AddRef_Proxy(LPUNKNOWN iface)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,PVtbl,iface);
+  StdProxyImpl *This = impl_from_proxy_obj( iface );
   TRACE("(%p)->AddRef() %s\n",This,This->name);
   return IUnknown_AddRef(This->pUnkOuter);
 }
 
 ULONG WINAPI IUnknown_Release_Proxy(LPUNKNOWN iface)
 {
-  ICOM_THIS_MULTI(StdProxyImpl,PVtbl,iface);
+  StdProxyImpl *This = impl_from_proxy_obj( iface );
   TRACE("(%p)->Release() %s\n",This,This->name);
   return IUnknown_Release(This->pUnkOuter);
 }
@@ -483,3 +489,24 @@ CreateProxyFromTypeInfo( LPTYPEINFO pTypeInfo, LPUNKNOWN pUnkOuter, REFIID riid,
     }
     return E_NOTIMPL;
 }
+
+HRESULT WINAPI
+CreateStubFromTypeInfo(ITypeInfo *pTypeInfo, REFIID riid, IUnknown *pUnkServer,
+                       IRpcStubBuffer **ppStub )
+{
+    typedef INT (WINAPI *MessageBoxA)(HWND,LPCSTR,LPCSTR,UINT);
+    HMODULE hUser32 = LoadLibraryA("user32");
+    MessageBoxA pMessageBoxA = (void *)GetProcAddress(hUser32, "MessageBoxA");
+
+    FIXME("%p %s %p %p\n", pTypeInfo, debugstr_guid(riid), pUnkServer, ppStub);
+    if (pMessageBoxA)
+    {
+        pMessageBoxA(NULL,
+            "The native implementation of OLEAUT32.DLL cannot be used "
+            "with Wine's RPCRT4.DLL. Remove OLEAUT32.DLL and try again.\n",
+            "Wine: Unimplemented CreateProxyFromTypeInfo",
+            0x10);
+        ExitProcess(1);
+    }
+    return E_NOTIMPL;
+}
