@@ -22,20 +22,20 @@
 #include "wine/debug.h"
 
 #include "shdocvw.h"
-#include "mshtml.h"
 #include "exdispid.h"
 #include "shellapi.h"
 #include "winreg.h"
 #include "shlwapi.h"
 #include "wininet.h"
+#include "mshtml.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(shdocvw);
 
 static const WCHAR emptyW[] = {0};
 
 typedef struct {
-    const IBindStatusCallbackVtbl  *lpBindStatusCallbackVtbl;
-    const IHttpNegotiateVtbl       *lpHttpNegotiateVtbl;
+    IBindStatusCallback  IBindStatusCallback_iface;
+    IHttpNegotiate       IHttpNegotiate_iface;
 
     LONG ref;
 
@@ -47,9 +47,6 @@ typedef struct {
     ULONG post_data_len;
 } BindStatusCallback;
 
-#define BINDSC(x)  ((IBindStatusCallback*) &(x)->lpBindStatusCallbackVtbl)
-#define HTTPNEG(x) ((IHttpNegotiate*)      &(x)->lpHttpNegotiateVtbl)
-
 static void dump_BINDINFO(BINDINFO *bi)
 {
     static const char * const BINDINFOF_str[] = {
@@ -111,38 +108,57 @@ static void set_status_text(BindStatusCallback *This, LPCWSTR str)
         IOleInPlaceFrame_SetStatusText(This->doc_host->frame, str);
 }
 
-#define BINDSC_THIS(iface) DEFINE_THIS(BindStatusCallback, BindStatusCallback, iface)
+static HRESULT set_dochost_url(DocHost *This, const WCHAR *url)
+{
+    WCHAR *new_url;
+
+    if(url) {
+        new_url = heap_strdupW(url);
+        if(!new_url)
+            return E_OUTOFMEMORY;
+    }else {
+        new_url = NULL;
+    }
+
+    heap_free(This->url);
+    This->url = new_url;
+
+    This->container_vtbl->SetURL(This, This->url);
+    return S_OK;
+}
+
+static inline BindStatusCallback *impl_from_IBindStatusCallback(IBindStatusCallback *iface)
+{
+    return CONTAINING_RECORD(iface, BindStatusCallback, IBindStatusCallback_iface);
+}
 
 static HRESULT WINAPI BindStatusCallback_QueryInterface(IBindStatusCallback *iface,
                                                         REFIID riid, void **ppv)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
-
-    *ppv = NULL;
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
 
     if(IsEqualGUID(&IID_IUnknown, riid)) {
         TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
-        *ppv = BINDSC(This);
+        *ppv = &This->IBindStatusCallback_iface;
     }else if(IsEqualGUID(&IID_IBindStatusCallback, riid)) {
         TRACE("(%p)->(IID_IBindStatusCallback %p)\n", This, ppv);
-        *ppv = BINDSC(This);
+        *ppv = &This->IBindStatusCallback_iface;
     }else if(IsEqualGUID(&IID_IHttpNegotiate, riid)) {
         TRACE("(%p)->(IID_IHttpNegotiate %p)\n", This, ppv);
-        *ppv = HTTPNEG(This);
+        *ppv = &This->IHttpNegotiate_iface;
+    }else {
+        *ppv = NULL;
+        WARN("(%p)->(%s %p)\n", This, debugstr_guid(riid), ppv);
+        return E_NOINTERFACE;
     }
 
-    if(*ppv) {
-        IBindStatusCallback_AddRef(BINDSC(This));
-        return S_OK;
-    }
-
-    WARN("(%p)->(%s %p)\n", This, debugstr_guid(riid), ppv);
-    return E_NOINTERFACE;
+    IUnknown_AddRef((IUnknown*)*ppv);
+    return S_OK;
 }
 
 static ULONG WINAPI BindStatusCallback_AddRef(IBindStatusCallback *iface)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
     LONG ref = InterlockedIncrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
@@ -152,14 +168,14 @@ static ULONG WINAPI BindStatusCallback_AddRef(IBindStatusCallback *iface)
 
 static ULONG WINAPI BindStatusCallback_Release(IBindStatusCallback *iface)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
     LONG ref = InterlockedDecrement(&This->ref);
 
     TRACE("(%p) ref=%d\n", This, ref);
 
     if(!ref) {
         if(This->doc_host)
-            IOleClientSite_Release(CLIENTSITE(This->doc_host));
+            IOleClientSite_Release(&This->doc_host->IOleClientSite_iface);
         if(This->post_data)
             GlobalFree(This->post_data);
         SysFreeString(This->headers);
@@ -173,7 +189,7 @@ static ULONG WINAPI BindStatusCallback_Release(IBindStatusCallback *iface)
 static HRESULT WINAPI BindStatusCallback_OnStartBinding(IBindStatusCallback *iface,
        DWORD dwReserved, IBinding *pbind)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
 
     TRACE("(%p)->(%d %p)\n", This, dwReserved, pbind);
 
@@ -183,7 +199,7 @@ static HRESULT WINAPI BindStatusCallback_OnStartBinding(IBindStatusCallback *ifa
 static HRESULT WINAPI BindStatusCallback_GetPriority(IBindStatusCallback *iface,
        LONG *pnPriority)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
     FIXME("(%p)->(%p)\n", This, pnPriority);
     return E_NOTIMPL;
 }
@@ -191,7 +207,7 @@ static HRESULT WINAPI BindStatusCallback_GetPriority(IBindStatusCallback *iface,
 static HRESULT WINAPI BindStatusCallback_OnLowResource(IBindStatusCallback *iface,
        DWORD reserved)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
     FIXME("(%p)->(%d)\n", This, reserved);
     return E_NOTIMPL;
 }
@@ -199,12 +215,14 @@ static HRESULT WINAPI BindStatusCallback_OnLowResource(IBindStatusCallback *ifac
 static HRESULT WINAPI BindStatusCallback_OnProgress(IBindStatusCallback *iface,
         ULONG ulProgress, ULONG ulProgressMax, ULONG ulStatusCode, LPCWSTR szStatusText)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
 
     TRACE("(%p)->(%d %d %d %s)\n", This, ulProgress, ulProgressMax, ulStatusCode,
           debugstr_w(szStatusText));
 
     switch(ulStatusCode) {
+    case BINDSTATUS_REDIRECTING:
+        return set_dochost_url(This->doc_host, szStatusText);
     case BINDSTATUS_BEGINDOWNLOADDATA:
         set_status_text(This, szStatusText); /* FIXME: "Start downloading from site: %s" */
         return S_OK;
@@ -220,20 +238,20 @@ static HRESULT WINAPI BindStatusCallback_OnProgress(IBindStatusCallback *iface,
         FIXME("status code %u\n", ulStatusCode);
     }
 
-    return E_NOTIMPL;
+    return S_OK;
 }
 
 static HRESULT WINAPI BindStatusCallback_OnStopBinding(IBindStatusCallback *iface,
         HRESULT hresult, LPCWSTR szError)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
 
     TRACE("(%p)->(%08x %s)\n", This, hresult, debugstr_w(szError));
 
     set_status_text(This, emptyW);
 
     if(This->doc_host) {
-        IOleClientSite_Release(CLIENTSITE(This->doc_host));
+        IOleClientSite_Release(&This->doc_host->IOleClientSite_iface);
         This->doc_host = NULL;
     }
 
@@ -243,7 +261,7 @@ static HRESULT WINAPI BindStatusCallback_OnStopBinding(IBindStatusCallback *ifac
 static HRESULT WINAPI BindStatusCallback_GetBindInfo(IBindStatusCallback *iface,
         DWORD *grfBINDF, BINDINFO *pbindinfo)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
 
     TRACE("(%p)->(%p %p)\n", This, grfBINDF, pbindinfo);
 
@@ -255,8 +273,8 @@ static HRESULT WINAPI BindStatusCallback_GetBindInfo(IBindStatusCallback *iface,
         pbindinfo->stgmedData.tymed = TYMED_HGLOBAL;
         pbindinfo->stgmedData.u.hGlobal = This->post_data;
         pbindinfo->cbstgmedData = This->post_data_len;
-        pbindinfo->stgmedData.pUnkForRelease = (IUnknown*)BINDSC(This);
-        IBindStatusCallback_AddRef(BINDSC(This));
+        pbindinfo->stgmedData.pUnkForRelease = (IUnknown*)&This->IBindStatusCallback_iface;
+        IBindStatusCallback_AddRef(&This->IBindStatusCallback_iface);
     }
 
     return S_OK;
@@ -265,58 +283,21 @@ static HRESULT WINAPI BindStatusCallback_GetBindInfo(IBindStatusCallback *iface,
 static HRESULT WINAPI BindStatusCallback_OnDataAvailable(IBindStatusCallback *iface,
         DWORD grfBSCF, DWORD dwSize, FORMATETC *pformatetc, STGMEDIUM *pstgmed)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
     FIXME("(%p)->(%08x %d %p %p)\n", This, grfBSCF, dwSize, pformatetc, pstgmed);
     return E_NOTIMPL;
 }
 
-static void object_available_proc(DocHost *This, task_header_t *task)
-{
-    object_available(This);
-}
-
 static HRESULT WINAPI BindStatusCallback_OnObjectAvailable(IBindStatusCallback *iface,
         REFIID riid, IUnknown *punk)
 {
-    BindStatusCallback *This = BINDSC_THIS(iface);
-    task_header_t *task;
-    IOleObject *oleobj;
-    HRESULT hres;
+    BindStatusCallback *This = impl_from_IBindStatusCallback(iface);
 
     TRACE("(%p)->(%s %p)\n", This, debugstr_guid(riid), punk);
 
-    IUnknown_AddRef(punk);
-    This->doc_host->document = punk;
-
-    hres = IUnknown_QueryInterface(punk, &IID_IOleObject, (void**)&oleobj);
-    if(SUCCEEDED(hres)) {
-        CLSID clsid;
-
-        hres = IOleObject_GetUserClassID(oleobj, &clsid);
-        if(SUCCEEDED(hres))
-            TRACE("Got clsid %s\n",
-                  IsEqualGUID(&clsid, &CLSID_HTMLDocument) ? "CLSID_HTMLDocument" : debugstr_guid(&clsid));
-
-        hres = IOleObject_SetClientSite(oleobj, CLIENTSITE(This->doc_host));
-        if(FAILED(hres))
-            FIXME("SetClientSite failed: %08x\n", hres);
-
-        IOleObject_Release(oleobj);
-    }else {
-        FIXME("Could not get IOleObject iface: %08x\n", hres);
-    }
-
-    /* FIXME: Call SetAdvise */
-    /* FIXME: Call Invoke(DISPID_READYSTATE) */
-
-    task = heap_alloc(sizeof(*task));
-    push_dochost_task(This->doc_host, task, object_available_proc, FALSE);
-
-    return S_OK;
+    return dochost_object_available(This->doc_host, punk);
 }
 
-#undef BSC_THIS
-
 static const IBindStatusCallbackVtbl BindStatusCallbackVtbl = {
     BindStatusCallback_QueryInterface,
     BindStatusCallback_AddRef,
@@ -331,33 +312,36 @@ static const IBindStatusCallbackVtbl BindStatusCallbackVtbl = {
     BindStatusCallback_OnObjectAvailable
 };
 
-#define HTTPNEG_THIS(iface) DEFINE_THIS(BindStatusCallback, HttpNegotiate, iface)
+static inline BindStatusCallback *impl_from_IHttpNegotiate(IHttpNegotiate *iface)
+{
+    return CONTAINING_RECORD(iface, BindStatusCallback, IHttpNegotiate_iface);
+}
 
 static HRESULT WINAPI HttpNegotiate_QueryInterface(IHttpNegotiate *iface,
                                                    REFIID riid, void **ppv)
 {
-    BindStatusCallback *This = HTTPNEG_THIS(iface);
-    return IBindStatusCallback_QueryInterface(BINDSC(This), riid, ppv);
+    BindStatusCallback *This = impl_from_IHttpNegotiate(iface);
+    return IBindStatusCallback_QueryInterface(&This->IBindStatusCallback_iface, riid, ppv);
 }
 
 static ULONG WINAPI HttpNegotiate_AddRef(IHttpNegotiate *iface)
 {
-    BindStatusCallback *This = HTTPNEG_THIS(iface);
-    return IBindStatusCallback_AddRef(BINDSC(This));
+    BindStatusCallback *This = impl_from_IHttpNegotiate(iface);
+    return IBindStatusCallback_AddRef(&This->IBindStatusCallback_iface);
 }
 
 static ULONG WINAPI HttpNegotiate_Release(IHttpNegotiate *iface)
 {
-    BindStatusCallback *This = HTTPNEG_THIS(iface);
-    return IBindStatusCallback_Release(BINDSC(This));
+    BindStatusCallback *This = impl_from_IHttpNegotiate(iface);
+    return IBindStatusCallback_Release(&This->IBindStatusCallback_iface);
 }
 
 static HRESULT WINAPI HttpNegotiate_BeginningTransaction(IHttpNegotiate *iface,
         LPCWSTR szURL, LPCWSTR szHeaders, DWORD dwReserved, LPWSTR *pszAdditionalHeaders)
 {
-    BindStatusCallback *This = HTTPNEG_THIS(iface);
+    BindStatusCallback *This = impl_from_IHttpNegotiate(iface);
 
-    FIXME("(%p)->(%s %s %d %p)\n", This, debugstr_w(szURL), debugstr_w(szHeaders),
+    TRACE("(%p)->(%s %s %d %p)\n", This, debugstr_w(szURL), debugstr_w(szHeaders),
           dwReserved, pszAdditionalHeaders);
 
     if(This->headers) {
@@ -373,14 +357,12 @@ static HRESULT WINAPI HttpNegotiate_OnResponse(IHttpNegotiate *iface,
         DWORD dwResponseCode, LPCWSTR szResponseHeaders, LPCWSTR szRequestHeaders,
         LPWSTR *pszAdditionalRequestHeaders)
 {
-    BindStatusCallback *This = HTTPNEG_THIS(iface);
-    FIXME("(%p)->(%d %s %s %p)\n", This, dwResponseCode, debugstr_w(szResponseHeaders),
+    BindStatusCallback *This = impl_from_IHttpNegotiate(iface);
+    TRACE("(%p)->(%d %s %s %p)\n", This, dwResponseCode, debugstr_w(szResponseHeaders),
           debugstr_w(szRequestHeaders), pszAdditionalRequestHeaders);
-    return E_NOTIMPL;
+    return S_OK;
 }
 
-#undef HTTPNEG_THIS
-
 static const IHttpNegotiateVtbl HttpNegotiateVtbl = {
     HttpNegotiate_QueryInterface,
     HttpNegotiate_AddRef,
@@ -394,8 +376,8 @@ static BindStatusCallback *create_callback(DocHost *doc_host, LPCWSTR url, PBYTE
 {
     BindStatusCallback *ret = heap_alloc(sizeof(BindStatusCallback));
 
-    ret->lpBindStatusCallbackVtbl = &BindStatusCallbackVtbl;
-    ret->lpHttpNegotiateVtbl      = &HttpNegotiateVtbl;
+    ret->IBindStatusCallback_iface.lpVtbl = &BindStatusCallbackVtbl;
+    ret->IHttpNegotiate_iface.lpVtbl      = &HttpNegotiateVtbl;
 
     ret->ref = 1;
     ret->url = heap_strdupW(url);
@@ -404,7 +386,7 @@ static BindStatusCallback *create_callback(DocHost *doc_host, LPCWSTR url, PBYTE
     ret->headers = headers ? SysAllocString(headers) : NULL;
 
     ret->doc_host = doc_host;
-    IOleClientSite_AddRef(CLIENTSITE(doc_host));
+    IOleClientSite_AddRef(&doc_host->IOleClientSite_iface);
 
     if(post_data) {
         ret->post_data = GlobalAlloc(0, post_data_len);
@@ -414,8 +396,7 @@ static BindStatusCallback *create_callback(DocHost *doc_host, LPCWSTR url, PBYTE
     return ret;
 }
 
-static void on_before_navigate2(DocHost *This, LPCWSTR url, const BYTE *post_data,
-                                ULONG post_data_len, LPWSTR headers, VARIANT_BOOL *cancel)
+static void on_before_navigate2(DocHost *This, LPCWSTR url, SAFEARRAY *post_data, LPWSTR headers, VARIANT_BOOL *cancel)
 {
     VARIANT var_url, var_flags, var_frame_name, var_post_data, var_post_data2, var_headers;
     DISPPARAMS dispparams;
@@ -440,18 +421,12 @@ static void on_before_navigate2(DocHost *This, LPCWSTR url, const BYTE *post_dat
     V_VARIANTREF(params+2) = &var_post_data2;
     V_VT(&var_post_data2) = (VT_BYREF|VT_VARIANT);
     V_VARIANTREF(&var_post_data2) = &var_post_data;
-    VariantInit(&var_post_data);
-
-    if(post_data_len) {
-        SAFEARRAYBOUND bound = {post_data_len, 0};
-        void *data;
 
+    if(post_data) {
         V_VT(&var_post_data) = VT_UI1|VT_ARRAY;
-        V_ARRAY(&var_post_data) = SafeArrayCreate(VT_UI1, 1, &bound);
-
-        SafeArrayAccessData(V_ARRAY(&var_post_data), &data);
-        memcpy(data, post_data, post_data_len);
-        SafeArrayUnaccessData(V_ARRAY(&var_post_data));
+        V_ARRAY(&var_post_data) = post_data;
+    }else {
+        V_VT(&var_post_data) = VT_EMPTY;
     }
 
     V_VT(params+3) = (VT_BYREF|VT_VARIANT);
@@ -475,8 +450,6 @@ static void on_before_navigate2(DocHost *This, LPCWSTR url, const BYTE *post_dat
     call_sink(This->cps.wbe2, DISPID_BEFORENAVIGATE2, &dispparams);
 
     SysFreeString(V_BSTR(&var_url));
-    if(post_data_len)
-        SafeArrayDestroy(V_ARRAY(&var_post_data));
 }
 
 /* FIXME: urlmon should handle it */
@@ -513,46 +486,6 @@ static BOOL try_application_url(LPCWSTR url)
     return ShellExecuteExW(&exec_info);
 }
 
-static HRESULT http_load_hack(DocHost *This, IMoniker *mon, IBindStatusCallback *callback, IBindCtx *bindctx)
-{
-    IPersistMoniker *persist;
-    IUnknown *doc;
-    HRESULT hres;
-
-    /*
-     * FIXME:
-     * We should use URLMoniker's BindToObject instead creating HTMLDocument here.
-     * This should be fixed when mshtml.dll and urlmon.dll will be good enough.
-     */
-
-    hres = CoCreateInstance(&CLSID_HTMLDocument, NULL,
-                            CLSCTX_INPROC_SERVER|CLSCTX_INPROC_HANDLER,
-                            &IID_IUnknown, (void**)&doc);
-
-    if(FAILED(hres)) {
-        ERR("Could not create HTMLDocument: %08x\n", hres);
-        return hres;
-    }
-
-    hres = IUnknown_QueryInterface(doc, &IID_IPersistMoniker, (void**)&persist);
-    if(FAILED(hres)) {
-        IUnknown_Release(doc);
-        return hres;
-    }
-
-    hres = IPersistMoniker_Load(persist, FALSE, mon, bindctx, 0);
-    IPersistMoniker_Release(persist);
-
-    if(SUCCEEDED(hres))
-        hres = IBindStatusCallback_OnObjectAvailable(callback, &IID_IUnknown, doc);
-    else
-        WARN("Load failed: %08x\n", hres);
-
-    IUnknown_Release(doc);
-
-    return IBindStatusCallback_OnStopBinding(callback, hres, NULL);
-}
-
 static HRESULT create_moniker(LPCWSTR url, IMoniker **mon)
 {
     WCHAR new_url[INTERNET_MAX_URL_LENGTH];
@@ -570,7 +503,7 @@ static HRESULT create_moniker(LPCWSTR url, IMoniker **mon)
             return hres;
         }
     }else {
-        size = sizeof(new_url);
+        size = sizeof(new_url)/sizeof(WCHAR);
         hres = UrlApplySchemeW(url, new_url, &size, URL_APPLY_GUESSSCHEME);
         TRACE("got %s\n", debugstr_w(new_url));
         if(FAILED(hres)) {
@@ -585,14 +518,10 @@ static HRESULT create_moniker(LPCWSTR url, IMoniker **mon)
 static HRESULT bind_to_object(DocHost *This, IMoniker *mon, LPCWSTR url, IBindCtx *bindctx,
                               IBindStatusCallback *callback)
 {
-    WCHAR schema[30];
-    DWORD schema_len;
+    IUnknown *unk = NULL;
+    WCHAR *display_name;
     HRESULT hres;
 
-    static const WCHAR httpW[] = {'h','t','t','p',0};
-    static const WCHAR httpsW[] = {'h','t','t','p','s',0};
-    static const WCHAR ftpW[]= {'f','t','p',0};
-
     if(mon) {
         IMoniker_AddRef(mon);
     }else {
@@ -601,45 +530,176 @@ static HRESULT bind_to_object(DocHost *This, IMoniker *mon, LPCWSTR url, IBindCt
             return hres;
     }
 
-    CoTaskMemFree(This->url);
-    hres = IMoniker_GetDisplayName(mon, 0, NULL, &This->url);
-    if(FAILED(hres))
+    hres = IMoniker_GetDisplayName(mon, 0, NULL, &display_name);
+    if(FAILED(hres)) {
         FIXME("GetDisplayName failed: %08x\n", hres);
+        return hres;
+    }
+
+    hres = set_dochost_url(This, display_name);
+    CoTaskMemFree(display_name);
+    if(FAILED(hres))
+        return hres;
 
     IBindCtx_RegisterObjectParam(bindctx, (LPOLESTR)SZ_HTML_CLIENTSITE_OBJECTPARAM,
-                                 (IUnknown*)CLIENTSITE(This));
+                                 (IUnknown*)&This->IOleClientSite_iface);
 
-    hres = CoInternetParseUrl(This->url, PARSE_SCHEMA, 0, schema, sizeof(schema)/sizeof(schema[0]),
-            &schema_len, 0);
-    if(SUCCEEDED(hres) &&
-       (!strcmpW(schema, httpW) || !strcmpW(schema, httpsW) || !strcmpW(schema, ftpW))) {
-        hres = http_load_hack(This, mon, callback, bindctx);
+    hres = IMoniker_BindToObject(mon, bindctx, NULL, &IID_IUnknown, (void**)&unk);
+    if(SUCCEEDED(hres)) {
+        hres = S_OK;
+        if(unk)
+            IUnknown_Release(unk);
+    }else if(try_application_url(url)) {
+        hres = S_OK;
     }else {
-        IUnknown *unk = NULL;
-
-        hres = IMoniker_BindToObject(mon, bindctx, NULL, &IID_IUnknown, (void**)&unk);
-        if(SUCCEEDED(hres)) {
-            hres = S_OK;
-            if(unk)
-                IUnknown_Release(unk);
-        }else if(try_application_url(url)) {
-            hres = S_OK;
-        }else {
-            FIXME("BindToObject failed: %08x\n", hres);
-        }
+        FIXME("BindToObject failed: %08x\n", hres);
     }
 
     IMoniker_Release(mon);
     return S_OK;
 }
 
-static HRESULT navigate_bsc(DocHost *This, BindStatusCallback *bsc, IMoniker *mon)
+static void html_window_navigate(DocHost *This, IHTMLPrivateWindow *window, BSTR url, BSTR headers, SAFEARRAY *post_data)
 {
-    IBindCtx *bindctx;
-    VARIANT_BOOL cancel = VARIANT_FALSE;
+    VARIANT headers_var, post_data_var;
+    BSTR empty_str;
     HRESULT hres;
 
-    on_before_navigate2(This, bsc->url, bsc->post_data, bsc->post_data_len, bsc->headers, &cancel);
+    hres = set_dochost_url(This, url);
+    if(FAILED(hres))
+        return;
+
+    empty_str = SysAllocStringLen(NULL, 0);
+
+    if(headers) {
+        V_VT(&headers_var) = VT_BSTR;
+        V_BSTR(&headers_var) = headers;
+    }else {
+        V_VT(&headers_var) = VT_EMPTY;
+    }
+
+    if(post_data) {
+        V_VT(&post_data_var) = VT_UI1|VT_ARRAY;
+        V_ARRAY(&post_data_var) = post_data;
+    }else {
+        V_VT(&post_data_var) = VT_EMPTY;
+    }
+
+    set_doc_state(This, READYSTATE_LOADING);
+    hres = IHTMLPrivateWindow_SuperNavigate(window, url, empty_str, NULL, NULL, &post_data_var, &headers_var, 0);
+    SysFreeString(empty_str);
+    if(FAILED(hres))
+        WARN("SuprtNavigate failed: %08x\n", hres);
+}
+
+typedef struct {
+    task_header_t header;
+    BSTR url;
+    BSTR headers;
+    SAFEARRAY *post_data;
+    BOOL async_notif;
+} task_doc_navigate_t;
+
+static HRESULT free_doc_navigate_task(task_doc_navigate_t *task, BOOL free_task)
+{
+    SysFreeString(task->url);
+    SysFreeString(task->headers);
+    if(task->post_data)
+        SafeArrayDestroy(task->post_data);
+    if(free_task)
+        heap_free(task);
+    return E_OUTOFMEMORY;
+}
+
+static void doc_navigate_proc(DocHost *This, task_header_t *t)
+{
+    task_doc_navigate_t *task = (task_doc_navigate_t*)t;
+    IHTMLPrivateWindow *priv_window;
+    HRESULT hres;
+
+    if(!This->doc_navigate)
+        return;
+
+    if(task->async_notif) {
+        VARIANT_BOOL cancel = VARIANT_FALSE;
+        on_before_navigate2(This, task->url, task->post_data, task->headers, &cancel);
+        if(cancel) {
+            TRACE("Navigation calnceled\n");
+            free_doc_navigate_task(task, FALSE);
+            return;
+        }
+    }
+
+    hres = IUnknown_QueryInterface(This->doc_navigate, &IID_IHTMLPrivateWindow, (void**)&priv_window);
+    if(SUCCEEDED(hres)) {
+        html_window_navigate(This, priv_window, task->url, task->headers, task->post_data);
+        IHTMLPrivateWindow_Release(priv_window);
+    }else {
+        WARN("Could not get IHTMLPrivateWindow iface: %08x\n", hres);
+    }
+
+    free_doc_navigate_task(task, FALSE);
+}
+
+static HRESULT async_doc_navigate(DocHost *This, LPCWSTR url, LPCWSTR headers, PBYTE post_data, ULONG post_data_size,
+        BOOL async_notif)
+{
+    task_doc_navigate_t *task;
+
+    task = heap_alloc_zero(sizeof(*task));
+    if(!task)
+        return E_OUTOFMEMORY;
+
+    task->url = SysAllocString(url);
+    if(!task->url)
+        return free_doc_navigate_task(task, TRUE);
+
+    if(headers) {
+        task->headers = SysAllocString(headers);
+        if(!task->headers)
+            return free_doc_navigate_task(task, TRUE);
+    }
+
+    if(post_data) {
+        task->post_data = SafeArrayCreateVector(VT_UI1, 0, post_data_size);
+        if(!task->post_data)
+            return free_doc_navigate_task(task, TRUE);
+        memcpy(task->post_data->pvData, post_data, post_data_size);
+    }
+
+    if(!async_notif) {
+        VARIANT_BOOL cancel = VARIANT_FALSE;
+
+        on_before_navigate2(This, task->url, task->post_data, task->headers, &cancel);
+        if(cancel) {
+            TRACE("Navigation calnceled\n");
+            free_doc_navigate_task(task, TRUE);
+            return S_OK;
+        }
+    }
+
+    task->async_notif = async_notif;
+    push_dochost_task(This, &task->header, doc_navigate_proc, FALSE);
+    return S_OK;
+}
+
+static HRESULT navigate_bsc(DocHost *This, BindStatusCallback *bsc, IMoniker *mon)
+{
+    VARIANT_BOOL cancel = VARIANT_FALSE;
+    SAFEARRAY *post_data = NULL;
+    IBindCtx *bindctx;
+    HRESULT hres;
+
+    set_doc_state(This, READYSTATE_LOADING);
+
+    if(bsc->post_data) {
+        post_data = SafeArrayCreateVector(VT_UI1, 0, bsc->post_data_len);
+        memcpy(post_data->pvData, post_data, bsc->post_data_len);
+    }
+
+    on_before_navigate2(This, bsc->url, post_data, bsc->headers, &cancel);
+    if(post_data)
+        SafeArrayDestroy(post_data);
     if(cancel) {
         FIXME("Navigation canceled\n");
         return S_OK;
@@ -648,12 +708,12 @@ static HRESULT navigate_bsc(DocHost *This, BindStatusCallback *bsc, IMoniker *mo
     if(This->document)
         deactivate_document(This);
 
-    CreateAsyncBindCtx(0, BINDSC(bsc), 0, &bindctx);
+    CreateAsyncBindCtx(0, &bsc->IBindStatusCallback_iface, 0, &bindctx);
 
     if(This->frame)
         IOleInPlaceFrame_EnableModeless(This->frame, FALSE);
 
-    hres = bind_to_object(This, mon, bsc->url, bindctx, BINDSC(bsc));
+    hres = bind_to_object(This, mon, bsc->url, bindctx, &bsc->IBindStatusCallback_iface);
 
     if(This->frame)
         IOleInPlaceFrame_EnableModeless(This->frame, TRUE);
@@ -677,17 +737,17 @@ static void navigate_bsc_proc(DocHost *This, task_header_t *t)
 
     navigate_bsc(This, task->bsc, NULL);
 
-    IBindStatusCallback_Release(BINDSC(task->bsc));
+    IBindStatusCallback_Release(&task->bsc->IBindStatusCallback_iface);
 }
 
 
 HRESULT navigate_url(DocHost *This, LPCWSTR url, const VARIANT *Flags,
                      const VARIANT *TargetFrameName, VARIANT *PostData, VARIANT *Headers)
 {
-    task_navigate_bsc_t *task;
     PBYTE post_data = NULL;
     ULONG post_data_len = 0;
     LPWSTR headers = NULL;
+    HRESULT hres = S_OK;
 
     TRACE("navigating to %s\n", debugstr_w(url));
 
@@ -697,32 +757,50 @@ HRESULT navigate_url(DocHost *This, LPCWSTR url, const VARIANT *Flags,
                 Flags, Flags ? V_VT(Flags) : -1, TargetFrameName,
                 TargetFrameName ? V_VT(TargetFrameName) : -1);
 
-    if(PostData) {
-        TRACE("PostData vt=%d\n", V_VT(PostData));
-
-        if(V_VT(PostData) == (VT_ARRAY | VT_UI1)) {
-            SafeArrayAccessData(V_ARRAY(PostData), (void**)&post_data);
-            post_data_len = V_ARRAY(PostData)->rgsabound[0].cElements;
-        }
+    if(PostData && V_VT(PostData) == (VT_ARRAY | VT_UI1) && V_ARRAY(PostData)) {
+        SafeArrayAccessData(V_ARRAY(PostData), (void**)&post_data);
+        post_data_len = V_ARRAY(PostData)->rgsabound[0].cElements;
     }
 
-    if(Headers && V_VT(Headers) != VT_EMPTY && V_VT(Headers) != VT_ERROR) {
-        if(V_VT(Headers) != VT_BSTR)
-            return E_INVALIDARG;
-
+    if(Headers && V_VT(Headers) == VT_BSTR) {
         headers = V_BSTR(Headers);
         TRACE("Headers: %s\n", debugstr_w(headers));
     }
 
-    task = heap_alloc(sizeof(*task));
-    task->bsc = create_callback(This, url, post_data, post_data_len, headers);
+    set_doc_state(This, READYSTATE_LOADING);
+    This->ready_state = READYSTATE_LOADING;
+
+    if(This->doc_navigate) {
+        WCHAR new_url[INTERNET_MAX_URL_LENGTH];
+
+        if(PathIsURLW(url)) {
+            new_url[0] = 0;
+        }else {
+            DWORD size;
+
+            size = sizeof(new_url)/sizeof(WCHAR);
+            hres = UrlApplySchemeW(url, new_url, &size,
+                    URL_APPLY_GUESSSCHEME | URL_APPLY_DEFAULT);
+            if(FAILED(hres)) {
+                WARN("UrlApplyScheme failed: %08x\n", hres);
+                new_url[0] = 0;
+            }
+        }
+
+        hres = async_doc_navigate(This, *new_url ? new_url : url, headers, post_data,
+                post_data_len, TRUE);
+    }else {
+        task_navigate_bsc_t *task;
+
+        task = heap_alloc(sizeof(*task));
+        task->bsc = create_callback(This, url, post_data, post_data_len, headers);
+        push_dochost_task(This, &task->header, navigate_bsc_proc, This->url == NULL);
+    }
 
     if(post_data)
         SafeArrayUnaccessData(V_ARRAY(PostData));
 
-    push_dochost_task(This, &task->header, navigate_bsc_proc, This->url == NULL);
-
-    return S_OK;
+    return hres;
 }
 
 static HRESULT navigate_hlink(DocHost *This, IMoniker *mon, IBindCtx *bindctx,
@@ -737,6 +815,10 @@ static HRESULT navigate_hlink(DocHost *This, IMoniker *mon, IBindCtx *bindctx,
     DWORD bindf = 0;
     HRESULT hres;
 
+    hres = IMoniker_GetDisplayName(mon, 0, NULL, &url);
+    if(FAILED(hres))
+        FIXME("GetDisplayName failed: %08x\n", hres);
+
     hres = IBindStatusCallback_QueryInterface(callback, &IID_IHttpNegotiate,
                                               (void**)&http_negotiate);
     if(SUCCEEDED(hres)) {
@@ -758,16 +840,15 @@ static HRESULT navigate_hlink(DocHost *This, IMoniker *mon, IBindCtx *bindctx,
             post_data = bindinfo.stgmedData.u.hGlobal;
     }
 
-    hres = IMoniker_GetDisplayName(mon, 0, NULL, &url);
-    if(FAILED(hres))
-        FIXME("GetDisplayName failed: %08x\n", hres);
+    if(This->doc_navigate) {
+        hres = async_doc_navigate(This, url, headers, post_data, post_data_len, FALSE);
+    }else {
+        bsc = create_callback(This, url, post_data, post_data_len, headers);
+        hres = navigate_bsc(This, bsc, mon);
+        IBindStatusCallback_Release(&bsc->IBindStatusCallback_iface);
+    }
 
-    bsc = create_callback(This, url, post_data, post_data_len, headers);
     CoTaskMemFree(url);
-
-    hres = navigate_bsc(This, bsc, mon);
-
-    IBindStatusCallback_Release(BINDSC(bsc));
     CoTaskMemFree(headers);
     ReleaseBindInfo(&bindinfo);
 
@@ -799,30 +880,33 @@ HRESULT go_home(DocHost *This)
     return navigate_url(This, wszPageName, NULL, NULL, NULL, NULL);
 }
 
-#define HLINKFRAME_THIS(iface) DEFINE_THIS(WebBrowser, HlinkFrame, iface)
+static inline HlinkFrame *impl_from_IHlinkFrame(IHlinkFrame *iface)
+{
+    return CONTAINING_RECORD(iface, HlinkFrame, IHlinkFrame_iface);
+}
 
 static HRESULT WINAPI HlinkFrame_QueryInterface(IHlinkFrame *iface, REFIID riid, void **ppv)
 {
-    WebBrowser *This = HLINKFRAME_THIS(iface);
-    return IWebBrowser2_QueryInterface(WEBBROWSER2(This), riid, ppv);
+    HlinkFrame *This = impl_from_IHlinkFrame(iface);
+    return IUnknown_QueryInterface(This->outer, riid, ppv);
 }
 
 static ULONG WINAPI HlinkFrame_AddRef(IHlinkFrame *iface)
 {
-    WebBrowser *This = HLINKFRAME_THIS(iface);
-    return IWebBrowser2_AddRef(WEBBROWSER2(This));
+    HlinkFrame *This = impl_from_IHlinkFrame(iface);
+    return IUnknown_AddRef(This->outer);
 }
 
 static ULONG WINAPI HlinkFrame_Release(IHlinkFrame *iface)
 {
-    WebBrowser *This = HLINKFRAME_THIS(iface);
-    return IWebBrowser2_Release(WEBBROWSER2(This));
+    HlinkFrame *This = impl_from_IHlinkFrame(iface);
+    return IUnknown_Release(This->outer);
 }
 
 static HRESULT WINAPI HlinkFrame_SetBrowseContext(IHlinkFrame *iface,
                                                   IHlinkBrowseContext *pihlbc)
 {
-    WebBrowser *This = HLINKFRAME_THIS(iface);
+    HlinkFrame *This = impl_from_IHlinkFrame(iface);
     FIXME("(%p)->(%p)\n", This, pihlbc);
     return E_NOTIMPL;
 }
@@ -830,7 +914,7 @@ static HRESULT WINAPI HlinkFrame_SetBrowseContext(IHlinkFrame *iface,
 static HRESULT WINAPI HlinkFrame_GetBrowseContext(IHlinkFrame *iface,
                                                   IHlinkBrowseContext **ppihlbc)
 {
-    WebBrowser *This = HLINKFRAME_THIS(iface);
+    HlinkFrame *This = impl_from_IHlinkFrame(iface);
     FIXME("(%p)->(%p)\n", This, ppihlbc);
     return E_NOTIMPL;
 }
@@ -838,7 +922,7 @@ static HRESULT WINAPI HlinkFrame_GetBrowseContext(IHlinkFrame *iface,
 static HRESULT WINAPI HlinkFrame_Navigate(IHlinkFrame *iface, DWORD grfHLNF, LPBC pbc,
                                           IBindStatusCallback *pibsc, IHlink *pihlNavigate)
 {
-    WebBrowser *This = HLINKFRAME_THIS(iface);
+    HlinkFrame *This = impl_from_IHlinkFrame(iface);
     IMoniker *mon;
     LPWSTR location = NULL;
 
@@ -863,13 +947,13 @@ static HRESULT WINAPI HlinkFrame_Navigate(IHlinkFrame *iface, DWORD grfHLNF, LPB
         return E_NOTIMPL;
     }
 
-    return navigate_hlink(&This->doc_host, mon, pbc, pibsc);
+    return navigate_hlink(This->doc_host, mon, pbc, pibsc);
 }
 
 static HRESULT WINAPI HlinkFrame_OnNavigate(IHlinkFrame *iface, DWORD grfHLNF,
         IMoniker *pimkTarget, LPCWSTR pwzLocation, LPCWSTR pwzFriendlyName, DWORD dwreserved)
 {
-    WebBrowser *This = HLINKFRAME_THIS(iface);
+    HlinkFrame *This = impl_from_IHlinkFrame(iface);
     FIXME("(%p)->(%08x %p %s %s %d)\n", This, grfHLNF, pimkTarget, debugstr_w(pwzLocation),
           debugstr_w(pwzFriendlyName), dwreserved);
     return E_NOTIMPL;
@@ -878,14 +962,12 @@ static HRESULT WINAPI HlinkFrame_OnNavigate(IHlinkFrame *iface, DWORD grfHLNF,
 static HRESULT WINAPI HlinkFrame_UpdateHlink(IHlinkFrame *iface, ULONG uHLID,
         IMoniker *pimkTarget, LPCWSTR pwzLocation, LPCWSTR pwzFriendlyName)
 {
-    WebBrowser *This = HLINKFRAME_THIS(iface);
+    HlinkFrame *This = impl_from_IHlinkFrame(iface);
     FIXME("(%p)->(%u %p %s %s)\n", This, uHLID, pimkTarget, debugstr_w(pwzLocation),
           debugstr_w(pwzFriendlyName));
     return E_NOTIMPL;
 }
 
-#undef HLINKFRAME_THIS
-
 static const IHlinkFrameVtbl HlinkFrameVtbl = {
     HlinkFrame_QueryInterface,
     HlinkFrame_AddRef,
@@ -897,7 +979,152 @@ static const IHlinkFrameVtbl HlinkFrameVtbl = {
     HlinkFrame_UpdateHlink
 };
 
-void WebBrowser_HlinkFrame_Init(WebBrowser *This)
+static inline HlinkFrame *impl_from_ITargetFrame2(ITargetFrame2 *iface)
 {
-    This->lpHlinkFrameVtbl = &HlinkFrameVtbl;
+    return CONTAINING_RECORD(iface, HlinkFrame, IHlinkFrame_iface);
+}
+
+static HRESULT WINAPI TargetFrame2_QueryInterface(ITargetFrame2 *iface, REFIID riid, void **ppv)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    return IUnknown_QueryInterface(This->outer, riid, ppv);
+}
+
+static ULONG WINAPI TargetFrame2_AddRef(ITargetFrame2 *iface)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    return IUnknown_AddRef(This->outer);
+}
+
+static ULONG WINAPI TargetFrame2_Release(ITargetFrame2 *iface)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    return IUnknown_Release(This->outer);
+}
+
+static HRESULT WINAPI TargetFrame2_SetFrameName(ITargetFrame2 *iface, LPCWSTR pszFrameName)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%s)\n", This, debugstr_w(pszFrameName));
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_GetFrameName(ITargetFrame2 *iface, LPWSTR *ppszFrameName)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%p)\n", This, ppszFrameName);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_GetParentFrame(ITargetFrame2 *iface, IUnknown **ppunkParent)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%p)\n", This, ppunkParent);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_SetFrameSrc(ITargetFrame2 *iface, LPCWSTR pszFrameSrc)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%s)\n", This, debugstr_w(pszFrameSrc));
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_GetFrameSrc(ITargetFrame2 *iface, LPWSTR *ppszFrameSrc)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->()\n", This);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_GetFramesContainer(ITargetFrame2 *iface, IOleContainer **ppContainer)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%p)\n", This, ppContainer);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_SetFrameOptions(ITargetFrame2 *iface, DWORD dwFlags)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%x)\n", This, dwFlags);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_GetFrameOptions(ITargetFrame2 *iface, DWORD *pdwFlags)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%p)\n", This, pdwFlags);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_SetFrameMargins(ITargetFrame2 *iface, DWORD dwWidth, DWORD dwHeight)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%d %d)\n", This, dwWidth, dwHeight);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_GetFrameMargins(ITargetFrame2 *iface, DWORD *pdwWidth, DWORD *pdwHeight)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%p %p)\n", This, pdwWidth, pdwHeight);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_FindFrame(ITargetFrame2 *iface, LPCWSTR pszTargetName, DWORD dwFlags, IUnknown **ppunkTargetFrame)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%s %x %p)\n", This, debugstr_w(pszTargetName), dwFlags, ppunkTargetFrame);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI TargetFrame2_GetTargetAlias(ITargetFrame2 *iface, LPCWSTR pszTargetName, LPWSTR *ppszTargetAlias)
+{
+    HlinkFrame *This = impl_from_ITargetFrame2(iface);
+    FIXME("(%p)->(%s %p)\n", This, debugstr_w(pszTargetName), ppszTargetAlias);
+    return E_NOTIMPL;
+}
+
+static const ITargetFrame2Vtbl TargetFrame2Vtbl = {
+    TargetFrame2_QueryInterface,
+    TargetFrame2_AddRef,
+    TargetFrame2_Release,
+    TargetFrame2_SetFrameName,
+    TargetFrame2_GetFrameName,
+    TargetFrame2_GetParentFrame,
+    TargetFrame2_SetFrameSrc,
+    TargetFrame2_GetFrameSrc,
+    TargetFrame2_GetFramesContainer,
+    TargetFrame2_SetFrameOptions,
+    TargetFrame2_GetFrameOptions,
+    TargetFrame2_SetFrameMargins,
+    TargetFrame2_GetFrameMargins,
+    TargetFrame2_FindFrame,
+    TargetFrame2_GetTargetAlias
+};
+
+BOOL HlinkFrame_QI(HlinkFrame *This, REFIID riid, void **ppv)
+{
+    if(IsEqualGUID(&IID_IHlinkFrame, riid)) {
+        TRACE("(%p)->(IID_IHlinkFrame %p)\n", This, ppv);
+        *ppv = &This->IHlinkFrame_iface;
+    }else if(IsEqualGUID(&IID_ITargetFrame2, riid)) {
+        TRACE("(%p)->(IID_ITargetFrame2 %p)\n", This, ppv);
+        *ppv = &This->ITargetFrame2_iface;
+    }else {
+        return FALSE;
+    }
+
+    IUnknown_AddRef((IUnknown*)*ppv);
+    return TRUE;
+}
+
+void HlinkFrame_Init(HlinkFrame *This, IUnknown *outer, DocHost *doc_host)
+{
+    This->IHlinkFrame_iface.lpVtbl   = &HlinkFrameVtbl;
+    This->ITargetFrame2_iface.lpVtbl = &TargetFrame2Vtbl;
+
+    This->outer = outer;
+    This->doc_host = doc_host;
 }
