@@ -200,7 +200,7 @@ static void InitializeTreeView( browse_info *info )
 
     TRACE("%p\n", info );
     
-    Shell_GetImageList(NULL, &hImageList);
+    Shell_GetImageLists(NULL, &hImageList);
 
     if (hImageList)
         SendMessageW( info->hwndTreeView, TVM_SETIMAGELIST, 0, (LPARAM)hImageList );
@@ -239,7 +239,7 @@ static void InitializeTreeView( browse_info *info )
         return;
     }
 
-    if (pidlChild && pidlChild->mkid.cb) {
+    if (!_ILIsEmpty(pidlChild)) {
         hr = IShellFolder_BindToObject(lpsfParent, pidlChild, 0, &IID_IShellFolder, (LPVOID*)&lpsfRoot);
     } else {
         lpsfRoot = lpsfParent;
@@ -490,7 +490,7 @@ static void BrsFolder_CheckValidSelection( browse_info *info, LPTV_ITEMDATA lptv
             bEnabled = FALSE;
         }
     }
-    SendMessageW(info->hWnd, BFFM_ENABLEOK, 0, (LPARAM)bEnabled);
+    SendMessageW(info->hWnd, BFFM_ENABLEOK, 0, bEnabled);
 }
 
 static LRESULT BrsFolder_Treeview_Delete( browse_info *info, NMTREEVIEWW *pnmtv )
@@ -519,7 +519,7 @@ static LRESULT BrsFolder_Treeview_Expand( browse_info *info, NMTREEVIEWW *pnmtv
     if ((pnmtv->itemNew.state & TVIS_EXPANDEDONCE))
         return 0;
 
-    if (lptvid->lpi && lptvid->lpi->mkid.cb) {
+    if (!_ILIsEmpty(lptvid->lpi)) {
         r = IShellFolder_BindToObject( lptvid->lpsfParent, lptvid->lpi, 0,
                                        &IID_IShellFolder, (LPVOID *)&lpsf2 );
     } else {
