@@ -368,8 +368,8 @@ static inline BOOL UNIXFS_is_pidl_of_type(LPCITEMIDLIST pIDL, SHCONTF fFilter) {
  */
 static BOOL UNIXFS_get_unix_path(LPCWSTR pszDosPath, char *pszCanonicalPath)
 {
-    char *pPathTail, *pElement, *pCanonicalTail, szPath[FILENAME_MAX], *pszUnixPath;
-    WCHAR wszDrive[] = { '?', ':', '\\', 0 };
+    char *pPathTail, *pElement, *pCanonicalTail, szPath[FILENAME_MAX], *pszUnixPath, has_failed = 0, mb_path[FILENAME_MAX];
+    WCHAR wszDrive[] = { '?', ':', '\\', 0 }, dospath[PATH_MAX], *dospath_end;
     int cDriveSymlinkLen;
     
     TRACE("(pszDosPath=%s, pszCanonicalPath=%p)\n", debugstr_w(pszDosPath), pszCanonicalPath);
@@ -388,11 +388,34 @@ static BOOL UNIXFS_get_unix_path(LPCWSTR pszDosPath, char *pszCanonicalPath)
     if (szPath[strlen(szPath)-1] != '/') strcat(szPath, "/");
 
     /* Append the part relative to the drive symbolic link target. */
-    pszUnixPath = wine_get_unix_file_name(pszDosPath);
-    if (!pszUnixPath) return FALSE;
+    lstrcpyW(dospath, pszDosPath);
+    dospath_end = dospath + lstrlenW(dospath);
+    /* search for the most valid UNIX path possible, then append missing
+     * path parts */
+    while(!(pszUnixPath = wine_get_unix_file_name(dospath))){
+        if(has_failed){
+            *dospath_end = '/';
+            --dospath_end;
+        }else
+            has_failed = 1;
+        while(*dospath_end != '\\' && *dospath_end != '/'){
+            --dospath_end;
+            if(dospath_end < dospath)
+                break;
+        }
+        *dospath_end = '\0';
+    }
+    if(dospath_end < dospath)
+        return FALSE;
     strcat(szPath, pszUnixPath + cDriveSymlinkLen);
     HeapFree(GetProcessHeap(), 0, pszUnixPath);
-    
+
+    if(has_failed && WideCharToMultiByte(CP_UNIXCP, 0, dospath_end + 1, -1,
+                mb_path, FILENAME_MAX, NULL, NULL) > 0){
+        strcat(szPath, "/");
+        strcat(szPath, mb_path);
+    }
+
     /* pCanonicalTail always points to the end of the canonical path constructed
      * thus far. pPathTail points to the still to be processed part of the input
      * path. pElement points to the path element currently investigated.
@@ -951,6 +974,10 @@ static HRESULT WINAPI UnixFolder_IShellFolder2_BindToObject(IShellFolder2* iface
     if (_ILIsEmpty(pidl))
         return E_INVALIDARG;
    
+    /* Don't bind to files */
+    if (_ILIsValue(ILFindLastID(pidl)))
+        return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
+
     if (IsEqualCLSID(This->m_pCLSID, &CLSID_FolderShortcut)) {
         /* Children of FolderShortcuts are ShellFSFolders on Windows. 
          * Unixfs' counterpart is UnixDosFolder. */
@@ -1023,9 +1050,17 @@ static HRESULT WINAPI UnixFolder_IShellFolder2_CompareIDs(IShellFolder2* iface,
     firstpidl = ILCloneFirst(pidl1);
     pidl1 = ILGetNext(pidl1);
     pidl2 = ILGetNext(pidl2);
-    
-    hr = IShellFolder2_BindToObject(iface, firstpidl, NULL, &IID_IShellFolder, (LPVOID*)&psf);
-    if (SUCCEEDED(hr)) {
+
+    isEmpty1 = _ILIsEmpty(pidl1);
+    isEmpty2 = _ILIsEmpty(pidl2);
+
+    if (isEmpty1 && isEmpty2)
+        return MAKE_HRESULT(SEVERITY_SUCCESS, 0, 0);
+    else if (isEmpty1)
+        return MAKE_HRESULT(SEVERITY_SUCCESS, 0, (WORD)-1);
+    else if (isEmpty2)
+        return MAKE_HRESULT(SEVERITY_SUCCESS, 0, (WORD)1);
+    else if (SUCCEEDED(IShellFolder2_BindToObject(iface, firstpidl, NULL, &IID_IShellFolder, (void**)&psf))) {
         hr = IShellFolder_CompareIDs(psf, lParam, pidl1, pidl2);
         IShellFolder2_Release(psf);
     }
@@ -1103,6 +1138,7 @@ static HRESULT WINAPI UnixFolder_IShellFolder2_GetUIObjectOf(IShellFolder2* ifac
 {
     UnixFolder *This = ADJUST_THIS(UnixFolder, IShellFolder2, iface);
     UINT i;
+    HRESULT hr;
     
     TRACE("(iface=%p, hwndOwner=%p, cidl=%d, apidl=%p, riid=%s, prgfInOut=%p, ppv=%p)\n",
         iface, hwndOwner, cidl, apidl, debugstr_guid(riid), prgfInOut, ppvOut);
@@ -1113,6 +1149,12 @@ static HRESULT WINAPI UnixFolder_IShellFolder2_GetUIObjectOf(IShellFolder2* ifac
     for (i=0; i<cidl; i++) 
         if (!apidl[i]) 
             return E_INVALIDARG;
+
+    if(cidl == 1) {
+        hr = SHELL32_CreateExtensionUIObject(iface, *apidl, riid, ppvOut);
+        if(hr != S_FALSE)
+            return hr;
+    }
     
     if (IsEqualIID(&IID_IContextMenu, riid)) {
         *ppvOut = ISvItemCm_Constructor((IShellFolder*)iface, This->m_pidlLocation, apidl, cidl);
@@ -1153,6 +1195,7 @@ static HRESULT WINAPI UnixFolder_IShellFolder2_GetDisplayNameOf(IShellFolder2* i
     LPCITEMIDLIST pidl, SHGDNF uFlags, STRRET* lpName)
 {
     UnixFolder *This = ADJUST_THIS(UnixFolder, IShellFolder2, iface);
+    SHITEMID emptyIDL = { 0, { 0 } };
     HRESULT hr = S_OK;    
 
     TRACE("(iface=%p, pidl=%p, uFlags=%x, lpName=%p)\n", iface, pidl, uFlags, lpName);
@@ -1176,15 +1219,66 @@ static HRESULT WINAPI UnixFolder_IShellFolder2_GetDisplayNameOf(IShellFolder2* i
                 PathRemoveBackslashW(lpName->u.pOleStr);
                 HeapFree(GetProcessHeap(), 0, pwszDosFileName);
             }
+        } else if (_ILIsValue(pidl)) {
+            STRRET str;
+            PWSTR path, file;
+
+            /* We are looking for the complete path to a file */
+
+            /* Get the complete path for the current folder object */
+            hr = IShellFolder_GetDisplayNameOf(iface, (LPITEMIDLIST)&emptyIDL, uFlags, &str);
+            if (SUCCEEDED(hr)) {
+                hr = StrRetToStrW(&str, NULL, &path);
+                if (SUCCEEDED(hr)) {
+
+                    /* Get the child filename */
+                    hr = IShellFolder_GetDisplayNameOf(iface, pidl, SHGDN_FORPARSING | SHGDN_INFOLDER, &str);
+                    if (SUCCEEDED(hr)) {
+                        hr = StrRetToStrW(&str, NULL, &file);
+                        if (SUCCEEDED(hr)) {
+                            static const WCHAR slashW = '/';
+                            UINT len_path = strlenW(path), len_file = strlenW(file);
+
+                            /* Now, combine them */
+                            lpName->uType = STRRET_WSTR;
+                            lpName->u.pOleStr = SHAlloc( (len_path + len_file + 2)*sizeof(WCHAR) );
+                            lstrcpyW(lpName->u.pOleStr, path);
+                            if (This->m_dwPathMode == PATHMODE_UNIX &&
+                               lpName->u.pOleStr[len_path-1] != slashW) {
+                                lpName->u.pOleStr[len_path] = slashW;
+                                lpName->u.pOleStr[len_path+1] = '\0';
+                            } else
+                                PathAddBackslashW(lpName->u.pOleStr);
+                            lstrcatW(lpName->u.pOleStr, file);
+
+                            CoTaskMemFree(file);
+                        } else
+                            WARN("Failed to convert strret (file)\n");
+                    }
+                    CoTaskMemFree(path);
+                } else
+                    WARN("Failed to convert strret (path)\n");
+            }
         } else {
             IShellFolder *pSubFolder;
-            SHITEMID emptyIDL = { 0, { 0 } };
 
             hr = IShellFolder_BindToObject(iface, pidl, NULL, &IID_IShellFolder, (void**)&pSubFolder);
-            if (FAILED(hr)) return hr;
+            if (SUCCEEDED(hr)) {
+                hr = IShellFolder_GetDisplayNameOf(pSubFolder, (LPITEMIDLIST)&emptyIDL, uFlags, lpName);
+                IShellFolder_Release(pSubFolder);
+            } else if (FAILED(hr) && !_ILIsPidlSimple(pidl)) {
+                LPITEMIDLIST pidl_parent = ILClone(pidl);
+                LPITEMIDLIST pidl_child = ILFindLastID(pidl);
 
-            hr = IShellFolder_GetDisplayNameOf(pSubFolder, (LPITEMIDLIST)&emptyIDL, uFlags, lpName);
-            IShellFolder_Release(pSubFolder);
+                /* Might be a file, try binding to its parent */
+                ILRemoveLastID(pidl_parent);
+                hr = IShellFolder_BindToObject(iface, pidl_parent, NULL, &IID_IShellFolder, (void**)&pSubFolder);
+                if (SUCCEEDED(hr)) {
+                    hr = IShellFolder_GetDisplayNameOf(pSubFolder, pidl_child, uFlags, lpName);
+                    IShellFolder_Release(pSubFolder);
+                }
+                ILFree(pidl_parent);
+            }
         }
     } else {
         WCHAR wszFileName[MAX_PATH];
