@@ -42,6 +42,10 @@
 #define IDS_SHV_COLUMN11	17
 #define IDS_SHV_COLUMN_DELFROM  18
 #define IDS_SHV_COLUMN_DELDATE  19
+#define IDS_SHV_COL_DOCS        80
+#define IDS_SHV_COL_STATUS      81
+#define IDS_SHV_COL_LOCATION    82
+#define IDS_SHV_COL_MODEL       83
 
 #define IDS_DESKTOP		20
 #define IDS_MYCOMPUTER		21
@@ -86,6 +90,28 @@
 #define IDS_COMMON_PICTURES         69
 #define IDS_COMMON_VIDEO            70
 #define IDS_CDBURN_AREA             71
+#define IDS_PROGRAM_FILESX86        72
+#define IDS_PROGRAM_FILES_COMMONX86 73
+#define IDS_CONTACTS                74
+#define IDS_LINKS                   75
+#define IDS_PHOTO_ALBUMS            76
+#define IDS_PICTURES_LIBRARY        77
+#define IDS_PLAYLISTS               78
+#define IDS_PUBLIC_DOWNLOADS        79
+#define IDS_PUBLIC_GAME_TASKS       84
+#define IDS_PUBLIC_LIBRARIES        85
+#define IDS_PUBLIC_RINGTONES        86
+#define IDS_SAMPLE_MUSIC            87
+#define IDS_SAMPLE_PICTURES         88
+#define IDS_SAMPLE_PLAYLISTS        89
+#define IDS_SAMPLE_VIDEOS           90
+#define IDS_SAVED_GAMES             91
+#define IDS_SAVED_SEARCHES          92
+#define IDS_USER_PROFILES           93
+#define IDS_COMMON_OEM_LINKS        94
+#define IDS_DOCUMENTS               95
+#define IDS_DOWNLOADS               96
+#define IDS_LOCAL_APPDATA_LOW       97
 
 #define IDS_CREATEFOLDER_DENIED     128
 #define IDS_CREATEFOLDER_CAPTION    129
@@ -170,8 +196,13 @@
 #define IDI_SHELL_CONTROL_PANEL     36
 #define IDI_SHELL_PRINTERS_FOLDER   38
 #define IDI_SHELL_FONTS_FOLDER      39
+#define IDI_SHELL_REMOTE_PRINTER   140
+#define IDI_SHELL_TO_FILE_PRINTER  141
 #define IDI_SHELL_TRASH_FILE       142
 #define IDI_SHELL_CONFIRM_DELETE   161
+#define IDI_SHELL_DEFAULT_PRINTER  168
+#define IDI_SHELL_DEFAULT_REMOTE_PRINTER   169
+#define IDI_SHELL_DEFAULT_TO_FILE_PRINTER  170
 #define IDI_SHELL_MY_DOCUMENTS     235
 
 /* 
