@@ -28,14 +28,16 @@
 #include "shlwapi.h"
 #include "wininet.h"
 
-static HMODULE hShlwapi;
 static HRESULT (WINAPI *pPathIsValidCharA)(char,DWORD);
 static HRESULT (WINAPI *pPathIsValidCharW)(WCHAR,DWORD);
 static LPWSTR  (WINAPI *pPathCombineW)(LPWSTR, LPCWSTR, LPCWSTR);
+static HRESULT (WINAPI *pPathCreateFromUrlA)(LPCSTR, LPSTR, LPDWORD, DWORD);
+static HRESULT (WINAPI *pPathCreateFromUrlW)(LPCWSTR, LPWSTR, LPDWORD, DWORD);
+static BOOL    (WINAPI *pPathAppendA)(LPSTR, LPCSTR);
 
 /* ################ */
 
-struct {
+static const struct {
     const char *url;
     const char *path;
     DWORD ret;
@@ -104,7 +106,7 @@ static struct {
     {NULL, FALSE}
 };
 
-struct {
+static const struct {
     const char *path;
     const char *result;
 } TEST_PATH_UNQUOTE_SPACES[] = {
@@ -207,31 +209,39 @@ static void test_PathCreateFromUrl(void)
     WCHAR *pathW, *urlW;
     static const char url[] = "http://www.winehq.org";
 
+    if (!pPathCreateFromUrlA) {
+        win_skip("PathCreateFromUrlA not found\n");
+        return;
+    }
+
     /* Check ret_path = NULL */
     len = sizeof(url);
-    ret = PathCreateFromUrlA(url, NULL, &len, 0); 
+    ret = pPathCreateFromUrlA(url, NULL, &len, 0);
     ok ( ret == E_INVALIDARG, "got 0x%08x expected E_INVALIDARG\n", ret);
 
     for(i = 0; i < sizeof(TEST_PATHFROMURL) / sizeof(TEST_PATHFROMURL[0]); i++) {
         len = INTERNET_MAX_URL_LENGTH;
-        ret = PathCreateFromUrlA(TEST_PATHFROMURL[i].url, ret_path, &len, 0);
+        ret = pPathCreateFromUrlA(TEST_PATHFROMURL[i].url, ret_path, &len, 0);
         ok(ret == TEST_PATHFROMURL[i].ret, "ret %08x from url %s\n", ret, TEST_PATHFROMURL[i].url);
         if(TEST_PATHFROMURL[i].path) {
            ok(!lstrcmpi(ret_path, TEST_PATHFROMURL[i].path), "got %s expected %s from url %s\n", ret_path, TEST_PATHFROMURL[i].path,  TEST_PATHFROMURL[i].url);
            ok(len == strlen(ret_path), "ret len %d from url %s\n", len, TEST_PATHFROMURL[i].url);
         }
-        len = INTERNET_MAX_URL_LENGTH;
-        pathW = GetWideString(TEST_PATHFROMURL[i].path);
-        urlW = GetWideString(TEST_PATHFROMURL[i].url);
-        ret = PathCreateFromUrlW(urlW, ret_pathW, &len, 0);
-        WideCharToMultiByte(CP_ACP, 0, ret_pathW, -1, ret_path, sizeof(ret_path),0,0);
-        ok(ret == TEST_PATHFROMURL[i].ret, "ret %08x from url L\"%s\"\n", ret, TEST_PATHFROMURL[i].url);
-        if(TEST_PATHFROMURL[i].path) {
-            ok(!lstrcmpiW(ret_pathW, pathW), "got %s expected %s from url L\"%s\"\n", ret_path, TEST_PATHFROMURL[i].path, TEST_PATHFROMURL[i].url);
-            ok(len == lstrlenW(ret_pathW), "ret len %d from url L\"%s\"\n", len, TEST_PATHFROMURL[i].url);
+        if (pPathCreateFromUrlW) {
+            len = INTERNET_MAX_URL_LENGTH;
+            pathW = GetWideString(TEST_PATHFROMURL[i].path);
+            urlW = GetWideString(TEST_PATHFROMURL[i].url);
+            ret = pPathCreateFromUrlW(urlW, ret_pathW, &len, 0);
+            WideCharToMultiByte(CP_ACP, 0, ret_pathW, -1, ret_path, sizeof(ret_path),0,0);
+            ok(ret == TEST_PATHFROMURL[i].ret, "ret %08x from url L\"%s\"\n", ret, TEST_PATHFROMURL[i].url);
+            if(TEST_PATHFROMURL[i].path) {
+                ok(!lstrcmpiW(ret_pathW, pathW), "got %s expected %s from url L\"%s\"\n",
+                    ret_path, TEST_PATHFROMURL[i].path, TEST_PATHFROMURL[i].url);
+                ok(len == lstrlenW(ret_pathW), "ret len %d from url L\"%s\"\n", len, TEST_PATHFROMURL[i].url);
+            }
+            FreeWideString(urlW);
+            FreeWideString(pathW);
         }
-        FreeWideString(urlW);
-        FreeWideString(pathW);
     }
 }
 
@@ -290,6 +300,15 @@ static void test_PathIsValidCharA(void)
     BOOL ret;
     unsigned int c;
 
+    /* For whatever reason, PathIsValidCharA and PathAppendA share the same
+     * ordinal number in some native versions. Check this to prevent a crash.
+     */
+    if (!pPathIsValidCharA || pPathIsValidCharA == (void*)pPathAppendA)
+    {
+        win_skip("PathIsValidCharA isn't available\n");
+        return;
+    }
+
     for (c = 0; c < 0x7f; c++)
     {
         ret = pPathIsValidCharA( c, ~0U );
@@ -308,6 +327,12 @@ static void test_PathIsValidCharW(void)
     BOOL ret;
     unsigned int c;
 
+    if (!pPathIsValidCharW)
+    {
+        win_skip("PathIsValidCharW isn't available\n");
+        return;
+    }
+
     for (c = 0; c < 0x7f; c++)
     {
         ret = pPathIsValidCharW( c, ~0U );
@@ -382,7 +407,13 @@ static void test_PathCombineW(void)
     WCHAR wbuf[MAX_PATH+1], wstr1[MAX_PATH] = {'C',':','\\',0}, wstr2[MAX_PATH];
     static const WCHAR expout[] = {'C',':','\\','A','A',0};
     int i;
-   
+
+    if (!pPathCombineW)
+    {
+        win_skip("PathCombineW isn't available\n");
+        return;
+    }
+
     wszString2 = HeapAlloc(GetProcessHeap(), 0, MAX_PATH * sizeof(WCHAR));
 
     /* NULL test */
@@ -690,7 +721,7 @@ static void test_PathAppendA(void)
 {
     char path[MAX_PATH];
     char too_long[LONG_LEN];
-    char one[HALF_LEN], two[HALF_LEN];
+    char half[HALF_LEN];
     BOOL res;
 
     lstrcpy(path, "C:\\one");
@@ -803,16 +834,16 @@ static void test_PathAppendA(void)
        "Expected length of path to be zero, got %i\n", lstrlen(path));
 
     /* both params combined are too long */
-    memset(one, 'a', HALF_LEN);
-    one[HALF_LEN - 1] = '\0';
-    memset(two, 'b', HALF_LEN);
-    two[HALF_LEN - 1] = '\0';
+    memset(path, 'a', HALF_LEN);
+    path[HALF_LEN - 1] = '\0';
+    memset(half, 'b', HALF_LEN);
+    half[HALF_LEN - 1] = '\0';
     SetLastError(0xdeadbeef);
-    res = PathAppendA(one, two);
+    res = PathAppendA(path, half);
     ok(!res, "Expected failure\n");
-    ok(lstrlen(one) == 0 ||
-       broken(lstrlen(one) == (HALF_LEN - 1)), /* Win95 and some W2K */
-       "Expected length of one to be zero, got %i\n", lstrlen(one));
+    ok(lstrlen(path) == 0 ||
+       broken(lstrlen(path) == (HALF_LEN - 1)), /* Win95 and some W2K */
+       "Expected length of path to be zero, got %i\n", lstrlen(path));
     ok(GetLastError() == 0xdeadbeef, "Expected 0xdeadbeef, got %d\n", GetLastError());
 }
 
@@ -1302,41 +1333,66 @@ static void test_PathUnquoteSpaces(void)
     }
 }
 
+static void test_PathGetDriveNumber(void)
+{
+    static const CHAR test1A[] = "a:\\test.file";
+    static const CHAR test2A[] = "file:////b:\\test.file";
+    static const CHAR test3A[] = "file:///c:\\test.file";
+    static const CHAR test4A[] = "file:\\\\c:\\test.file";
+    int ret;
+
+    SetLastError(0xdeadbeef);
+    ret = PathGetDriveNumberA(NULL);
+    ok(ret == -1, "got %d\n", ret);
+    ok(GetLastError() == 0xdeadbeef, "got %d\n", GetLastError());
+
+    ret = PathGetDriveNumberA(test1A);
+    ok(ret == 0, "got %d\n", ret);
+    ret = PathGetDriveNumberA(test2A);
+    ok(ret == -1, "got %d\n", ret);
+    ret = PathGetDriveNumberA(test3A);
+    ok(ret == -1, "got %d\n", ret);
+    ret = PathGetDriveNumberA(test4A);
+    ok(ret == -1, "got %d\n", ret);
+}
+
 /* ################ */
 
 START_TEST(path)
 {
-  hShlwapi = GetModuleHandleA("shlwapi.dll");
+    HMODULE hShlwapi = GetModuleHandleA("shlwapi.dll");
 
-  test_PathSearchAndQualify();
-  test_PathCreateFromUrl();
-  test_PathIsUrl();
+    /* SHCreateStreamOnFileEx was introduced in shlwapi v6.0 */
+    if(!GetProcAddress(hShlwapi, "SHCreateStreamOnFileEx")){
+        win_skip("Too old shlwapi version\n");
+        return;
+    }
 
-  test_PathAddBackslash();
-  test_PathMakePretty();
-  test_PathMatchSpec();
+    pPathCreateFromUrlA = (void*)GetProcAddress(hShlwapi, "PathCreateFromUrlA");
+    pPathCreateFromUrlW = (void*)GetProcAddress(hShlwapi, "PathCreateFromUrlW");
+    pPathCombineW = (void*)GetProcAddress(hShlwapi, "PathCombineW");
+    pPathIsValidCharA = (void*)GetProcAddress(hShlwapi, (LPSTR)455);
+    pPathIsValidCharW = (void*)GetProcAddress(hShlwapi, (LPSTR)456);
+    pPathAppendA = (void*)GetProcAddress(hShlwapi, "PathAppendA");
+
+    test_PathSearchAndQualify();
+    test_PathCreateFromUrl();
+    test_PathIsUrl();
+
+    test_PathAddBackslash();
+    test_PathMakePretty();
+    test_PathMatchSpec();
 
-  /* For whatever reason, PathIsValidCharA and PathAppendA share the same
-   * ordinal number in some native versions. Check this to prevent a crash.
-   */
-  pPathIsValidCharA = (void*)GetProcAddress(hShlwapi, (LPSTR)455);
-  if (pPathIsValidCharA && pPathIsValidCharA != (void*)GetProcAddress(hShlwapi, "PathAppendA"))
-  {
     test_PathIsValidCharA();
+    test_PathIsValidCharW();
 
-     pPathIsValidCharW = (void*)GetProcAddress(hShlwapi, (LPSTR)456);
-     if (pPathIsValidCharW) test_PathIsValidCharW();
-  }
-
-  pPathCombineW = (void*)GetProcAddress(hShlwapi, "PathCombineW");
-  if (pPathCombineW)
     test_PathCombineW();
-
-  test_PathCombineA();
-  test_PathAppendA();
-  test_PathCanonicalizeA();
-  test_PathFindExtensionA();
-  test_PathBuildRootA();
-  test_PathCommonPrefixA();
-  test_PathUnquoteSpaces();
+    test_PathCombineA();
+    test_PathAppendA();
+    test_PathCanonicalizeA();
+    test_PathFindExtensionA();
+    test_PathBuildRootA();
+    test_PathCommonPrefixA();
+    test_PathUnquoteSpaces();
+    test_PathGetDriveNumber();
 }
