@@ -25,6 +25,7 @@
 #include "config.h"
 #include "wine/port.h"
 
+#include <assert.h>
 #include <stdarg.h>
 #include <string.h>
 #include <stdlib.h>
@@ -37,7 +38,10 @@
 #include "wine/exception.h"
 #include "wine/server.h"
 #include "controls.h"
+#include "win.h"
 #include "user_private.h"
+#include "wine/list.h"
+#include "wine/unicode.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(cursor);
@@ -73,38 +77,7 @@ static HDC screen_dc;
 
 static const WCHAR DISPLAYW[] = {'D','I','S','P','L','A','Y',0};
 
-
-/**********************************************************************
- * ICONCACHE for cursors/icons loaded with LR_SHARED.
- *
- * FIXME: This should not be allocated on the system heap, but on a
- *        subsystem-global heap (i.e. one for all Win16 processes,
- *        and one for each Win32 process).
- */
-typedef struct tagICONCACHE
-{
-    struct tagICONCACHE *next;
-
-    HMODULE              hModule;
-    HRSRC                hRsrc;
-    HRSRC                hGroupRsrc;
-    HICON                hIcon;
-
-    INT                  count;
-
-} ICONCACHE;
-
-static ICONCACHE *IconAnchor = NULL;
-
-static CRITICAL_SECTION IconCrst;
-static CRITICAL_SECTION_DEBUG critsect_debug =
-{
-    0, 0, &IconCrst,
-    { &critsect_debug.ProcessLocksList, &critsect_debug.ProcessLocksList },
-      0, 0, { (DWORD_PTR)(__FILE__ ": IconCrst") }
-};
-static CRITICAL_SECTION IconCrst = { &critsect_debug, -1, 0, 0, 0, 0 };
-
+static struct list icon_cache = LIST_INIT( icon_cache );
 
 /**********************************************************************
  * User objects management
@@ -120,7 +93,11 @@ struct cursoricon_frame
 struct cursoricon_object
 {
     struct user_object      obj;        /* object header */
+    struct list             entry;      /* entry in shared icons list */
     ULONG_PTR               param;      /* opaque param used by 16-bit code */
+    HMODULE                 module;     /* module for icons loaded from resources */
+    LPWSTR                  resname;    /* resource name for icons loaded from resources */
+    HRSRC                   rsrc;       /* resource for shared icons */
     BOOL                    is_icon;    /* whether icon or cursor */
     UINT                    width;
     UINT                    height;
@@ -166,12 +143,15 @@ static BOOL free_icon_handle( HICON handle )
         ULONG_PTR param = obj->param;
         UINT i;
 
+        assert( !obj->rsrc );  /* shared icons can't be freed */
+
         for (i=0; i<obj->num_frames; i++)
         {
             if (obj->frames[i].alpha) DeleteObject( obj->frames[i].alpha );
             if (obj->frames[i].color) DeleteObject( obj->frames[i].color );
             DeleteObject( obj->frames[i].mask );
         }
+        if (!IS_INTRESOURCE( obj->resname )) HeapFree( GetProcessHeap(), 0, obj->resname );
         HeapFree( GetProcessHeap(), 0, obj );
         if (wow_handlers.free_icon_param && param) wow_handlers.free_icon_param( param );
         USER_Driver->pDestroyCursorIcon( handle );
@@ -305,19 +285,22 @@ static int bitmap_info_size( const BITMAPINFO * info, WORD coloruse )
  */
 static HBITMAP copy_bitmap( HBITMAP bitmap )
 {
-    HDC src, dst;
-    HBITMAP new_bitmap;
+    HDC src, dst = 0;
+    HBITMAP new_bitmap = 0;
     BITMAP bmp;
 
     if (!bitmap) return 0;
     if (!GetObjectW( bitmap, sizeof(bmp), &bmp )) return 0;
 
-    src = CreateCompatibleDC( 0 );
-    dst = CreateCompatibleDC( 0 );
-    SelectObject( src, bitmap );
-    new_bitmap = CreateCompatibleBitmap( src, bmp.bmWidth, bmp.bmHeight );
-    SelectObject( dst, new_bitmap );
-    BitBlt( dst, 0, 0, bmp.bmWidth, bmp.bmHeight, src, 0, 0, SRCCOPY );
+    if ((src = CreateCompatibleDC( 0 )) && (dst = CreateCompatibleDC( 0 )))
+    {
+        SelectObject( src, bitmap );
+        if ((new_bitmap = CreateCompatibleBitmap( src, bmp.bmWidth, bmp.bmHeight )))
+        {
+            SelectObject( dst, new_bitmap );
+            BitBlt( dst, 0, 0, bmp.bmWidth, bmp.bmHeight, src, 0, 0, SRCCOPY );
+        }
+    }
     DeleteDC( dst );
     DeleteDC( src );
     return new_bitmap;
@@ -403,107 +386,6 @@ static int DIB_GetBitmapInfo( const BITMAPINFOHEADER *header, LONG *width,
     return -1;
 }
 
-/**********************************************************************
- *	    CURSORICON_FindSharedIcon
- */
-static HICON CURSORICON_FindSharedIcon( HMODULE hModule, HRSRC hRsrc )
-{
-    HICON hIcon = 0;
-    ICONCACHE *ptr;
-
-    EnterCriticalSection( &IconCrst );
-
-    for ( ptr = IconAnchor; ptr; ptr = ptr->next )
-        if ( ptr->hModule == hModule && ptr->hRsrc == hRsrc )
-        {
-            ptr->count++;
-            hIcon = ptr->hIcon;
-            break;
-        }
-
-    LeaveCriticalSection( &IconCrst );
-
-    return hIcon;
-}
-
-/*************************************************************************
- * CURSORICON_FindCache
- *
- * Given a handle, find the corresponding cache element
- *
- * PARAMS
- *      Handle     [I] handle to an Image
- *
- * RETURNS
- *     Success: The cache entry
- *     Failure: NULL
- *
- */
-static ICONCACHE* CURSORICON_FindCache(HICON hIcon)
-{
-    ICONCACHE *ptr;
-    ICONCACHE *pRet=NULL;
-    BOOL IsFound = FALSE;
-
-    EnterCriticalSection( &IconCrst );
-
-    for (ptr = IconAnchor; ptr != NULL && !IsFound; ptr = ptr->next)
-    {
-        if ( hIcon == ptr->hIcon )
-        {
-            IsFound = TRUE;
-            pRet = ptr;
-        }
-    }
-
-    LeaveCriticalSection( &IconCrst );
-
-    return pRet;
-}
-
-/**********************************************************************
- *	    CURSORICON_AddSharedIcon
- */
-static void CURSORICON_AddSharedIcon( HMODULE hModule, HRSRC hRsrc, HRSRC hGroupRsrc, HICON hIcon )
-{
-    ICONCACHE *ptr = HeapAlloc( GetProcessHeap(), 0, sizeof(ICONCACHE) );
-    if ( !ptr ) return;
-
-    ptr->hModule = hModule;
-    ptr->hRsrc   = hRsrc;
-    ptr->hIcon  = hIcon;
-    ptr->hGroupRsrc = hGroupRsrc;
-    ptr->count   = 1;
-
-    EnterCriticalSection( &IconCrst );
-    ptr->next    = IconAnchor;
-    IconAnchor   = ptr;
-    LeaveCriticalSection( &IconCrst );
-}
-
-/**********************************************************************
- *	    CURSORICON_DelSharedIcon
- */
-static INT CURSORICON_DelSharedIcon( HICON hIcon )
-{
-    INT count = -1;
-    ICONCACHE *ptr;
-
-    EnterCriticalSection( &IconCrst );
-
-    for ( ptr = IconAnchor; ptr; ptr = ptr->next )
-        if ( ptr->hIcon == hIcon )
-        {
-            if ( ptr->count > 0 ) ptr->count--;
-            count = ptr->count;
-            break;
-        }
-
-    LeaveCriticalSection( &IconCrst );
-
-    return count;
-}
-
 /**********************************************************************
  *              get_icon_size
  */
@@ -879,22 +761,13 @@ done:
     return ret;
 }
 
-static HICON CURSORICON_CreateIconFromBMI( BITMAPINFO *bmi,
-					   POINT hotspot, BOOL bIcon,
-					   DWORD dwVersion,
-					   INT width, INT height,
-					   UINT cFlag )
+static HICON CURSORICON_CreateIconFromBMI( BITMAPINFO *bmi, HMODULE module, LPCWSTR resname, HRSRC rsrc,
+                                           POINT hotspot, BOOL bIcon, INT width, INT height, UINT cFlag )
 {
     HICON hObj;
     HBITMAP color = 0, mask = 0, alpha = 0;
     BOOL do_stretch;
 
-    if (dwVersion == 0x00020000)
-    {
-        FIXME_(cursor)("\t2.xx resources are not supported\n");
-        return 0;
-    }
-
     /* Check bitmap header */
 
     if ( (bmi->bmiHeader.biSize != sizeof(BITMAPCOREHEADER)) &&
@@ -933,12 +806,25 @@ static HICON CURSORICON_CreateIconFromBMI( BITMAPINFO *bmi,
         struct cursoricon_object *info = get_icon_ptr( hObj );
 
         info->is_icon = bIcon;
+        info->module  = module;
         info->hotspot = hotspot;
         info->width   = width;
         info->height  = height;
         info->frames[0].color = color;
         info->frames[0].mask  = mask;
         info->frames[0].alpha = alpha;
+        if (!IS_INTRESOURCE(resname))
+        {
+            info->resname = HeapAlloc( GetProcessHeap(), 0, (strlenW(resname) + 1) * sizeof(WCHAR) );
+            if (info->resname) strcpyW( info->resname, resname );
+        }
+        else info->resname = MAKEINTRESOURCEW( LOWORD(resname) );
+
+        if (module && (cFlag & LR_SHARED))
+        {
+            info->rsrc = rsrc;
+            list_add_head( &icon_cache, &info->entry );
+        }
         release_icon_ptr( hObj, info );
         USER_Driver->pCreateCursorIcon( hObj );
     }
@@ -1191,6 +1077,12 @@ HICON WINAPI CreateIconFromResourceEx( LPBYTE bits, UINT cbSize,
 
     if (!bits) return 0;
 
+    if (dwVersion == 0x00020000)
+    {
+        FIXME_(cursor)("\t2.xx resources are not supported\n");
+        return 0;
+    }
+
     if (bIcon)
     {
         hotspot.x = width / 2;
@@ -1205,8 +1097,7 @@ HICON WINAPI CreateIconFromResourceEx( LPBYTE bits, UINT cbSize,
         bmi = (BITMAPINFO *)(pt + 2);
     }
 
-    return CURSORICON_CreateIconFromBMI( bmi, hotspot, bIcon, dwVersion,
-					 width, height, cFlag );
+    return CURSORICON_CreateIconFromBMI( bmi, NULL, NULL, NULL, hotspot, bIcon, width, height, cFlag );
 }
 
 
@@ -1268,9 +1159,8 @@ static HICON CURSORICON_LoadFromFile( LPCWSTR filename,
 
     hotspot.x = entry->xHotspot;
     hotspot.y = entry->yHotspot;
-    hIcon = CURSORICON_CreateIconFromBMI( (BITMAPINFO *)&bits[entry->dwDIBOffset],
-					  hotspot, !fCursor, 0x00030000,
-					  width, height, loadflags );
+    hIcon = CURSORICON_CreateIconFromBMI( (BITMAPINFO *)&bits[entry->dwDIBOffset], NULL, NULL, NULL,
+					  hotspot, !fCursor, width, height, loadflags );
 end:
     TRACE("loaded %s -> %p\n", debugstr_w( filename ), hIcon );
     UnmapViewOfFile( bits );
@@ -1288,12 +1178,12 @@ static HICON CURSORICON_Load(HINSTANCE hInstance, LPCWSTR name,
 {
     HANDLE handle = 0;
     HICON hIcon = 0;
-    HRSRC hRsrc, hGroupRsrc;
+    HRSRC hRsrc;
     CURSORICONDIR *dir;
     CURSORICONDIRENTRY *dirEntry;
     LPBYTE bits;
     WORD wResId;
-    DWORD dwBytesInRes;
+    POINT hotspot;
 
     TRACE("%p, %s, %dx%d, depth %d, fCursor %d, flags 0x%04x\n",
           hInstance, debugstr_w(name), width, height, depth, fCursor, loadflags);
@@ -1311,7 +1201,6 @@ static HICON CURSORICON_Load(HINSTANCE hInstance, LPCWSTR name,
     if (!(hRsrc = FindResourceW( hInstance, name,
                                  (LPWSTR)(fCursor ? RT_GROUP_CURSOR : RT_GROUP_ICON) )))
         return 0;
-    hGroupRsrc = hRsrc;
 
     /* Find the best entry in the directory */
 
@@ -1323,7 +1212,6 @@ static HICON CURSORICON_Load(HINSTANCE hInstance, LPCWSTR name,
         dirEntry = CURSORICON_FindBestIconRes( dir, width, height, depth );
     if (!dirEntry) return 0;
     wResId = dirEntry->wResId;
-    dwBytesInRes = dirEntry->dwBytesInRes;
     FreeResource( handle );
 
     /* Load the resource */
@@ -1332,167 +1220,44 @@ static HICON CURSORICON_Load(HINSTANCE hInstance, LPCWSTR name,
                                 (LPWSTR)(fCursor ? RT_CURSOR : RT_ICON) ))) return 0;
 
     /* If shared icon, check whether it was already loaded */
-    if (    (loadflags & LR_SHARED)
-         && (hIcon = CURSORICON_FindSharedIcon( hInstance, hRsrc ) ) != 0 )
-        return hIcon;
+    if (loadflags & LR_SHARED)
+    {
+        struct cursoricon_object *ptr;
+
+        USER_Lock();
+        LIST_FOR_EACH_ENTRY( ptr, &icon_cache, struct cursoricon_object, entry )
+        {
+            if (ptr->module != hInstance) continue;
+            if (ptr->rsrc != hRsrc) continue;
+            hIcon = ptr->obj.handle;
+            break;
+        }
+        USER_Unlock();
+        if (hIcon) return hIcon;
+    }
 
     if (!(handle = LoadResource( hInstance, hRsrc ))) return 0;
     bits = LockResource( handle );
-    hIcon = CreateIconFromResourceEx( bits, dwBytesInRes,
-                                      !fCursor, 0x00030000, width, height, loadflags);
+
+    if (!fCursor)
+    {
+        hotspot.x = width / 2;
+        hotspot.y = height / 2;
+    }
+    else /* get the hotspot */
+    {
+        SHORT *pt = (SHORT *)bits;
+        hotspot.x = pt[0];
+        hotspot.y = pt[1];
+        bits += 2 * sizeof(SHORT);
+    }
+    hIcon = CURSORICON_CreateIconFromBMI( (BITMAPINFO *)bits, hInstance, name, hRsrc,
+                                          hotspot, !fCursor, width, height, loadflags );
     FreeResource( handle );
-
-    /* If shared icon, add to icon cache */
-
-    if ( hIcon && (loadflags & LR_SHARED) )
-        CURSORICON_AddSharedIcon( hInstance, hRsrc, hGroupRsrc, hIcon );
-
     return hIcon;
 }
 
 
-/*************************************************************************
- * CURSORICON_ExtCopy
- *
- * Copies an Image from the Cache if LR_COPYFROMRESOURCE is specified
- *
- * PARAMS
- *      Handle     [I] handle to an Image
- *      nType      [I] Type of Handle (IMAGE_CURSOR | IMAGE_ICON)
- *      iDesiredCX [I] The Desired width of the Image
- *      iDesiredCY [I] The desired height of the Image
- *      nFlags     [I] The flags from CopyImage
- *
- * RETURNS
- *     Success: The new handle of the Image
- *
- * NOTES
- *     LR_COPYDELETEORG and LR_MONOCHROME are currently not implemented.
- *     LR_MONOCHROME should be implemented by CreateIconFromResourceEx.
- *     LR_COPYFROMRESOURCE will only work if the Image is in the Cache.
- *
- *
- */
-
-static HICON CURSORICON_ExtCopy(HICON hIcon, UINT nType,
-                                INT iDesiredCX, INT iDesiredCY,
-                                UINT nFlags)
-{
-    HICON hNew=0;
-
-    TRACE_(icon)("hIcon %p, nType %u, iDesiredCX %i, iDesiredCY %i, nFlags %u\n",
-                 hIcon, nType, iDesiredCX, iDesiredCY, nFlags);
-
-    if(hIcon == 0)
-    {
-        return 0;
-    }
-
-    /* Best Fit or Monochrome */
-    if( (nFlags & LR_COPYFROMRESOURCE
-        && (iDesiredCX > 0 || iDesiredCY > 0))
-        || nFlags & LR_MONOCHROME)
-    {
-        ICONCACHE* pIconCache = CURSORICON_FindCache(hIcon);
-
-        /* Not Found in Cache, then do a straight copy
-        */
-        if(pIconCache == NULL)
-        {
-            hNew = CopyIcon( hIcon );
-            if(nFlags & LR_COPYFROMRESOURCE)
-            {
-                TRACE_(icon)("LR_COPYFROMRESOURCE: Failed to load from cache\n");
-            }
-        }
-        else
-        {
-            int iTargetCY = iDesiredCY, iTargetCX = iDesiredCX;
-            LPBYTE pBits;
-            HANDLE hMem;
-            HRSRC hRsrc;
-            DWORD dwBytesInRes;
-            WORD wResId;
-            CURSORICONDIR *pDir;
-            CURSORICONDIRENTRY *pDirEntry;
-            BOOL bIsIcon = (nType == IMAGE_ICON);
-
-            /* Completing iDesiredCX CY for Monochrome Bitmaps if needed
-            */
-            if(((nFlags & LR_MONOCHROME) && !(nFlags & LR_COPYFROMRESOURCE))
-                || (iDesiredCX == 0 && iDesiredCY == 0))
-            {
-                iDesiredCY = GetSystemMetrics(bIsIcon ?
-                    SM_CYICON : SM_CYCURSOR);
-                iDesiredCX = GetSystemMetrics(bIsIcon ?
-                    SM_CXICON : SM_CXCURSOR);
-            }
-
-            /* Retrieve the CURSORICONDIRENTRY
-            */
-            if (!(hMem = LoadResource( pIconCache->hModule ,
-                            pIconCache->hGroupRsrc)))
-            {
-                return 0;
-            }
-            if (!(pDir = LockResource( hMem )))
-            {
-                return 0;
-            }
-
-            /* Find Best Fit
-            */
-            if(bIsIcon)
-            {
-                pDirEntry = CURSORICON_FindBestIconRes(
-                                pDir, iDesiredCX, iDesiredCY, 256 );
-            }
-            else
-            {
-                pDirEntry = CURSORICON_FindBestCursorRes(
-                                pDir, iDesiredCX, iDesiredCY, 1);
-            }
-
-            wResId = pDirEntry->wResId;
-            dwBytesInRes = pDirEntry->dwBytesInRes;
-            FreeResource(hMem);
-
-            TRACE_(icon)("ResID %u, BytesInRes %u, Width %d, Height %d DX %d, DY %d\n",
-                wResId, dwBytesInRes,  pDirEntry->ResInfo.icon.bWidth,
-                pDirEntry->ResInfo.icon.bHeight, iDesiredCX, iDesiredCY);
-
-            /* Get the Best Fit
-            */
-            if (!(hRsrc = FindResourceW(pIconCache->hModule ,
-                MAKEINTRESOURCEW(wResId), (LPWSTR)(bIsIcon ? RT_ICON : RT_CURSOR))))
-            {
-                return 0;
-            }
-            if (!(hMem = LoadResource( pIconCache->hModule , hRsrc )))
-            {
-                return 0;
-            }
-
-            pBits = LockResource( hMem );
-
-            if(nFlags & LR_DEFAULTSIZE)
-            {
-                iTargetCY = GetSystemMetrics(SM_CYICON);
-                iTargetCX = GetSystemMetrics(SM_CXICON);
-            }
-
-            /* Create a New Icon with the proper dimension
-            */
-            hNew = CreateIconFromResourceEx( pBits, dwBytesInRes,
-                       bIsIcon, 0x00030000, iTargetCX, iTargetCY, nFlags);
-            FreeResource(hMem);
-        }
-    }
-    else hNew = CopyIcon( hIcon );
-    return hNew;
-}
-
-
 /***********************************************************************
  *		CreateCursor (USER32.@)
  */
@@ -1576,7 +1341,11 @@ HICON WINAPI CopyIcon( HICON hIcon )
     struct cursoricon_object *ptrOld, *ptrNew;
     HICON hNew;
 
-    if (!(ptrOld = get_icon_ptr( hIcon ))) return 0;
+    if (!(ptrOld = get_icon_ptr( hIcon )))
+    {
+        SetLastError( ERROR_INVALID_CURSOR_HANDLE );
+        return 0;
+    }
     if ((hNew = alloc_icon_handle(1)))
     {
         ptrNew = get_icon_ptr( hNew );
@@ -1600,11 +1369,19 @@ HICON WINAPI CopyIcon( HICON hIcon )
  */
 BOOL WINAPI DestroyIcon( HICON hIcon )
 {
+    BOOL ret = FALSE;
+    struct cursoricon_object *obj = get_icon_ptr( hIcon );
+
     TRACE_(icon)("%p\n", hIcon );
 
-    if (CURSORICON_DelSharedIcon( hIcon ) == -1)
-        free_icon_handle( hIcon );
-    return TRUE;
+    if (obj)
+    {
+        BOOL shared = (obj->rsrc != NULL);
+        release_icon_ptr( hIcon, obj );
+        ret = (GetCursor() != hIcon);
+        if (!shared) free_icon_handle( hIcon );
+    }
+    return ret;
 }
 
 
@@ -1613,11 +1390,6 @@ BOOL WINAPI DestroyIcon( HICON hIcon )
  */
 BOOL WINAPI DestroyCursor( HCURSOR hCursor )
 {
-    if (GetCursor() == hCursor)
-    {
-        WARN_(cursor)("Destroying active cursor!\n" );
-        return FALSE;
-    }
     return DestroyIcon( hCursor );
 }
 
@@ -1639,6 +1411,7 @@ BOOL WINAPI DrawIcon( HDC hdc, INT x, INT y, HICON hIcon )
  */
 HCURSOR WINAPI DECLSPEC_HOTPATCH SetCursor( HCURSOR hCursor /* [in] Handle of cursor to show */ )
 {
+    struct cursoricon_object *obj;
     HCURSOR hOldCursor;
     int show_count;
     BOOL ret;
@@ -1661,6 +1434,9 @@ HCURSOR WINAPI DECLSPEC_HOTPATCH SetCursor( HCURSOR hCursor /* [in] Handle of cu
 
     /* Change the cursor shape only if it is visible */
     if (show_count >= 0 && hOldCursor != hCursor) USER_Driver->pSetCursor( hCursor );
+
+    if (!(obj = get_icon_ptr( hOldCursor ))) return 0;
+    release_icon_ptr( hOldCursor, obj );
     return hOldCursor;
 }
 
@@ -1863,22 +1639,88 @@ HICON WINAPI LoadIconA(HINSTANCE hInstance, LPCSTR name)
  */
 BOOL WINAPI GetIconInfo(HICON hIcon, PICONINFO iconinfo)
 {
-    struct cursoricon_object *ptr;
-
-    if (!(ptr = get_icon_ptr( hIcon ))) return FALSE;
-
-    TRACE("%p => %dx%d\n", hIcon, ptr->width, ptr->height);
-
-    iconinfo->fIcon    = ptr->is_icon;
-    iconinfo->xHotspot = ptr->hotspot.x;
-    iconinfo->yHotspot = ptr->hotspot.y;
-    iconinfo->hbmColor = copy_bitmap( ptr->frames[0].color );
-    iconinfo->hbmMask  = copy_bitmap( ptr->frames[0].mask );
-    release_icon_ptr( hIcon, ptr );
+    ICONINFOEXW infoW;
 
+    infoW.cbSize = sizeof(infoW);
+    if (!GetIconInfoExW( hIcon, &infoW )) return FALSE;
+    iconinfo->fIcon    = infoW.fIcon;
+    iconinfo->xHotspot = infoW.xHotspot;
+    iconinfo->yHotspot = infoW.yHotspot;
+    iconinfo->hbmColor = infoW.hbmColor;
+    iconinfo->hbmMask  = infoW.hbmMask;
     return TRUE;
 }
 
+/**********************************************************************
+ *              GetIconInfoExA (USER32.@)
+ */
+BOOL WINAPI GetIconInfoExA( HICON icon, ICONINFOEXA *info )
+{
+    ICONINFOEXW infoW;
+
+    if (info->cbSize != sizeof(*info))
+    {
+        SetLastError( ERROR_INVALID_PARAMETER );
+        return FALSE;
+    }
+    infoW.cbSize = sizeof(infoW);
+    if (!GetIconInfoExW( icon, &infoW )) return FALSE;
+    info->fIcon    = infoW.fIcon;
+    info->xHotspot = infoW.xHotspot;
+    info->yHotspot = infoW.yHotspot;
+    info->hbmColor = infoW.hbmColor;
+    info->hbmMask  = infoW.hbmMask;
+    info->wResID   = infoW.wResID;
+    WideCharToMultiByte( CP_ACP, 0, infoW.szModName, -1, info->szModName, MAX_PATH, NULL, NULL );
+    WideCharToMultiByte( CP_ACP, 0, infoW.szResName, -1, info->szResName, MAX_PATH, NULL, NULL );
+    return TRUE;
+}
+
+/**********************************************************************
+ *              GetIconInfoExW (USER32.@)
+ */
+BOOL WINAPI GetIconInfoExW( HICON icon, ICONINFOEXW *info )
+{
+    struct cursoricon_object *ptr;
+    BOOL ret = TRUE;
+
+    if (info->cbSize != sizeof(*info))
+    {
+        SetLastError( ERROR_INVALID_PARAMETER );
+        return FALSE;
+    }
+    if (!(ptr = get_icon_ptr( icon )))
+    {
+        SetLastError( ERROR_INVALID_CURSOR_HANDLE );
+        return FALSE;
+    }
+
+    TRACE("%p => %dx%d\n", icon, ptr->width, ptr->height);
+
+    info->fIcon        = ptr->is_icon;
+    info->xHotspot     = ptr->hotspot.x;
+    info->yHotspot     = ptr->hotspot.y;
+    info->hbmColor     = copy_bitmap( ptr->frames[0].color );
+    info->hbmMask      = copy_bitmap( ptr->frames[0].mask );
+    info->wResID       = 0;
+    info->szModName[0] = 0;
+    info->szResName[0] = 0;
+    if (ptr->module)
+    {
+        GetModuleFileNameW( ptr->module, info->szModName, MAX_PATH );
+        if (IS_INTRESOURCE( ptr->resname )) info->wResID = LOWORD( ptr->resname );
+        else lstrcpynW( info->szResName, ptr->resname, MAX_PATH );
+    }
+    if (!info->hbmMask || (!info->hbmColor && ptr->frames[0].color))
+    {
+        DeleteObject( info->hbmMask );
+        DeleteObject( info->hbmColor );
+        ret = FALSE;
+    }
+    release_icon_ptr( icon, ptr );
+    return ret;
+}
+
 /* copy an icon bitmap, even when it can't be selected into a DC */
 /* helper for CreateIconIndirect */
 static void stretch_blt_icon( HDC hdc_dst, int dst_x, int dst_y, int dst_width, int dst_height,
@@ -2674,13 +2516,30 @@ HANDLE WINAPI CopyImage( HANDLE hnd, UINT type, INT desiredx,
             return res;
         }
         case IMAGE_ICON:
-                return CURSORICON_ExtCopy(hnd,type, desiredx, desiredy, flags);
         case IMAGE_CURSOR:
-                /* Should call CURSORICON_ExtCopy but more testing
-                 * needs to be done before we change this
-                 */
-                if (flags) FIXME("Flags are ignored\n");
-                return CopyCursor(hnd);
+        {
+            struct cursoricon_object *icon;
+            HICON res = 0;
+            int depth = (flags & LR_MONOCHROME) ? 1 : GetDeviceCaps( screen_dc, BITSPIXEL );
+
+            if (flags & LR_DEFAULTSIZE)
+            {
+                if (!desiredx) desiredx = GetSystemMetrics( type == IMAGE_ICON ? SM_CXICON : SM_CXCURSOR );
+                if (!desiredy) desiredy = GetSystemMetrics( type == IMAGE_ICON ? SM_CYICON : SM_CYCURSOR );
+            }
+
+            if (!(icon = get_icon_ptr( hnd ))) return 0;
+
+            if (icon->rsrc && (flags & LR_COPYFROMRESOURCE))
+                res = CURSORICON_Load( icon->module, icon->resname, desiredx, desiredy, depth,
+                                       type == IMAGE_CURSOR, flags );
+            else
+                res = CopyIcon( hnd ); /* FIXME: change size if necessary */
+            release_icon_ptr( hnd, icon );
+
+            if (res && (flags & LR_COPYDELETEORG)) DeleteObject( hnd );
+            return res;
+        }
     }
     return 0;
 }
