@@ -38,6 +38,15 @@ struct edit_notify {
 
 static struct edit_notify notifications;
 
+static BOOL (WINAPI *pEndMenu) (void);
+
+static void init_function_pointers(void)
+{
+    HMODULE hdll = GetModuleHandleA("user32");
+
+    pEndMenu = (void*)GetProcAddress(hdll, "EndMenu");
+}
+
 static INT_PTR CALLBACK multi_edit_dialog_proc(HWND hdlg, UINT msg, WPARAM wparam, LPARAM lparam)
 {
     static int num_ok_commands = 0;
@@ -537,6 +546,7 @@ static HINSTANCE hinst;
 static HWND hwndET2;
 static const char szEditTest2Class[] = "EditTest2Class";
 static const char szEditTest3Class[] = "EditTest3Class";
+static const char szEditTest4Class[] = "EditTest4Class";
 static const char szEditTextPositionClass[] = "EditTextPositionWindowClass";
 
 static HWND create_editcontrol (DWORD style, DWORD exstyle)
@@ -1371,13 +1381,12 @@ static void test_margins(void)
 {
     HWND hwEdit;
     RECT old_rect, new_rect;
-    INT old_left_margin, old_right_margin;
+    INT old_right_margin;
     DWORD old_margins, new_margins;
 
     hwEdit = create_editcontrol(WS_BORDER | ES_AUTOHSCROLL | ES_AUTOVSCROLL, 0);
     
     old_margins = SendMessage(hwEdit, EM_GETMARGINS, 0, 0);
-    old_left_margin = LOWORD(old_margins);
     old_right_margin = HIWORD(old_margins);
     
     /* Check if setting the margins works */
@@ -2081,10 +2090,63 @@ static void test_child_edit_wmkeydown(void)
     destroy_child_editcontrol(hwEdit);
 }
 
+static int got_en_setfocus = 0;
+static int got_wm_capturechanged = 0;
+
+static LRESULT CALLBACK edit4_wnd_procA(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
+{
+    switch (msg) {
+        case WM_COMMAND:
+            switch (HIWORD(wParam)) {
+                case EN_SETFOCUS:
+                    got_en_setfocus = 1;
+                    break;
+            }
+            break;
+        case WM_CAPTURECHANGED:
+            if (hWnd != (HWND)lParam)
+            {
+                got_wm_capturechanged = 1;
+                pEndMenu();
+            }
+            break;
+    }
+    return DefWindowProcA(hWnd, msg, wParam, lParam);
+}
+
+static void test_contextmenu_focus(void)
+{
+    HWND hwndMain, hwndEdit;
+
+    hwndMain = CreateWindow(szEditTest4Class, "ET4", WS_OVERLAPPEDWINDOW|WS_VISIBLE,
+                            0, 0, 200, 200, NULL, NULL, hinst, NULL);
+    assert(hwndMain);
+
+    hwndEdit = CreateWindow("EDIT", NULL,
+                           WS_CHILD|WS_BORDER|WS_VISIBLE|ES_LEFT|ES_AUTOHSCROLL,
+                           0, 0, 150, 50, /* important this not be 0 size. */
+                           hwndMain, (HMENU) ID_EDITTEST2, hinst, NULL);
+    assert(hwndEdit);
+
+    SetFocus(NULL);
+
+    SetCapture(hwndMain);
+
+    SendMessage(hwndEdit, WM_CONTEXTMENU, (WPARAM)hwndEdit, MAKEWORD(10, 10));
+
+    ok(got_en_setfocus, "edit box didn't get focused\n");
+
+    ok(got_wm_capturechanged, "main window capture did not change\n");
+
+    DestroyWindow (hwndEdit);
+    DestroyWindow (hwndMain);
+}
+
 static BOOL RegisterWindowClasses (void)
 {
     WNDCLASSA test2;
     WNDCLASSA test3;
+    WNDCLASSA test4;
     WNDCLASSA text_position;
     
     test2.style = 0;
@@ -2110,6 +2172,18 @@ static BOOL RegisterWindowClasses (void)
     test3.lpszMenuName = NULL;
     test3.lpszClassName = szEditTest3Class;
     if (!RegisterClassA(&test3)) return FALSE;
+    
+    test4.style = 0;
+    test4.lpfnWndProc = edit4_wnd_procA;
+    test4.cbClsExtra = 0;
+    test4.cbWndExtra = 0;
+    test4.hInstance = hinst;
+    test4.hIcon = NULL;
+    test4.hCursor = LoadCursorA (NULL, IDC_ARROW);
+    test4.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
+    test4.lpszMenuName = NULL;
+    test4.lpszClassName = szEditTest4Class;
+    if (!RegisterClassA(&test4)) return FALSE;
 
     text_position.style = CS_HREDRAW | CS_VREDRAW;
     text_position.cbClsExtra = 0;
@@ -2130,6 +2204,7 @@ static void UnregisterWindowClasses (void)
 {
     UnregisterClassA(szEditTest2Class, hinst);
     UnregisterClassA(szEditTest3Class, hinst);
+    UnregisterClassA(szEditTest4Class, hinst);
     UnregisterClassA(szEditTextPositionClass, hinst);
 }
 
@@ -2360,6 +2435,8 @@ static void test_dialogmode(void)
 
 START_TEST(edit)
 {
+    init_function_pointers();
+
     hinst = GetModuleHandleA(NULL);
     assert(RegisterWindowClasses());
 
@@ -2385,6 +2462,10 @@ START_TEST(edit)
     test_child_edit_wmkeydown();
     test_fontsize();
     test_dialogmode();
+    if (pEndMenu)
+        test_contextmenu_focus();
+    else
+        win_skip("EndMenu is not available\n");
 
     UnregisterWindowClasses();
 }
