@@ -32,6 +32,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(d3d);
 struct wined3d_wndproc
 {
     HWND window;
+    BOOL unicode;
     WNDPROC proc;
     IWineD3DDeviceImpl *device;
 };
@@ -394,6 +395,7 @@ static LRESULT CALLBACK wined3d_wndproc(HWND window, UINT message, WPARAM wparam
 {
     struct wined3d_wndproc *entry;
     IWineD3DDeviceImpl *device;
+    BOOL unicode;
     WNDPROC proc;
 
     wined3d_mutex_lock();
@@ -407,10 +409,11 @@ static LRESULT CALLBACK wined3d_wndproc(HWND window, UINT message, WPARAM wparam
     }
 
     device = entry->device;
+    unicode = entry->unicode;
     proc = entry->proc;
     wined3d_mutex_unlock();
 
-    return device_process_message(device, window, message, wparam, lparam, proc);
+    return device_process_message(device, window, unicode, message, wparam, lparam, proc);
 }
 
 BOOL wined3d_register_window(HWND window, IWineD3DDeviceImpl *device)
@@ -419,6 +422,13 @@ BOOL wined3d_register_window(HWND window, IWineD3DDeviceImpl *device)
 
     wined3d_mutex_lock();
 
+    if (wined3d_find_wndproc(window))
+    {
+        wined3d_mutex_unlock();
+        WARN("Window %p is already registered with wined3d.\n", window);
+        return TRUE;
+    }
+
     if (wndproc_table.size == wndproc_table.count)
     {
         unsigned int new_size = max(1, wndproc_table.size * 2);
@@ -440,7 +450,14 @@ BOOL wined3d_register_window(HWND window, IWineD3DDeviceImpl *device)
 
     entry = &wndproc_table.entries[wndproc_table.count++];
     entry->window = window;
-    entry->proc = (WNDPROC)SetWindowLongPtrW(window, GWLP_WNDPROC, (LONG_PTR)wined3d_wndproc);
+    entry->unicode = IsWindowUnicode(window);
+    /* Set a window proc that matches the window. Some applications (e.g. NoX)
+     * replace the window proc after we've set ours, and expect to be able to
+     * call the previous one (ours) directly, without using CallWindowProc(). */
+    if (entry->unicode)
+        entry->proc = (WNDPROC)SetWindowLongPtrW(window, GWLP_WNDPROC, (LONG_PTR)wined3d_wndproc);
+    else
+        entry->proc = (WNDPROC)SetWindowLongPtrA(window, GWLP_WNDPROC, (LONG_PTR)wined3d_wndproc);
     entry->device = device;
 
     wined3d_mutex_unlock();
@@ -450,27 +467,49 @@ BOOL wined3d_register_window(HWND window, IWineD3DDeviceImpl *device)
 
 void wined3d_unregister_window(HWND window)
 {
-    unsigned int i;
+    struct wined3d_wndproc *entry, *last;
+    LONG_PTR proc;
 
     wined3d_mutex_lock();
-    for (i = 0; i < wndproc_table.count; ++i)
+
+    if (!(entry = wined3d_find_wndproc(window)))
     {
-        if (wndproc_table.entries[i].window == window)
+        wined3d_mutex_unlock();
+        ERR("Window %p is not registered with wined3d.\n", window);
+        return;
+    }
+
+    if (entry->unicode)
+    {
+        proc = GetWindowLongPtrW(window, GWLP_WNDPROC);
+        if (proc != (LONG_PTR)wined3d_wndproc)
         {
-            struct wined3d_wndproc *entry = &wndproc_table.entries[i];
-            struct wined3d_wndproc *last = &wndproc_table.entries[--wndproc_table.count];
-
-            if (GetWindowLongPtrW(window, GWLP_WNDPROC) == (LONG_PTR)wined3d_wndproc)
-                SetWindowLongPtrW(window, GWLP_WNDPROC, (LONG_PTR)entry->proc);
-            if (entry != last) *entry = *last;
             wined3d_mutex_unlock();
-
+            WARN("Not unregistering window %p, window proc %#lx doesn't match wined3d window proc %p.\n",
+                    window, proc, wined3d_wndproc);
             return;
         }
-    }
-    wined3d_mutex_unlock();
 
-    ERR("Window %p is not registered with wined3d.\n", window);
+        SetWindowLongPtrW(window, GWLP_WNDPROC, (LONG_PTR)entry->proc);
+    }
+    else
+    {
+        proc = GetWindowLongPtrA(window, GWLP_WNDPROC);
+        if (proc != (LONG_PTR)wined3d_wndproc)
+        {
+            wined3d_mutex_unlock();
+            WARN("Not unregistering window %p, window proc %#lx doesn't match wined3d window proc %p.\n",
+                    window, proc, wined3d_wndproc);
+            return;
+        }
+
+        SetWindowLongPtrA(window, GWLP_WNDPROC, (LONG_PTR)entry->proc);
+    }
+
+    last = &wndproc_table.entries[--wndproc_table.count];
+    if (entry != last) *entry = *last;
+
+    wined3d_mutex_unlock();
 }
 
 /* At process attach */
