@@ -271,6 +271,7 @@ static LPDEVMODEA DEVMODEdupWtoA(HANDLE heap, const DEVMODEW *dmW)
     LPDEVMODEA dmA;
     DWORD size;
     BOOL Formname;
+    /* there is no pointer dereference here, if your code checking tool complains it's broken */
     ptrdiff_t off_formname = (const char *)dmW->dmFormName - (const char *)dmW;
 
     if(!dmW) return NULL;
@@ -510,8 +511,6 @@ INT CDECL PSDRV_GetDeviceCaps( PSDRV_PDEVICE *physDev, INT cap )
     case VREFRESH:
     case BLTALIGNMENT:
         return 0;
-    case SHADEBLENDCAPS:
-        return SB_NONE;
     default:
         FIXME("(%p): unsupported capability %d, will return 0\n", physDev->hdc, cap );
         return 0;
