@@ -30,9 +30,7 @@
 
 #include "windef.h"
 #include "winbase.h"
-#include "winerror.h"
 #include "wingdi.h"
-#include "winspool.h"
 
 #include "psdrv.h"
 #include "wine/debug.h"
@@ -168,7 +166,7 @@ TYPE42 *T42_download_header(PSDRV_PDEVICE *physDev, char *ps_name,
       "currentdict end dup /FontName get exch definefont pop\n";
 
 
-    t42 = HeapAlloc(GetProcessHeap(), 0, sizeof(*t42));
+    t42 = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*t42));
     memcpy(t42->tables, tables_templ, sizeof(tables_templ));
     t42->loca_tab = t42->glyf_tab = t42->head_tab = t42->hmtx_tab = -1;
     t42->emsize = emsize;
@@ -287,8 +285,6 @@ BOOL T42_download_glyph(PSDRV_PDEVICE *physDev, DOWNLOAD *pdl, DWORD index,
     DWORD start, end, i;
     char *buf;
     TYPE42 *t42;
-    WORD awidth;
-    short lsb;
 
     const char glyph_def[] = 
       "/%s findfont exch 1 index\n"
@@ -316,15 +312,9 @@ BOOL T42_download_glyph(PSDRV_PDEVICE *physDev, DOWNLOAD *pdl, DWORD index,
 				      t42->glyph_sent_size * sizeof(*(t42->glyph_sent)));
     }
 
-    buf = HeapAlloc(GetProcessHeap(), 0, sizeof(glyph_def) +
-		    strlen(pdl->ps_name) + 100);
-
     if(!get_glyf_pos(t42, index, &start, &end)) return FALSE;
     TRACE("start = %x end = %x\n", start, end);
 
-    awidth = GET_BE_WORD(t42->tables[t42->hmtx_tab].data + index * 4);
-    lsb = GET_BE_WORD(t42->tables[t42->hmtx_tab].data + index * 4 + 2);
-
     if(GET_BE_WORD(t42->tables[t42->glyf_tab].data + start) == 0xffff) {
       /* Composite glyph */
         BYTE *sg_start = t42->tables[t42->glyf_tab].data + start + 10;
@@ -354,6 +344,10 @@ BOOL T42_download_glyph(PSDRV_PDEVICE *physDev, DOWNLOAD *pdl, DWORD index,
 
     for(i = 1; t42->glyf_blocks[i]; i++)
         if(start < t42->glyf_blocks[i]) break;
+
+    buf = HeapAlloc(GetProcessHeap(), 0, sizeof(glyph_def) +
+		    strlen(pdl->ps_name) + 100);
+
     /* we don't have a string for the gdir and glyf tables, but we do have a 
        string for the TT header.  So the offset we need is tables - 2 */
     sprintf(buf, "%d %d\n", t42->num_of_written_tables - 2 + i, start - t42->glyf_blocks[i-1]);
