@@ -78,6 +78,7 @@ static XContext win_data_context;
 static Time last_user_time;
 static Window user_time_window;
 
+static const char foreign_window_prop[] = "__wine_x11_foreign_window";
 static const char whole_window_prop[] = "__wine_x11_whole_window";
 static const char client_window_prop[]= "__wine_x11_client_window";
 static const char icon_window_prop[]  = "__wine_x11_icon_window";
@@ -478,8 +479,10 @@ static void sync_window_region( Display *display, struct x11drv_win_data *data,
     }
     else
     {
-        RGNDATA *pRegionData = X11DRV_GetRegionData( hrgn, 0 );
-        if (pRegionData)
+        RGNDATA *pRegionData;
+
+        if (GetWindowLongW( data->hwnd, GWL_EXSTYLE ) & WS_EX_LAYOUTRTL) MirrorRgn( data->hwnd, hrgn );
+        if ((pRegionData = X11DRV_GetRegionData( hrgn, 0 )))
         {
             wine_tsx11_lock();
             XShapeCombineRectangles( display, data->whole_window, ShapeBounding,
@@ -1354,6 +1357,8 @@ static void set_xembed_flags( Display *display, struct x11drv_win_data *data, un
 {
     unsigned long info[2];
 
+    if (!data->whole_window) return;
+
     info[0] = 0; /* protocol version */
     info[1] = flags;
     wine_tsx11_lock();
@@ -1416,18 +1421,19 @@ static void unmap_window( Display *display, struct x11drv_win_data *data )
  */
 void make_window_embedded( Display *display, struct x11drv_win_data *data )
 {
-    if (data->mapped)
-    {
-        /* the window cannot be mapped before being embedded */
-        unmap_window( display, data );
-        data->embedded = TRUE;
+    BOOL was_mapped = data->mapped;
+    /* the window cannot be mapped before being embedded */
+    if (data->mapped) unmap_window( display, data );
+
+    data->embedded = TRUE;
+    data->managed = TRUE;
+    SetPropA( data->hwnd, managed_prop, (HANDLE)1 );
+    sync_window_style( display, data );
+
+    if (was_mapped)
         map_window( display, data, 0 );
-    }
     else
-    {
-        data->embedded = TRUE;
         set_xembed_flags( display, data, 0 );
-    }
 }
 
 
@@ -1600,7 +1606,7 @@ static void move_window_bits( struct x11drv_win_data *data, const RECT *old_rect
     RECT dst_rect = *new_rect;
     HDC hdc_src, hdc_dst;
     INT code;
-    HRGN rgn;
+    HRGN rgn = 0;
     HWND parent = 0;
 
     if (!data->whole_window)
@@ -1619,9 +1625,6 @@ static void move_window_bits( struct x11drv_win_data *data, const RECT *old_rect
         hdc_src = hdc_dst = GetDCEx( data->hwnd, 0, DCX_CACHE );
     }
 
-    rgn = CreateRectRgnIndirect( &dst_rect );
-    SelectClipRgn( hdc_dst, rgn );
-    DeleteObject( rgn );
     ExcludeUpdateRgn( hdc_dst, data->hwnd );
 
     code = X11DRV_START_EXPOSURES;
@@ -1634,7 +1637,6 @@ static void move_window_bits( struct x11drv_win_data *data, const RECT *old_rect
             dst_rect.right - dst_rect.left, dst_rect.bottom - dst_rect.top,
             hdc_src, src_rect.left, src_rect.top, SRCCOPY );
 
-    rgn = 0;
     code = X11DRV_END_EXPOSURES;
     ExtEscape( hdc_dst, X11DRV_ESCAPE, sizeof(code), (LPSTR)&code, sizeof(rgn), (LPSTR)&rgn );
 
@@ -1752,7 +1754,23 @@ done:
  */
 static void destroy_whole_window( Display *display, struct x11drv_win_data *data, BOOL already_destroyed )
 {
-    if (!data->whole_window) return;
+    if (!data->whole_window)
+    {
+        if (data->embedded)
+        {
+            Window xwin = (Window)GetPropA( data->hwnd, foreign_window_prop );
+            if (xwin)
+            {
+                wine_tsx11_lock();
+                if (!already_destroyed) XSelectInput( display, xwin, 0 );
+                XDeleteContext( display, xwin, winContext );
+                wine_tsx11_unlock();
+                RemovePropA( data->hwnd, foreign_window_prop );
+            }
+        }
+        return;
+    }
+
 
     TRACE( "win %p xwin %lx/%lx\n", data->hwnd, data->whole_window, data->client_window );
     wine_tsx11_lock();
@@ -1883,9 +1901,10 @@ void X11DRV_DestroyNotify( HWND hwnd, XEvent *event )
     struct x11drv_win_data *data;
 
     if (!(data = X11DRV_get_win_data( hwnd ))) return;
+    if (!data->embedded) FIXME( "window %p/%lx destroyed from the outside\n", hwnd, data->whole_window );
 
-    FIXME( "window %p/%lx destroyed from the outside\n", hwnd, data->whole_window );
     destroy_whole_window( display, data, TRUE );
+    if (data->embedded) SendMessageW( hwnd, WM_CLOSE, 0, 0 );
 }
 
 
@@ -1932,9 +1951,10 @@ BOOL CDECL X11DRV_CreateDesktopWindow( HWND hwnd )
     SERVER_START_REQ( get_window_rectangles )
     {
         req->handle = wine_server_user_handle( hwnd );
+        req->relative = COORDS_CLIENT;
         wine_server_call( req );
-        width  = reply->window.right - reply->window.left;
-        height = reply->window.bottom - reply->window.top;
+        width  = reply->window.right;
+        height = reply->window.bottom;
     }
     SERVER_END_REQ;
 
@@ -2036,6 +2056,119 @@ struct x11drv_win_data *X11DRV_create_win_data( HWND hwnd )
 }
 
 
+/* window procedure for foreign windows */
+static LRESULT WINAPI foreign_window_proc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
+{
+    switch(msg)
+    {
+    case WM_PARENTNOTIFY:
+        if (LOWORD(wparam) == WM_DESTROY)
+        {
+            TRACE( "%p: got parent notify destroy for win %lx\n", hwnd, lparam );
+            PostMessageW( hwnd, WM_CLOSE, 0, 0 );  /* so that we come back here once the child is gone */
+        }
+        return 0;
+    case WM_CLOSE:
+        if (GetWindow( hwnd, GW_CHILD )) return 0;  /* refuse to die if we still have children */
+        break;
+    }
+    return DefWindowProcW( hwnd, msg, wparam, lparam );
+}
+
+
+/***********************************************************************
+ *		create_foreign_window
+ *
+ * Create a foreign window for the specified X window and its ancestors
+ */
+HWND create_foreign_window( Display *display, Window xwin )
+{
+    static const WCHAR classW[] = {'_','_','w','i','n','e','_','x','1','1','_','f','o','r','e','i','g','n','_','w','i','n','d','o','w',0};
+    static BOOL class_registered;
+    struct x11drv_win_data *data;
+    HWND hwnd, parent;
+    Window xparent, xroot;
+    Window *xchildren;
+    unsigned int nchildren;
+    XWindowAttributes attr;
+    DWORD style = WS_CLIPCHILDREN;
+
+    if (!class_registered)
+    {
+        WNDCLASSEXW class;
+
+        memset( &class, 0, sizeof(class) );
+        class.cbSize        = sizeof(class);
+        class.lpfnWndProc   = foreign_window_proc;
+        class.lpszClassName = classW;
+        if (!RegisterClassExW( &class ) && GetLastError() != ERROR_CLASS_ALREADY_EXISTS)
+        {
+            ERR( "Could not register foreign window class\n" );
+            return FALSE;
+        }
+        class_registered = TRUE;
+    }
+
+    wine_tsx11_lock();
+    if (XFindContext( display, xwin, winContext, (char **)&hwnd )) hwnd = 0;
+    if (hwnd)  /* already created */
+    {
+        wine_tsx11_unlock();
+        return hwnd;
+    }
+
+    XSelectInput( display, xwin, StructureNotifyMask );
+    if (!XGetWindowAttributes( display, xwin, &attr ) ||
+        !XQueryTree( display, xwin, &xroot, &xparent, &xchildren, &nchildren ))
+    {
+        XSelectInput( display, xwin, 0 );
+        wine_tsx11_unlock();
+        return 0;
+    }
+    XFree( xchildren );
+    wine_tsx11_unlock();
+
+    if (xparent == xroot)
+    {
+        parent = GetDesktopWindow();
+        style |= WS_POPUP;
+        attr.x += virtual_screen_rect.left;
+        attr.y += virtual_screen_rect.top;
+    }
+    else
+    {
+        parent = create_foreign_window( display, xparent );
+        style |= WS_CHILD;
+    }
+
+    hwnd = CreateWindowW( classW, NULL, style, attr.x, attr.y, attr.width, attr.height,
+                          parent, 0, 0, NULL );
+
+    if (!(data = alloc_win_data( display, hwnd )))
+    {
+        DestroyWindow( hwnd );
+        return 0;
+    }
+    SetRect( &data->window_rect, attr.x, attr.y, attr.x + attr.width, attr.y + attr.height );
+    data->whole_rect = data->client_rect = data->window_rect;
+    data->whole_window = data->client_window = 0;
+    data->embedded = TRUE;
+    data->mapped = TRUE;
+
+    SetPropA( hwnd, foreign_window_prop, (HANDLE)xwin );
+    wine_tsx11_lock();
+    XSaveContext( display, xwin, winContext, (char *)data->hwnd );
+    wine_tsx11_unlock();
+
+    ShowWindow( hwnd, SW_SHOW );
+
+    TRACE( "win %lx parent %p style %08x %s -> hwnd %p\n",
+           xwin, parent, style, wine_dbgstr_rect(&data->window_rect), hwnd );
+
+    return hwnd;
+}
+
+
 /***********************************************************************
  *		X11DRV_get_whole_window
  *
@@ -2099,6 +2232,7 @@ void CDECL X11DRV_GetDC( HDC hdc, HWND hwnd, HWND top, const RECT *win_rect,
 {
     struct x11drv_escape_set_drawable escape;
     struct x11drv_win_data *data = X11DRV_get_win_data( hwnd );
+    HWND parent;
 
     escape.code        = X11DRV_SET_DRAWABLE;
     escape.mode        = IncludeInferiors;
@@ -2107,6 +2241,15 @@ void CDECL X11DRV_GetDC( HDC hdc, HWND hwnd, HWND top, const RECT *win_rect,
     escape.pixmap      = 0;
     escape.gl_copy     = FALSE;
 
+    escape.dc_rect.left         = win_rect->left - top_rect->left;
+    escape.dc_rect.top          = win_rect->top - top_rect->top;
+    escape.dc_rect.right        = win_rect->right - top_rect->left;
+    escape.dc_rect.bottom       = win_rect->bottom - top_rect->top;
+    escape.drawable_rect.left   = top_rect->left;
+    escape.drawable_rect.top    = top_rect->top;
+    escape.drawable_rect.right  = top_rect->right;
+    escape.drawable_rect.bottom = top_rect->bottom;
+
     if (top == hwnd)
     {
         escape.fbconfig_id = data ? data->fbconfig_id : (XID)GetPropA( hwnd, fbconfig_id_prop );
@@ -2123,7 +2266,19 @@ void CDECL X11DRV_GetDC( HDC hdc, HWND hwnd, HWND top, const RECT *win_rect,
     }
     else
     {
-        escape.drawable    = X11DRV_get_client_window( top );
+        /* find the first ancestor that has a drawable */
+        for (parent = hwnd; parent && parent != top; parent = GetAncestor( parent, GA_PARENT ))
+            if ((escape.drawable = X11DRV_get_client_window( parent ))) break;
+
+        if (escape.drawable)
+        {
+            POINT pt = { 0, 0 };
+            MapWindowPoints( top, parent, &pt, 1 );
+            OffsetRect( &escape.dc_rect, pt.x, pt.y );
+            OffsetRect( &escape.drawable_rect, -pt.x, -pt.y );
+        }
+        else escape.drawable = X11DRV_get_client_window( top );
+
         escape.fbconfig_id = data ? data->fbconfig_id : (XID)GetPropA( hwnd, fbconfig_id_prop );
         escape.gl_drawable = data ? data->gl_drawable : (Drawable)GetPropA( hwnd, gl_drawable_prop );
         escape.pixmap      = data ? data->pixmap : (Pixmap)GetPropA( hwnd, pixmap_prop );
@@ -2131,15 +2286,6 @@ void CDECL X11DRV_GetDC( HDC hdc, HWND hwnd, HWND top, const RECT *win_rect,
         if (flags & DCX_CLIPCHILDREN) escape.mode = ClipByChildren;
     }
 
-    escape.dc_rect.left         = win_rect->left - top_rect->left;
-    escape.dc_rect.top          = win_rect->top - top_rect->top;
-    escape.dc_rect.right        = win_rect->right - top_rect->left;
-    escape.dc_rect.bottom       = win_rect->bottom - top_rect->top;
-    escape.drawable_rect.left   = top_rect->left;
-    escape.drawable_rect.top    = top_rect->top;
-    escape.drawable_rect.right  = top_rect->right;
-    escape.drawable_rect.bottom = top_rect->bottom;
-
     ExtEscape( hdc, X11DRV_ESCAPE, sizeof(escape), (LPSTR)&escape, 0, NULL );
 }
 
@@ -2210,6 +2356,7 @@ void CDECL X11DRV_SetParent( HWND hwnd, HWND parent, HWND old_parent )
 
     if (!data) return;
     if (parent == old_parent) return;
+    if (data->embedded) return;
 
     if (parent != GetDesktopWindow()) /* a child window */
     {
@@ -2360,13 +2507,14 @@ void CDECL X11DRV_WindowPosChanged( HWND hwnd, HWND insert_after, UINT swp_flags
     if (thread_data->current_event && thread_data->current_event->xany.window == data->whole_window)
         event_type = thread_data->current_event->type;
 
-    if (event_type != ConfigureNotify && event_type != PropertyNotify)
+    if (event_type != ConfigureNotify && event_type != PropertyNotify &&
+        event_type != GravityNotify && event_type != ReparentNotify)
         event_type = 0;  /* ignore other events */
 
-    if (data->mapped)
+    if (data->mapped && event_type != ReparentNotify)
     {
         if (((swp_flags & SWP_HIDEWINDOW) && !(new_style & WS_VISIBLE)) ||
-            (!event_type &&
+            (event_type != ConfigureNotify &&
              !is_window_rect_mapped( rectWindow ) && is_window_rect_mapped( &old_window_rect )))
             unmap_window( display, data );
     }
