@@ -18,8 +18,7 @@
 #ifndef __WINTRUST_PRIV_H__
 #define __WINTRUST_PRIV_H__
 
-void * WINAPI WINTRUST_Alloc(DWORD cb);
-void * WINAPI WINTRUST_ReAlloc(void *ptr, DWORD cb);
+void * WINAPI WINTRUST_Alloc(DWORD cb) __WINE_ALLOC_SIZE(1);
 void WINAPI WINTRUST_Free(void *p);
 BOOL WINAPI WINTRUST_AddStore(CRYPT_PROVIDER_DATA *data, HCERTSTORE store);
 BOOL WINAPI WINTRUST_AddSgnr(CRYPT_PROVIDER_DATA *data,
