@@ -122,6 +122,9 @@ INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP)
                                 )
 
 #define INTERNET_ERROR_MASK_INSERT_CDROM                    0x1
+#define INTERNET_ERROR_MASK_COMBINED_SEC_CERT               0x2
+#define INTERNET_ERROR_MASK_NEED_MSN_SSPI_PKG               0x4
+#define INTERNET_ERROR_MASK_LOGIN_FAILURE_DISPLAY_ENTITY_BODY 0x8
 
 #define INTERNET_OPTIONS_MASK   (~INTERNET_FLAGS_MASK)
 #define WININET_API_FLAG_ASYNC          0x00000001
@@ -371,6 +374,19 @@ typedef struct _INTERNET_PER_CONN_OPTION_LISTW {
 DECL_WINELIB_TYPE_AW(INTERNET_PER_CONN_OPTION_LIST)
 DECL_WINELIB_TYPE_AW(LPINTERNET_PER_CONN_OPTION_LIST)
 
+typedef struct _INTERNET_DIAGNOSTIC_SOCKET_INFO
+{
+    DWORD_PTR Socket;
+    DWORD     SourcePort;
+    DWORD     DestPort;
+    DWORD     Flags;
+} INTERNET_DIAGNOSTIC_SOCKET_INFO, *LPINTERNET_DIAGNOSTIC_SOCKET_INFO;
+
+#define IDSI_FLAG_KEEP_ALIVE 0x00000001
+#define IDSI_FLAG_SECURE     0x00000002
+#define IDSI_FLAG_PROXY      0x00000004
+#define IDSI_FLAG_TUNNEL     0x00000008
+
 BOOLAPI InternetTimeFromSystemTimeA(CONST SYSTEMTIME *,DWORD ,LPSTR ,DWORD);
 BOOLAPI InternetTimeFromSystemTimeW(CONST SYSTEMTIME *,DWORD ,LPWSTR ,DWORD);
 #define InternetTimeFromSystemTime WINELIB_NAME_AW(InternetTimeFromSystemTime)
