@@ -141,7 +141,7 @@ static char *get_runtime_libdir(void)
 static char *get_runtime_bindir( const char *argv0 )
 {
     char *p, *bindir, *cwd;
-    size_t len, size;
+    int len, size;
 
 #ifdef linux
     for (size = 256; ; size *= 2)
@@ -470,6 +470,7 @@ static void preloader_exec( char **argv, int use_preloader )
         execv( full_name, new_argv );
         free( new_argv );
         free( full_name );
+        return;
     }
 #endif
     execv( argv[0], argv );
