@@ -25,8 +25,6 @@
  *   functions which can be implemented as macros)
  * - finish buffer scrolling (mainly, need to decide of a nice way for 
  *   requesting the UP/DOWN operations
- * - Resizing (unix) terminal does not change (Win32) console size.
- * - Initial console size comes from registry and not from terminal size.
  */
 
 #include "config.h"
@@ -35,6 +33,9 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
+#ifdef HAVE_POLL_H
+# include <poll.h>
+#endif
 #ifdef HAVE_NCURSES_H
 # include <ncurses.h>
 #elif defined(HAVE_CURSES_H)
@@ -53,14 +54,13 @@
 #include "winecon_private.h"
 
 #include "wine/library.h"
-#include "wine/server.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(curses);
 
 #define PRIVATE(data)   ((struct inner_data_curse*)((data)->private))
 
-#if defined(HAVE_CURSES_H) || defined(HAVE_NCURSES_H)
+#if defined(SONAME_LIBCURSES) || defined(SONAME_LIBNCURSES)
 
 #ifdef HAVE_NCURSES_H
 # define CURSES_NAME "ncurses"
@@ -71,13 +71,14 @@ WINE_DEFAULT_DEBUG_CHANNEL(curses);
 struct inner_data_curse 
 {
     unsigned long       initial_mouse_mask;
-    HANDLE              hInput;
+    int                 sync_pipe[2];
+    HANDLE              input_thread;
+    CRITICAL_SECTION    lock;
     WINDOW*             pad;
     chtype*             line;
     int                 allow_scroll;
 };
 
-
 static void *nc_handle = NULL;
 
 #ifdef initscr  /* work around Solaris breakage */
@@ -122,6 +123,7 @@ MAKE_FUNCPTR(wgetch)
 MAKE_FUNCPTR(mouseinterval)
 MAKE_FUNCPTR(mousemask)
 #endif
+MAKE_FUNCPTR(acs_map)
 
 #undef MAKE_FUNCPTR
 
@@ -129,10 +131,10 @@ MAKE_FUNCPTR(mousemask)
 
 static BOOL WCCURSES_bind_libcurses(void)
 {
-#ifdef HAVE_NCURSES_H
-    static const char *ncname = SONAME_LIBNCURSES;
+#ifdef SONAME_LIBNCURSES
+    static const char ncname[] = SONAME_LIBNCURSES;
 #else
-    static const char *ncname = SONAME_LIBCURSES;
+    static const char ncname[] = SONAME_LIBCURSES;
 #endif
 
     nc_handle = wine_dlopen(ncname, RTLD_NOW, NULL, 0);
@@ -185,6 +187,7 @@ static BOOL WCCURSES_bind_libcurses(void)
     LOAD_FUNCPTR(mouseinterval)
     LOAD_FUNCPTR(mousemask)
 #endif
+    LOAD_FUNCPTR(acs_map)
 
 #undef LOAD_FUNCPTR
 
@@ -233,6 +236,7 @@ sym_not_found:
 #define waddchnstr p_waddchnstr
 #define wmove p_wmove
 #define wgetch p_wgetch
+#define acs_map (*p_acs_map)
 
 /******************************************************************
  *		WCCURSES_ResizeScreenBuffer
@@ -335,8 +339,8 @@ static void	WCCURSES_SetTitle(const struct inner_data* data)
     if (WINECON_GetConsoleTitle(data->hConIn, wbuf, sizeof(wbuf)/sizeof(WCHAR)))
     {
         char        buffer[256];
-        
-        WideCharToMultiByte(CP_ACP, 0, wbuf, -1, buffer, sizeof(buffer), 
+
+        WideCharToMultiByte(CP_UNIXCP, 0, wbuf, -1, buffer, sizeof(buffer),
                             NULL, NULL);
         fputs("\033]2;", stdout);
         fputs(buffer, stdout);
@@ -356,16 +360,80 @@ static void WCCURSES_Refresh(const struct inner_data* data, int tp, int bm)
     int         y;
     CHAR_INFO*	cell;
     DWORD       attr;
-    char        ch;
 
     for (y = tp; y <= bm; y++)
     {
 	cell = &data->cells[y * data->curcfg.sb_width];
         for (x = 0; x < data->curcfg.sb_width; x++)
         {
-            WideCharToMultiByte(CP_ACP, 0, &cell[x].Char.UnicodeChar, 1, 
-                                &ch, 1, NULL, NULL);
-            attr = ((BYTE)ch < 32 || (BYTE)ch > 127) ? 32 : (BYTE)ch;
+            /* check for some mapping to ACS characters (drawing boxes, arrows) */
+            if ((cell[x].Char.UnicodeChar >= 0x2500 && cell[x].Char.UnicodeChar <= 0x257F) ||
+                (cell[x].Char.UnicodeChar >= 0x2190 && cell[x].Char.UnicodeChar <= 0x21FF))
+            {
+                /* FIXME: we're also mapping heavy and lines item to single lines
+                 * (that's ugly, but that's better than crap)
+                 * Moreover, as the ACS_ macros refer to values in array acs_map[], we
+                 * cannot simply build static tables for the mapping (FIXME: this could be done
+                 * at load time)
+                 */
+                switch (cell[x].Char.UnicodeChar)
+                {
+                case 0x2190: case 0x219E: case 0x21A2: case 0x21A4:
+                case 0x21BC: case 0x21BD: case 0x21D0: case 0x21E6: attr = ACS_LARROW;   break;
+                case 0x2191: case 0x219F: case 0x21A3: case 0x21A5:
+                case 0x21BE: case 0x21BF: case 0x21D1: case 0x21E7: attr = ACS_UARROW;   break;
+                case 0x2192: case 0x21A0: case 0x21A6: case 0x21C0:
+                case 0x21C1: case 0x21D2: case 0x21E8:              attr = ACS_RARROW;   break;
+                case 0x2193: case 0x21A1: case 0x21A7: case 0x21C2:
+                case 0x21C3: case 0x21D3: case 0x21E9:              attr = ACS_DARROW;   break;
+
+                case 0x2500: case 0x2501: case 0x257C: case 0x257E: attr = ACS_HLINE;    break;
+                case 0x2502: case 0x2503: case 0x257D: case 0x257F: attr = ACS_VLINE;    break;
+                case 0x250C: case 0x250D: case 0x250E: case 0x250F: attr = ACS_ULCORNER; break;
+                case 0x2510: case 0x2511: case 0x2512: case 0x2513: attr = ACS_URCORNER; break;
+                case 0x2514: case 0x2515: case 0x2516: case 0x2517: attr = ACS_LLCORNER; break;
+                case 0x2518: case 0x2519: case 0x251A: case 0x251B: attr = ACS_LRCORNER; break;
+                case 0x251C: case 0x251D: case 0x251E: case 0x251F:
+                case 0x2520: case 0x2521: case 0x2522: case 0x2523: attr = ACS_LTEE;     break;
+                case 0x2524: case 0x2525: case 0x2526: case 0x2527:
+                case 0x2528: case 0x2529: case 0x252A: case 0x252B: attr = ACS_RTEE;     break;
+
+                case 0x252C: case 0x252D: case 0x252E: case 0x252F:
+                case 0x2530: case 0x2531: case 0x2532: case 0x2533: attr = ACS_TTEE;     break;
+                case 0x2534: case 0x2535: case 0x2536: case 0x2537:
+                case 0x2538: case 0x2539: case 0x253A: case 0x253B: attr = ACS_BTEE;     break;
+
+                case 0x253C: case 0x253D: case 0x253E: case 0x253F:
+                case 0x2540: case 0x2541: case 0x2542: case 0x2543:
+                case 0x2544: case 0x2545: case 0x2546: case 0x2547:
+                case 0x2548: case 0x2549: case 0x254A: case 0x254B: attr = ACS_PLUS;     break;
+
+                case 0x2550:                                        attr = ACS_HLINE;    break;
+                case 0x2551:                                        attr = ACS_VLINE;    break;
+                case 0x2552: case 0x2553: case 0x2554:              attr = ACS_ULCORNER; break;
+                case 0x2555: case 0x2556: case 0x2557:              attr = ACS_URCORNER; break;
+                case 0x2558: case 0x2559: case 0x255A:              attr = ACS_LLCORNER; break;
+                case 0x255B: case 0x255C: case 0x255D:              attr = ACS_LRCORNER; break;
+                case 0x255E: case 0x255F: case 0x2560:              attr = ACS_LTEE;     break;
+                case 0x2561: case 0x2562: case 0x2563:              attr = ACS_RTEE;     break;
+                case 0x2564: case 0x2565: case 0x2566:              attr = ACS_TTEE;     break;
+                case 0x2567: case 0x2568: case 0x2569:              attr = ACS_BTEE;     break;
+                case 0x256A: case 0x256B: case 0x256C:              attr = ACS_PLUS;     break;
+                default:
+                    WINE_FIXME("Unmapped special character (%x)\n", cell[x].Char.UnicodeChar);
+                    attr = ' ';
+                }
+            }
+            else
+            {
+                char     ch[2];
+
+                if (WideCharToMultiByte(CP_UNIXCP, 0, &cell[x].Char.UnicodeChar, 1,
+                                        ch, sizeof(ch), NULL, NULL) == 1)
+                    attr = ((BYTE)ch[0] < 32) ? 32 : (BYTE)ch[0];
+                else
+                    attr = 32;
+            }
 
             if (cell[x].Attributes & FOREGROUND_RED)       attr |= COLOR_PAIR(COLOR_RED);
             if (cell[x].Attributes & FOREGROUND_BLUE)      attr |= COLOR_PAIR(COLOR_BLUE);
@@ -412,6 +480,18 @@ static void WCCURSES_SetFont(struct inner_data* data, const WCHAR* font,
     /* FIXME: really not much to do ? */
 }
 
+/******************************************************************
+ *		WCCURSES_Resize
+ *
+ */
+static void WCCURSES_Resize(struct inner_data* data)
+{
+    int width, height;
+
+    getmaxyx(stdscr, height, width);
+    WINECON_ResizeWithContainer(data, width, height);
+}
+
 /******************************************************************
  *		WCCURSES_ScrollV
  *
@@ -419,22 +499,21 @@ static void WCCURSES_SetFont(struct inner_data* data, const WCHAR* font,
  */
 static void WCCURSES_ScrollV(struct inner_data* data, int delta)
 {
-    int	pos = data->curcfg.win_pos.Y;
+    struct config_data  cfg = data->curcfg;
 
-    pos += delta;
-    if (pos < 0) pos = 0;
-    if (pos > data->curcfg.sb_height - data->curcfg.win_height)
-        pos = data->curcfg.sb_height - data->curcfg.win_height;
-    if (pos != data->curcfg.win_pos.Y)
+    cfg.win_pos.Y += delta;
+    if (cfg.win_pos.Y < 0) cfg.win_pos.Y = 0;
+    if (cfg.win_pos.Y > data->curcfg.sb_height - data->curcfg.win_height)
+        cfg.win_pos.Y = data->curcfg.sb_height - data->curcfg.win_height;
+    if (cfg.win_pos.Y != data->curcfg.win_pos.Y)
     {
-        data->curcfg.win_pos.Y = pos;
         WCCURSES_PosCursor(data);
-        WINECON_NotifyWindowChange(data);
+        WINECON_SetConfig(data, &cfg);
     }
 }
 
 /* Ascii -> VK, generated by calling VkKeyScanA(i) */
-static int vkkeyscan_table[256] = 
+static const int vkkeyscan_table[256] =
 {
      0,0,0,0,0,0,0,0,8,9,0,0,0,13,0,0,0,0,0,19,145,556,0,0,0,0,0,27,0,0,0,
      0,32,305,478,307,308,309,311,222,313,304,312,443,188,189,190,191,48,
@@ -448,7 +527,7 @@ static int vkkeyscan_table[256] =
      0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,400,0,0,0,0,0,0
 };
 
-static int mapvkey_0[256] = 
+static const int mapvkey_0[256] =
 {
      0,0,0,0,0,0,0,0,14,15,0,0,0,28,0,0,42,29,56,69,58,0,0,0,0,0,0,1,0,0,
      0,0,57,73,81,79,71,75,72,77,80,0,0,0,55,82,83,0,11,2,3,4,5,6,7,8,9,
@@ -487,6 +566,7 @@ static unsigned WCCURSES_FillSimpleChar(INPUT_RECORD* ir, unsigned real_inchar)
 {
     unsigned vk;
     unsigned inchar;
+    char ch;
     unsigned numEvent = 0;
     DWORD    cks = 0;
 
@@ -538,8 +618,9 @@ static unsigned WCCURSES_FillSimpleChar(INPUT_RECORD* ir, unsigned real_inchar)
         ir[numEvent].Event.KeyEvent.dwControlKeyState |= LEFT_ALT_PRESSED;
     ir[numEvent].Event.KeyEvent.wVirtualKeyCode = vk;
     ir[numEvent].Event.KeyEvent.wVirtualScanCode = mapvkey_0[vk & 0x00ff]; /* VirtualKeyCodes to ScanCode */
-    ir[numEvent].Event.KeyEvent.uChar.UnicodeChar = (unsigned char)inchar;
 
+    ch = inchar;
+    MultiByteToWideChar(CP_UNIXCP, 0,&ch,1,&ir[numEvent].Event.KeyEvent.uChar.UnicodeChar, 1);
     ir[numEvent + 1] = ir[numEvent];
     ir[numEvent + 1].Event.KeyEvent.bKeyDown      = 0;
 
@@ -763,7 +844,12 @@ static unsigned WCCURSES_FillCode(struct inner_data* data, INPUT_RECORD* ir, int
     case KEY_MOUSE:
         numEvent = WCCURSES_FillMouse(ir);
         break;
-        
+#ifdef KEY_RESIZE
+    case KEY_RESIZE:
+        WCCURSES_Resize(data);
+        break;
+#endif
+
     case KEY_MOVE:
     case KEY_NEXT:
     case KEY_OPEN:
@@ -781,9 +867,6 @@ static unsigned WCCURSES_FillCode(struct inner_data* data, INPUT_RECORD* ir, int
     case KEY_SCOMMAND:
     case KEY_SCOPY:
     case KEY_SCREATE:
-#ifdef KEY_RESIZE
-    case KEY_RESIZE:
-#endif
         goto notFound;
 
     case KEY_SDC:
@@ -844,31 +927,47 @@ static unsigned WCCURSES_FillCode(struct inner_data* data, INPUT_RECORD* ir, int
 }
 
 /******************************************************************
- *		WCCURSES_GetEvents
- *
- *
+ *		input_thread
  */
-static void WCCURSES_GetEvents(struct inner_data* data)
+static DWORD CALLBACK input_thread( void *arg )
 {
+    struct inner_data* data = arg;
     int		        inchar;
     INPUT_RECORD        ir[8];
     unsigned		numEvent;
     DWORD               n;
-    
-    if ((inchar = wgetch(stdscr)) == ERR) {WINE_FIXME("Ooch. somebody beat us\n");return;}
-    
-    WINE_TRACE("Got o%o (0x%x)\n", inchar,inchar);
-    
-    if (inchar >= KEY_MIN && inchar <= KEY_MAX)
+    struct pollfd pfd[2];
+
+    pfd[0].fd = 0;
+    pfd[0].events = POLLIN;
+    pfd[1].fd = PRIVATE(data)->sync_pipe[0];
+    pfd[1].events = POLLHUP;
+
+    for (;;)
     {
-        numEvent = WCCURSES_FillCode(data, ir, inchar);
+        pfd[0].revents = pfd[1].revents = 0;
+        if (poll( pfd, 2, -1 ) == -1) break;
+        if (pfd[0].revents & (POLLHUP|POLLERR)) break;
+        if (pfd[1].revents & (POLLHUP|POLLERR)) break;
+        if (!(pfd[0].revents & POLLIN)) continue;
+
+        /* we're called from input thread (not main thread), so force unique access */
+        EnterCriticalSection(&PRIVATE(data)->lock);
+        if ((inchar = wgetch(stdscr)) != ERR)
+        {
+            WINE_TRACE("Got o%o (0x%x)\n", inchar,inchar);
+
+            if (inchar >= KEY_MIN && inchar <= KEY_MAX)
+                numEvent = WCCURSES_FillCode(data, ir, inchar);
+            else
+                numEvent = WCCURSES_FillSimpleChar(ir, inchar);
+
+            if (numEvent) WriteConsoleInputW(data->hConIn, ir, numEvent, &n);
+        }
+        LeaveCriticalSection(&PRIVATE(data)->lock);
     }
-    else
-    {
-        numEvent = WCCURSES_FillSimpleChar(ir, inchar);
-    }
-    if (numEvent)
-        WriteConsoleInput(data->hConIn, ir, numEvent, &n);
+    close( PRIVATE(data)->sync_pipe[0] );
+    return 0;
 }
 
 /******************************************************************
@@ -880,7 +979,13 @@ static void WCCURSES_DeleteBackend(struct inner_data* data)
 {
     if (!PRIVATE(data)) return;
 
-    CloseHandle(PRIVATE(data)->hInput);
+    if (PRIVATE(data)->input_thread)
+    {
+        close( PRIVATE(data)->sync_pipe[1] );
+        WaitForSingleObject( PRIVATE(data)->input_thread, INFINITE );
+        CloseHandle( PRIVATE(data)->input_thread );
+    }
+    DeleteCriticalSection(&PRIVATE(data)->lock);
 
     delwin(PRIVATE(data)->pad);
 #ifdef HAVE_MOUSEMASK
@@ -903,28 +1008,26 @@ static void WCCURSES_DeleteBackend(struct inner_data* data)
  */
 static int WCCURSES_MainLoop(struct inner_data* data)
 {
-    HANDLE hin[2];
+    DWORD       id;
+    BOOL        cont = TRUE;
 
-    hin[0] = PRIVATE(data)->hInput;
-    hin[1] = data->hSynchro;
+    WCCURSES_Resize(data);
 
-    for (;;) 
+    if (pipe( PRIVATE(data)->sync_pipe ) == -1) return 0;
+    PRIVATE(data)->input_thread = CreateThread( NULL, 0, input_thread, data, 0, &id );
+
+    while (cont && WaitForSingleObject(data->hSynchro, INFINITE) == WAIT_OBJECT_0)
     {
-        unsigned ret = WaitForMultipleObjects(2, hin, FALSE, INFINITE);
-        switch (ret)
-        {
-        case WAIT_OBJECT_0:
-            WCCURSES_GetEvents(data);
-            break;
-        case WAIT_OBJECT_0+1:
-            if (!WINECON_GrabChanges(data)) return 0;
-            break;
-	default:
-	    WINE_ERR("got pb\n");
-	    /* err */
-	    break;
-        }
+        EnterCriticalSection(&PRIVATE(data)->lock);
+        cont = WINECON_GrabChanges(data);
+        LeaveCriticalSection(&PRIVATE(data)->lock);
     }
+
+    close( PRIVATE(data)->sync_pipe[1] );
+    WaitForSingleObject( PRIVATE(data)->input_thread, INFINITE );
+    CloseHandle( PRIVATE(data)->input_thread );
+    PRIVATE(data)->input_thread = 0;
+    return 0;
 }
 
 /******************************************************************
@@ -951,13 +1054,7 @@ enum init_return WCCURSES_InitBackend(struct inner_data* data)
     data->fnScroll             = WCCURSES_Scroll;
     data->fnSetFont            = WCCURSES_SetFont;
     data->fnDeleteBackend      = WCCURSES_DeleteBackend;
-
-    if (wine_server_fd_to_handle(0, GENERIC_READ|SYNCHRONIZE, 0,
-                                 (obj_handle_t*)&PRIVATE(data)->hInput))
-    {
-        WINE_FIXME("Cannot open 0\n");
-        return init_failed;
-    }
+    data->hWnd                 = NULL;
 
     /* FIXME: should find a good way to enable buffer scrolling
      * For the time being, setting this to 1 will allow scrolling up/down 
@@ -1006,6 +1103,7 @@ enum init_return WCCURSES_InitBackend(struct inner_data* data)
         PRIVATE(data)->initial_mouse_mask = mm;
     }
 #endif
+    InitializeCriticalSection(&PRIVATE(data)->lock);
 
     return init_success;
 }
@@ -1013,6 +1111,8 @@ enum init_return WCCURSES_InitBackend(struct inner_data* data)
 #else
 enum init_return WCCURSES_InitBackend(struct inner_data* data)
 {
+    WINE_ERR("(n)curses was not found at configuration time.\n"
+             "If you want (n)curses support, please install relevant packages.\n");
     return init_not_supported;
 }
 #endif
