@@ -30,12 +30,18 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
+#ifdef HAVE_SYS_STAT_H
+# include <sys/stat.h>
+#endif
+#ifdef HAVE_SYS_MMAN_H
+#include <sys/mman.h>
+#endif
 
 #include "build.h"
 
-#define MAX_TMP_FILES 8
-static const char *tmp_files[MAX_TMP_FILES];
+static const char **tmp_files;
 static unsigned int nb_tmp_files;
+static unsigned int max_tmp_files;
 
 static const struct
 {
@@ -48,17 +54,19 @@ static const struct
     { "i586",    CPU_x86 },
     { "i686",    CPU_x86 },
     { "i786",    CPU_x86 },
+    { "amd64",   CPU_x86_64 },
     { "x86_64",  CPU_x86_64 },
     { "sparc",   CPU_SPARC },
     { "alpha",   CPU_ALPHA },
-    { "powerpc", CPU_POWERPC }
+    { "powerpc", CPU_POWERPC },
+    { "arm", CPU_ARM }
 };
 
 /* atexit handler to clean tmp files */
 static void cleanup_tmp_files(void)
 {
     unsigned int i;
-    for (i = 0; i < MAX_TMP_FILES; i++) if (tmp_files[i]) unlink( tmp_files[i] );
+    for (i = 0; i < nb_tmp_files; i++) if (tmp_files[i]) unlink( tmp_files[i] );
 }
 
 
@@ -111,6 +119,65 @@ int strendswith(const char* str, const char* end)
     return l >= m && strcmp(str + l - m, end) == 0;
 }
 
+char *strmake( const char* fmt, ... )
+{
+    int n;
+    size_t size = 100;
+    va_list ap;
+
+    for (;;)
+    {
+        char *p = xmalloc( size );
+        va_start( ap, fmt );
+	n = vsnprintf( p, size, fmt, ap );
+	va_end( ap );
+        if (n == -1) size *= 2;
+        else if ((size_t)n >= size) size = n + 1;
+        else return p;
+        free( p );
+    }
+}
+
+struct strarray *strarray_init(void)
+{
+    struct strarray *array = xmalloc( sizeof(*array) );
+    array->count = 0;
+    array->max = 16;
+    array->str = xmalloc( array->max * sizeof(*array->str) );
+    return array;
+}
+
+static void strarray_add_one( struct strarray *array, const char *str )
+{
+    if (array->count == array->max)
+    {
+        array->max *= 2;
+        array->str = xrealloc( array->str, array->max * sizeof(*array->str) );
+    }
+    array->str[array->count++] = str;
+}
+
+void strarray_add( struct strarray *array, ... )
+{
+    va_list valist;
+    const char *str;
+
+    va_start( valist, array );
+    while ((str = va_arg( valist, const char *))) strarray_add_one( array, str );
+    va_end( valist );
+}
+
+void strarray_addv( struct strarray *array, char * const *argv )
+{
+    while (*argv) strarray_add_one( array, *argv++ );
+}
+
+void strarray_free( struct strarray *array )
+{
+    free( array->str );
+    free( array );
+}
+
 void fatal_error( const char *msg, ... )
 {
     va_list valist;
@@ -191,16 +258,37 @@ int output( const char *format, ... )
     return ret;
 }
 
+void spawn( struct strarray *args )
+{
+    unsigned int i;
+    int status;
+
+    strarray_add_one( args, NULL );
+    if (verbose)
+        for (i = 0; args->str[i]; i++)
+            fprintf( stderr, "%s%c", args->str[i], args->str[i+1] ? ' ' : '\n' );
+
+    if ((status = spawnvp( _P_WAIT, args->str[0], args->str )))
+    {
+	if (status > 0) fatal_error( "%s failed with status %u\n", args->str[0], status );
+	else fatal_perror( "winebuild" );
+	exit( 1 );
+    }
+}
+
 /* find a build tool in the path, trying the various names */
-char *find_tool( const char * const *names )
+char *find_tool( const char *name, const char * const *names )
 {
     static char **dirs;
     static unsigned int count, maxlen;
 
     char *p, *file;
+    const char *alt_names[2];
     unsigned int i, len;
     struct stat st;
 
+    if (target_alias) return strmake( "%s-%s", target_alias, name );
+
     if (!dirs)
     {
         char *path;
@@ -223,9 +311,16 @@ char *find_tool( const char * const *names )
         for (i = 0; i < count; i++) maxlen = max( maxlen, strlen(dirs[i])+2 );
     }
 
+    if (!names)
+    {
+        alt_names[0] = name;
+        alt_names[1] = NULL;
+        names = alt_names;
+    }
+
     while (*names)
     {
-        len = strlen(*names) + 1;
+        len = strlen(*names) + sizeof(EXEEXT) + 1;
         file = xmalloc( maxlen + len );
 
         for (i = 0; i < count; i++)
@@ -235,111 +330,81 @@ char *find_tool( const char * const *names )
             if (p == file) *p++ = '.';
             if (p[-1] != '/') *p++ = '/';
             strcpy( p, *names );
+            strcat( p, EXEEXT );
 
             if (!stat( file, &st ) && S_ISREG(st.st_mode) && (st.st_mode & 0111)) return file;
         }
         free( file );
         names++;
     }
-    return NULL;
+    return xstrdup( name );
 }
 
-const char *get_as_command(void)
+struct strarray *get_as_command(void)
 {
+    struct strarray *args = strarray_init();
+
     if (!as_command)
     {
-        if (target_alias)
-        {
-            as_command = xmalloc( strlen(target_alias) + sizeof("-as") );
-            strcpy( as_command, target_alias );
-            strcat( as_command, "-as" );
-        }
-        else
-        {
-            static const char * const commands[] = { "gas", "as", NULL };
-            if (!(as_command = find_tool( commands ))) as_command = xstrdup("as");
-        }
+        static const char * const commands[] = { "gas", "as", NULL };
+        as_command = find_tool( "as", commands );
+    }
+    strarray_add_one( args, as_command );
 
-        if (force_pointer_size)
+    if (force_pointer_size)
+    {
+        switch (target_platform)
         {
-            const char *args = (target_platform == PLATFORM_APPLE) ?
-                ((force_pointer_size == 8) ? " -arch x86_64" : " -arch i386") :
-                ((force_pointer_size == 8) ? " --64" : " --32");
-            as_command = xrealloc( as_command, strlen(as_command) + strlen(args) + 1 );
-            strcat( as_command, args );
+        case PLATFORM_APPLE:
+            strarray_add( args, "-arch", (force_pointer_size == 8) ? "x86_64" : "i386", NULL );
+            break;
+        default:
+            strarray_add_one( args, (force_pointer_size == 8) ? "--64" : "--32" );
+            break;
         }
     }
-    return as_command;
+    return args;
 }
 
-const char *get_ld_command(void)
+struct strarray *get_ld_command(void)
 {
+    struct strarray *args = strarray_init();
+
     if (!ld_command)
     {
-        if (target_alias)
-        {
-            ld_command = xmalloc( strlen(target_alias) + sizeof("-ld") );
-            strcpy( ld_command, target_alias );
-            strcat( ld_command, "-ld" );
-        }
-        else
-        {
-            static const char * const commands[] = { "ld", "gld", NULL };
-            if (!(ld_command = find_tool( commands ))) ld_command = xstrdup("ld");
-        }
+        static const char * const commands[] = { "ld", "gld", NULL };
+        ld_command = find_tool( "ld", commands );
+    }
+    strarray_add_one( args, ld_command );
 
-        if (force_pointer_size)
+    if (force_pointer_size)
+    {
+        switch (target_platform)
         {
-            const char *args = (target_platform == PLATFORM_APPLE) ?
-                ((force_pointer_size == 8) ? " -arch x86_64" : " -arch i386") :
-                ((force_pointer_size == 8) ? " -m elf_x86_64" : " -m elf_i386");
-            ld_command = xrealloc( ld_command, strlen(ld_command) + strlen(args) + 1 );
-            strcat( ld_command, args );
+        case PLATFORM_APPLE:
+            strarray_add( args, "-arch", (force_pointer_size == 8) ? "x86_64" : "i386", NULL );
+            break;
+        case PLATFORM_FREEBSD:
+            strarray_add( args, "-m", (force_pointer_size == 8) ? "elf_x86_64_fbsd" : "elf_i386_fbsd", NULL );
+            break;
+        default:
+            strarray_add( args, "-m", (force_pointer_size == 8) ? "elf_x86_64" : "elf_i386", NULL );
+            break;
         }
     }
-    return ld_command;
+    return args;
 }
 
 const char *get_nm_command(void)
 {
     if (!nm_command)
     {
-        if (target_alias)
-        {
-            nm_command = xmalloc( strlen(target_alias) + sizeof("-nm") );
-            strcpy( nm_command, target_alias );
-            strcat( nm_command, "-nm" );
-        }
-        else
-        {
-            static const char * const commands[] = { "nm", "gnm", NULL };
-            if (!(nm_command = find_tool( commands ))) nm_command = xstrdup("nm");
-        }
+        static const char * const commands[] = { "nm", "gnm", NULL };
+        nm_command = find_tool( "nm", commands );
     }
     return nm_command;
 }
 
-const char *get_windres_command(void)
-{
-    static char *windres_command;
-
-    if (!windres_command)
-    {
-        if (target_alias)
-        {
-            windres_command = xmalloc( strlen(target_alias) + sizeof("-windres") );
-            strcpy( windres_command, target_alias );
-            strcat( windres_command, "-windres" );
-        }
-        else
-        {
-            static const char * const commands[] = { "windres", NULL };
-            if (!(windres_command = find_tool( commands ))) windres_command = xstrdup("windres");
-        }
-    }
-    return windres_command;
-}
-
 /* get a name for a temp file, automatically cleaned up on exit */
 char *get_temp_file_name( const char *prefix, const char *suffix )
 {
@@ -347,7 +412,6 @@ char *get_temp_file_name( const char *prefix, const char *suffix )
     const char *ext;
     int fd;
 
-    assert( nb_tmp_files < MAX_TMP_FILES );
     if (!nb_tmp_files && !save_temps) atexit( cleanup_tmp_files );
 
     if (!prefix || !prefix[0]) prefix = "winebuild";
@@ -366,10 +430,165 @@ char *get_temp_file_name( const char *prefix, const char *suffix )
         fatal_error( "could not generate a temp file\n" );
 
     close( fd );
+    if (nb_tmp_files >= max_tmp_files)
+    {
+        max_tmp_files = max( 2 * max_tmp_files, 8 );
+        tmp_files = xrealloc( tmp_files, max_tmp_files * sizeof(tmp_files[0]) );
+    }
     tmp_files[nb_tmp_files++] = name;
     return name;
 }
 
+/*******************************************************************
+ *         buffer management
+ *
+ * Function for reading from/writing to a memory buffer.
+ */
+
+int byte_swapped = 0;
+const char *input_buffer_filename;
+const unsigned char *input_buffer;
+size_t input_buffer_pos;
+size_t input_buffer_size;
+unsigned char *output_buffer;
+size_t output_buffer_pos;
+size_t output_buffer_size;
+
+static void check_output_buffer_space( size_t size )
+{
+    if (output_buffer_pos + size >= output_buffer_size)
+    {
+        output_buffer_size = max( output_buffer_size * 2, output_buffer_pos + size );
+        output_buffer = xrealloc( output_buffer, output_buffer_size );
+    }
+}
+
+void init_input_buffer( const char *file )
+{
+    int fd;
+    struct stat st;
+
+    if ((fd = open( file, O_RDONLY | O_BINARY )) == -1) fatal_perror( "Cannot open %s", file );
+    if ((fstat( fd, &st ) == -1)) fatal_perror( "Cannot stat %s", file );
+    if (!st.st_size) fatal_error( "%s is an empty file\n", file );
+#ifdef	HAVE_MMAP
+    if ((input_buffer = mmap( NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0 )) == (void*)-1)
+#endif
+    {
+        unsigned char *buffer = xmalloc( st.st_size );
+        if (read( fd, buffer, st.st_size ) != st.st_size) fatal_error( "Cannot read %s\n", file );
+        input_buffer = buffer;
+    }
+    close( fd );
+    input_buffer_filename = xstrdup( file );
+    input_buffer_size = st.st_size;
+    input_buffer_pos = 0;
+    byte_swapped = 0;
+}
+
+void init_output_buffer(void)
+{
+    output_buffer_size = 1024;
+    output_buffer_pos = 0;
+    output_buffer = xmalloc( output_buffer_size );
+}
+
+void flush_output_buffer(void)
+{
+    if (fwrite( output_buffer, 1, output_buffer_pos, output_file ) != output_buffer_pos)
+        fatal_error( "Error writing to %s\n", output_file_name );
+    free( output_buffer );
+}
+
+unsigned char get_byte(void)
+{
+    if (input_buffer_pos >= input_buffer_size)
+        fatal_error( "%s is a truncated file\n", input_buffer_filename );
+    return input_buffer[input_buffer_pos++];
+}
+
+unsigned short get_word(void)
+{
+    unsigned short ret;
+
+    if (input_buffer_pos + sizeof(ret) > input_buffer_size)
+        fatal_error( "%s is a truncated file\n", input_buffer_filename );
+    memcpy( &ret, input_buffer + input_buffer_pos, sizeof(ret) );
+    if (byte_swapped) ret = (ret << 8) | (ret >> 8);
+    input_buffer_pos += sizeof(ret);
+    return ret;
+}
+
+unsigned int get_dword(void)
+{
+    unsigned int ret;
+
+    if (input_buffer_pos + sizeof(ret) > input_buffer_size)
+        fatal_error( "%s is a truncated file\n", input_buffer_filename );
+    memcpy( &ret, input_buffer + input_buffer_pos, sizeof(ret) );
+    if (byte_swapped)
+        ret = ((ret << 24) | ((ret << 8) & 0x00ff0000) | ((ret >> 8) & 0x0000ff00) | (ret >> 24));
+    input_buffer_pos += sizeof(ret);
+    return ret;
+}
+
+void put_data( const void *data, size_t size )
+{
+    check_output_buffer_space( size );
+    memcpy( output_buffer + output_buffer_pos, data, size );
+    output_buffer_pos += size;
+}
+
+void put_byte( unsigned char val )
+{
+    check_output_buffer_space( 1 );
+    output_buffer[output_buffer_pos++] = val;
+}
+
+void put_word( unsigned short val )
+{
+    if (byte_swapped) val = (val << 8) | (val >> 8);
+    put_data( &val, sizeof(val) );
+}
+
+void put_dword( unsigned int val )
+{
+    if (byte_swapped)
+        val = ((val << 24) | ((val << 8) & 0x00ff0000) | ((val >> 8) & 0x0000ff00) | (val >> 24));
+    put_data( &val, sizeof(val) );
+}
+
+void put_qword( unsigned int val )
+{
+    if (byte_swapped)
+    {
+        put_dword( 0 );
+        put_dword( val );
+    }
+    else
+    {
+        put_dword( val );
+        put_dword( 0 );
+    }
+}
+
+/* pointer-sized word */
+void put_pword( unsigned int val )
+{
+    if (get_ptr_size() == 8) put_qword( val );
+    else put_dword( val );
+}
+
+void align_output( unsigned int align )
+{
+    size_t size = align - (output_buffer_pos % align);
+
+    if (size == align) return;
+    check_output_buffer_space( size );
+    memset( output_buffer + output_buffer_pos, 0, size );
+    output_buffer_pos += size;
+}
+
 /* output a standard header for generated files */
 void output_standard_file_header(void)
 {
@@ -409,10 +628,7 @@ FILE *open_input_file( const char *srcdir, const char *name )
 
     if (!file && srcdir)
     {
-        fullname = xmalloc( strlen(srcdir) + strlen(name) + 2 );
-        strcpy( fullname, srcdir );
-        strcat( fullname, "/" );
-        strcat( fullname, name );
+        fullname = strmake( "%s/%s", srcdir, name );
         file = fopen( fullname, "r" );
     }
     else fullname = xstrdup( name );
@@ -462,16 +678,10 @@ int remove_stdcall_decoration( char *name )
  */
 void assemble_file( const char *src_file, const char *obj_file )
 {
-    const char *prog = get_as_command();
-    char *cmd;
-    int err;
-
-    cmd = xmalloc( strlen(prog) + strlen(obj_file) + strlen(src_file) + 6 );
-    sprintf( cmd, "%s -o %s %s", prog, obj_file, src_file );
-    if (verbose) fprintf( stderr, "%s\n", cmd );
-    err = system( cmd );
-    if (err) fatal_error( "%s failed with status %d\n", prog, err );
-    free( cmd );
+    struct strarray *args = get_as_command();
+    strarray_add( args, "-o", obj_file, src_file, NULL );
+    spawn( args );
+    strarray_free( args );
 }
 
 
@@ -569,16 +779,19 @@ const char *make_c_identifier( const char *str )
  */
 const char *get_stub_name( const ORDDEF *odp, const DLLSPEC *spec )
 {
-    static char buffer[256];
+    static char *buffer;
+
+    free( buffer );
     if (odp->name || odp->export_name)
     {
         char *p;
-        sprintf( buffer, "__wine_stub_%s", odp->name ? odp->name : odp->export_name );
+        buffer = strmake( "__wine_stub_%s", odp->name ? odp->name : odp->export_name );
         /* make sure name is a legal C identifier */
         for (p = buffer; *p; p++) if (!isalnum(*p) && *p != '_') break;
         if (!*p) return buffer;
+        free( buffer );
     }
-    sprintf( buffer, "__wine_stub_%s_%d", make_c_identifier(spec->file_name), odp->ordinal );
+    buffer = strmake( "__wine_stub_%s_%d", make_c_identifier(spec->file_name), odp->ordinal );
     return buffer;
 }
 
@@ -631,6 +844,7 @@ unsigned int get_alignment(unsigned int align)
         /* fall through */
     case CPU_POWERPC:
     case CPU_ALPHA:
+    case CPU_ARM:
         n = 0;
         while ((1u << n) != align) n++;
         return n;
@@ -648,6 +862,7 @@ unsigned int get_page_size(void)
     case CPU_x86:     return 4096;
     case CPU_x86_64:  return 4096;
     case CPU_POWERPC: return 4096;
+    case CPU_ARM:     return 4096;
     case CPU_SPARC:   return 8192;
     case CPU_ALPHA:   return 8192;
     }
@@ -665,6 +880,7 @@ unsigned int get_ptr_size(void)
     case CPU_POWERPC:
     case CPU_SPARC:
     case CPU_ALPHA:
+    case CPU_ARM:
         return 4;
     case CPU_x86_64:
         return 8;
@@ -674,18 +890,47 @@ unsigned int get_ptr_size(void)
     return 0;
 }
 
+/* return the total size in bytes of the arguments on the stack */
+unsigned int get_args_size( const ORDDEF *odp )
+{
+    unsigned int i, size;
+
+    for (i = size = 0; i < odp->u.func.nb_args; i++)
+    {
+        switch (odp->u.func.args[i])
+        {
+        case ARG_INT64:
+        case ARG_DOUBLE:
+            size += 8;
+            break;
+        case ARG_INT128:
+            /* int128 is passed as pointer on x86_64 */
+            if (target_cpu != CPU_x86_64)
+            {
+                size += 16;
+                break;
+            }
+            /* fall through */
+        default:
+            size += get_ptr_size();
+            break;
+        }
+    }
+    return size;
+}
+
 /* return the assembly name for a C symbol */
 const char *asm_name( const char *sym )
 {
-    static char buffer[256];
+    static char *buffer;
 
     switch (target_platform)
     {
     case PLATFORM_APPLE:
     case PLATFORM_WINDOWS:
         if (sym[0] == '.' && sym[1] == 'L') return sym;
-        buffer[0] = '_';
-        strcpy( buffer + 1, sym );
+        free( buffer );
+        buffer = strmake( "_%s", sym );
         return buffer;
     default:
         return sym;
@@ -695,17 +940,27 @@ const char *asm_name( const char *sym )
 /* return an assembly function declaration for a C function name */
 const char *func_declaration( const char *func )
 {
-    static char buffer[256];
+    static char *buffer;
 
     switch (target_platform)
     {
     case PLATFORM_APPLE:
         return "";
     case PLATFORM_WINDOWS:
-        sprintf( buffer, ".def _%s; .scl 2; .type 32; .endef", func );
+        free( buffer );
+        buffer = strmake( ".def _%s; .scl 2; .type 32; .endef", func );
         break;
     default:
-        sprintf( buffer, ".type %s,@function", func );
+        free( buffer );
+        switch(target_cpu)
+        {
+        case CPU_ARM:
+            buffer = strmake( ".type %s,%%function", func );
+            break;
+        default:
+            buffer = strmake( ".type %s,@function", func );
+            break;
+        }
         break;
     }
     return buffer;
@@ -725,6 +980,19 @@ void output_function_size( const char *name )
     }
 }
 
+/* output a .cfi directive */
+void output_cfi( const char *format, ... )
+{
+    va_list valist;
+
+    if (!unwind_tables) return;
+    va_start( valist, format );
+    fputc( '\t', output_file );
+    vfprintf( output_file, format, valist );
+    fputc( '\n', output_file );
+    va_end( valist );
+}
+
 /* output the GNU note for non-exec stack */
 void output_gnu_stack_note(void)
 {
@@ -734,7 +1002,15 @@ void output_gnu_stack_note(void)
     case PLATFORM_APPLE:
         break;
     default:
-        output( "\t.section .note.GNU-stack,\"\",@progbits\n" );
+        switch(target_cpu)
+        {
+        case CPU_ARM:
+            output( "\t.section .note.GNU-stack,\"\",%%progbits\n" );
+            break;
+        default:
+            output( "\t.section .note.GNU-stack,\"\",@progbits\n" );
+            break;
+        }
         break;
     }
 }
@@ -742,20 +1018,22 @@ void output_gnu_stack_note(void)
 /* return a global symbol declaration for an assembly symbol */
 const char *asm_globl( const char *func )
 {
-    static char buffer[256];
+    static char *buffer;
 
+    free( buffer );
     switch (target_platform)
     {
     case PLATFORM_APPLE:
-        sprintf( buffer, "\t.globl _%s\n\t.private_extern _%s\n_%s:", func, func, func );
-        return buffer;
+        buffer = strmake( "\t.globl _%s\n\t.private_extern _%s\n_%s:", func, func, func );
+        break;
     case PLATFORM_WINDOWS:
-        sprintf( buffer, "\t.globl _%s\n_%s:", func, func );
-        return buffer;
+        buffer = strmake( "\t.globl _%s\n_%s:", func, func );
+        break;
     default:
-        sprintf( buffer, "\t.globl %s\n\t.hidden %s\n%s:", func, func, func );
-        return buffer;
+        buffer = strmake( "\t.globl %s\n\t.hidden %s\n%s:", func, func, func );
+        break;
     }
+    return buffer;
 }
 
 const char *get_asm_ptr_keyword(void)
